@@ -41,18 +41,29 @@
 #include "DFGFailedFinalizer.h"
 #include "DFGFlushLivenessAnalysisPhase.h"
 #include "DFGFixupPhase.h"
+#include "DFGInvalidationPointInjectionPhase.h"
 #include "DFGJITCompiler.h"
 #include "DFGLICMPhase.h"
 #include "DFGLivenessAnalysisPhase.h"
 #include "DFGLoopPreHeaderCreationPhase.h"
 #include "DFGOSRAvailabilityAnalysisPhase.h"
+#include "DFGOSREntrypointCreationPhase.h"
 #include "DFGPredictionInjectionPhase.h"
 #include "DFGPredictionPropagationPhase.h"
+#include "DFGResurrectionForValidationPhase.h"
 #include "DFGSSAConversionPhase.h"
+#include "DFGSSALoweringPhase.h"
+#include "DFGStackLayoutPhase.h"
+#include "DFGStoreBarrierElisionPhase.h"
+#include "DFGStrengthReductionPhase.h"
+#include "DFGTierUpCheckInjectionPhase.h"
 #include "DFGTypeCheckHoistingPhase.h"
 #include "DFGUnificationPhase.h"
 #include "DFGValidate.h"
 #include "DFGVirtualRegisterAllocationPhase.h"
+#include "DFGWatchpointCollectionPhase.h"
+#include "Debugger.h"
+#include "OperandsInlines.h"
 #include "Operations.h"
 #include <wtf/CurrentTime.h>
 
@@ -63,6 +74,7 @@
 #include "FTLLink.h"
 #include "FTLLowerDFGToLLVM.h"
 #include "FTLState.h"
+#include "InitializeLLVM.h"
 #endif
 
 namespace JSC { namespace DFG {
@@ -80,14 +92,13 @@ static void dumpAndVerifyGraph(Graph& graph, const char* text)
 }
 
 Plan::Plan(
-    CompileMode compileMode, PassRefPtr<CodeBlock> passedCodeBlock, unsigned osrEntryBytecodeIndex,
-    unsigned numVarsWithValues)
-    : compileMode(compileMode)
-    , vm(*passedCodeBlock->vm())
+    PassRefPtr<CodeBlock> passedCodeBlock, CompilationMode mode,
+    unsigned osrEntryBytecodeIndex, const Operands<JSValue>& mustHandleValues)
+    : vm(*passedCodeBlock->vm())
     , codeBlock(passedCodeBlock)
+    , mode(mode)
     , osrEntryBytecodeIndex(osrEntryBytecodeIndex)
-    , numVarsWithValues(numVarsWithValues)
-    , mustHandleValues(codeBlock->numParameters(), numVarsWithValues)
+    , mustHandleValues(mustHandleValues)
     , compilation(codeBlock->vm()->m_perBytecodeProfiler ? adoptRef(new Profiler::Compilation(codeBlock->vm()->m_perBytecodeProfiler->ensureBytecodesFor(codeBlock.get()), Profiler::DFG)) : 0)
     , identifiers(codeBlock.get())
     , weakReferences(codeBlock.get())
@@ -109,7 +120,7 @@ void Plan::compileInThread(LongLivedState& longLivedState)
     CompilationScope compilationScope;
 
     if (logCompilationChanges())
-        dataLog("DFG(Plan) compiling ", *codeBlock, ", number of instructions = ", codeBlock->instructionCount(), "\n");
+        dataLog("DFG(Plan) compiling ", *codeBlock, " with ", mode, ", number of instructions = ", codeBlock->instructionCount(), "\n");
 
     CompilationPath path = compileInThreadImpl(longLivedState);
 
@@ -133,7 +144,7 @@ void Plan::compileInThread(LongLivedState& longLivedState)
             break;
         }
         double now = currentTimeMS();
-        dataLog("Optimized ", *codeBlock->alternative(), " with ", pathName, " in ", now - before, " ms");
+        dataLog("Optimized ", *codeBlock->alternative(), " using ", mode, " with ", pathName, " in ", now - before, " ms");
         if (path == FTLPath)
             dataLog(" (DFG: ", beforeFTL - before, ", LLVM: ", now - beforeFTL, ")");
         dataLog(".\n");
@@ -142,6 +153,12 @@ void Plan::compileInThread(LongLivedState& longLivedState)
 
 Plan::CompilationPath Plan::compileInThreadImpl(LongLivedState& longLivedState)
 {
+    if (verboseCompilationEnabled() && osrEntryBytecodeIndex != UINT_MAX) {
+        dataLog("\n");
+        dataLog("Compiler must handle OSR entry from bc#", osrEntryBytecodeIndex, " with values: ", mustHandleValues, "\n");
+        dataLog("\n");
+    }
+    
     Graph dfg(vm, *this, longLivedState);
     
     if (!parse(dfg)) {
@@ -162,12 +179,22 @@ Plan::CompilationPath Plan::compileInThreadImpl(LongLivedState& longLivedState)
     performUnification(dfg);
     performPredictionInjection(dfg);
     
+    if (mode == FTLForOSREntryMode) {
+        bool result = performOSREntrypointCreation(dfg);
+        if (!result) {
+            finalizer = adoptPtr(new FailedFinalizer(*this));
+            return FailPath;
+        }
+        performCPSRethreading(dfg);
+    }
+    
     if (validationEnabled())
         validate(dfg);
     
     performBackwardsPropagation(dfg);
     performPredictionPropagation(dfg);
     performFixup(dfg);
+    performInvalidationPointInjection(dfg);
     performTypeCheckHoisting(dfg);
     
     unsigned count = 1;
@@ -180,6 +207,7 @@ Plan::CompilationPath Plan::compileInThreadImpl(LongLivedState& longLivedState)
         if (validationEnabled())
             validate(dfg);
         
+        changed |= performStrengthReduction(dfg);
         performCFA(dfg);
         changed |= performConstantFolding(dfg);
         changed |= performArgumentsSimplification(dfg);
@@ -197,6 +225,7 @@ Plan::CompilationPath Plan::compileInThreadImpl(LongLivedState& longLivedState)
 
     dfg.m_fixpointState = FixpointConverged;
 
+    performStoreBarrierElision(dfg);
     performStoreElimination(dfg);
     
     // If we're doing validation, then run some analyses, to give them an opportunity
@@ -206,29 +235,60 @@ Plan::CompilationPath Plan::compileInThreadImpl(LongLivedState& longLivedState)
         dfg.m_naturalLoops.computeIfNecessary(dfg);
     }
 
+    switch (mode) {
+    case DFGMode: {
+        performTierUpCheckInjection(dfg);
+
+        performCPSRethreading(dfg);
+        performDCE(dfg);
+        performStackLayout(dfg);
+        performVirtualRegisterAllocation(dfg);
+        performWatchpointCollection(dfg);
+        dumpAndVerifyGraph(dfg, "Graph after optimization:");
+        
+        JITCompiler dataFlowJIT(dfg);
+        if (codeBlock->codeType() == FunctionCode) {
+            dataFlowJIT.compileFunction();
+            dataFlowJIT.linkFunction();
+        } else {
+            dataFlowJIT.compile();
+            dataFlowJIT.link();
+        }
+        
+        return DFGPath;
+    }
+    
+    case FTLMode:
+    case FTLForOSREntryMode: {
 #if ENABLE(FTL_JIT)
-    if (Options::useExperimentalFTL()
-        && compileMode == CompileFunction
-        && FTL::canCompile(dfg)) {
+        if (FTL::canCompile(dfg) == FTL::CannotCompile) {
+            finalizer = adoptPtr(new FailedFinalizer(*this));
+            return FailPath;
+        }
         
         performCriticalEdgeBreaking(dfg);
         performLoopPreHeaderCreation(dfg);
         performCPSRethreading(dfg);
         performSSAConversion(dfg);
+        performSSALowering(dfg);
         performLivenessAnalysis(dfg);
         performCFA(dfg);
         performLICM(dfg);
+        performCSE(dfg);
         performLivenessAnalysis(dfg);
         performCFA(dfg);
+        if (Options::validateFTLOSRExitLiveness())
+            performResurrectionForValidation(dfg);
         performDCE(dfg); // We rely on this to convert dead SetLocals into the appropriate hint, and to kill dead code that won't be recognized as dead by LLVM.
+        performStackLayout(dfg);
         performLivenessAnalysis(dfg);
         performFlushLivenessAnalysis(dfg);
         performOSRAvailabilityAnalysis(dfg);
+        performWatchpointCollection(dfg);
         
         dumpAndVerifyGraph(dfg, "Graph just before FTL lowering:");
         
-        // FIXME: Support OSR entry.
-        // https://bugs.webkit.org/show_bug.cgi?id=113625
+        initializeLLVM();
         
         FTL::State state(dfg);
         FTL::lowerDFGToLLVM(state);
@@ -236,36 +296,30 @@ Plan::CompilationPath Plan::compileInThreadImpl(LongLivedState& longLivedState)
         if (Options::reportCompileTimes())
             beforeFTL = currentTimeMS();
         
-        if (Options::llvmAlwaysFails()) {
+        if (Options::llvmAlwaysFailsBeforeCompile()) {
             FTL::fail(state);
             return FTLPath;
         }
         
         FTL::compile(state);
+
+        if (Options::llvmAlwaysFailsBeforeLink()) {
+            FTL::fail(state);
+            return FTLPath;
+        }
+        
         FTL::link(state);
         return FTLPath;
-    }
 #else
-    RELEASE_ASSERT(!Options::useExperimentalFTL());
+        RELEASE_ASSERT_NOT_REACHED();
+        return FailPath;
 #endif // ENABLE(FTL_JIT)
-    
-    performCPSRethreading(dfg);
-    performDCE(dfg);
-    performVirtualRegisterAllocation(dfg);
-    dumpAndVerifyGraph(dfg, "Graph after optimization:");
-
-    JITCompiler dataFlowJIT(dfg);
-    if (compileMode == CompileFunction) {
-        dataFlowJIT.compileFunction();
-        dataFlowJIT.linkFunction();
-    } else {
-        ASSERT(compileMode == CompileOther);
-        
-        dataFlowJIT.compile();
-        dataFlowJIT.link();
     }
-    
-    return DFGPath;
+        
+    default:
+        RELEASE_ASSERT_NOT_REACHED();
+        return FailPath;
+    }
 }
 
 bool Plan::isStillValid()
@@ -276,35 +330,53 @@ bool Plan::isStillValid()
 
 void Plan::reallyAdd(CommonData* commonData)
 {
-    watchpoints.reallyAdd();
+    watchpoints.reallyAdd(codeBlock.get(), *commonData);
     identifiers.reallyAdd(vm, commonData);
     weakReferences.reallyAdd(vm, commonData);
     transitions.reallyAdd(vm, commonData);
     writeBarriers.trigger(vm);
 }
 
-CompilationResult Plan::finalize(RefPtr<JSC::JITCode>& jitCode, MacroAssemblerCodePtr* jitCodeWithArityCheck)
+void Plan::notifyReady()
+{
+    callback->compilationDidBecomeReadyAsynchronously(codeBlock.get());
+    isCompiled = true;
+}
+
+CompilationResult Plan::finalizeWithoutNotifyingCallback()
 {
     if (!isStillValid())
         return CompilationInvalidated;
-    
+
+    if (vm.enabledProfiler())
+        return CompilationInvalidated;
+
+    Debugger* debugger = codeBlock->globalObject()->debugger();
+    if (debugger && (debugger->isStepping() || codeBlock->baselineAlternative()->hasDebuggerRequests()))
+        return CompilationInvalidated;
+
     bool result;
-    if (compileMode == CompileFunction)
-        result = finalizer->finalizeFunction(jitCode, *jitCodeWithArityCheck);
+    if (codeBlock->codeType() == FunctionCode)
+        result = finalizer->finalizeFunction();
     else
-        result = finalizer->finalize(jitCode);
+        result = finalizer->finalize();
     
     if (!result)
         return CompilationFailed;
     
-    reallyAdd(jitCode->dfgCommon());
+    reallyAdd(codeBlock->jitCode()->dfgCommon());
     
     return CompilationSuccessful;
 }
 
-CodeBlock* Plan::key()
+void Plan::finalizeAndNotifyCallback()
 {
-    return codeBlock->alternative();
+    callback->compilationDidComplete(codeBlock.get(), finalizeWithoutNotifyingCallback());
+}
+
+CompilationKey Plan::key()
+{
+    return CompilationKey(codeBlock->alternative(), mode);
 }
 
 } } // namespace JSC::DFG
