@@ -27,54 +27,38 @@
 #define IncrementalSweeper_h
 
 #include "HeapTimer.h"
-#include "MarkedBlock.h"
-#include <wtf/HashSet.h>
 #include <wtf/PassOwnPtr.h>
-#include <wtf/RetainPtr.h>
 #include <wtf/Vector.h>
 
 namespace JSC {
 
 class Heap;
-
-struct CopyFunctor : public MarkedBlock::VoidFunctor {
-    CopyFunctor(Vector<MarkedBlock*>& blocks) 
-        : m_index(0) 
-        , m_blocks(blocks)
-    {
-    }
-
-    void operator()(MarkedBlock* block) { m_blocks[m_index++] = block; }
-
-    size_t m_index;
-    Vector<MarkedBlock*>& m_blocks;
-};
+class MarkedBlock;
 
 class IncrementalSweeper : public HeapTimer {
 public:
-    static IncrementalSweeper* create(Heap*);
-    void startSweeping(const HashSet<MarkedBlock*>& blockSnapshot);
-    virtual void doWork();
+    static PassOwnPtr<IncrementalSweeper> create(Heap*);
+    void startSweeping(Vector<MarkedBlock*>&);
+    JS_EXPORT_PRIVATE virtual void doWork() override;
     void sweepNextBlock();
-    bool structuresCanBeSwept();
     void willFinishSweeping();
 
-private:
+protected:
 #if USE(CF)
-    IncrementalSweeper(Heap*, CFRunLoopRef);
-    
+    JS_EXPORT_PRIVATE IncrementalSweeper(Heap*, CFRunLoopRef);
+#else
+    IncrementalSweeper(VM*);
+#endif
+
+#if USE(CF)
+private:
     void doSweep(double startTime);
     void scheduleTimer();
     void cancelTimer();
     
     unsigned m_currentBlockToSweepIndex;
-    Vector<MarkedBlock*> m_blocksToSweep;
-#else
-    
-    IncrementalSweeper(JSGlobalData*);
-    
+    Vector<MarkedBlock*>& m_blocksToSweep;
 #endif
-    bool m_structuresCanBeSwept;
 };
 
 } // namespace JSC
