@@ -32,30 +32,52 @@
 #include <wtf/ThreadSafeRefCounted.h>
 #include <wtf/Threading.h>
 
+#if USE(WEB_THREAD)
+#include <wtf/MainThread.h>
+#endif
+
 namespace WTF {
 
-namespace Internal {
-
 template<typename T>
-class WeakReference : public ThreadSafeRefCounted<WeakReference<T> > {
+class WeakReference : public ThreadSafeRefCounted<WeakReference<T>> {
     WTF_MAKE_NONCOPYABLE(WeakReference<T>);
     WTF_MAKE_FAST_ALLOCATED;
 public:
-    static PassRefPtr<WeakReference<T> > create(T* ptr) { return adoptRef(new WeakReference(ptr)); }
+    static PassRefPtr<WeakReference<T>> create(T* ptr) { return adoptRef(new WeakReference(ptr)); }
+    static PassRefPtr<WeakReference<T>> createUnbound() { return adoptRef(new WeakReference()); }
 
     T* get() const
     {
+#if USE(WEB_THREAD)
+        ASSERT(canAccessThreadLocalDataForThread(m_boundThread));
+#else
         ASSERT(m_boundThread == currentThread());
+#endif
         return m_ptr;
     }
 
     void clear()
     {
+#if USE(WEB_THREAD)
+        ASSERT(canAccessThreadLocalDataForThread(m_boundThread));
+#else
         ASSERT(m_boundThread == currentThread());
+#endif
         m_ptr = 0;
     }
 
+    void bindTo(T* ptr)
+    {
+        ASSERT(!m_ptr);
+#ifndef NDEBUG
+        m_boundThread = currentThread();
+#endif
+        m_ptr = ptr;
+    }
+
 private:
+    WeakReference() : m_ptr(0) { }
+
     explicit WeakReference(T* ptr)
         : m_ptr(ptr)
 #ifndef NDEBUG
@@ -70,19 +92,18 @@ private:
 #endif
 };
 
-}
-
 template<typename T>
 class WeakPtr {
     WTF_MAKE_FAST_ALLOCATED;
 public:
     WeakPtr() { }
-    WeakPtr(PassRefPtr<Internal::WeakReference<T> > ref) : m_ref(ref) { }
+    WeakPtr(PassRefPtr<WeakReference<T>> ref) : m_ref(ref) { }
 
     T* get() const { return m_ref->get(); }
+    bool operator!() const { return !m_ref->get(); }
 
 private:
-    RefPtr<Internal::WeakReference<T> > m_ref;
+    RefPtr<WeakReference<T>> m_ref;
 };
 
 template<typename T>
@@ -90,7 +111,14 @@ class WeakPtrFactory {
     WTF_MAKE_NONCOPYABLE(WeakPtrFactory<T>);
     WTF_MAKE_FAST_ALLOCATED;
 public:
-    explicit WeakPtrFactory(T* ptr) { m_ref = Internal::WeakReference<T>::create(ptr); }
+    explicit WeakPtrFactory(T* ptr) : m_ref(WeakReference<T>::create(ptr)) { }
+
+    WeakPtrFactory(PassRefPtr<WeakReference<T>> ref, T* ptr)
+        : m_ref(ref)
+    {
+        m_ref->bindTo(ptr);
+    }
+
     ~WeakPtrFactory() { m_ref->clear(); }
 
     // We should consider having createWeakPtr populate m_ref the first time createWeakPtr is called.
@@ -101,16 +129,17 @@ public:
         T* ptr = m_ref->get();
         m_ref->clear();
         // We create a new WeakReference so that future calls to createWeakPtr() create nonzero WeakPtrs.
-        m_ref = Internal::WeakReference<T>::create(ptr);
+        m_ref = WeakReference<T>::create(ptr);
     }
 
 private:
-    RefPtr<Internal::WeakReference<T> > m_ref;
+    RefPtr<WeakReference<T>> m_ref;
 };
 
 } // namespace WTF
 
 using WTF::WeakPtr;
 using WTF::WeakPtrFactory;
+using WTF::WeakReference;
 
 #endif
