@@ -34,6 +34,7 @@
 #include "AccessibilityTableColumn.h"
 #include "AccessibilityTableHeaderContainer.h"
 #include "AccessibilityTableRow.h"
+#include "ElementIterator.h"
 #include "HTMLNames.h"
 #include "HTMLTableCaptionElement.h"
 #include "HTMLTableCellElement.h"
@@ -48,8 +49,9 @@ namespace WebCore {
 using namespace HTMLNames;
 
 AccessibilityTable::AccessibilityTable(RenderObject* renderer)
-    : AccessibilityRenderObject(renderer),
-    m_headerContainer(0)
+    : AccessibilityRenderObject(renderer)
+    , m_headerContainer(0)
+    , m_isAccessibilityTable(true)
 {
 }
 
@@ -60,11 +62,7 @@ AccessibilityTable::~AccessibilityTable()
 void AccessibilityTable::init()
 {
     AccessibilityRenderObject::init();
-#if ACCESSIBILITY_TABLES
     m_isAccessibilityTable = isTableExposableThroughAccessibility();
-#else
-    m_isAccessibilityTable = false;
-#endif
 }
 
 PassRefPtr<AccessibilityTable> AccessibilityTable::create(RenderObject* renderer)
@@ -104,7 +102,7 @@ bool AccessibilityTable::isDataTable() const
     // When a section of the document is contentEditable, all tables should be
     // treated as data tables, otherwise users may not be able to work with rich
     // text editors that allow creating and editing tables.
-    if (node() && node()->rendererIsEditable())
+    if (node() && node()->hasEditableStyle())
         return true;
 
     // This employs a heuristic to determine if this table should appear.
@@ -113,12 +111,11 @@ bool AccessibilityTable::isDataTable() const
     // between a "layout" table and a "data" table.
     
     RenderTable* table = toRenderTable(m_renderer);
-    Node* tableNode = table->node();
-    if (!tableNode || !tableNode->hasTagName(tableTag))
+    if (!table->element() || !isHTMLTableElement(table->element()))
         return false;
 
     // if there is a caption element, summary, THEAD, or TFOOT section, it's most certainly a data table
-    HTMLTableElement* tableElement = static_cast<HTMLTableElement*>(tableNode);
+    HTMLTableElement* tableElement = toHTMLTableElement(table->element());
     if (!tableElement->summary().isEmpty() || tableElement->tHead() || tableElement->tFoot() || tableElement->caption())
         return true;
     
@@ -127,8 +124,8 @@ bool AccessibilityTable::isDataTable() const
         return true;    
 
     // if there's a colgroup or col element, it's probably a data table.
-    for (Node* child = tableElement->firstChild(); child; child = child->nextSibling()) {
-        if (child->hasTagName(colTag) || child->hasTagName(colgroupTag))
+    for (const auto& child : childrenOfType<Element>(*tableElement)) {
+        if (child.hasTagName(colTag) || child.hasTagName(colgroupTag))
             return true;
     }
     
@@ -151,10 +148,8 @@ bool AccessibilityTable::isDataTable() const
         return true;
     
     // Store the background color of the table to check against cell's background colors.
-    RenderStyle* tableStyle = table->style();
-    if (!tableStyle)
-        return false;
-    Color tableBGColor = tableStyle->visitedDependentColor(CSSPropertyBackgroundColor);
+    const RenderStyle& tableStyle = table->style();
+    Color tableBGColor = tableStyle.visitedDependentColor(CSSPropertyBackgroundColor);
     
     // check enough of the cells to find if the table matches our criteria
     // Criteria: 
@@ -180,8 +175,7 @@ bool AccessibilityTable::isDataTable() const
             RenderTableCell* cell = firstBody->primaryCellAt(row, col);
             if (!cell)
                 continue;
-            Node* cellNode = cell->node();
-            if (!cellNode)
+            if (!cell->element())
                 continue;
             
             if (cell->width() < 1 || cell->height() < 1)
@@ -189,7 +183,7 @@ bool AccessibilityTable::isDataTable() const
             
             validCellCount++;
             
-            HTMLTableCellElement* cellElement = static_cast<HTMLTableCellElement*>(cellNode);
+            HTMLTableCellElement* cellElement = toHTMLTableCellElement(cell->element());
             
             bool isTHCell = cellElement->hasTagName(thTag);
             // If the first row is comprised of all <th> tags, assume it is a data table.
@@ -205,12 +199,10 @@ bool AccessibilityTable::isDataTable() const
                 || !cellElement->axis().isEmpty() || !cellElement->scope().isEmpty())
                 return true;
             
-            RenderStyle* renderStyle = cell->style();
-            if (!renderStyle)
-                continue;
+            const RenderStyle& renderStyle = cell->style();
 
             // If the empty-cells style is set, we'll call it a data table.
-            if (renderStyle->emptyCells() == HIDE)
+            if (renderStyle.emptyCells() == HIDE)
                 return true;
 
             // If a cell has matching bordered sides, call it a (fully) bordered cell.
@@ -231,7 +223,7 @@ bool AccessibilityTable::isDataTable() const
             
             // If the cell has a different color from the table and there is cell spacing,
             // then it is probably a data table cell (spacing and colors take the place of borders).
-            Color cellColor = renderStyle->visitedDependentColor(CSSPropertyBackgroundColor);
+            Color cellColor = renderStyle.visitedDependentColor(CSSPropertyBackgroundColor);
             if (table->hBorderSpacing() > 0 && table->vBorderSpacing() > 0
                 && tableBGColor != cellColor && cellColor.alpha() != 1)
                 backgroundDifferenceCellCount++;
@@ -245,10 +237,8 @@ bool AccessibilityTable::isDataTable() const
                 RenderObject* renderRow = cell->parent();
                 if (!renderRow || !renderRow->isBoxModelObject() || !toRenderBoxModelObject(renderRow)->isTableRow())
                     continue;
-                RenderStyle* rowRenderStyle = renderRow->style();
-                if (!rowRenderStyle)
-                    continue;
-                Color rowColor = rowRenderStyle->visitedDependentColor(CSSPropertyBackgroundColor);
+                const RenderStyle& rowRenderStyle = renderRow->style();
+                Color rowColor = rowRenderStyle.visitedDependentColor(CSSPropertyBackgroundColor);
                 alternatingRowColors[alternatingRowColorCount] = rowColor;
                 alternatingRowColorCount++;
             }
@@ -311,9 +301,9 @@ bool AccessibilityTable::isTableExposableThroughAccessibility() const
         return false;
 
     // Gtk+ ATs expect all tables to be exposed as tables.
-#if PLATFORM(GTK)
-    Node* tableNode = toRenderTable(m_renderer)->node();
-    return tableNode && tableNode->hasTagName(tableTag);
+#if PLATFORM(GTK) || PLATFORM(EFL)
+    Element* tableNode = toRenderTable(m_renderer)->element();
+    return tableNode && isHTMLTableElement(tableNode);
 #endif
 
     return isDataTable();
@@ -345,62 +335,54 @@ void AccessibilityTable::addChildren()
         return;
     
     RenderTable* table = toRenderTable(m_renderer);
-    AXObjectCache* axCache = m_renderer->document()->axObjectCache();
+    AXObjectCache* axCache = m_renderer->document().axObjectCache();
 
-    // go through all the available sections to pull out the rows
-    // and add them as children
-    // FIXME: This will skip a table with just a tfoot. Should fix by using RenderTable::topSection.
-    RenderTableSection* tableSection = table->header();
-    if (!tableSection)
-        tableSection = table->firstBody();
-    
+    // Go through all the available sections to pull out the rows and add them as children.
+    table->recalcSectionsIfNeeded();
+    RenderTableSection* tableSection = table->topSection();
     if (!tableSection)
         return;
     
-    RenderTableSection* initialTableSection = tableSection;
-    
+    unsigned maxColumnCount = 0;
     while (tableSection) {
         
         HashSet<AccessibilityObject*> appendedRows;
-
         unsigned numRows = tableSection->numRows();
-        unsigned numCols = tableSection->numColumns();
         for (unsigned rowIndex = 0; rowIndex < numRows; ++rowIndex) {
-            for (unsigned colIndex = 0; colIndex < numCols; ++colIndex) {
-                
-                RenderTableCell* cell = tableSection->primaryCellAt(rowIndex, colIndex);
-                if (!cell)
-                    continue;
-                
-                AccessibilityObject* rowObject = axCache->getOrCreate(cell->parent());
-                if (!rowObject->isTableRow())
-                    continue;
-                
-                AccessibilityTableRow* row = static_cast<AccessibilityTableRow*>(rowObject);
-                // we need to check every cell for a new row, because cell spans
-                // can cause us to mess rows if we just check the first column
-                if (appendedRows.contains(row))
-                    continue;
-                
-                row->setRowIndex((int)m_rows.size());        
-                m_rows.append(row);
-                if (!row->accessibilityIsIgnored())
-                    m_children.append(row);
-#if PLATFORM(GTK)
-                else
-                    m_children.append(row->children());
+            
+            RenderTableRow* renderRow = tableSection->rowRendererAt(rowIndex);
+            if (!renderRow)
+                continue;
+            
+            AccessibilityObject* rowObject = axCache->getOrCreate(renderRow);
+            if (!rowObject->isTableRow())
+                continue;
+            
+            AccessibilityTableRow* row = toAccessibilityTableRow(rowObject);
+            // We need to check every cell for a new row, because cell spans
+            // can cause us to miss rows if we just check the first column.
+            if (appendedRows.contains(row))
+                continue;
+            
+            row->setRowIndex(static_cast<int>(m_rows.size()));
+            m_rows.append(row);
+            if (!row->accessibilityIsIgnored())
+                m_children.append(row);
+#if PLATFORM(GTK) || PLATFORM(EFL)
+            else
+                m_children.appendVector(row->children());
 #endif
-                appendedRows.add(row);
-            }
+            appendedRows.add(row);
         }
-        
+    
+        maxColumnCount = std::max(tableSection->numColumns(), maxColumnCount);
         tableSection = table->sectionBelow(tableSection, SkipEmptySections);
     }
     
     // make the columns based on the number of columns in the first body
-    unsigned length = initialTableSection->numColumns();
+    unsigned length = maxColumnCount;
     for (unsigned i = 0; i < length; ++i) {
-        AccessibilityTableColumn* column = static_cast<AccessibilityTableColumn*>(axCache->getOrCreate(ColumnRole));
+        AccessibilityTableColumn* column = toAccessibilityTableColumn(axCache->getOrCreate(ColumnRole));
         column->setColumnIndex((int)i);
         column->setParent(this);
         m_columns.append(column);
@@ -425,35 +407,19 @@ AccessibilityObject* AccessibilityTable::headerContainer()
     return m_headerContainer.get();
 }
 
-AccessibilityObject::AccessibilityChildrenVector& AccessibilityTable::columns()
+const AccessibilityObject::AccessibilityChildrenVector& AccessibilityTable::columns()
 {
     updateChildrenIfNecessary();
         
     return m_columns;
 }
 
-AccessibilityObject::AccessibilityChildrenVector& AccessibilityTable::rows()
+const AccessibilityObject::AccessibilityChildrenVector& AccessibilityTable::rows()
 {
     updateChildrenIfNecessary();
     
     return m_rows;
 }
-    
-void AccessibilityTable::rowHeaders(AccessibilityChildrenVector& headers)
-{
-    if (!m_renderer)
-        return;
-    
-    updateChildrenIfNecessary();
-    
-    unsigned rowCount = m_rows.size();
-    for (unsigned k = 0; k < rowCount; ++k) {
-        AccessibilityObject* header = static_cast<AccessibilityTableRow*>(m_rows[k].get())->headerObject();
-        if (!header)
-            continue;
-        headers.append(header);
-    }
-}
 
 void AccessibilityTable::columnHeaders(AccessibilityChildrenVector& headers)
 {
@@ -462,15 +428,38 @@ void AccessibilityTable::columnHeaders(AccessibilityChildrenVector& headers)
     
     updateChildrenIfNecessary();
     
-    unsigned colCount = m_columns.size();
-    for (unsigned k = 0; k < colCount; ++k) {
-        AccessibilityObject* header = static_cast<AccessibilityTableColumn*>(m_columns[k].get())->headerObject();
-        if (!header)
-            continue;
-        headers.append(header);
+    for (const auto& column : m_columns) {
+        if (AccessibilityObject* header = toAccessibilityTableColumn(column.get())->headerObject())
+            headers.append(header);
     }
 }
+
+void AccessibilityTable::rowHeaders(AccessibilityChildrenVector& headers)
+{
+    if (!m_renderer)
+        return;
     
+    updateChildrenIfNecessary();
+    
+    for (const auto& row : m_rows) {
+        if (AccessibilityObject* header = toAccessibilityTableRow(row.get())->headerObject())
+            headers.append(header);
+    }
+}
+
+void AccessibilityTable::visibleRows(AccessibilityChildrenVector& rows)
+{
+    if (!m_renderer)
+        return;
+    
+    updateChildrenIfNecessary();
+    
+    for (const auto& row : m_rows) {
+        if (row && !row->isOffScreen())
+            rows.append(row);
+    }
+}
+
 void AccessibilityTable::cells(AccessibilityObject::AccessibilityChildrenVector& cells)
 {
     if (!m_renderer)
@@ -478,11 +467,8 @@ void AccessibilityTable::cells(AccessibilityObject::AccessibilityChildrenVector&
     
     updateChildrenIfNecessary();
     
-    int numRows = m_rows.size();
-    for (int row = 0; row < numRows; ++row) {
-        AccessibilityChildrenVector rowChildren = m_rows[row]->children();
-        cells.append(rowChildren);
-    }
+    for (const auto& row : m_rows)
+        cells.appendVector(row->children());
 }
     
 unsigned AccessibilityTable::columnCount()
@@ -512,76 +498,36 @@ int AccessibilityTable::tableLevel() const
 
 AccessibilityTableCell* AccessibilityTable::cellForColumnAndRow(unsigned column, unsigned row)
 {
-    if (!m_renderer || !m_renderer->isTable())
+    updateChildrenIfNecessary();
+    if (column >= columnCount() || row >= rowCount())
         return 0;
     
-    updateChildrenIfNecessary();
-    
-    RenderTable* table = toRenderTable(m_renderer);
-    // FIXME: This will skip a table with just a tfoot. Should fix by using RenderTable::topSection.
-    RenderTableSection* tableSection = table->header();
-    if (!tableSection)
-        tableSection = table->firstBody();
-    
-    RenderTableCell* cell = 0;
-    unsigned rowCount = 0;
-    unsigned rowOffset = 0;
-    while (tableSection) {
-        
-        unsigned numRows = tableSection->numRows();
-        unsigned numCols = tableSection->numColumns();
-        
-        rowCount += numRows;
-        
-        unsigned sectionSpecificRow = row - rowOffset;            
-        if (row < rowCount && column < numCols && sectionSpecificRow < numRows) {
-            cell = tableSection->primaryCellAt(sectionSpecificRow, column);
+    // Iterate backwards through the rows in case the desired cell has a rowspan and exists in a previous row.
+    for (unsigned rowIndexCounter = row + 1; rowIndexCounter > 0; --rowIndexCounter) {
+        unsigned rowIndex = rowIndexCounter - 1;
+        const auto& children = m_rows[rowIndex]->children();
+        // Since some cells may have colspans, we have to check the actual range of each
+        // cell to determine which is the right one.
+        for (unsigned colIndexCounter = std::min(static_cast<unsigned>(children.size()), column + 1); colIndexCounter > 0; --colIndexCounter) {
+            unsigned colIndex = colIndexCounter - 1;
+            AccessibilityObject* child = children[colIndex].get();
+            ASSERT(child->isTableCell());
+            if (!child->isTableCell())
+                continue;
             
-            // we didn't find the cell, which means there's spanning happening
-            // search backwards to find the spanning cell
-            if (!cell) {
-                
-                // first try rows
-                for (int testRow = sectionSpecificRow - 1; testRow >= 0; --testRow) {
-                    cell = tableSection->primaryCellAt(testRow, column);
-                    // cell overlapped. use this one
-                    ASSERT(!cell || cell->rowSpan() >= 1);
-                    if (cell && ((cell->rowIndex() + (cell->rowSpan() - 1)) >= sectionSpecificRow))
-                        break;
-                    cell = 0;
-                }
-                
-                if (!cell) {
-                    // try cols
-                    for (int testCol = column - 1; testCol >= 0; --testCol) {
-                        cell = tableSection->primaryCellAt(sectionSpecificRow, testCol);
-                        // cell overlapped. use this one
-                        ASSERT(!cell || cell->rowSpan() >= 1);
-                        if (cell && ((cell->col() + (cell->colSpan() - 1)) >= column))
-                            break;
-                        cell = 0;
-                    }
-                }
-            }
+            std::pair<unsigned, unsigned> columnRange;
+            std::pair<unsigned, unsigned> rowRange;
+            AccessibilityTableCell* tableCellChild = toAccessibilityTableCell(child);
+            tableCellChild->columnIndexRange(columnRange);
+            tableCellChild->rowIndexRange(rowRange);
+            
+            if ((column >= columnRange.first && column < (columnRange.first + columnRange.second))
+                && (row >= rowRange.first && row < (rowRange.first + rowRange.second)))
+                return tableCellChild;
         }
-        
-        if (cell)
-            break;
-        
-        rowOffset += numRows;
-        // we didn't find anything between the rows we should have
-        if (row < rowCount)
-            break;
-        tableSection = table->sectionBelow(tableSection, SkipEmptySections);
     }
     
-    if (!cell)
-        return 0;
-    
-    AccessibilityObject* cellObject = axObjectCache()->getOrCreate(cell);
-    ASSERT_WITH_SECURITY_IMPLICATION(cellObject->isTableCell());
-    
-    return static_cast<AccessibilityTableCell*>(cellObject);
+    return 0;
 }
 
 AccessibilityRole AccessibilityTable::roleValue() const
@@ -592,20 +538,27 @@ AccessibilityRole AccessibilityTable::roleValue() const
     return TableRole;
 }
     
-bool AccessibilityTable::accessibilityIsIgnored() const
+bool AccessibilityTable::computeAccessibilityIsIgnored() const
 {
-    AccessibilityObjectInclusion decision = accessibilityIsIgnoredBase();
+    AccessibilityObjectInclusion decision = defaultObjectInclusion();
     if (decision == IncludeObject)
         return false;
     if (decision == IgnoreObject)
         return true;
     
     if (!isAccessibilityTable())
-        return AccessibilityRenderObject::accessibilityIsIgnored();
+        return AccessibilityRenderObject::computeAccessibilityIsIgnored();
         
     return false;
 }
-    
+
+void AccessibilityTable::titleElementText(Vector<AccessibilityText>& textOrder) const
+{
+    String title = this->title();
+    if (!title.isEmpty())
+        textOrder.append(AccessibilityText(title, LabelByElementText));
+}
+
 String AccessibilityTable::title() const
 {
     if (!isAccessibilityTable())
@@ -617,8 +570,8 @@ String AccessibilityTable::title() const
     
     // see if there is a caption
     Node* tableElement = m_renderer->node();
-    if (tableElement && tableElement->hasTagName(tableTag)) {
-        HTMLTableCaptionElement* caption = static_cast<HTMLTableElement*>(tableElement)->caption();
+    if (tableElement && isHTMLTableElement(tableElement)) {
+        HTMLTableCaptionElement* caption = toHTMLTableElement(tableElement)->caption();
         if (caption)
             title = caption->innerText();
     }
