@@ -42,17 +42,17 @@ namespace WebCore {
 
 static JSValue getNamedItems(ExecState* exec, JSHTMLAllCollection* collection, PropertyName propertyName)
 {
-    Vector<RefPtr<Node> > namedItems;
-    collection->impl()->namedItems(propertyNameToAtomicString(propertyName), namedItems);
+    Vector<Ref<Element>> namedItems;
+    collection->impl().namedItems(propertyNameToAtomicString(propertyName), namedItems);
 
     if (namedItems.isEmpty())
         return jsUndefined();
     if (namedItems.size() == 1)
-        return toJS(exec, collection->globalObject(), namedItems[0].get());
+        return toJS(exec, collection->globalObject(), &namedItems[0].get());
 
     // FIXME: HTML5 specification says this should be a HTMLCollection.
     // http://www.whatwg.org/specs/web-apps/current-work/multipage/common-dom-interfaces.html#htmlallcollection
-    return toJS(exec, collection->globalObject(), StaticNodeList::adopt(namedItems).get());
+    return toJS(exec, collection->globalObject(), StaticElementList::adopt(namedItems).get());
 }
 
 // HTMLAllCollections are strange objects, they support both get and call.
@@ -63,7 +63,7 @@ static EncodedJSValue JSC_HOST_CALL callHTMLAllCollection(ExecState* exec)
 
     // Do not use thisObj here. It can be the JSHTMLDocument, in the document.forms(i) case.
     JSHTMLAllCollection* jsCollection = jsCast<JSHTMLAllCollection*>(exec->callee());
-    HTMLAllCollection* collection = static_cast<HTMLAllCollection*>(jsCollection->impl());
+    HTMLAllCollection& collection = jsCollection->impl();
 
     // Also, do we need the TypeError test here ?
 
@@ -72,7 +72,7 @@ static EncodedJSValue JSC_HOST_CALL callHTMLAllCollection(ExecState* exec)
         String string = exec->argument(0).toString(exec)->value(exec);
         unsigned index = toUInt32FromStringImpl(string.impl());
         if (index != PropertyName::NotAnIndex)
-            return JSValue::encode(toJS(exec, jsCollection->globalObject(), collection->item(index)));
+            return JSValue::encode(toJS(exec, jsCollection->globalObject(), collection.item(index)));
 
         // Support for document.images('<name>') etc.
         return JSValue::encode(getNamedItems(exec, jsCollection, Identifier(exec, string)));
@@ -82,7 +82,7 @@ static EncodedJSValue JSC_HOST_CALL callHTMLAllCollection(ExecState* exec)
     String string = exec->argument(0).toString(exec)->value(exec);
     unsigned index = toUInt32FromStringImpl(exec->argument(1).toWTFString(exec).impl());
     if (index != PropertyName::NotAnIndex) {
-        if (Node* node = collection->namedItemWithIndex(string, index))
+        if (Node* node = collection.namedItemWithIndex(string, index))
             return JSValue::encode(toJS(exec, jsCollection->globalObject(), node));
     }
 
@@ -100,23 +100,24 @@ bool JSHTMLAllCollection::canGetItemsForName(ExecState*, HTMLAllCollection* coll
     return collection->hasNamedItem(propertyNameToAtomicString(propertyName));
 }
 
-JSValue JSHTMLAllCollection::nameGetter(ExecState* exec, JSValue slotBase, PropertyName propertyName)
+EncodedJSValue JSHTMLAllCollection::nameGetter(ExecState* exec, EncodedJSValue slotBase, EncodedJSValue, PropertyName propertyName)
 {
-    JSHTMLAllCollection* thisObj = jsCast<JSHTMLAllCollection*>(asObject(slotBase));
-    return getNamedItems(exec, thisObj, propertyName);
+    JSHTMLAllCollection* thisObj = jsCast<JSHTMLAllCollection*>(JSValue::decode(slotBase));
+    return JSValue::encode(getNamedItems(exec, thisObj, propertyName));
 }
 
 JSValue JSHTMLAllCollection::item(ExecState* exec)
 {
     uint32_t index = toUInt32FromStringImpl(exec->argument(0).toString(exec)->value(exec).impl());
     if (index != PropertyName::NotAnIndex)
-        return toJS(exec, globalObject(), impl()->item(index));
+        return toJS(exec, globalObject(), impl().item(index));
     return getNamedItems(exec, this, Identifier(exec, exec->argument(0).toString(exec)->value(exec)));
 }
 
 JSValue JSHTMLAllCollection::namedItem(ExecState* exec)
 {
-    return getNamedItems(exec, this, Identifier(exec, exec->argument(0).toString(exec)->value(exec)));
+    JSValue value = getNamedItems(exec, this, Identifier(exec, exec->argument(0).toString(exec)->value(exec)));
+    return value.isUndefined() ? jsNull() : value;
 }
 
 } // namespace WebCore
