@@ -36,39 +36,52 @@
 #include "CSSSelectorList.h"
 #include "CSSValueKeywords.h"
 #include "HTMLElement.h"
+#include "InspectorInstrumentation.h"
 #include "RenderRegion.h"
 #include "SVGElement.h"
 #include "SelectorCheckerFastPath.h"
-#include "StylePropertySet.h"
+#include "SelectorCompiler.h"
+#include "StyleProperties.h"
 #include "StyledElement.h"
 
 #include <wtf/TemporaryChange.h>
 
 namespace WebCore {
 
-static const StylePropertySet& leftToRightDeclaration()
+static const StyleProperties& leftToRightDeclaration()
 {
-    static MutableStylePropertySet* leftToRightDecl = MutableStylePropertySet::create().leakRef();
-    if (leftToRightDecl->isEmpty())
-        leftToRightDecl->setProperty(CSSPropertyDirection, CSSValueLtr);
-    return *leftToRightDecl;
+    static NeverDestroyed<Ref<MutableStyleProperties>> leftToRightDecl(MutableStyleProperties::create());
+    if (leftToRightDecl.get()->isEmpty())
+        leftToRightDecl.get()->setProperty(CSSPropertyDirection, CSSValueLtr);
+    return leftToRightDecl.get().get();
 }
 
-static const StylePropertySet& rightToLeftDeclaration()
+static const StyleProperties& rightToLeftDeclaration()
 {
-    static MutableStylePropertySet* rightToLeftDecl = MutableStylePropertySet::create().leakRef();
-    if (rightToLeftDecl->isEmpty())
-        rightToLeftDecl->setProperty(CSSPropertyDirection, CSSValueRtl);
-    return *rightToLeftDecl;
+    static NeverDestroyed<Ref<MutableStyleProperties>> rightToLeftDecl(MutableStyleProperties::create());
+    if (rightToLeftDecl.get()->isEmpty())
+        rightToLeftDecl.get()->setProperty(CSSPropertyDirection, CSSValueRtl);
+    return rightToLeftDecl.get().get();
 }
 
+class MatchRequest {
+public:
+    MatchRequest(RuleSet* ruleSet, bool includeEmptyRules = false)
+        : ruleSet(ruleSet)
+        , includeEmptyRules(includeEmptyRules)
+    {
+    }
+    const RuleSet* ruleSet;
+    const bool includeEmptyRules;
+};
+
 StyleResolver::MatchResult& ElementRuleCollector::matchedResult()
 {
     ASSERT(m_mode == SelectorChecker::ResolvingStyle);
     return m_result;
 }
 
-const Vector<RefPtr<StyleRuleBase> >& ElementRuleCollector::matchedRuleList() const
+const Vector<RefPtr<StyleRuleBase>>& ElementRuleCollector::matchedRuleList() const
 {
     ASSERT(m_mode == SelectorChecker::CollectingRules);
     return m_matchedRuleList;
@@ -77,7 +90,7 @@ const Vector<RefPtr<StyleRuleBase> >& ElementRuleCollector::matchedRuleList() co
 inline void ElementRuleCollector::addMatchedRule(const RuleData* rule)
 {
     if (!m_matchedRules)
-        m_matchedRules = adoptPtr(new Vector<const RuleData*, 32>);
+        m_matchedRules = std::make_unique<Vector<const RuleData*, 32>>();
     m_matchedRules->append(rule);
 }
 
@@ -88,7 +101,7 @@ inline void ElementRuleCollector::clearMatchedRules()
     m_matchedRules->clear();
 }
 
-inline void ElementRuleCollector::addElementStyleProperties(const StylePropertySet* propertySet, bool isCacheable)
+inline void ElementRuleCollector::addElementStyleProperties(const StyleProperties* propertySet, bool isCacheable)
 {
     if (!propertySet)
         return;
@@ -151,10 +164,8 @@ void ElementRuleCollector::collectMatchingRules(const MatchRequest& matchRequest
     // a) it's a UA rule
     // b) the tree scope allows author rules
     // c) the rules comes from a scoped style sheet within the same tree scope
-    TreeScope* treeScope = element->treeScope();
     if (!MatchingUARulesScope::isMatchingUARules()
-        && !treeScope->applyAuthorStyles()
-        && (!matchRequest.scope || matchRequest.scope->treeScope() != treeScope))
+        && !element->treeScope().applyAuthorStyles())
         return;
 
     // We need to collect the rules for id, class, tag, and everything else into a buffer and
@@ -182,10 +193,10 @@ void ElementRuleCollector::collectMatchingRulesForRegion(const MatchRequest& mat
     unsigned size = matchRequest.ruleSet->regionSelectorsAndRuleSets().size();
     for (unsigned i = 0; i < size; ++i) {
         const CSSSelector* regionSelector = matchRequest.ruleSet->regionSelectorsAndRuleSets().at(i).selector;
-        if (checkRegionSelector(regionSelector, toElement(m_regionForStyling->node()))) {
+        if (checkRegionSelector(regionSelector, m_regionForStyling->generatingElement())) {
             RuleSet* regionRules = matchRequest.ruleSet->regionSelectorsAndRuleSets().at(i).ruleSet.get();
             ASSERT(regionRules);
-            collectMatchingRules(MatchRequest(regionRules, matchRequest.includeEmptyRules, matchRequest.scope), ruleRange);
+            collectMatchingRules(MatchRequest(regionRules, matchRequest.includeEmptyRules), ruleRange);
         }
     }
 }
@@ -214,68 +225,6 @@ void ElementRuleCollector::sortAndTransferMatchedRules()
     }
 }
 
-void ElementRuleCollector::matchScopedAuthorRules(bool includeEmptyRules)
-{
-#if ENABLE(STYLE_SCOPED) || ENABLE(SHADOW_DOM)
-    if (!m_scopeResolver)
-        return;
-
-    // Match scoped author rules by traversing the scoped element stack (rebuild it if it got inconsistent).
-    if (m_scopeResolver->hasScopedStyles() && m_scopeResolver->ensureStackConsistency(m_state.element())) {
-        bool applyAuthorStyles = m_state.element()->treeScope()->applyAuthorStyles();
-        bool documentScope = true;
-        unsigned scopeSize = m_scopeResolver->stackSize();
-        for (unsigned i = 0; i < scopeSize; ++i) {
-            clearMatchedRules();
-            m_result.ranges.lastAuthorRule = m_result.matchedProperties.size() - 1;
-
-            const StyleScopeResolver::StackFrame& frame = m_scopeResolver->stackFrameAt(i);
-            documentScope = documentScope && !frame.m_scope->isInShadowTree();
-            if (documentScope) {
-                if (!applyAuthorStyles)
-                    continue;
-            } else {
-                if (!m_scopeResolver->matchesStyleBounds(frame))
-                    continue;
-            }
-
-            MatchRequest matchRequest(frame.m_ruleSet, includeEmptyRules, frame.m_scope);
-            StyleResolver::RuleRange ruleRange = m_result.ranges.authorRuleRange();
-            collectMatchingRules(matchRequest, ruleRange);
-            collectMatchingRulesForRegion(matchRequest, ruleRange);
-            sortAndTransferMatchedRules();
-        }
-    }
-
-    matchHostRules(includeEmptyRules);
-#else
-    UNUSED_PARAM(includeEmptyRules);
-#endif
-}
-
-void ElementRuleCollector::matchHostRules(bool includeEmptyRules)
-{
-#if ENABLE(SHADOW_DOM)
-    ASSERT(m_scopeResolver);
-
-    clearMatchedRules();
-    m_result.ranges.lastAuthorRule = m_result.matchedProperties.size() - 1;
-
-    Vector<RuleSet*> matchedRules;
-    m_scopeResolver->matchHostRules(m_state.element(), matchedRules);
-    if (matchedRules.isEmpty())
-        return;
-
-    for (unsigned i = matchedRules.size(); i > 0; --i) {
-        StyleResolver::RuleRange ruleRange = m_result.ranges.authorRuleRange();
-        collectMatchingRules(MatchRequest(matchedRules.at(i-1), includeEmptyRules, m_state.element()), ruleRange);
-    }
-    sortAndTransferMatchedRules();
-#else
-    UNUSED_PARAM(includeEmptyRules);
-#endif
-}
-
 void ElementRuleCollector::matchAuthorRules(bool includeEmptyRules)
 {
     clearMatchedRules();
@@ -291,8 +240,6 @@ void ElementRuleCollector::matchAuthorRules(bool includeEmptyRules)
     collectMatchingRulesForRegion(matchRequest, ruleRange);
 
     sortAndTransferMatchedRules();
-
-    matchScopedAuthorRules(includeEmptyRules);
 }
 
 void ElementRuleCollector::matchUserRules(bool includeEmptyRules)
@@ -323,11 +270,11 @@ void ElementRuleCollector::matchUARules()
     matchUARules(userAgentStyleSheet);
 
     // In quirks mode, we match rules from the quirks user agent sheet.
-    if (document()->inQuirksMode())
+    if (document().inQuirksMode())
         matchUARules(CSSDefaultStyleSheets::defaultQuirksStyle);
 
     // If document uses view source styles (in view source mode or in xml viewer mode), then we match rules from the view source style sheet.
-    if (document()->isViewSource())
+    if (document().isViewSource())
         matchUARules(CSSDefaultStyleSheets::viewSourceStyle());
 }
 
@@ -342,11 +289,12 @@ void ElementRuleCollector::matchUARules(RuleSet* rules)
     sortAndTransferMatchedRules();
 }
 
-inline bool ElementRuleCollector::ruleMatches(const RuleData& ruleData, const ContainerNode* scope, PseudoId& dynamicPseudo)
+inline bool ElementRuleCollector::ruleMatches(const RuleData& ruleData, PseudoId& dynamicPseudo)
 {
     const StyleResolver::State& state = m_state;
 
-    if (ruleData.hasFastCheckableSelector()) {
+    bool fastCheckableSelector = ruleData.hasFastCheckableSelector();
+    if (fastCheckableSelector) {
         // We know this selector does not include any pseudo elements.
         if (m_pseudoStyleRequest.pseudoId != NOPSEUDO)
             return false;
@@ -356,6 +304,38 @@ inline bool ElementRuleCollector::ruleMatches(const RuleData& ruleData, const Co
             if (!ruleData.hasMultipartSelector())
                 return true;
         }
+    }
+
+#if ENABLE(CSS_SELECTOR_JIT)
+    void* compiledSelectorChecker = ruleData.compiledSelectorCodeRef().code().executableAddress();
+    if (!compiledSelectorChecker && ruleData.compilationStatus() == SelectorCompilationStatus::NotCompiled) {
+        JSC::VM* vm = document().scriptExecutionContext()->vm();
+        SelectorCompilationStatus compilationStatus;
+        JSC::MacroAssemblerCodeRef compiledSelectorCodeRef;
+        compilationStatus = SelectorCompiler::compileSelector(ruleData.selector(), vm, compiledSelectorCodeRef);
+
+        ruleData.setCompiledSelector(compilationStatus, compiledSelectorCodeRef);
+        compiledSelectorChecker = ruleData.compiledSelectorCodeRef().code().executableAddress();
+    }
+    if (compiledSelectorChecker) {
+        if (m_pseudoStyleRequest.pseudoId != NOPSEUDO)
+            return false;
+
+        if (ruleData.compilationStatus() == SelectorCompilationStatus::SimpleSelectorChecker) {
+            SelectorCompiler::SimpleSelectorChecker selectorChecker = SelectorCompiler::simpleSelectorCheckerFunction(compiledSelectorChecker, ruleData.compilationStatus());
+            return selectorChecker(state.element());
+        }
+        ASSERT(ruleData.compilationStatus() == SelectorCompilationStatus::SelectorCheckerWithCheckingContext);
+
+        SelectorCompiler::SelectorCheckerWithCheckingContext selectorChecker = SelectorCompiler::selectorCheckerFunctionWithCheckingContext(compiledSelectorChecker, ruleData.compilationStatus());
+        SelectorCompiler::CheckingContext context;
+        context.elementStyle = state.style();
+        context.resolvingMode = m_mode;
+        return selectorChecker(state.element(), &context);
+    }
+#endif // ENABLE(CSS_SELECTOR_JIT)
+
+    if (fastCheckableSelector) {
         if (ruleData.selector()->m_match == CSSSelector::Tag && !SelectorChecker::tagMatches(state.element(), ruleData.selector()->tagQName()))
             return false;
         SelectorCheckerFastPath selectorCheckerFastPath(ruleData.selector(), state.element());
@@ -369,12 +349,10 @@ inline bool ElementRuleCollector::ruleMatches(const RuleData& ruleData, const Co
     SelectorChecker selectorChecker(document(), m_mode);
     SelectorChecker::SelectorCheckingContext context(ruleData.selector(), state.element(), SelectorChecker::VisitedMatchEnabled);
     context.elementStyle = state.style();
-    context.scope = scope;
     context.pseudoId = m_pseudoStyleRequest.pseudoId;
     context.scrollbar = m_pseudoStyleRequest.scrollbar;
     context.scrollbarPart = m_pseudoStyleRequest.scrollbarPart;
-    SelectorChecker::Match match = selectorChecker.match(context, dynamicPseudo);
-    if (match != SelectorChecker::SelectorMatches)
+    if (!selectorChecker.match(context, dynamicPseudo))
         return false;
     if (m_pseudoStyleRequest.pseudoId != NOPSEUDO && m_pseudoStyleRequest.pseudoId != dynamicPseudo)
         return false;
@@ -382,16 +360,6 @@ inline bool ElementRuleCollector::ruleMatches(const RuleData& ruleData, const Co
 }
 
 void ElementRuleCollector::collectMatchingRulesForList(const Vector<RuleData>* rules, const MatchRequest& matchRequest, StyleResolver::RuleRange& ruleRange)
-{
-    if (UNLIKELY(InspectorInstrumentation::hasFrontends())) {
-        doCollectMatchingRulesForList<true>(rules, matchRequest, ruleRange);
-        return;
-    }
-    doCollectMatchingRulesForList<false>(rules, matchRequest, ruleRange);
-}
-
-template<bool hasInspectorFrontends>
-void ElementRuleCollector::doCollectMatchingRulesForList(const Vector<RuleData>* rules, const MatchRequest& matchRequest, StyleResolver::RuleRange& ruleRange)
 {
     if (!rules)
         return;
@@ -404,32 +372,26 @@ void ElementRuleCollector::doCollectMatchingRulesForList(const Vector<RuleData>*
             continue;
 
         StyleRule* rule = ruleData.rule();
-        InspectorInstrumentationCookie cookie;
-        if (hasInspectorFrontends)
-            cookie = InspectorInstrumentation::willMatchRule(document(), rule, m_inspectorCSSOMWrappers, document()->styleSheetCollection());
         PseudoId dynamicPseudo = NOPSEUDO;
-        if (ruleMatches(ruleData, matchRequest.scope, dynamicPseudo)) {
+        if (ruleMatches(ruleData, dynamicPseudo)) {
+            // For SharingRules testing, any match is good enough, we don't care what is matched.
+            if (m_mode == SelectorChecker::SharingRules) {
+                addMatchedRule(&ruleData);
+                break;
+            }
+
             // If the rule has no properties to apply, then ignore it in the non-debug mode.
-            const StylePropertySet& properties = rule->properties();
-            if (properties.isEmpty() && !matchRequest.includeEmptyRules) {
-                if (hasInspectorFrontends)
-                    InspectorInstrumentation::didMatchRule(cookie, false);
+            const StyleProperties& properties = rule->properties();
+            if (properties.isEmpty() && !matchRequest.includeEmptyRules)
                 continue;
-            }
             // FIXME: Exposing the non-standard getMatchedCSSRules API to web is the only reason this is needed.
-            if (m_sameOriginOnly && !ruleData.hasDocumentSecurityOrigin()) {
-                if (hasInspectorFrontends)
-                    InspectorInstrumentation::didMatchRule(cookie, false);
+            if (m_sameOriginOnly && !ruleData.hasDocumentSecurityOrigin())
                 continue;
-            }
             // If we're matching normal rules, set a pseudo bit if
             // we really just matched a pseudo-element.
             if (dynamicPseudo != NOPSEUDO && m_pseudoStyleRequest.pseudoId == NOPSEUDO) {
-                if (m_mode == SelectorChecker::CollectingRules) {
-                    if (hasInspectorFrontends)
-                        InspectorInstrumentation::didMatchRule(cookie, false);
+                if (m_mode == SelectorChecker::CollectingRules)
                     continue;
-                }
                 if (dynamicPseudo < FIRST_INTERNAL_PSEUDOID)
                     state.style()->setHasPseudoStyle(dynamicPseudo);
             } else {
@@ -440,13 +402,9 @@ void ElementRuleCollector::doCollectMatchingRulesForList(const Vector<RuleData>*
 
                 // Add this rule to our list of matched rules.
                 addMatchedRule(&ruleData);
-                if (hasInspectorFrontends)
-                    InspectorInstrumentation::didMatchRule(cookie, true);
                 continue;
             }
         }
-        if (hasInspectorFrontends)
-            InspectorInstrumentation::didMatchRule(cookie, false);
     }
 }
 
