@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
- * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Apple Inc. All rights reserved.
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Apple Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,10 +22,9 @@
 #ifndef StyleResolver_h
 #define StyleResolver_h
 
-#include "CSSRule.h"
-#include "CSSRuleList.h"
 #include "CSSToStyleMap.h"
 #include "CSSValueList.h"
+#include "DocumentRuleSets.h"
 #include "InspectorCSSOMWrappers.h"
 #include "LinkHash.h"
 #include "MediaQueryExp.h"
@@ -33,14 +32,13 @@
 #include "RuleFeature.h"
 #include "RuleSet.h"
 #include "RuntimeEnabledFeatures.h"
+#include "ScrollTypes.h"
 #include "SelectorChecker.h"
 #include "SelectorFilter.h"
 #include "StyleInheritedData.h"
 #include "StyleScopeResolver.h"
 #include "ViewportStyleResolver.h"
-#if ENABLE(CSS_FILTERS) && ENABLE(SVG)
-#include "WebKitCSSSVGDocumentValue.h"
-#endif
+#include <memory>
 #include <wtf/HashMap.h>
 #include <wtf/HashSet.h>
 #include <wtf/RefPtr.h>
@@ -50,8 +48,6 @@
 
 namespace WebCore {
 
-enum ESmartMinimumForFontSize { DoNotUseSmartMinimumForFontSize, UseSmartMinimumForFontFize };
-
 class CSSCursorImageValue;
 class CSSFontSelector;
 class CSSFontFace;
@@ -62,34 +58,30 @@ class CSSImageValue;
 class CSSPageRule;
 class CSSPrimitiveValue;
 class CSSProperty;
-class CSSRuleList;
 class CSSSelector;
 class CSSStyleSheet;
 class CSSValue;
 class ContainerNode;
-class CustomFilterOperation;
-class CustomFilterParameter;
-class CustomFilterParameterList;
 class Document;
+class DeprecatedStyleBuilder;
 class Element;
 class Frame;
 class FrameView;
-class KURL;
+class URL;
 class KeyframeList;
 class KeyframeValue;
 class MediaQueryEvaluator;
 class Node;
 class RenderRegion;
+class RenderScrollbar;
 class RuleData;
 class RuleSet;
 class Settings;
-class StaticCSSRuleList;
-class StyleBuilder;
 class StyleScopeResolver;
 class StyleImage;
 class StyleKeyframe;
 class StylePendingImage;
-class StylePropertySet;
+class StyleProperties;
 class StyleRule;
 #if ENABLE(SHADOW_DOM)
 class StyleRuleHost;
@@ -97,14 +89,11 @@ class StyleRuleHost;
 class StyleRuleKeyframes;
 class StyleRulePage;
 class StyleRuleRegion;
-class StyleShader;
 class StyleSheet;
 class StyleSheetList;
 class StyledElement;
 class ViewportStyleResolver;
 class WebKitCSSFilterValue;
-class WebKitCSSShaderValue;
-class WebKitCSSSVGDocumentValue;
 
 class MediaQueryResult {
     WTF_MAKE_NONCOPYABLE(MediaQueryResult); WTF_MAKE_FAST_ALLOCATED;
@@ -114,7 +103,6 @@ public:
         , m_result(result)
     {
     }
-    void reportMemoryUsage(MemoryObjectInfo*) const;
 
     MediaQueryExp m_expression;
     bool m_result;
@@ -135,11 +123,25 @@ enum RuleMatchingBehavior {
     MatchOnlyUserAgentRules,
 };
 
+class PseudoStyleRequest {
+public:
+    PseudoStyleRequest(PseudoId pseudoId, RenderScrollbar* scrollbar = 0, ScrollbarPart scrollbarPart = NoPart)
+        : pseudoId(pseudoId)
+        , scrollbarPart(scrollbarPart)
+        , scrollbar(scrollbar)
+    {
+    }
+
+    PseudoId pseudoId;
+    ScrollbarPart scrollbarPart;
+    RenderScrollbar* scrollbar;
+};
+
 // This class selects a RenderStyle for a given element based on a collection of stylesheets.
 class StyleResolver {
     WTF_MAKE_NONCOPYABLE(StyleResolver); WTF_MAKE_FAST_ALLOCATED;
 public:
-    StyleResolver(Document*, bool matchAuthorAndUserStyles);
+    StyleResolver(Document&, bool matchAuthorAndUserStyles);
     ~StyleResolver();
 
     // Using these during tree walk will allow style selector to optimize child and descendant selector lookups.
@@ -151,59 +153,44 @@ public:
     void addHostRule(StyleRuleHost* rule, bool hasDocumentSecurityOrigin, const ContainerNode* scope) { ensureScopeResolver()->addHostRule(rule, hasDocumentSecurityOrigin, scope); }
 #endif
 
-    PassRefPtr<RenderStyle> styleForElement(Element*, RenderStyle* parentStyle = 0, StyleSharingBehavior = AllowStyleSharing,
+    PassRef<RenderStyle> styleForElement(Element*, RenderStyle* parentStyle = 0, StyleSharingBehavior = AllowStyleSharing,
         RuleMatchingBehavior = MatchAllRules, RenderRegion* regionForStyling = 0);
 
     void keyframeStylesForAnimation(Element*, const RenderStyle*, KeyframeList&);
 
-    PassRefPtr<RenderStyle> pseudoStyleForElement(PseudoId, Element*, RenderStyle* parentStyle);
+    PassRefPtr<RenderStyle> pseudoStyleForElement(Element*, const PseudoStyleRequest&, RenderStyle* parentStyle);
 
-    PassRefPtr<RenderStyle> styleForPage(int pageIndex);
-    PassRefPtr<RenderStyle> defaultStyleForElement();
-    PassRefPtr<RenderStyle> styleForText(Text*);
+    PassRef<RenderStyle> styleForPage(int pageIndex);
+    PassRef<RenderStyle> defaultStyleForElement();
 
-    static PassRefPtr<RenderStyle> styleForDocument(Document*, CSSFontSelector* = 0);
+    RenderStyle* style() const { return m_state.style(); }
+    RenderStyle* parentStyle() const { return m_state.parentStyle(); }
+    RenderStyle* rootElementStyle() const { return m_state.rootElementStyle(); }
+    Element* element() { return m_state.element(); }
+    Document& document() { return m_document; }
+    StyleScopeResolver* scopeResolver() const { return m_scopeResolver.get(); }
+    bool hasParentNode() const { return m_state.parentNode(); }
 
-    RenderStyle* style() const { return m_state.style.get(); }
-    RenderStyle* parentStyle() const { return m_state.parentStyle; }
-    RenderStyle* rootElementStyle() const { return m_state.rootElementStyle; }
-    Element* element() const { return m_state.element; }
-    Document* document() const { return m_document; }
-    const FontDescription& fontDescription() { return style()->fontDescription(); }
-    const FontDescription& parentFontDescription() { return parentStyle()->fontDescription(); }
-    void setFontDescription(const FontDescription& fontDescription) { m_state.fontDirty |= style()->setFontDescription(fontDescription); }
-    void setZoom(float f) { m_state.fontDirty |= style()->setZoom(f); }
-    void setEffectiveZoom(float f) { m_state.fontDirty |= style()->setEffectiveZoom(f); }
-    void setTextSizeAdjust(bool b) { m_state.fontDirty |= style()->setTextSizeAdjust(b); }
-    void setWritingMode(WritingMode writingMode) { m_state.fontDirty |= style()->setWritingMode(writingMode); }
-    void setTextOrientation(TextOrientation textOrientation) { m_state.fontDirty |= style()->setTextOrientation(textOrientation); }
-    bool hasParentNode() const { return m_state.parentNode; }
+    // FIXME: It could be better to call m_ruleSets.appendAuthorStyleSheets() directly after we factor StyleRsolver further.
+    // https://bugs.webkit.org/show_bug.cgi?id=108890
+    void appendAuthorStyleSheets(unsigned firstNew, const Vector<RefPtr<CSSStyleSheet>>&);
 
-    void resetAuthorStyle();
-    void appendAuthorStyleSheets(unsigned firstNew, const Vector<RefPtr<CSSStyleSheet> >&);
+    DocumentRuleSets& ruleSets() { return m_ruleSets; }
+    const DocumentRuleSets& ruleSets() const { return m_ruleSets; }
+    SelectorFilter& selectorFilter() { return m_selectorFilter; }
 
-private:
-#if ENABLE(STYLE_SCOPED) || ENABLE(SHADOW_DOM)
+#if ENABLE(SHADOW_DOM)
     StyleScopeResolver* ensureScopeResolver()
     {
-#if ENABLE(STYLE_SCOPED)
-#if ENABLE(SHADOW_DOM)
-        ASSERT(RuntimeEnabledFeatures::shadowDOMEnabled() || RuntimeEnabledFeatures::styleScopedEnabled());
-#else
-        ASSERT(RuntimeEnabledFeatures::styleScopedEnabled());
-#endif
-#else
-        ASSERT(RuntimeEnabledFeatures::shadowDOMEnabled());
-#endif
+        ASSERT(RuntimeEnabledFeatures::sharedFeatures().shadowDOMEnabled());
         if (!m_scopeResolver)
-            m_scopeResolver = adoptPtr(new StyleScopeResolver());
+            m_scopeResolver = std::make_unique<StyleScopeResolver>();
         return m_scopeResolver.get();
     }
 #endif
 
-    void initForStyleResolve(Element*, RenderStyle* parentStyle = 0, PseudoId = NOPSEUDO);
+private:
     void initElement(Element*);
-    void collectFeatures();
     RenderStyle* locateSharedStyle();
     bool styleSharingCandidateMatchesRuleSet(RuleSet*);
     bool styleSharingCandidateMatchesHostRules();
@@ -211,7 +198,7 @@ private:
     StyledElement* findSiblingForStyleSharing(Node*, unsigned& count) const;
     bool canShareStyleWithElement(StyledElement*) const;
 
-    PassRefPtr<RenderStyle> styleForKeyframe(const RenderStyle*, const StyleKeyframe*, KeyframeValue&);
+    PassRef<RenderStyle> styleForKeyframe(const RenderStyle*, const StyleKeyframe*, KeyframeValue&);
 
 public:
     // These methods will give back the set of rules that matched for a given element (or a pseudo-element).
@@ -223,15 +210,8 @@ public:
         AllButEmptyCSSRules = UAAndUserCSSRules | AuthorCSSRules | CrossOriginCSSRules,
         AllCSSRules         = AllButEmptyCSSRules | EmptyCSSRules,
     };
-    PassRefPtr<CSSRuleList> styleRulesForElement(Element*, unsigned rulesToInclude = AllButEmptyCSSRules);
-    PassRefPtr<CSSRuleList> pseudoStyleRulesForElement(Element*, PseudoId, unsigned rulesToInclude = AllButEmptyCSSRules);
-
-    // Given a CSS keyword in the range (xx-small to -webkit-xxx-large), this function will return
-    // the correct font size scaled relative to the user's default (medium).
-    static float fontSizeForKeyword(Document*, int keyword, bool shouldUseFixedDefaultSize);
-
-    // Given a font size in pixel, this function will return legacy font size between 1 and 7.
-    static int legacyFontSize(Document*, int pixelFontSize, bool shouldUseFixedDefaultSize);
+    Vector<RefPtr<StyleRuleBase>> styleRulesForElement(Element*, unsigned rulesToInclude = AllButEmptyCSSRules);
+    Vector<RefPtr<StyleRuleBase>> pseudoStyleRulesForElement(Element*, PseudoId, unsigned rulesToInclude = AllButEmptyCSSRules);
 
 public:
     void applyPropertyToStyle(CSSPropertyID, CSSValue*, RenderStyle*);
@@ -241,13 +221,8 @@ public:
     void updateFont();
     void initializeFontStyle(Settings*);
 
-    static float getComputedSizeFromSpecifiedSize(Document*, float zoomFactor, bool isAbsoluteSize, float specifiedSize, ESmartMinimumForFontSize = UseSmartMinimumForFontFize);
-
     void setFontSize(FontDescription&, float size);
 
-private:
-    static float getComputedSizeFromSpecifiedSize(Document*, RenderStyle*, bool isAbsoluteSize, float specifiedSize, bool useSVGZoomRules);
-
 public:
     bool useSVGZoomRules();
 
@@ -271,29 +246,14 @@ public:
 
     bool checkRegionStyle(Element* regionElement);
 
-    bool usesSiblingRules() const { return !m_features.siblingRules.isEmpty(); }
-    bool usesFirstLineRules() const { return m_features.usesFirstLineRules; }
-    bool usesBeforeAfterRules() const { return m_features.usesBeforeAfterRules; }
-
-    static bool createTransformOperations(CSSValue* inValue, RenderStyle* inStyle, RenderStyle* rootStyle, TransformOperations& outOperations);
+    bool usesSiblingRules() const { return !m_ruleSets.features().siblingRules.isEmpty(); }
+    bool usesFirstLineRules() const { return m_ruleSets.features().usesFirstLineRules; }
+    bool usesBeforeAfterRules() const { return m_ruleSets.features().usesBeforeAfterRules; }
     
     void invalidateMatchedPropertiesCache();
 
 #if ENABLE(CSS_FILTERS)
-    bool createFilterOperations(CSSValue* inValue, RenderStyle* inStyle, RenderStyle* rootStyle, FilterOperations& outOperations);
-#if ENABLE(CSS_SHADERS)
-    StyleShader* styleShader(CSSValue*);
-    StyleShader* cachedOrPendingStyleShaderFromValue(WebKitCSSShaderValue*);
-    bool parseCustomFilterParameterList(CSSValue*, CustomFilterParameterList&);
-    PassRefPtr<CustomFilterParameter> parseCustomFilterParameter(const String& name, CSSValue*);
-    PassRefPtr<CustomFilterParameter> parseCustomFilterArrayParameter(const String& name, CSSValueList*);
-    PassRefPtr<CustomFilterParameter> parseCustomFilterNumberParameter(const String& name, CSSValueList*);
-    PassRefPtr<CustomFilterParameter> parseCustomFilterTransformParameter(const String& name, CSSValueList*);
-    PassRefPtr<CustomFilterOperation> createCustomFilterOperationWithAtRuleReferenceSyntax(WebKitCSSFilterValue*);
-    PassRefPtr<CustomFilterOperation> createCustomFilterOperationWithInlineSyntax(WebKitCSSFilterValue*);
-    PassRefPtr<CustomFilterOperation> createCustomFilterOperation(WebKitCSSFilterValue*);
-    void loadPendingShaders();
-#endif
+    bool createFilterOperations(CSSValue* inValue, FilterOperations& outOperations);
 #if ENABLE(SVG)
     void loadPendingSVGDocuments();
 #endif
@@ -301,15 +261,7 @@ public:
 
     void loadPendingResources();
 
-private:
-    // This function fixes up the default font size if it detects that the current generic font family has changed. -dwh
-    void checkForGenericFamilyChange(RenderStyle*, RenderStyle* parentStyle);
-    void checkForZoomChange(RenderStyle*, RenderStyle* parentStyle);
-    void checkForTextSizeAdjust();
-
-    void adjustRenderStyle(RenderStyle* styleToAdjust, RenderStyle* parentStyle, Element*);
-
-    void addMatchedRule(const RuleData* rule) { m_state.matchedRules.append(rule); }
+    int viewportPercentageValue(CSSPrimitiveValue& unit, int percentage);
 
     struct RuleRange {
         RuleRange(int& firstRuleIndex, int& lastRuleIndex): firstRuleIndex(firstRuleIndex), lastRuleIndex(lastRuleIndex) { }
@@ -331,10 +283,10 @@ private:
     };
 
     struct MatchedProperties {
-        MatchedProperties() : possiblyPaddedMember(0) { }
-        void reportMemoryUsage(MemoryObjectInfo*) const;
+        MatchedProperties();
+        ~MatchedProperties();
         
-        RefPtr<StylePropertySet> properties;
+        RefPtr<StyleProperties> properties;
         union {
             struct {
                 unsigned linkMatchType : 2;
@@ -351,181 +303,188 @@ private:
         Vector<StyleRule*, 64> matchedRules;
         MatchRanges ranges;
         bool isCacheable;
+
+        void addMatchedProperties(const StyleProperties&, StyleRule* = 0, unsigned linkMatchType = SelectorChecker::MatchAll, PropertyWhitelistType = PropertyWhitelistNone);
     };
 
-    struct MatchRequest {
-        MatchRequest(RuleSet* ruleSet, bool includeEmptyRules = false, const ContainerNode* scope = 0)
-            : ruleSet(ruleSet)
-            , includeEmptyRules(includeEmptyRules)
-            , scope(scope) { }
-        const RuleSet* ruleSet;
-        const bool includeEmptyRules;
-        const ContainerNode* scope;
-    };
+private:
+    // This function fixes up the default font size if it detects that the current generic font family has changed. -dwh
+    void checkForGenericFamilyChange(RenderStyle*, RenderStyle* parentStyle);
+    void checkForZoomChange(RenderStyle*, RenderStyle* parentStyle);
+#if ENABLE(IOS_TEXT_AUTOSIZING)
+    void checkForTextSizeAdjust(RenderStyle*);
+#endif
 
-    static void addMatchedProperties(MatchResult&, const StylePropertySet* properties, StyleRule* = 0, unsigned linkMatchType = SelectorChecker::MatchAll, PropertyWhitelistType = PropertyWhitelistNone);
-    void addElementStyleProperties(MatchResult&, const StylePropertySet*, bool isCacheable = true);
-
-    void matchAllRules(MatchResult&, bool includeSMILProperties);
-    void matchUARules(MatchResult&);
-    void matchUARules(MatchResult&, RuleSet*);
-    void matchAuthorRules(MatchResult&, bool includeEmptyRules);
-    void matchUserRules(MatchResult&, bool includeEmptyRules);
-    void matchScopedAuthorRules(MatchResult&, bool includeEmptyRules);
-    void matchHostRules(MatchResult&, bool includeEmptyRules);
-
-    void collectMatchingRules(const MatchRequest&, RuleRange&);
-    void collectMatchingRulesForRegion(const MatchRequest&, RuleRange&);
-    void collectMatchingRulesForList(const Vector<RuleData>*, const MatchRequest&, RuleRange&);
+    void adjustRenderStyle(RenderStyle& styleToAdjust, const RenderStyle& parentStyle, Element*);
+    void adjustGridItemPosition(RenderStyle& styleToAdjust, const RenderStyle& parentStyle) const;
 
     bool fastRejectSelector(const RuleData&) const;
-    void sortMatchedRules();
-    void sortAndTransferMatchedRules(MatchResult&);
 
-    bool ruleMatches(const RuleData&, const ContainerNode* scope);
-    bool checkRegionSelector(const CSSSelector* regionSelector, Element* regionElement);
-    void applyMatchedProperties(const MatchResult&, const Element*);
-    enum StyleApplicationPass {
-#if ENABLE(CSS_VARIABLES)
-        VariableDefinitions,
-#endif
-        HighPriorityProperties,
-        LowPriorityProperties
-    };
-    template <StyleApplicationPass pass>
-    void applyMatchedProperties(const MatchResult&, bool important, int startIndex, int endIndex, bool inheritedOnly);
-    template <StyleApplicationPass pass>
-    void applyProperties(const StylePropertySet* properties, StyleRule*, bool isImportant, bool inheritedOnly, PropertyWhitelistType = PropertyWhitelistNone);
-#if ENABLE(CSS_VARIABLES)
-    void resolveVariables(CSSPropertyID, CSSValue*, Vector<std::pair<CSSPropertyID, String> >& knownExpressions);
-#endif
+    enum ShouldUseMatchedPropertiesCache { DoNotUseMatchedPropertiesCache = 0, UseMatchedPropertiesCache };
+    void applyMatchedProperties(const MatchResult&, const Element*, ShouldUseMatchedPropertiesCache = UseMatchedPropertiesCache);
+
+    class CascadedProperties;
+
+    void applyCascadedProperties(CascadedProperties&, int firstProperty, int lastProperty);
+    void cascadeMatches(CascadedProperties&, const MatchResult&, bool important, int startIndex, int endIndex, bool inheritedOnly);
+
     static bool isValidRegionStyleProperty(CSSPropertyID);
 #if ENABLE(VIDEO_TRACK)
     static bool isValidCueStyleProperty(CSSPropertyID);
 #endif
     void matchPageRules(MatchResult&, RuleSet*, bool isLeftPage, bool isFirstPage, const String& pageName);
     void matchPageRulesForList(Vector<StyleRulePage*>& matchedRules, const Vector<StyleRulePage*>&, bool isLeftPage, bool isFirstPage, const String& pageName);
-    Settings* documentSettings() { return m_document->settings(); }
+    Settings* documentSettings() { return m_document.settings(); }
 
     bool isLeftPage(int pageIndex) const;
     bool isRightPage(int pageIndex) const { return !isLeftPage(pageIndex); }
     bool isFirstPage(int pageIndex) const;
     String pageName(int pageIndex) const;
 
-    OwnPtr<RuleSet> m_authorStyle;
-    OwnPtr<RuleSet> m_userStyle;
+    DocumentRuleSets m_ruleSets;
 
-    RuleFeatureSet m_features;
-    OwnPtr<RuleSet> m_siblingRuleSet;
-    OwnPtr<RuleSet> m_uncommonAttributeRuleSet;
-
-    typedef HashMap<AtomicStringImpl*, RefPtr<StyleRuleKeyframes> > KeyframesRuleMap;
+    typedef HashMap<AtomicStringImpl*, RefPtr<StyleRuleKeyframes>> KeyframesRuleMap;
     KeyframesRuleMap m_keyframesRuleMap;
 
 public:
-    typedef HashMap<CSSPropertyID, RefPtr<CSSValue> > PendingImagePropertyMap;
-#if ENABLE(CSS_FILTERS) && ENABLE(SVG)
-    typedef HashMap<FilterOperation*, RefPtr<WebKitCSSSVGDocumentValue> > PendingSVGDocumentMap;
-#endif
+    typedef HashMap<CSSPropertyID, RefPtr<CSSValue>> PendingImagePropertyMap;
 
     class State {
         WTF_MAKE_NONCOPYABLE(State);
     public:
         State()
-        : element(0)
-        , styledElement(0)
-        , parentNode(0)
-        , parentStyle(0)
-        , rootElementStyle(0)
-        , regionForStyling(0)
-        , sameOriginOnly(false)
-        , pseudoStyle(NOPSEUDO)
-        , elementLinkState(NotInsideLink)
-        , distributedToInsertionPoint(false)
-        , elementAffectedByClassRules(false)
-        , applyPropertyToRegularStyle(true)
-        , applyPropertyToVisitedLinkStyle(false)
-#if ENABLE(CSS_SHADERS)
-        , hasPendingShaders(false)
+        : m_element(0)
+        , m_styledElement(0)
+        , m_parentNode(0)
+        , m_parentStyle(0)
+        , m_rootElementStyle(0)
+        , m_regionForStyling(0)
+        , m_elementLinkState(NotInsideLink)
+        , m_elementAffectedByClassRules(false)
+        , m_applyPropertyToRegularStyle(true)
+        , m_applyPropertyToVisitedLinkStyle(false)
+        , m_lineHeightValue(0)
+        , m_fontDirty(false)
+        , m_hasUAAppearance(false)
+        , m_backgroundData(BackgroundFillLayer) { }
+
+    public:
+        void initElement(Element*);
+        void initForStyleResolve(Document&, Element*, RenderStyle* parentStyle = 0, RenderRegion* regionForStyling = 0);
+        void clear();
+
+        Document& document() const { return m_element->document(); }
+        Element* element() const { return m_element; }
+        StyledElement* styledElement() const { return m_styledElement; }
+        void setStyle(PassRef<RenderStyle> style) { m_style = std::move(style); }
+        RenderStyle* style() const { return m_style.get(); }
+        PassRef<RenderStyle> takeStyle() { return m_style.releaseNonNull(); }
+
+        const ContainerNode* parentNode() const { return m_parentNode; }
+        void setParentStyle(PassRef<RenderStyle> parentStyle) { m_parentStyle = std::move(parentStyle); }
+        RenderStyle* parentStyle() const { return m_parentStyle.get(); }
+        RenderStyle* rootElementStyle() const { return m_rootElementStyle; }
+
+        const RenderRegion* regionForStyling() const { return m_regionForStyling; }
+        EInsideLink elementLinkState() const { return m_elementLinkState; }
+        void setElementAffectedByClassRules(bool isAffected) { m_elementAffectedByClassRules = isAffected; }
+        bool elementAffectedByClassRules() const { return m_elementAffectedByClassRules; }
+
+        void setApplyPropertyToRegularStyle(bool isApply) { m_applyPropertyToRegularStyle = isApply; }
+        void setApplyPropertyToVisitedLinkStyle(bool isApply) { m_applyPropertyToVisitedLinkStyle = isApply; }
+        bool applyPropertyToRegularStyle() const { return m_applyPropertyToRegularStyle; }
+        bool applyPropertyToVisitedLinkStyle() const { return m_applyPropertyToVisitedLinkStyle; }
+        PendingImagePropertyMap& pendingImageProperties() { return m_pendingImageProperties; }
+#if ENABLE(CSS_FILTERS) && ENABLE(SVG)
+        Vector<RefPtr<ReferenceFilterOperation>>& filtersWithPendingSVGDocuments() { return m_filtersWithPendingSVGDocuments; }
 #endif
-        , lineHeightValue(0)
-        , fontDirty(false)
-        , hasUAAppearance(false)
-        , backgroundData(BackgroundFillLayer) { }
-        
-public:
+
+        void setLineHeightValue(CSSValue* value) { m_lineHeightValue = value; }
+        CSSValue* lineHeightValue() { return m_lineHeightValue; }
+        void setFontDirty(bool isDirty) { m_fontDirty = isDirty; }
+        bool fontDirty() const { return m_fontDirty; }
+
+        void cacheBorderAndBackground();
+        bool hasUAAppearance() const { return m_hasUAAppearance; }
+        BorderData borderData() const { return m_borderData; }
+        FillLayer backgroundData() const { return m_backgroundData; }
+        Color backgroundColor() const { return m_backgroundColor; }
+
+        const FontDescription& fontDescription() { return m_style->fontDescription(); }
+        const FontDescription& parentFontDescription() { return m_parentStyle->fontDescription(); }
+        void setFontDescription(const FontDescription& fontDescription) { m_fontDirty |= m_style->setFontDescription(fontDescription); }
+        void setZoom(float f) { m_fontDirty |= m_style->setZoom(f); }
+        void setEffectiveZoom(float f) { m_fontDirty |= m_style->setEffectiveZoom(f); }
+        void setWritingMode(WritingMode writingMode) { m_fontDirty |= m_style->setWritingMode(writingMode); }
+        void setTextOrientation(TextOrientation textOrientation) { m_fontDirty |= m_style->setTextOrientation(textOrientation); }
+
+        bool useSVGZoomRules() const { return m_element && m_element->isSVGElement(); }
+
+    private:
         // FIXME(bug 108563): to make it easier to review, these member
         // variables are public. However we should add methods to access
         // these variables.
-        Element* element;
-        RefPtr<RenderStyle> style;
-        StyledElement* styledElement;
-        ContainerNode* parentNode;
-        RenderStyle* parentStyle;
-        RenderStyle* rootElementStyle;
+        Element* m_element;
+        RefPtr<RenderStyle> m_style;
+        StyledElement* m_styledElement;
+        ContainerNode* m_parentNode;
+        RefPtr<RenderStyle> m_parentStyle;
+        RenderStyle* m_rootElementStyle;
+
+        // Required to ASSERT in applyProperties.
+        RenderRegion* m_regionForStyling;
         
-        RenderRegion* regionForStyling;
-        bool sameOriginOnly;
-        PseudoId pseudoStyle;
+        EInsideLink m_elementLinkState;
 
-        EInsideLink elementLinkState;
+        bool m_elementAffectedByClassRules;
 
-        bool distributedToInsertionPoint;
+        bool m_applyPropertyToRegularStyle;
+        bool m_applyPropertyToVisitedLinkStyle;
 
-        bool elementAffectedByClassRules;
-
-        PseudoId dynamicPseudo;
-
-        // A buffer used to hold the set of matched rules for an element,
-        // and a temporary buffer used for merge sorting.
-        Vector<const RuleData*, 32> matchedRules;
-        RefPtr<StaticCSSRuleList> ruleList;
-
-        bool applyPropertyToRegularStyle;
-        bool applyPropertyToVisitedLinkStyle;
-
-        PendingImagePropertyMap pendingImageProperties;
-#if ENABLE(CSS_SHADERS)
-        bool hasPendingShaders;
-#endif
+        PendingImagePropertyMap m_pendingImageProperties;
 #if ENABLE(CSS_FILTERS) && ENABLE(SVG)
-        PendingSVGDocumentMap pendingSVGDocuments;
+        Vector<RefPtr<ReferenceFilterOperation>> m_filtersWithPendingSVGDocuments;
 #endif
-        CSSValue* lineHeightValue;
-        bool fontDirty;
+        CSSValue* m_lineHeightValue;
+        bool m_fontDirty;
 
-        bool hasUAAppearance;
-        BorderData borderData;
-        FillLayer backgroundData;
-        Color backgroundColor;
+        bool m_hasUAAppearance;
+        BorderData m_borderData;
+        FillLayer m_backgroundData;
+        Color m_backgroundColor;
     };
 
+    State& state() { return m_state; }
+
     static RenderStyle* styleNotYetAvailable() { return s_styleNotYetAvailable; }
 
     PassRefPtr<StyleImage> styleImage(CSSPropertyID, CSSValue*);
     PassRefPtr<StyleImage> cachedOrPendingFromValue(CSSPropertyID, CSSImageValue*);
-    PassRefPtr<StyleImage> generatedOrPendingFromValue(CSSPropertyID, CSSImageGeneratorValue*);
+    PassRefPtr<StyleImage> generatedOrPendingFromValue(CSSPropertyID, CSSImageGeneratorValue&);
 #if ENABLE(CSS_IMAGE_SET)
     PassRefPtr<StyleImage> setOrPendingFromValue(CSSPropertyID, CSSImageSetValue*);
 #endif
     PassRefPtr<StyleImage> cursorOrPendingFromValue(CSSPropertyID, CSSCursorImageValue*);
 
-    bool applyPropertyToRegularStyle() const { return m_state.applyPropertyToRegularStyle; }
-    bool applyPropertyToVisitedLinkStyle() const { return m_state.applyPropertyToVisitedLinkStyle; }
+    bool applyPropertyToRegularStyle() const { return m_state.applyPropertyToRegularStyle(); }
+    bool applyPropertyToVisitedLinkStyle() const { return m_state.applyPropertyToVisitedLinkStyle(); }
 
-    static Length convertToIntLength(CSSPrimitiveValue*, RenderStyle*, RenderStyle* rootStyle, double multiplier = 1);
-    static Length convertToFloatLength(CSSPrimitiveValue*, RenderStyle*, RenderStyle* rootStyle, double multiplier = 1);
+    static Length convertToIntLength(const CSSPrimitiveValue*, const RenderStyle*, const RenderStyle* rootStyle, double multiplier = 1);
+    static Length convertToFloatLength(const CSSPrimitiveValue*, const RenderStyle*, const RenderStyle* rootStyle, double multiplier = 1);
 
     CSSToStyleMap* styleMap() { return &m_styleMap; }
     InspectorCSSOMWrappers& inspectorCSSOMWrappers() { return m_inspectorCSSOMWrappers; }
+    const FontDescription& fontDescription() { return m_state.fontDescription(); }
+    const FontDescription& parentFontDescription() { return m_state.parentFontDescription(); }
+    void setFontDescription(const FontDescription& fontDescription) { m_state.setFontDescription(fontDescription); }
+    void setZoom(float f) { m_state.setZoom(f); }
+    void setEffectiveZoom(float f) { m_state.setEffectiveZoom(f); }
+    void setWritingMode(WritingMode writingMode) { m_state.setWritingMode(writingMode); }
+    void setTextOrientation(TextOrientation textOrientation) { m_state.setTextOrientation(textOrientation); }
 
-    void reportMemoryUsage(MemoryObjectInfo*) const;
-    
 private:
     static RenderStyle* s_styleNotYetAvailable;
 
-    void collectRulesFromUserStyleSheets(const Vector<RefPtr<CSSStyleSheet> >&, RuleSet& userStyle);
-
     void cacheBorderAndBackground();
 
 private:
@@ -537,12 +496,15 @@ private:
     void applySVGProperty(CSSPropertyID, CSSValue*);
 #endif
 
+    PassRefPtr<StyleImage> loadPendingImage(StylePendingImage*, const ResourceLoaderOptions&);
     PassRefPtr<StyleImage> loadPendingImage(StylePendingImage*);
     void loadPendingImages();
+#if ENABLE(CSS_SHAPES)
+    void loadPendingShapeImage(ShapeValue*);
+#endif
 
     static unsigned computeMatchedPropertiesHash(const MatchedProperties*, unsigned size);
     struct MatchedPropertiesCacheItem {
-        void reportMemoryUsage(MemoryObjectInfo*) const;
         Vector<MatchedProperties> matchedProperties;
         MatchRanges ranges;
         RefPtr<RenderStyle> renderStyle;
@@ -558,6 +520,7 @@ private:
     bool classNamesAffectedByRules(const SpaceSplitString&) const;
     bool sharingCandidateHasIdenticalStyleAffectingAttributes(StyledElement*) const;
 
+
     unsigned m_matchedPropertiesCacheAdditionsSinceLastSweep;
 
     typedef HashMap<unsigned, MatchedPropertiesCacheItem> MatchedPropertiesCache;
@@ -565,35 +528,30 @@ private:
 
     Timer<StyleResolver> m_matchedPropertiesCacheSweepTimer;
 
-    OwnPtr<MediaQueryEvaluator> m_medium;
+    std::unique_ptr<MediaQueryEvaluator> m_medium;
     RefPtr<RenderStyle> m_rootDefaultStyle;
 
-    Document* m_document;
-    // FIXME: Make SelectorChecker an on-stack object and
-    // move selectorChecker.mode() to State.
-    // selectorChecker.stringParsing() can be replaced with
-    // !document.inQuirksMode().
-    SelectorChecker m_selectorChecker;
+    Document& m_document;
     SelectorFilter m_selectorFilter;
 
     bool m_matchAuthorAndUserStyles;
 
     RefPtr<CSSFontSelector> m_fontSelector;
-    Vector<OwnPtr<MediaQueryResult> > m_viewportDependentMediaQueryResults;
+    Vector<std::unique_ptr<MediaQueryResult>> m_viewportDependentMediaQueryResults;
 
 #if ENABLE(CSS_DEVICE_ADAPTATION)
     RefPtr<ViewportStyleResolver> m_viewportStyleResolver;
 #endif
 
-    const StyleBuilder& m_styleBuilder;
+    const DeprecatedStyleBuilder& m_deprecatedStyleBuilder;
 
-    OwnPtr<StyleScopeResolver> m_scopeResolver;
+    std::unique_ptr<StyleScopeResolver> m_scopeResolver;
     CSSToStyleMap m_styleMap;
     InspectorCSSOMWrappers m_inspectorCSSOMWrappers;
 
     State m_state;
 
-    friend class StyleBuilder;
+    friend class DeprecatedStyleBuilder;
     friend bool operator==(const MatchedProperties&, const MatchedProperties&);
     friend bool operator!=(const MatchedProperties&, const MatchedProperties&);
     friend bool operator==(const MatchRanges&, const MatchRanges&);
@@ -603,21 +561,66 @@ private:
 inline bool StyleResolver::hasSelectorForAttribute(const AtomicString &attributeName) const
 {
     ASSERT(!attributeName.isEmpty());
-    return m_features.attrsInRules.contains(attributeName.impl());
+    return m_ruleSets.features().attrsInRules.contains(attributeName.impl());
 }
 
 inline bool StyleResolver::hasSelectorForClass(const AtomicString& classValue) const
 {
     ASSERT(!classValue.isEmpty());
-    return m_features.classesInRules.contains(classValue.impl());
+    return m_ruleSets.features().classesInRules.contains(classValue.impl());
 }
 
 inline bool StyleResolver::hasSelectorForId(const AtomicString& idValue) const
 {
     ASSERT(!idValue.isEmpty());
-    return m_features.idsInRules.contains(idValue.impl());
+    return m_ruleSets.features().idsInRules.contains(idValue.impl());
 }
 
+inline bool checkRegionSelector(const CSSSelector* regionSelector, Element* regionElement)
+{
+    if (!regionSelector || !regionElement)
+        return false;
+
+    SelectorChecker selectorChecker(regionElement->document(), SelectorChecker::QueryingRules);
+    for (const CSSSelector* s = regionSelector; s; s = CSSSelectorList::next(s)) {
+        SelectorChecker::SelectorCheckingContext selectorCheckingContext(s, regionElement, SelectorChecker::VisitedMatchDisabled);
+        PseudoId ignoreDynamicPseudo = NOPSEUDO;
+        if (selectorChecker.match(selectorCheckingContext, ignoreDynamicPseudo))
+            return true;
+    }
+    return false;
+}
+
+class StyleResolverParentPusher {
+public:
+    StyleResolverParentPusher(Element* parent)
+        : m_parent(parent)
+        , m_pushedStyleResolver(0)
+    { }
+    void push()
+    {
+        if (m_pushedStyleResolver)
+            return;
+        m_pushedStyleResolver = &m_parent->document().ensureStyleResolver();
+        m_pushedStyleResolver->pushParentElement(m_parent);
+    }
+    ~StyleResolverParentPusher()
+    {
+        if (!m_pushedStyleResolver)
+            return;
+        // This tells us that our pushed style selector is in a bad state,
+        // so we should just bail out in that scenario.
+        ASSERT(m_pushedStyleResolver == &m_parent->document().ensureStyleResolver());
+        if (m_pushedStyleResolver != &m_parent->document().ensureStyleResolver())
+            return;
+        m_pushedStyleResolver->popParentElement(m_parent);
+    }
+    
+private:
+    Element* m_parent;
+    StyleResolver* m_pushedStyleResolver;
+};
+
 } // namespace WebCore
 
 #endif // StyleResolver_h
