@@ -24,10 +24,10 @@
 #include "Attr.h"
 
 #include "ExceptionCode.h"
-#include "HTMLNames.h"
 #include "ScopedEventQueue.h"
+#include "StyleProperties.h"
 #include "StyledElement.h"
-#include "Text.h"
+#include "TextNodeTraversal.h"
 #include "XMLNSNames.h"
 #include <wtf/text/AtomicString.h>
 #include <wtf/text/StringBuilder.h>
@@ -37,21 +37,19 @@ namespace WebCore {
 using namespace HTMLNames;
 
 Attr::Attr(Element* element, const QualifiedName& name)
-    : ContainerNode(element->document())
+    : ContainerNode(&element->document())
     , m_element(element)
     , m_name(name)
     , m_ignoreChildrenChanged(0)
-    , m_specified(true)
 {
 }
 
-Attr::Attr(Document* document, const QualifiedName& name, const AtomicString& standaloneValue)
-    : ContainerNode(document)
+Attr::Attr(Document& document, const QualifiedName& name, const AtomicString& standaloneValue)
+    : ContainerNode(&document)
     , m_element(0)
     , m_name(name)
     , m_standaloneValue(standaloneValue)
     , m_ignoreChildrenChanged(0)
-    , m_specified(true)
 {
 }
 
@@ -62,7 +60,7 @@ PassRefPtr<Attr> Attr::create(Element* element, const QualifiedName& name)
     return attr.release();
 }
 
-PassRefPtr<Attr> Attr::create(Document* document, const QualifiedName& name, const AtomicString& value)
+PassRefPtr<Attr> Attr::create(Document& document, const QualifiedName& name, const AtomicString& value)
 {
     RefPtr<Attr> attr = adoptRef(new Attr(document, name, value));
     attr->createTextChild();
@@ -77,11 +75,11 @@ void Attr::createTextChild()
 {
     ASSERT(refCount());
     if (!value().isEmpty()) {
-        RefPtr<Text> textNode = document()->createTextNode(value().string());
+        RefPtr<Text> textNode = document().createTextNode(value().string());
 
         // This does everything appendChild() would do in this situation (assuming m_ignoreChildrenChanged was set),
         // but much more efficiently.
-        textNode->setParentOrShadowHostNode(this);
+        textNode->setParentNode(this);
         setFirstChild(textNode.get());
         setLastChild(textNode.get());
     }
@@ -119,7 +117,7 @@ void Attr::setValue(const AtomicString& value)
     createTextChild();
     m_ignoreChildrenChanged--;
 
-    invalidateNodeListCachesInAncestors(&m_name, m_element);
+    invalidateNodeListAndCollectionCachesInAncestors(&m_name, m_element);
 }
 
 void Attr::setValue(const AtomicString& value, ExceptionCode&)
@@ -157,22 +155,17 @@ bool Attr::childTypeAllowed(NodeType type) const
     }
 }
 
-void Attr::childrenChanged(bool, Node*, Node*, int)
+void Attr::childrenChanged(const ChildChange&)
 {
     if (m_ignoreChildrenChanged > 0)
         return;
 
-    invalidateNodeListCachesInAncestors(&qualifiedName(), m_element);
-
-    // FIXME: We should include entity references in the value
+    invalidateNodeListAndCollectionCachesInAncestors(&qualifiedName(), m_element);
 
     StringBuilder valueBuilder;
-    for (Node *n = firstChild(); n; n = n->nextSibling()) {
-        if (n->isTextNode())
-            valueBuilder.append(toText(n)->data());
-    }
+    TextNodeTraversal::appendContents(this, valueBuilder);
 
-    AtomicString newValue = valueBuilder.toString();
+    AtomicString newValue = valueBuilder.toAtomicString();
     if (m_element)
         m_element->willModifyAttribute(qualifiedName(), value(), newValue);
 
@@ -187,16 +180,16 @@ void Attr::childrenChanged(bool, Node*, Node*, int)
 
 bool Attr::isId() const
 {
-    return qualifiedName().matches(document()->idAttributeName());
+    return qualifiedName().matches(document().idAttributeName());
 }
 
 CSSStyleDeclaration* Attr::style()
 {
     // This function only exists to support the Obj-C bindings.
     if (!m_element || !m_element->isStyledElement())
-        return 0;
-    m_style = StylePropertySet::create();
-    static_cast<StyledElement*>(m_element)->collectStyleForPresentationAttribute(elementAttribute(), m_style.get());
+        return nullptr;
+    m_style = MutableStyleProperties::create();
+    toStyledElement(m_element)->collectStyleForPresentationAttribute(qualifiedName(), value(), *m_style);
     return m_style->ensureCSSStyleDeclaration();
 }
 
@@ -210,8 +203,8 @@ const AtomicString& Attr::value() const
 Attribute& Attr::elementAttribute()
 {
     ASSERT(m_element);
-    ASSERT(m_element->attributeData());
-    return *m_element->getAttributeItem(qualifiedName());
+    ASSERT(m_element->elementData());
+    return *m_element->ensureUniqueElementData().findAttributeByName(qualifiedName());
 }
 
 void Attr::detachFromElementWithValue(const AtomicString& value)
