@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 
-# Copyright (C) 2005, 2006, 2007, 2009 Apple Inc. All rights reserved.
+# Copyright (C) 2005, 2006, 2007, 2009, 2013 Apple Inc. All rights reserved.
 # Copyright (C) 2009, Julien Chaffraix <jchaffraix@webkit.org>
 # Copyright (C) 2009 Torch Mobile Inc. All rights reserved. (http://www.torchmobile.com/)
 # Copyright (C) 2011 Ericsson AB. All rights reserved.
@@ -65,10 +65,12 @@ require Config;
 my $gccLocation = "";
 if ($ENV{CC}) {
     $gccLocation = $ENV{CC};
-} elsif (($Config::Config{'osname'}) =~ /solaris/i) {
+} elsif (($Config::Config{"osname"}) =~ /solaris/i) {
     $gccLocation = "/usr/sfw/bin/gcc";
+} elsif ($Config::Config{"osname"} eq "darwin" && $ENV{SDKROOT}) {
+    chomp($gccLocation = `xcrun -find cc -sdk '$ENV{SDKROOT}'`);
 } else {
-    $gccLocation = "/usr/bin/gcc";
+    $gccLocation = "/usr/bin/cc";
 }
 my $preprocessor = $gccLocation . " -E -x c++";
 
@@ -127,7 +129,10 @@ if (length($fontNamesIn)) {
     print F StaticString::GenerateStringAsserts(\%parameters);
 
     while ( my ($name, $identifier) = each %parameters ) {
-        print F "    new (NotNull, (void*)&$name) AtomicString(${name}Impl);\n";
+        # FIXME: Would like to use static_cast here, but there are differences in const
+        # depending on whether SKIP_STATIC_CONSTRUCTORS_ON_GCC is used, so stick with a
+        # C-style cast for now.
+        print F "    new (NotNull, (void*)&$name) AtomicString(reinterpret_cast<StringImpl*>(&${name}Data));\n";
     }
 
     print F "}\n}\n}\n";
@@ -155,12 +160,14 @@ die "You must specify a namespaceURI (e.g. http://www.w3.org/2000/svg)" unless $
 $parameters{namespacePrefix} = $parameters{namespace} unless $parameters{namespacePrefix};
 $parameters{fallbackJSInterfaceName} = $parameters{fallbackInterfaceName} unless $parameters{fallbackJSInterfaceName};
 
+my $typeHelpersBasePath = "$outputDir/$parameters{namespace}ElementTypeHelpers";
 my $namesBasePath = "$outputDir/$parameters{namespace}Names";
 my $factoryBasePath = "$outputDir/$parameters{namespace}ElementFactory";
 my $wrapperFactoryFileName = "$parameters{namespace}ElementWrapperFactory";
 
 printNamesHeaderFile("$namesBasePath.h");
 printNamesCppFile("$namesBasePath.cpp");
+printTypeHelpersHeaderFile("$typeHelpersBasePath.h");
 
 if ($printFactory) {
     printFactoryCppFile("$factoryBasePath.cpp");
@@ -186,8 +193,8 @@ sub defaultTagPropertyHash
         'mapToTagName' => '',
         'wrapperOnlyIfMediaIsAvailable' => 0,
         'conditional' => 0,
-        'contextConditional' => 0,
-        'runtimeConditional' => 0
+        'runtimeConditional' => 0,
+        'generateTypeHelpers' => 0
     );
 }
 
@@ -201,7 +208,7 @@ sub defaultParametersHash
         'tagsNullNamespace' => 0,
         'attrsNullNamespace' => 0,
         'fallbackInterfaceName' => '',
-        'fallbackJSInterfaceName' => ''
+        'fallbackJSInterfaceName' => '',
     );
 }
 
@@ -374,7 +381,7 @@ sub printConstructorSignature
 {
     my ($F, $tagName, $constructorName, $constructorTagName) = @_;
 
-    print F "static PassRefPtr<$parameters{namespace}Element> ${constructorName}Constructor(const QualifiedName& $constructorTagName, Document* document";
+    print F "static PassRefPtr<$parameters{namespace}Element> ${constructorName}Constructor(const QualifiedName& $constructorTagName, Document& document";
     if ($parameters{namespace} eq "HTML") {
         print F ", HTMLFormElement*";
         print F " formElement" if $enabledTags{$tagName}{constructorNeedsFormElement};
@@ -392,9 +399,13 @@ sub printConstructorInterior
     my ($F, $tagName, $interfaceName, $constructorTagName) = @_;
 
     # Handle media elements.
+    # Note that wrapperOnlyIfMediaIsAvailable is a misnomer, because media availability
+    # does not just control the wrapper; it controls the element object that is created.
+    # FIXME: Could we instead do this entirely in the wrapper, and use custom wrappers
+    # instead of having all the support for this here in this script?
     if ($enabledTags{$tagName}{wrapperOnlyIfMediaIsAvailable}) {
         print F <<END
-    Settings* settings = document->settings();
+    Settings* settings = document.settings();
     if (!MediaPlayer::isAvailable() || (settings && !settings->mediaEnabled()))
         return 0;
     
@@ -402,19 +413,10 @@ END
 ;
     }
 
-    my $contextConditional = $enabledTags{$tagName}{contextConditional};
-    if ($contextConditional) {
-        print F <<END
-    if (!ContextFeatures::${contextConditional}Enabled(document))
-        return 0;
-END
-;
-    }
-
     my $runtimeConditional = $enabledTags{$tagName}{runtimeConditional};
     if ($runtimeConditional) {
         print F <<END
-    if (!RuntimeEnabledFeatures::${runtimeConditional}Enabled())
+    if (!RuntimeEnabledFeatures::sharedFeatures().${runtimeConditional}Enabled())
         return 0;
 END
 ;
@@ -473,7 +475,7 @@ sub printConstructors
     }
 }
 
-sub printFunctionInits
+sub printFunctionTable
 {
     my ($F, $tagConstructorMap) = @_;
     my %tagConstructorMap = %$tagConstructorMap;
@@ -488,9 +490,9 @@ sub printFunctionInits
         }
 
         if ($enabledTags{$tagName}{mapToTagName}) {
-            print F "    addTag(${tagName}Tag, $enabledTags{$tagName}{mapToTagName}To${tagName}Constructor);\n";
+            print F "        { ${tagName}Tag, $enabledTags{$tagName}{mapToTagName}To${tagName}Constructor },\n";
         } else {
-            print F "    addTag(${tagName}Tag, $tagConstructorMap{$tagName}Constructor);\n";
+            print F "        { ${tagName}Tag, $tagConstructorMap{$tagName}Constructor },\n";
         }
 
         if ($conditional) {
@@ -586,7 +588,7 @@ sub printLicenseHeader
  *
  * This file was generated by the dom/make_names.pl script.
  *
- * Copyright (C) 2005, 2006, 2007, 2008, 2009 Apple Inc.  All rights reserved.
+ * Copyright (C) 2005, 2006, 2007, 2008, 2009, 2013 Apple Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -613,6 +615,85 @@ sub printLicenseHeader
 ";
 }
 
+sub printTypeHelpers
+{
+    my ($F, $namesRef) = @_;
+    my %names = %$namesRef;
+
+    for my $name (sort keys %names) {
+        if (!$parsedTags{$name}{generateTypeHelpers}) {
+            next;
+        }
+
+        my $class = $parsedTags{$name}{interfaceName};
+        my $checkHelper = "is$class";
+
+        print F <<END
+class $class;
+void $checkHelper(const $class&); // Catch unnecessary runtime check of type known at compile time.
+void $checkHelper(const $class*); // Catch unnecessary runtime check of type known at compile time.
+END
+        ;
+
+        if ($parameters{namespace} eq "HTML") {
+            if ($parsedTags{$name}{wrapperOnlyIfMediaIsAvailable}) {
+                # We need to check for HTMLUnknownElement if it might have been created by the factory.
+                print F <<END
+inline bool $checkHelper(const HTMLElement& element) { return !element.isHTMLUnknownElement() && element.hasLocalName($parameters{namespace}Names::${name}Tag); }
+inline bool $checkHelper(const HTMLElement* element) { ASSERT(element); return $checkHelper(*element); }
+END
+                ;
+            } else {
+                print F <<END
+inline bool $checkHelper(const HTMLElement& element) { return element.hasLocalName(HTMLNames::${name}Tag); }
+inline bool $checkHelper(const HTMLElement* element) { ASSERT(element); return $checkHelper(*element); }
+END
+                ;
+            }
+
+                print F <<END
+inline bool $checkHelper(const Node& node) { return node.isHTMLElement() && $checkHelper(toHTMLElement(node)); }
+inline bool $checkHelper(const Node* node) { ASSERT(node); return $checkHelper(*node); }
+template <> inline bool isElementOfType<const $class>(const HTMLElement& element) { return $checkHelper(element); }
+template <> inline bool isElementOfType<const $class>(const Element& element) { return $checkHelper(element); }
+END
+                ;
+
+        } else {
+            print F <<END
+inline bool $checkHelper(const Element& element) { return element.hasTagName($parameters{namespace}Names::${name}Tag); }
+inline bool $checkHelper(const Element* element) { ASSERT(element); return $checkHelper(*element); }
+inline bool $checkHelper(const Node& node) { return node.isElementNode() && $checkHelper(toElement(node)); }
+inline bool $checkHelper(const Node* node) { ASSERT(node); return node->isElementNode() && $checkHelper(toElement(node)); }
+template <> inline bool isElementOfType<const $class>(const Element& element) { return $checkHelper(element); }
+END
+            ;
+        }
+
+        print F "\n";
+    }
+}
+
+sub printTypeHelpersHeaderFile
+{
+    my ($headerPath) = shift;
+    my $F;
+    open F, ">$headerPath";
+    printLicenseHeader($F);
+
+    print F "#ifndef ".$parameters{namespace}."ElementTypeHelpers_h\n";
+    print F "#define ".$parameters{namespace}."ElementTypeHelpers_h\n\n";
+    print F "#include \"".$parameters{namespace}."Names.h\"\n\n";
+    print F "namespace WebCore {\n\n";
+
+    printTypeHelpers($F, \%allTags);
+
+    print F "}\n\n";
+    print F "#endif\n";
+
+    close F;
+}
+
 sub printNamesHeaderFile
 {
     my ($headerPath) = shift;
@@ -639,12 +720,12 @@ sub printNamesHeaderFile
 
     if (keys %allTags) {
         print F "const unsigned $parameters{namespace}TagsCount = ", scalar(keys %allTags), ";\n";
-        print F "WebCore::QualifiedName** get$parameters{namespace}Tags();\n";
+        print F "const WebCore::QualifiedName* const * get$parameters{namespace}Tags();\n";
     }
 
     if (keys %allAttrs) {
         print F "const unsigned $parameters{namespace}AttrsCount = ", scalar(keys %allAttrs), ";\n";
-        print F "WebCore::QualifiedName** get$parameters{namespace}Attrs();\n";
+        print F "const WebCore::QualifiedName* const * get$parameters{namespace}Attrs();\n";
     }
 
     printInit($F, 1);
@@ -672,10 +753,10 @@ sub printNamesCppFile
             print F "DEFINE_GLOBAL(QualifiedName, ", $name, "Tag)\n";
         }
         
-        print F "\n\nWebCore::QualifiedName** get$parameters{namespace}Tags()\n";
-        print F "{\n    static WebCore::QualifiedName* $parameters{namespace}Tags[] = {\n";
+        print F "\n\nconst WebCore::QualifiedName* const * get$parameters{namespace}Tags()\n";
+        print F "{\n    static const WebCore::QualifiedName* const $parameters{namespace}Tags[] = {\n";
         for my $name (sort keys %allTags) {
-            print F "        (WebCore::QualifiedName*)&${name}Tag,\n";
+            print F "        reinterpret_cast<const WebCore::QualifiedName*>(&${name}Tag),\n";
         }
         print F "    };\n";
         print F "    return $parameters{namespace}Tags;\n";
@@ -687,13 +768,13 @@ sub printNamesCppFile
         for my $name (sort keys %allAttrs) {
             print F "DEFINE_GLOBAL(QualifiedName, ", $name, "Attr)\n";
         }
-        print F "\n\nWebCore::QualifiedName** get$parameters{namespace}Attrs()\n";
-        print F "{\n    static WebCore::QualifiedName* $parameters{namespace}Attr[] = {\n";
+        print F "\n\nconst WebCore::QualifiedName* const * get$parameters{namespace}Attrs()\n";
+        print F "{\n    static const WebCore::QualifiedName* const $parameters{namespace}Attrs[] = {\n";
         for my $name (sort keys %allAttrs) {
-            print F "        (WebCore::QualifiedName*)&${name}Attr,\n";
+            print F "        reinterpret_cast<const WebCore::QualifiedName*>(&${name}Attr),\n";
         }
         print F "    };\n";
-        print F "    return $parameters{namespace}Attr;\n";
+        print F "    return $parameters{namespace}Attrs;\n";
         print F "}\n";
     }
 
@@ -798,20 +879,34 @@ sub printConditionalElementIncludes
 sub printDefinitions
 {
     my ($F, $namesRef, $type, $namespaceURI) = @_;
-    my $singularType = substr($type, 0, -1);
-    my $shortType = substr($singularType, 0, 4);
-    my $shortCamelType = ucfirst($shortType);
-    my $shortUpperType = uc($shortType);
-    
-    print F "    // " . ucfirst($type) . "\n";
 
+    my $shortCamelType = ucfirst(substr(substr($type, 0, -1), 0, 4));
+    my $capitalizedType = ucfirst($type);
+    
+print F <<END
+
+    struct ${capitalizedType}TableEntry {
+        void* targetAddress;
+        StringImpl& name;
+    };
+
+    static const ${capitalizedType}TableEntry ${type}Table[] = {
+END
+;
     for my $name (sort keys %$namesRef) {
-        # To generate less code in init(), the common case of nullAtom for the namespace, we call createQualifiedName() without passing $namespaceURI.
-        if ($namespaceURI eq "nullAtom") {
-            print F "    createQualifiedName((void*)&$name","${shortCamelType}, ${name}Impl);\n";
-        } else {
-            print F "    createQualifiedName((void*)&$name","${shortCamelType}, ${name}Impl, $namespaceURI);\n";
-        }
+        print F "        { (void*)&$name$shortCamelType, *reinterpret_cast<StringImpl*>(&${name}Data) },\n";
+    }
+
+print F <<END
+    };
+
+    for (unsigned i = 0; i < WTF_ARRAY_LENGTH(${type}Table); ++i)
+END
+;
+    if ($namespaceURI eq "nullAtom") {
+        print F "        createQualifiedName(${type}Table[i].targetAddress, &${type}Table[i].name);\n";
+    } else {
+        print F "        createQualifiedName(${type}Table[i].targetAddress, &${type}Table[i].name, $namespaceURI);\n";
     }
 }
 
@@ -823,143 +918,115 @@ sub printFactoryCppFile
     my $F;
     open F, ">$cppPath";
 
-printLicenseHeader($F);
+    my $formElementArgumentForDeclaration = "";
+    my $formElementArgumentForDefinition = "";
+    $formElementArgumentForDeclaration = ", HTMLFormElement*" if $parameters{namespace} eq "HTML";
+    $formElementArgumentForDefinition = ", HTMLFormElement* formElement" if $parameters{namespace} eq "HTML";
 
-print F <<END
+    printLicenseHeader($F);
+
+    print F <<END
 #include "config.h"
 END
-;
+    ;
 
-print F "\n#if $parameters{guardFactoryWith}\n\n" if $parameters{guardFactoryWith};
+    print F "\n#if $parameters{guardFactoryWith}\n\n" if $parameters{guardFactoryWith};
 
-print F <<END
+    print F <<END
 #include "$parameters{namespace}ElementFactory.h"
+
 #include "$parameters{namespace}Names.h"
+
 END
-;
+    ;
 
-printElementIncludes($F);
+    printElementIncludes($F);
+    printConditionalElementIncludes($F, 0);
 
-print F "\n#include <wtf/HashMap.h>\n";
+    print F <<END
 
-printConditionalElementIncludes($F, 0);
-
-print F <<END
-
-#include "ContextFeatures.h"
-#include "RuntimeEnabledFeatures.h"
-
-#if ENABLE(CUSTOM_ELEMENTS)
-#include "CustomElementConstructor.h"
-#include "CustomElementRegistry.h"
-#endif
-
-#if ENABLE(DASHBOARD_SUPPORT) || ENABLE(VIDEO)
 #include "Document.h"
+#include "RuntimeEnabledFeatures.h"
 #include "Settings.h"
-#endif
+#include <wtf/HashMap.h>
+#include <wtf/NeverDestroyed.h>
 
 namespace WebCore {
 
 using namespace $parameters{namespace}Names;
 
-END
-;
-
-print F "typedef PassRefPtr<$parameters{namespace}Element> (*ConstructorFunction)(const QualifiedName&, Document*";
-print F ", HTMLFormElement*" if $parameters{namespace} eq "HTML";
-print F ", bool createdByParser);\n";
-print F <<END
-typedef HashMap<AtomicStringImpl*, ConstructorFunction> FunctionMap;
-
-static FunctionMap* gFunctionMap = 0;
+typedef PassRefPtr<$parameters{namespace}Element> (*$parameters{namespace}ConstructorFunction)(const QualifiedName&, Document&$formElementArgumentForDeclaration, bool createdByParser);
 
 END
-;
+    ;
 
-my %tagConstructorMap = buildConstructorMap();
+    my %tagConstructorMap = buildConstructorMap();
 
-printConstructors($F, \%tagConstructorMap);
+    printConstructors($F, \%tagConstructorMap);
 
-print F <<END
-static void addTag(const QualifiedName& tag, ConstructorFunction func)
+    print F <<END
+static NEVER_INLINE void populate$parameters{namespace}FactoryMap(HashMap<AtomicStringImpl*, $parameters{namespace}ConstructorFunction>& map)
 {
-    gFunctionMap->set(tag.localName().impl(), func);
+    struct TableEntry {
+        const QualifiedName& name;
+        $parameters{namespace}ConstructorFunction function;
+    };
+
+    static const TableEntry table[] = {
+END
+    ;
+
+    printFunctionTable($F, \%tagConstructorMap);
+
+    print F <<END
+    };
+
+    for (unsigned i = 0; i < WTF_ARRAY_LENGTH(table); ++i)
+        map.add(table[i].name.localName().impl(), table[i].function);
 }
 
-static void createFunctionMap()
+PassRefPtr<$parameters{namespace}Element> $parameters{namespace}ElementFactory::createElement(const QualifiedName& name, Document& document$formElementArgumentForDefinition, bool createdByParser)
 {
-    ASSERT(!gFunctionMap);
-
-    // Create the table.
-    gFunctionMap = new FunctionMap;
-    
-    // Populate it with constructor functions.
 END
-;
+    ;
 
-printFunctionInits($F, \%tagConstructorMap);
-
-print F "}\n";
-
-
-print F "\nPassRefPtr<$parameters{namespace}Element> $parameters{namespace}ElementFactory::create$parameters{namespace}Element(const QualifiedName& qName, Document* document";
-print F ", HTMLFormElement* formElement" if $parameters{namespace} eq "HTML";
-print F ", bool createdByParser)\n{\n";
-
-print F <<END
-    if (!document)
-        return 0;
-
-END
-;
-
-if ($parameters{namespace} ne "HTML" and $parameters{namespace} ne "SVG") {
-print F <<END
+    if ($parameters{namespace} ne "HTML" and $parameters{namespace} ne "SVG") {
+        print F <<END
 #if ENABLE(DASHBOARD_SUPPORT)
-    Settings* settings = document->settings();
+    Settings* settings = document.settings();
     if (settings && settings->usesDashboardBackwardCompatibilityMode())
         return 0;
 #endif
 END
-;
-
-}
-
-print F <<END
-#if ENABLE(CUSTOM_ELEMENTS)
-    if (document->registry()) {
-        if (RefPtr<CustomElementConstructor> constructor = document->registry()->find(nullQName(), qName)) {
-            RefPtr<Element> element = constructor->createElement();
-            ASSERT(element->is$parameters{namespace}Element());
-            return static_pointer_cast<$parameters{namespace}Element>(element.release());
-        }
+        ;
     }
-#endif
 
-    if (!gFunctionMap)
-        createFunctionMap();
-    if (ConstructorFunction function = gFunctionMap->get(qName.localName().impl())) {
+    print F <<END
+    static NeverDestroyed<HashMap<AtomicStringImpl*, $parameters{namespace}ConstructorFunction>> functions;
+    if (functions.get().isEmpty())
+        populate$parameters{namespace}FactoryMap(functions);
+    if ($parameters{namespace}ConstructorFunction function = functions.get().get(name.localName().impl())) {
 END
-;
+    ;
 
-if ($parameters{namespace} eq "HTML") {
-    print F "        if (PassRefPtr<$parameters{namespace}Element> element = function(qName, document, formElement, createdByParser))\n";
-    print F "            return element;\n";
-} else {
-    print F "        if (PassRefPtr<$parameters{namespace}Element> element = function(qName, document, createdByParser))\n";
-    print F "            return element;\n";
-}
-print F <<END
+    if ($parameters{namespace} eq "HTML") {
+        print F "        if (RefPtr<$parameters{namespace}Element> element = function(name, document, formElement, createdByParser))\n";
+        print F "            return element.release();\n";
+    } else {
+        print F "        if (RefPtr<$parameters{namespace}Element> element = function(name, document, createdByParser))\n";
+        print F "            return element.release();\n";
     }
 
-    return $parameters{fallbackInterfaceName}::create(qName, document);
+    print F "   }\n";
+    print F "   return $parameters{fallbackInterfaceName}::create(name, document);\n";
+
+    print F <<END
 }
 
 } // namespace WebCore
 
 END
-;
+    ;
 
     print F "#endif\n" if $parameters{guardFactoryWith};
 
@@ -979,33 +1046,23 @@ sub printFactoryHeaderFile
 #define $parameters{namespace}ElementFactory_h
 
 #include <wtf/Forward.h>
-#include <wtf/PassRefPtr.h>
 
 namespace WebCore {
-    class Element;
+
     class Document;
+    class HTMLFormElement;
     class QualifiedName;
-}
-
-namespace WebCore {
 
     class $parameters{namespace}Element;
-END
-;
 
-print F "     class HTMLFormElement;\n" if $parameters{namespace} eq "HTML";
-
-print F<<END
-    // The idea behind this class is that there will eventually be a mapping from namespace URIs to ElementFactories that can dispense
-    // elements. In a compound document world, the generic createElement function (will end up being virtual) will be called.
     class $parameters{namespace}ElementFactory {
     public:
-        PassRefPtr<Element> createElement(const WebCore::QualifiedName&, WebCore::Document*, bool createdByParser = true);
 END
 ;
-print F "        static PassRefPtr<$parameters{namespace}Element> create$parameters{namespace}Element(const WebCore::QualifiedName&, WebCore::Document*";
-print F ", HTMLFormElement* = 0" if $parameters{namespace} eq "HTML";
-print F ", bool createdByParser = true);\n";
+
+print F "        static PassRefPtr<$parameters{namespace}Element> createElement(const QualifiedName&, Document&";
+print F ", HTMLFormElement* = nullptr" if $parameters{namespace} eq "HTML";
+print F ", bool createdByParser = false);\n";
 
 printf F<<END
     };
@@ -1046,40 +1103,23 @@ sub printWrapperFunctions
             print F "#if ${conditionalString}\n\n";
         }
 
-        # Hack for the media tags
-        # FIXME: This should have been done via a CustomWrapper attribute and a separate *Custom file.
         if ($enabledTags{$tagName}{wrapperOnlyIfMediaIsAvailable}) {
             print F <<END
 static JSDOMWrapper* create${JSInterfaceName}Wrapper(ExecState* exec, JSDOMGlobalObject* globalObject, PassRefPtr<$parameters{namespace}Element> element)
 {
-    Settings* settings = element->document()->settings();
-    if (!MediaPlayer::isAvailable() || (settings && !settings->mediaEnabled()))
+    if (element->isHTMLUnknownElement())
         return CREATE_DOM_WRAPPER(exec, globalObject, $parameters{namespace}Element, element.get());
     return CREATE_DOM_WRAPPER(exec, globalObject, ${JSInterfaceName}, element.get());
 }
 
 END
-    ;
-        } elsif ($enabledTags{$tagName}{contextConditional}) {
-            my $contextConditional = $enabledTags{$tagName}{contextConditional};
-            print F <<END
-static JSDOMWrapper* create${JSInterfaceName}Wrapper(ExecState* exec, JSDOMGlobalObject* globalObject, PassRefPtr<$parameters{namespace}Element> element)
-{
-    if (!ContextFeatures::${contextConditional}Enabled(element->document())) {
-        ASSERT(!element || element->is$parameters{fallbackInterfaceName}());
-        return CREATE_DOM_WRAPPER(exec, globalObject, $parameters{fallbackJSInterfaceName}, element.get());
-    }
-
-    return CREATE_DOM_WRAPPER(exec, globalObject, ${JSInterfaceName}, element.get());
-}
-END
-    ;
+            ;
         } elsif ($enabledTags{$tagName}{runtimeConditional}) {
             my $runtimeConditional = $enabledTags{$tagName}{runtimeConditional};
             print F <<END
 static JSDOMWrapper* create${JSInterfaceName}Wrapper(ExecState* exec, JSDOMGlobalObject* globalObject, PassRefPtr<$parameters{namespace}Element> element)
 {
-    if (!RuntimeEnabledFeatures::${runtimeConditional}Enabled()) {
+    if (!RuntimeEnabledFeatures::sharedFeatures().${runtimeConditional}Enabled()) {
         ASSERT(!element || element->is$parameters{fallbackInterfaceName}());
         return CREATE_DOM_WRAPPER(exec, globalObject, $parameters{fallbackJSInterfaceName}, element.get());
     }
@@ -1115,47 +1155,34 @@ sub printWrapperFactoryCppFile
     printLicenseHeader($F);
 
     print F "#include \"config.h\"\n";
-    print F "#include \"JS$parameters{namespace}ElementWrapperFactory.h\"\n";
+    print F "#include \"JS$parameters{namespace}ElementWrapperFactory.h\"\n\n";
 
     print F "\n#if $parameters{guardFactoryWith}\n\n" if $parameters{guardFactoryWith};
 
     printJSElementIncludes($F);
-
-    print F "\n#include \"$parameters{namespace}Names.h\"\n\n";
-
     printElementIncludes($F);
 
-    print F "\n#include <wtf/StdLibExtras.h>\n";
+    print F "\n#include \"$parameters{namespace}Names.h\"\n";
+    print F <<END
+
+#include "Document.h"
+#include "RuntimeEnabledFeatures.h"
+#include "Settings.h"
+#include <wtf/NeverDestroyed.h>
+#include <wtf/StdLibExtras.h>
+END
+;
 
     printConditionalElementIncludes($F, 1);
 
     print F <<END
 
-#include "ContextFeatures.h"
-#include "RuntimeEnabledFeatures.h"
-
-#if ENABLE(VIDEO)
-#include "Document.h"
-#include "Settings.h"
-#endif
-
-END
-;
-
-    print F <<END
 using namespace JSC;
-END
-;
-
-    print F <<END
 
 namespace WebCore {
 
 using namespace $parameters{namespace}Names;
 
-END
-;
-print F <<END
 typedef JSDOMWrapper* (*Create$parameters{namespace}ElementWrapperFunction)(ExecState*, JSDOMGlobalObject*, PassRefPtr<$parameters{namespace}Element>);
 
 END
@@ -1164,11 +1191,15 @@ END
     printWrapperFunctions($F);
 
 print F <<END
-JSDOMWrapper* createJS$parameters{namespace}Wrapper(ExecState* exec, JSDOMGlobalObject* globalObject, PassRefPtr<$parameters{namespace}Element> element)
+
+static NEVER_INLINE void populate$parameters{namespace}WrapperMap(HashMap<AtomicStringImpl*, Create$parameters{namespace}ElementWrapperFunction>& map)
 {
-    typedef HashMap<WTF::AtomicStringImpl*, Create$parameters{namespace}ElementWrapperFunction> FunctionMap;
-    DEFINE_STATIC_LOCAL(FunctionMap, map, ());
-    if (map.isEmpty()) {
+    struct TableEntry {
+        const QualifiedName& name;
+        Create$parameters{namespace}ElementWrapperFunction function;
+    };
+
+    static const TableEntry table[] = {
 END
 ;
 
@@ -1183,7 +1214,10 @@ END
         }
 
         my $ucTag = $enabledTags{$tag}{JSInterfaceName};
-        print F "       map.set(${tag}Tag.localName().impl(), create${ucTag}Wrapper);\n";
+
+        # FIXME Remove unnecessary '&' from the following (print) line once we switch to a non-broken Visual Studio compiler.
+        # https://bugs.webkit.org/show_bug.cgi?id=121235:
+        print F "        { ${tag}Tag, &create${ucTag}Wrapper },\n";
 
         if ($conditional) {
             print F "#endif\n";
@@ -1191,27 +1225,27 @@ END
     }
 
     print F <<END
-    }
-END
-;
+    };
 
-    print F <<END
-    Create$parameters{namespace}ElementWrapperFunction createWrapperFunction = map.get(element->localName().impl());
-    if (createWrapperFunction)
-END
-;
-    print F <<END
-        return createWrapperFunction(exec, globalObject, element);
+    for (unsigned i = 0; i < WTF_ARRAY_LENGTH(table); ++i)
+        map.add(table[i].name.localName().impl(), table[i].function);
+}
+
+JSDOMWrapper* createJS$parameters{namespace}Wrapper(ExecState* exec, JSDOMGlobalObject* globalObject, PassRefPtr<$parameters{namespace}Element> element)
+{
+    static NeverDestroyed<HashMap<AtomicStringImpl*, Create$parameters{namespace}ElementWrapperFunction>> functions;
+    if (functions.get().isEmpty())
+        populate$parameters{namespace}WrapperMap(functions);
+    if (auto function = functions.get().get(element->localName().impl()))
+        return function(exec, globalObject, element);
     return CREATE_DOM_WRAPPER(exec, globalObject, $parameters{fallbackJSInterfaceName}, element.get());
-END
-;
-    print F <<END
+}
+
 }
 END
 ;
 
-    print F "}\n\n";
-    print F "#endif\n" if $parameters{guardFactoryWith};
+    print F "\n#endif\n" if $parameters{guardFactoryWith};
 
     close F;
 }
