@@ -44,21 +44,21 @@ namespace WebCore {
 using namespace HTMLNames;
 
 // FIXME: Share more code with MediaDocumentParser.
-class PluginDocumentParser : public RawDataDocumentParser {
+class PluginDocumentParser final : public RawDataDocumentParser {
 public:
-    static PassRefPtr<PluginDocumentParser> create(PluginDocument* document)
+    static PassRefPtr<PluginDocumentParser> create(PluginDocument& document)
     {
         return adoptRef(new PluginDocumentParser(document));
     }
 
 private:
-    PluginDocumentParser(Document* document)
+    PluginDocumentParser(Document& document)
         : RawDataDocumentParser(document)
         , m_embedElement(0)
     {
     }
 
-    virtual void appendBytes(DocumentWriter*, const char*, size_t);
+    virtual void appendBytes(DocumentWriter&, const char*, size_t) override;
 
     void createDocumentStructure();
 
@@ -69,21 +69,30 @@ void PluginDocumentParser::createDocumentStructure()
 {
     RefPtr<Element> rootElement = document()->createElement(htmlTag, false);
     document()->appendChild(rootElement, IGNORE_EXCEPTION);
-    static_cast<HTMLHtmlElement*>(rootElement.get())->insertedByParser();
+    toHTMLHtmlElement(rootElement.get())->insertedByParser();
 
     if (document()->frame())
-        document()->frame()->loader().dispatchDocumentElementAvailable();
+        document()->frame()->injectUserScripts(InjectAtDocumentStart);
+
+#if PLATFORM(IOS)
+    // Should not be able to zoom into standalone plug-in documents.
+    document()->processViewport(ASCIILiteral("user-scalable=no"), ViewportArguments::PluginDocument);
+#endif
 
     RefPtr<Element> body = document()->createElement(bodyTag, false);
-    body->setAttribute(marginwidthAttr, "0");
-    body->setAttribute(marginheightAttr, "0");
-    body->setAttribute(styleAttr, "background-color: rgb(38,38,38)");
+    body->setAttribute(marginwidthAttr, AtomicString("0", AtomicString::ConstructFromLiteral));
+    body->setAttribute(marginheightAttr, AtomicString("0", AtomicString::ConstructFromLiteral));
+#if PLATFORM(IOS)
+    body->setAttribute(styleAttr, AtomicString("background-color: rgb(217,224,233)", AtomicString::ConstructFromLiteral));
+#else
+    body->setAttribute(styleAttr, AtomicString("background-color: rgb(38,38,38)", AtomicString::ConstructFromLiteral));
+#endif
 
     rootElement->appendChild(body, IGNORE_EXCEPTION);
         
     RefPtr<Element> embedElement = document()->createElement(embedTag, false);
         
-    m_embedElement = static_cast<HTMLEmbedElement*>(embedElement.get());
+    m_embedElement = toHTMLEmbedElement(embedElement.get());
     m_embedElement->setAttribute(widthAttr, "100%");
     m_embedElement->setAttribute(heightAttr, "100%");
     
@@ -93,14 +102,14 @@ void PluginDocumentParser::createDocumentStructure()
     DocumentLoader* loader = document()->loader();
     ASSERT(loader);
     if (loader)
-        m_embedElement->setAttribute(typeAttr, loader->writer()->mimeType());
+        m_embedElement->setAttribute(typeAttr, loader->writer().mimeType());
 
     toPluginDocument(document())->setPluginElement(m_embedElement);
 
     body->appendChild(embedElement, IGNORE_EXCEPTION);
 }
 
-void PluginDocumentParser::appendBytes(DocumentWriter*, const char*, size_t)
+void PluginDocumentParser::appendBytes(DocumentWriter&, const char*, size_t)
 {
     if (m_embedElement)
         return;
@@ -120,7 +129,7 @@ void PluginDocumentParser::appendBytes(DocumentWriter*, const char*, size_t)
     // can synchronously redirect data to the plugin.
     frame->view()->flushAnyPendingPostLayoutTasks();
 
-    if (RenderPart* renderer = m_embedElement->renderPart()) {
+    if (RenderWidget* renderer = m_embedElement->renderWidget()) {
         if (Widget* widget = renderer->widget()) {
             frame->loader().client().redirectDataToPlugin(widget);
             // In a plugin document, the main resource is the plugin. If we have a null widget, that means
@@ -131,7 +140,7 @@ void PluginDocumentParser::appendBytes(DocumentWriter*, const char*, size_t)
     }
 }
 
-PluginDocument::PluginDocument(Frame* frame, const KURL& url)
+PluginDocument::PluginDocument(Frame* frame, const URL& url)
     : HTMLDocument(frame, url, PluginDocumentClass)
     , m_shouldLoadPluginManually(true)
 {
@@ -141,7 +150,7 @@ PluginDocument::PluginDocument(Frame* frame, const KURL& url)
 
 PassRefPtr<DocumentParser> PluginDocument::createParser()
 {
-    return PluginDocumentParser::create(this);
+    return PluginDocumentParser::create(*this);
 }
 
 Widget* PluginDocument::pluginWidget()
@@ -158,12 +167,11 @@ void PluginDocument::setPluginElement(PassRefPtr<HTMLPlugInElement> element)
     m_pluginElement = element;
 }
 
-void PluginDocument::detach()
+void PluginDocument::detachFromPluginElement()
 {
     // Release the plugin Element so that we don't have a circular reference.
     m_pluginElement = 0;
     frame()->loader().client().redirectDataToPlugin(0);
-    Document::detach();
 }
 
 void PluginDocument::cancelManualPluginLoad()
