@@ -56,7 +56,7 @@
 
 namespace WebCore {
 
-ApplicationCacheGroup::ApplicationCacheGroup(const KURL& manifestURL, bool isCopy)
+ApplicationCacheGroup::ApplicationCacheGroup(const URL& manifestURL, bool isCopy)
     : m_manifestURL(manifestURL)
     , m_origin(SecurityOrigin::create(manifestURL))
     , m_updateStatus(Idle)
@@ -101,7 +101,7 @@ ApplicationCache* ApplicationCacheGroup::cacheForMainRequest(const ResourceReque
     if (!ApplicationCache::requestIsHTTPOrHTTPSGet(request))
         return 0;
 
-    KURL url(request.url());
+    URL url(request.url());
     if (url.hasFragmentIdentifier())
         url.removeFragmentIdentifier();
 
@@ -120,7 +120,7 @@ ApplicationCache* ApplicationCacheGroup::fallbackCacheForMainRequest(const Resou
     if (!ApplicationCache::requestIsHTTPOrHTTPSGet(request))
         return 0;
 
-    KURL url(request.url());
+    URL url(request.url());
     if (url.hasFragmentIdentifier())
         url.removeFragmentIdentifier();
 
@@ -134,14 +134,14 @@ ApplicationCache* ApplicationCacheGroup::fallbackCacheForMainRequest(const Resou
     return 0;
 }
 
-void ApplicationCacheGroup::selectCache(Frame* frame, const KURL& passedManifestURL)
+void ApplicationCacheGroup::selectCache(Frame* frame, const URL& passedManifestURL)
 {
     ASSERT(frame && frame->page());
     
     if (!frame->settings().offlineWebApplicationCacheEnabled())
         return;
 
-    if (!frame->document()->securityOrigin()->canAccessApplicationCache(frame->tree().top()->document()->securityOrigin()))
+    if (!frame->document()->securityOrigin()->canAccessApplicationCache(frame->tree().top().document()->securityOrigin()))
         return;
     
     DocumentLoader* documentLoader = frame->loader().documentLoader();
@@ -152,7 +152,7 @@ void ApplicationCacheGroup::selectCache(Frame* frame, const KURL& passedManifest
         return;
     }
 
-    KURL manifestURL(passedManifestURL);
+    URL manifestURL(passedManifestURL);
     if (manifestURL.hasFragmentIdentifier())
         manifestURL.removeFragmentIdentifier();
 
@@ -167,7 +167,7 @@ void ApplicationCacheGroup::selectCache(Frame* frame, const KURL& passedManifest
             mainResourceCache->group()->update(frame, ApplicationCacheUpdateWithBrowsingContext);
         } else {
             // The main resource was loaded from cache, so the cache must have an entry for it. Mark it as foreign.
-            KURL resourceURL(documentLoader->responseURL());
+            URL resourceURL(documentLoader->responseURL());
             if (resourceURL.hasFragmentIdentifier())
                 resourceURL.removeFragmentIdentifier();
             ApplicationCacheResource* resource = mainResourceCache->resourceForURL(resourceURL);
@@ -217,7 +217,7 @@ void ApplicationCacheGroup::selectCacheWithoutManifestURL(Frame* frame)
     if (!frame->settings().offlineWebApplicationCacheEnabled())
         return;
 
-    if (!frame->document()->securityOrigin()->canAccessApplicationCache(frame->tree().top()->document()->securityOrigin()))
+    if (!frame->document()->securityOrigin()->canAccessApplicationCache(frame->tree().top().document()->securityOrigin()))
         return;
 
     DocumentLoader* documentLoader = frame->loader().documentLoader();
@@ -235,7 +235,7 @@ void ApplicationCacheGroup::finishedLoadingMainResource(DocumentLoader* loader)
 {
     ASSERT(m_pendingMasterResourceLoaders.contains(loader));
     ASSERT(m_completionType == None || m_pendingEntries.isEmpty());
-    KURL url = loader->url();
+    URL url = loader->url();
     if (url.hasFragmentIdentifier())
         url.removeFragmentIdentifier();
 
@@ -362,10 +362,7 @@ void ApplicationCacheGroup::stopLoading()
 
 void ApplicationCacheGroup::disassociateDocumentLoader(DocumentLoader* loader)
 {
-    HashSet<DocumentLoader*>::iterator it = m_associatedDocumentLoaders.find(loader);
-    if (it != m_associatedDocumentLoaders.end())
-        m_associatedDocumentLoaders.remove(it);
-
+    m_associatedDocumentLoaders.remove(loader);
     m_pendingMasterResourceLoaders.remove(loader);
 
     loader->applicationCacheHost()->setApplicationCache(0); // Will set candidate to 0, too.
@@ -390,12 +387,7 @@ void ApplicationCacheGroup::disassociateDocumentLoader(DocumentLoader* loader)
 
 void ApplicationCacheGroup::cacheDestroyed(ApplicationCache* cache)
 {
-    if (!m_caches.contains(cache))
-        return;
-    
-    m_caches.remove(cache);
-
-    if (m_caches.isEmpty()) {
+    if (m_caches.remove(cache) && m_caches.isEmpty()) {
         ASSERT(m_associatedDocumentLoaders.isEmpty());
         ASSERT(m_pendingMasterResourceLoaders.isEmpty());
         delete this;
@@ -483,7 +475,7 @@ void ApplicationCacheGroup::abort(Frame* frame)
     cacheUpdateFailed();
 }
 
-PassRefPtr<ResourceHandle> ApplicationCacheGroup::createResourceHandle(const KURL& url, ApplicationCacheResource* newestCachedResource)
+PassRefPtr<ResourceHandle> ApplicationCacheGroup::createResourceHandle(const URL& url, ApplicationCacheResource* newestCachedResource)
 {
     ResourceRequest request(url);
     m_frame->loader().applyUserAgent(request);
@@ -500,24 +492,6 @@ PassRefPtr<ResourceHandle> ApplicationCacheGroup::createResourceHandle(const KUR
         }
     }
 
-#if PLATFORM(BLACKBERRY)
-    ResourceRequest::TargetType target = ResourceRequest::TargetIsUnspecified;
-    if (newestCachedResource) {
-        const String& mimeType = newestCachedResource->response().mimeType();
-        if (!mimeType.isEmpty())
-            target = ResourceRequest::targetTypeFromMimeType(mimeType);
-    }
-    if (target == ResourceRequest::TargetIsUnspecified) {
-        String mimeType;
-        if (url.protocolIsData())
-            mimeType = mimeTypeFromDataURL(url);
-        if (!mimeType.isEmpty())
-            target = ResourceRequest::targetTypeFromMimeType(mimeType);
-    }
-
-    request.setTargetType(target);
-#endif
-
     RefPtr<ResourceHandle> handle = ResourceHandle::create(m_frame->loader().networkingContext(), request, this, false, true);
 #if ENABLE(INSPECTOR)
     // Because willSendRequest only gets called during redirects, we initialize
@@ -544,7 +518,7 @@ void ApplicationCacheGroup::didReceiveResponse(ResourceHandle* handle, const Res
     
     ASSERT(handle == m_currentHandle);
 
-    KURL url(handle->firstRequest().url());
+    URL url(handle->firstRequest().url());
     if (url.hasFragmentIdentifier())
         url.removeFragmentIdentifier();
     
@@ -603,7 +577,7 @@ void ApplicationCacheGroup::didReceiveResponse(ResourceHandle* handle, const Res
     m_currentResource = ApplicationCacheResource::create(url, response, type);
 }
 
-void ApplicationCacheGroup::didReceiveData(ResourceHandle* handle, const char* data, int length, int encodedDataLength)
+void ApplicationCacheGroup::didReceiveData(ResourceHandle* handle, const char* data, unsigned length, int encodedDataLength)
 {
     UNUSED_PARAM(encodedDataLength);
 
@@ -678,7 +652,7 @@ void ApplicationCacheGroup::didFail(ResourceHandle* handle, const ResourceError&
     ASSERT(handle == m_currentHandle);
 
     unsigned type = m_currentResource ? m_currentResource->type() : m_pendingEntries.get(handle->firstRequest().url());
-    KURL url(handle->firstRequest().url());
+    URL url(handle->firstRequest().url());
     if (url.hasFragmentIdentifier())
         url.removeFragmentIdentifier();
 
@@ -789,17 +763,15 @@ void ApplicationCacheGroup::didFinishLoadingManifest()
     ASSERT(m_pendingEntries.isEmpty());
 
     if (isUpgradeAttempt) {
-        ApplicationCache::ResourceMap::const_iterator end = m_newestCache->end();
-        for (ApplicationCache::ResourceMap::const_iterator it = m_newestCache->begin(); it != end; ++it) {
-            unsigned type = it->value->type();
+        for (const auto& urlAndResource : *m_newestCache) {
+            unsigned type = urlAndResource.value->type();
             if (type & ApplicationCacheResource::Master)
-                addEntry(it->key, type);
+                addEntry(urlAndResource.key, type);
         }
     }
     
-    HashSet<String>::const_iterator end = manifest.explicitURLs.end();
-    for (HashSet<String>::const_iterator it = manifest.explicitURLs.begin(); it != end; ++it)
-        addEntry(*it, ApplicationCacheResource::Explicit);
+    for (const auto& explicitURL : manifest.explicitURLs)
+        addEntry(explicitURL, ApplicationCacheResource::Explicit);
 
     size_t fallbackCount = manifest.fallbackURLs.size();
     for (size_t i = 0; i  < fallbackCount; ++i)
@@ -1032,7 +1004,7 @@ void ApplicationCacheGroup::startLoadingEntry()
 
     ASSERT(!m_currentHandle);
     
-    m_currentHandle = createResourceHandle(KURL(ParsedURLString, it->key), m_newestCache ? m_newestCache->resourceForURL(it->key) : 0);
+    m_currentHandle = createResourceHandle(URL(ParsedURLString, it->key), m_newestCache ? m_newestCache->resourceForURL(it->key) : 0);
 }
 
 void ApplicationCacheGroup::deliverDelayedMainResources()
@@ -1059,7 +1031,7 @@ void ApplicationCacheGroup::deliverDelayedMainResources()
 void ApplicationCacheGroup::addEntry(const String& url, unsigned type)
 {
     ASSERT(m_cacheBeingUpdated);
-    ASSERT(!KURL(ParsedURLString, url).hasFragmentIdentifier());
+    ASSERT(!URL(ParsedURLString, url).hasFragmentIdentifier());
     
     // Don't add the URL if we already have an master resource in the cache
     // (i.e., the main resource finished loading before the manifest).
@@ -1106,7 +1078,7 @@ public:
     }
 
 private:
-    virtual void fired()
+    virtual void fired() override
     {
         m_cacheGroup->didReachMaxAppCacheSize();
         delete this;
@@ -1132,7 +1104,7 @@ public:
         return adoptPtr(new CallCacheListenerTask(loader, eventID, progressTotal, progressDone));
     }
 
-    virtual void performTask(ScriptExecutionContext* context)
+    virtual void performTask(ScriptExecutionContext* context) override
     {
         
         ASSERT_UNUSED(context, context->isDocument());
@@ -1187,9 +1159,8 @@ void ApplicationCacheGroup::clearStorageID()
 {
     m_storageID = 0;
     
-    HashSet<ApplicationCache*>::const_iterator end = m_caches.end();
-    for (HashSet<ApplicationCache*>::const_iterator it = m_caches.begin(); it != end; ++it)
-        (*it)->clearStorageID();
+    for (const auto& cache : m_caches)
+        cache->clearStorageID();
 }
 
 }
