@@ -39,8 +39,6 @@
 #include <algorithm>
 #include <assert.h>
 #include <math.h>
-#include <wtf/OwnPtr.h>
-#include <wtf/PassOwnPtr.h>
 
 namespace WebCore {
 
@@ -48,21 +46,26 @@ using namespace VectorMath;
 
 const unsigned MaxBusChannels = 32;
 
+PassRefPtr<AudioBus> AudioBus::create(unsigned numberOfChannels, size_t length, bool allocate)
+{
+    ASSERT(numberOfChannels <= MaxBusChannels);
+    if (numberOfChannels > MaxBusChannels)
+        return 0;
+
+    return adoptRef(new AudioBus(numberOfChannels, length, allocate));
+}
+
 AudioBus::AudioBus(unsigned numberOfChannels, size_t length, bool allocate)
     : m_length(length)
     , m_busGain(1)
     , m_isFirstTime(true)
     , m_sampleRate(0)
 {
-    ASSERT(numberOfChannels <= MaxBusChannels);
-    if (numberOfChannels > MaxBusChannels)
-        return;
-
     m_channels.reserveInitialCapacity(numberOfChannels);
 
     for (unsigned i = 0; i < numberOfChannels; ++i) {
-        PassOwnPtr<AudioChannel> channel = allocate ? adoptPtr(new AudioChannel(length)) : adoptPtr(new AudioChannel(0, length));
-        m_channels.append(channel);
+        auto channel = allocate ? std::make_unique<AudioChannel>(length) : std::make_unique<AudioChannel>(nullptr, length);
+        m_channels.append(std::move(channel));
     }
 
     m_layout = LayoutCanonical; // for now this is the only layout we define
@@ -76,6 +79,16 @@ void AudioBus::setChannelMemory(unsigned channelIndex, float* storage, size_t le
     }
 }
 
+void AudioBus::resizeSmaller(size_t newLength)
+{
+    ASSERT(newLength <= m_length);
+    if (newLength <= m_length)
+        m_length = newLength;
+
+    for (unsigned i = 0; i < m_channels.size(); ++i)
+        m_channels[i]->resizeSmaller(newLength);
+}
+
 void AudioBus::zero()
 {
     for (unsigned i = 0; i < m_channels.size(); ++i)
@@ -154,7 +167,7 @@ bool AudioBus::topologyMatches(const AudioBus& bus) const
     return true;
 }
 
-PassOwnPtr<AudioBus> AudioBus::createBufferFromRange(const AudioBus* sourceBuffer, unsigned startFrame, unsigned endFrame)
+PassRefPtr<AudioBus> AudioBus::createBufferFromRange(const AudioBus* sourceBuffer, unsigned startFrame, unsigned endFrame)
 {
     size_t numberOfSourceFrames = sourceBuffer->length();
     unsigned numberOfChannels = sourceBuffer->numberOfChannels();
@@ -163,17 +176,17 @@ PassOwnPtr<AudioBus> AudioBus::createBufferFromRange(const AudioBus* sourceBuffe
     bool isRangeSafe = startFrame < endFrame && endFrame <= numberOfSourceFrames;
     ASSERT(isRangeSafe);
     if (!isRangeSafe)
-        return nullptr;
+        return 0;
 
     size_t rangeLength = endFrame - startFrame;
 
-    OwnPtr<AudioBus> audioBus = adoptPtr(new AudioBus(numberOfChannels, rangeLength));
+    RefPtr<AudioBus> audioBus = create(numberOfChannels, rangeLength);
     audioBus->setSampleRate(sourceBuffer->sampleRate());
 
     for (unsigned i = 0; i < numberOfChannels; ++i)
         audioBus->channel(i)->copyFromRange(sourceBuffer->channel(i), startFrame, endFrame);
 
-    return audioBus.release();
+    return audioBus;
 }
 
 float AudioBus::maxAbsValue() const
@@ -200,10 +213,7 @@ void AudioBus::scale(float scale)
         channel(i)->scale(scale);
 }
 
-// Just copies the samples from the source bus to this one.
-// This is just a simple copy if the number of channels match, otherwise a mixup or mixdown is done.
-// For now, we just support mixup from mono -> stereo and mixdown from stereo -> mono.
-void AudioBus::copyFrom(const AudioBus& sourceBus)
+void AudioBus::copyFrom(const AudioBus& sourceBus, ChannelInterpretation channelInterpretation)
 {
     if (&sourceBus == this)
         return;
@@ -214,7 +224,54 @@ void AudioBus::copyFrom(const AudioBus& sourceBus)
     if (numberOfDestinationChannels == numberOfSourceChannels) {
         for (unsigned i = 0; i < numberOfSourceChannels; ++i)
             channel(i)->copyFrom(sourceBus.channel(i));
-    } else if (numberOfDestinationChannels == 2 && numberOfSourceChannels == 1) {
+    } else {
+        switch (channelInterpretation) {
+        case Speakers:
+            speakersCopyFrom(sourceBus);
+            break;
+        case Discrete:
+            discreteCopyFrom(sourceBus);
+            break;
+        default:
+            ASSERT_NOT_REACHED();
+        }
+    }
+}
+
+void AudioBus::sumFrom(const AudioBus& sourceBus, ChannelInterpretation channelInterpretation)
+{
+    if (&sourceBus == this)
+        return;
+
+    unsigned numberOfSourceChannels = sourceBus.numberOfChannels();
+    unsigned numberOfDestinationChannels = numberOfChannels();
+
+    if (numberOfDestinationChannels == numberOfSourceChannels) {
+        for (unsigned i = 0; i < numberOfSourceChannels; ++i)
+            channel(i)->sumFrom(sourceBus.channel(i));
+    } else {
+        switch (channelInterpretation) {
+        case Speakers:
+            speakersSumFrom(sourceBus);
+            break;
+        case Discrete:
+            discreteSumFrom(sourceBus);
+            break;
+        default:
+            ASSERT_NOT_REACHED();
+        }
+    }
+}
+
+void AudioBus::speakersCopyFrom(const AudioBus& sourceBus)
+{
+    // FIXME: Implement down mixing 5.1 to stereo.
+    // https://bugs.webkit.org/show_bug.cgi?id=79192
+
+    unsigned numberOfSourceChannels = sourceBus.numberOfChannels();
+    unsigned numberOfDestinationChannels = numberOfChannels();
+
+    if (numberOfDestinationChannels == 2 && numberOfSourceChannels == 1) {
         // Handle mono -> stereo case (for now simply copy mono channel into both left and right)
         // FIXME: Really we should apply an equal-power scaling factor here, since we're effectively panning center...
         const AudioChannel* sourceChannel = sourceBus.channel(0);
@@ -240,40 +297,25 @@ void AudioBus::copyFrom(const AudioBus& sourceBus)
         channel(4)->zero();
         channel(5)->zero();
     } else if (numberOfDestinationChannels == 1 && numberOfSourceChannels == 6) {
-        // Handle 5.1 -> mono case, copy center channel into mono.
-        // FIXME: We should have a better algorithm for this down mixing.
-        channel(0)->copyFrom(sourceBus.channel(2));
-    } else if (numberOfDestinationChannels < numberOfSourceChannels) {
-        // Default down mixing handling, just match the source channels with the first available destination channels. 
-        // 5.1 -> stereo case covered here. 
-        // FIXME: We should have a better algorithm for down mixing 5.1 to stereo.
-        // https://bugs.webkit.org/show_bug.cgi?id=79192
-        for (unsigned i = 0; i < numberOfDestinationChannels; ++i)
-            channel(i)->copyFrom(sourceBus.channel(i));
-    } else if (numberOfDestinationChannels > numberOfSourceChannels) {
-        // Default up mixing handling, just match the destination channels with the first available source channels. 
-        // Stereo -> 5.1 case covered here. 
-        for (unsigned i = 0; i < numberOfSourceChannels; ++i) 
-            channel(i)->copyFrom(sourceBus.channel(i));
-        for (unsigned i = numberOfSourceChannels; i < numberOfDestinationChannels; ++i)
-            channel(i)->zero();
+        // Handle 5.1 -> mono case.
+        zero();
+        speakersSumFrom5_1_ToMono(sourceBus);
     } else {
-        // Case not handled
-        ASSERT_NOT_REACHED();
+        // Fallback for unknown combinations.
+        discreteCopyFrom(sourceBus);
     }
 }
 
-void AudioBus::sumFrom(const AudioBus &sourceBus)
+void AudioBus::speakersSumFrom(const AudioBus& sourceBus)
 {
+    // FIXME: Implement down mixing 5.1 to stereo.
+    // https://bugs.webkit.org/show_bug.cgi?id=79192
+    
     unsigned numberOfSourceChannels = sourceBus.numberOfChannels();
     unsigned numberOfDestinationChannels = numberOfChannels();
 
-    if (numberOfDestinationChannels == numberOfSourceChannels) {
-        for (unsigned i = 0; i < numberOfChannels(); ++i)
-            channel(i)->sumFrom(sourceBus.channel(i));
-    } else if (numberOfDestinationChannels == 2 && numberOfSourceChannels == 1) {
-        // Handle mono -> stereo case (for now simply sum mono channel into both left and right)
-        // FIXME: Really we should apply an equal-power scaling factor here, since we're effectively panning center...
+    if (numberOfDestinationChannels == 2 && numberOfSourceChannels == 1) {
+        // Handle mono -> stereo case (summing mono channel into both left and right).
         const AudioChannel* sourceChannel = sourceBus.channel(0);
         channel(0)->sumFrom(sourceChannel);
         channel(1)->sumFrom(sourceChannel);
@@ -292,24 +334,76 @@ void AudioBus::sumFrom(const AudioBus &sourceBus)
         // Handle mono -> 5.1 case, sum mono channel into center.
         channel(2)->sumFrom(sourceBus.channel(0));
     } else if (numberOfDestinationChannels == 1 && numberOfSourceChannels == 6) {
-        // Handle 5.1 -> mono case, sum center channel into mono.
-        // FIXME: We should have a better algorithm for this down mixing.
-        channel(0)->sumFrom(sourceBus.channel(2));
-    } else if (numberOfDestinationChannels < numberOfSourceChannels) {
-        // Default down mixing, just summing the first available destination channels. 
-        // 5.1 -> stereo case covered here.
-        // FIXME: We should have a better algorithm for down mixing 5.1 to stereo.
-        // https://bugs.webkit.org/show_bug.cgi?id=79192
+        // Handle 5.1 -> mono case.
+        speakersSumFrom5_1_ToMono(sourceBus);
+    } else {
+        // Fallback for unknown combinations.
+        discreteSumFrom(sourceBus);
+    }
+}
+
+void AudioBus::speakersSumFrom5_1_ToMono(const AudioBus& sourceBus)
+{
+    AudioBus& sourceBusSafe = const_cast<AudioBus&>(sourceBus);
+
+    const float* sourceL = sourceBusSafe.channelByType(ChannelLeft)->data();
+    const float* sourceR = sourceBusSafe.channelByType(ChannelRight)->data();
+    const float* sourceC = sourceBusSafe.channelByType(ChannelCenter)->data();
+    const float* sourceSL = sourceBusSafe.channelByType(ChannelSurroundLeft)->data();
+    const float* sourceSR = sourceBusSafe.channelByType(ChannelSurroundRight)->data();
+
+    float* destination = channelByType(ChannelLeft)->mutableData();
+
+    AudioFloatArray temp(length());
+    float* tempData = temp.data();
+
+    // Sum in L and R.
+    vadd(sourceL, 1, sourceR, 1, tempData, 1, length());
+    float scale = 0.7071;
+    vsmul(tempData, 1, &scale, tempData, 1, length());
+    vadd(tempData, 1, destination, 1, destination, 1, length());
+
+    // Sum in SL and SR.
+    vadd(sourceSL, 1, sourceSR, 1, tempData, 1, length());
+    scale = 0.5;
+    vsmul(tempData, 1, &scale, tempData, 1, length());
+    vadd(tempData, 1, destination, 1, destination, 1, length());
+
+    // Sum in center.
+    vadd(sourceC, 1, destination, 1, destination, 1, length());
+}
+
+void AudioBus::discreteCopyFrom(const AudioBus& sourceBus)
+{
+    unsigned numberOfSourceChannels = sourceBus.numberOfChannels();
+    unsigned numberOfDestinationChannels = numberOfChannels();
+    
+    if (numberOfDestinationChannels < numberOfSourceChannels) {
+        // Down-mix by copying channels and dropping the remaining.
+        for (unsigned i = 0; i < numberOfDestinationChannels; ++i)
+            channel(i)->copyFrom(sourceBus.channel(i));
+    } else if (numberOfDestinationChannels > numberOfSourceChannels) {
+        // Up-mix by copying as many channels as we have, then zeroing remaining channels.
+        for (unsigned i = 0; i < numberOfSourceChannels; ++i) 
+            channel(i)->copyFrom(sourceBus.channel(i));
+        for (unsigned i = numberOfSourceChannels; i < numberOfDestinationChannels; ++i)
+            channel(i)->zero();
+    }
+}
+
+void AudioBus::discreteSumFrom(const AudioBus& sourceBus)
+{
+    unsigned numberOfSourceChannels = sourceBus.numberOfChannels();
+    unsigned numberOfDestinationChannels = numberOfChannels();
+
+    if (numberOfDestinationChannels < numberOfSourceChannels) {
+        // Down-mix by summing channels and dropping the remaining.
         for (unsigned i = 0; i < numberOfDestinationChannels; ++i) 
             channel(i)->sumFrom(sourceBus.channel(i));
     } else if (numberOfDestinationChannels > numberOfSourceChannels) {
-        // Default up mixing, just summing the first available source channels. 
-        // stereo -> 5.1 case covered here.
+        // Up-mix by summing as many channels as we have.
         for (unsigned i = 0; i < numberOfSourceChannels; ++i) 
             channel(i)->sumFrom(sourceBus.channel(i));
-    } else {
-        // Case not handled
-        ASSERT_NOT_REACHED();
     }
 }
 
@@ -368,7 +462,7 @@ void AudioBus::copyWithGainFrom(const AudioBus &sourceBus, float* lastMixGain, f
 
     if (framesToDezipper) {
         if (!m_dezipperGainValues.get() || m_dezipperGainValues->size() < framesToDezipper)
-            m_dezipperGainValues = adoptPtr(new AudioFloatArray(framesToDezipper));
+            m_dezipperGainValues = std::make_unique<AudioFloatArray>(framesToDezipper);
 
         float* gainValues = m_dezipperGainValues->data();
         for (unsigned i = 0; i < framesToDezipper; ++i) {
@@ -427,12 +521,12 @@ void AudioBus::copyWithSampleAccurateGainValuesFrom(const AudioBus &sourceBus, f
     }
 }
 
-PassOwnPtr<AudioBus> AudioBus::createBySampleRateConverting(const AudioBus* sourceBus, bool mixToMono, double newSampleRate)
+PassRefPtr<AudioBus> AudioBus::createBySampleRateConverting(const AudioBus* sourceBus, bool mixToMono, double newSampleRate)
 {
     // sourceBus's sample-rate must be known.
     ASSERT(sourceBus && sourceBus->sampleRate());
     if (!sourceBus || !sourceBus->sampleRate())
-        return nullptr;
+        return 0;
 
     double sourceSampleRate = sourceBus->sampleRate();
     double destinationSampleRate = newSampleRate;
@@ -452,14 +546,14 @@ PassOwnPtr<AudioBus> AudioBus::createBySampleRateConverting(const AudioBus* sour
     }
 
     if (sourceBus->isSilent()) {
-        OwnPtr<AudioBus> silentBus = adoptPtr(new AudioBus(numberOfSourceChannels, sourceBus->length() / sampleRateRatio));
+        RefPtr<AudioBus> silentBus = create(numberOfSourceChannels, sourceBus->length() / sampleRateRatio);
         silentBus->setSampleRate(newSampleRate);
-        return silentBus.release();
+        return silentBus;
     }
 
     // First, mix to mono (if necessary) then sample-rate convert.
     const AudioBus* resamplerSourceBus;
-    OwnPtr<AudioBus> mixedMonoBus;
+    RefPtr<AudioBus> mixedMonoBus;
     if (mixToMono) {
         mixedMonoBus = AudioBus::createByMixingToMono(sourceBus);
         resamplerSourceBus = mixedMonoBus.get();
@@ -474,7 +568,7 @@ PassOwnPtr<AudioBus> AudioBus::createBySampleRateConverting(const AudioBus* sour
 
     // Create destination bus with same number of channels.
     unsigned numberOfDestinationChannels = resamplerSourceBus->numberOfChannels();
-    OwnPtr<AudioBus> destinationBus(adoptPtr(new AudioBus(numberOfDestinationChannels, destinationLength)));
+    RefPtr<AudioBus> destinationBus = create(numberOfDestinationChannels, destinationLength);
 
     // Sample-rate convert each channel.
     for (unsigned i = 0; i < numberOfDestinationChannels; ++i) {
@@ -487,13 +581,13 @@ PassOwnPtr<AudioBus> AudioBus::createBySampleRateConverting(const AudioBus* sour
 
     destinationBus->clearSilentFlag();
     destinationBus->setSampleRate(newSampleRate);    
-    return destinationBus.release();
+    return destinationBus;
 }
 
-PassOwnPtr<AudioBus> AudioBus::createByMixingToMono(const AudioBus* sourceBus)
+PassRefPtr<AudioBus> AudioBus::createByMixingToMono(const AudioBus* sourceBus)
 {
     if (sourceBus->isSilent())
-        return adoptPtr(new AudioBus(1, sourceBus->length()));
+        return create(1, sourceBus->length());
 
     switch (sourceBus->numberOfChannels()) {
     case 1:
@@ -502,7 +596,7 @@ PassOwnPtr<AudioBus> AudioBus::createByMixingToMono(const AudioBus* sourceBus)
     case 2:
         {
             unsigned n = sourceBus->length();
-            OwnPtr<AudioBus> destinationBus(adoptPtr(new AudioBus(1, n)));
+            RefPtr<AudioBus> destinationBus = create(1, n);
 
             const float* sourceL = sourceBus->channel(0)->data();
             const float* sourceR = sourceBus->channel(1)->data();
@@ -514,12 +608,12 @@ PassOwnPtr<AudioBus> AudioBus::createByMixingToMono(const AudioBus* sourceBus)
 
             destinationBus->clearSilentFlag();
             destinationBus->setSampleRate(sourceBus->sampleRate());    
-            return destinationBus.release();
+            return destinationBus;
         }
     }
 
     ASSERT_NOT_REACHED();
-    return nullptr;
+    return 0;
 }
 
 bool AudioBus::isSilent() const
