@@ -27,13 +27,14 @@
 #define IntRect_h
 
 #include "IntPoint.h"
+#include "LayoutUnit.h"
 #include <wtf/Vector.h>
 
 #if USE(CG)
 typedef struct CGRect CGRect;
 #endif
 
-#if PLATFORM(MAC)
+#if PLATFORM(MAC) && !PLATFORM(IOS)
 #ifdef NSGEOMETRY_TYPES_SAME_AS_CGGEOMETRY_TYPES
 typedef struct CGRect NSRect;
 #else
@@ -41,24 +42,20 @@ typedef struct _NSRect NSRect;
 #endif
 #endif
 
+#if PLATFORM(IOS)
+#ifndef NSRect
+#define NSRect CGRect
+#endif
+#endif
+
 #if PLATFORM(WIN)
 typedef struct tagRECT RECT;
-#elif PLATFORM(QT)
-QT_BEGIN_NAMESPACE
-class QRect;
-QT_END_NAMESPACE
 #elif PLATFORM(GTK)
 #ifdef GTK_API_VERSION_2
 typedef struct _GdkRectangle GdkRectangle;
 #endif
 #elif PLATFORM(EFL)
 typedef struct _Eina_Rectangle Eina_Rectangle;
-#elif PLATFORM(BLACKBERRY)
-namespace BlackBerry {
-namespace Platform {
-class IntRect;
-}
-}
 #endif
 
 #if USE(CAIRO)
@@ -177,9 +174,6 @@ public:
 #if PLATFORM(WIN)
     IntRect(const RECT&);
     operator RECT() const;
-#elif PLATFORM(QT)
-    IntRect(const QRect&);
-    operator QRect() const;
 #elif PLATFORM(GTK)
 #ifdef GTK_API_VERSION_2
     IntRect(const GdkRectangle&);
@@ -199,17 +193,17 @@ public:
     operator CGRect() const;
 #endif
 
+#if !PLATFORM(IOS)
 #if (PLATFORM(MAC) && !defined(NSGEOMETRY_TYPES_SAME_AS_CGGEOMETRY_TYPES))
     operator NSRect() const;
 #endif
-
-#if PLATFORM(BLACKBERRY)
-    IntRect(const BlackBerry::Platform::IntRect&);
-    operator BlackBerry::Platform::IntRect() const;
-#endif
+#endif // !PLATFORM(IOS)
 
     void dump(PrintStream& out) const;
 
+    static IntRect infiniteRect();
+    bool isInfinite() const;
+
 private:
     IntPoint m_location;
     IntSize m_size;
@@ -241,13 +235,26 @@ inline bool operator!=(const IntRect& a, const IntRect& b)
     return a.location() != b.location() || a.size() != b.size();
 }
 
+inline IntRect IntRect::infiniteRect()
+{
+    static IntRect infiniteRect(-LayoutUnit::max() / 2, -LayoutUnit::max() / 2, LayoutUnit::max(), LayoutUnit::max());
+    return infiniteRect;
+}
+
+inline bool IntRect::isInfinite() const
+{
+    return *this == infiniteRect();
+}
+
 #if USE(CG)
 IntRect enclosingIntRect(const CGRect&);
 #endif
 
+#if !PLATFORM(IOS)
 #if (PLATFORM(MAC) && !defined(NSGEOMETRY_TYPES_SAME_AS_CGGEOMETRY_TYPES))
 IntRect enclosingIntRect(const NSRect&);
 #endif
+#endif // !PLATFORM(IOS)
 
 } // namespace WebCore
 
