@@ -28,7 +28,9 @@
 #include "Filter.h"
 #include "ImageBuffer.h"
 #include "TextStream.h"
-#include <wtf/Uint8ClampedArray.h>
+#include <runtime/Operations.h>
+#include <runtime/TypedArrayInlines.h>
+#include <runtime/Uint8ClampedArray.h>
 
 #if HAVE(ARM_NEON_INTRINSICS)
 #include <arm_neon.h>
@@ -44,7 +46,7 @@ FilterEffect::FilterEffect(Filter* filter)
     , m_hasWidth(false)
     , m_hasHeight(false)
     , m_clipsToBounds(true)
-    , m_colorSpace(ColorSpaceLinearRGB)
+    , m_operatingColorSpace(ColorSpaceLinearRGB)
     , m_resultColorSpace(ColorSpaceDeviceRGB)
 {
     ASSERT(m_filter);
@@ -93,10 +95,28 @@ IntRect FilterEffect::drawingRegionOfInputImage(const IntRect& srcRect) const
 
 FilterEffect* FilterEffect::inputEffect(unsigned number) const
 {
-    ASSERT(number < m_inputEffects.size());
+    ASSERT_WITH_SECURITY_IMPLICATION(number < m_inputEffects.size());
     return m_inputEffects.at(number).get();
 }
 
+#if ENABLE(OPENCL)
+void FilterEffect::applyAll()
+{
+    if (hasResult())
+        return;
+    FilterContextOpenCL* context = FilterContextOpenCL::context();
+    if (context) {
+        apply();
+        if (!context->inError())
+            return;
+        clearResultsRecursive();
+        context->destroyContext();
+    }
+    // Software code path.
+    apply();
+}
+#endif
+
 void FilterEffect::apply()
 {
     if (hasResult())
@@ -109,11 +129,11 @@ void FilterEffect::apply()
             return;
 
         // Convert input results to the current effect's color space.
-        in->transformResultColorSpace(m_colorSpace);
+        transformResultColorSpace(in, i);
     }
 
     determineAbsolutePaintRect();
-    m_resultColorSpace = m_colorSpace;
+    setResultColorSpace(m_operatingColorSpace);
 
     if (!isFilterSizeValid(m_absolutePaintRect))
         return;
@@ -127,10 +147,6 @@ void FilterEffect::apply()
 #if ENABLE(OPENCL)
     if (platformApplyOpenCL())
         return;
-#endif
-#if USE(SKIA)
-    if (platformApplySkia())
-        return;
 #endif
     platformApplySoftware();
 }
@@ -212,7 +228,7 @@ void FilterEffect::forceValidPreMultipliedPixels()
 void FilterEffect::clearResult()
 {
     if (m_imageBufferResult)
-        m_imageBufferResult.clear();
+        m_imageBufferResult.reset();
     if (m_unmultipliedImageResult)
         m_unmultipliedImageResult.clear();
     if (m_premultipliedImageResult)
@@ -223,6 +239,18 @@ void FilterEffect::clearResult()
 #endif
 }
 
+void FilterEffect::clearResultsRecursive()
+{
+    // Clear all results, regardless that the current effect has
+    // a result. Can be used if an effect is in an erroneous state.
+    if (hasResult())
+        clearResult();
+
+    unsigned size = m_inputEffects.size();
+    for (unsigned i = 0; i < size; ++i)
+        m_inputEffects.at(i).get()->clearResultsRecursive();
+}
+
 ImageBuffer* FilterEffect::asImageBuffer()
 {
     if (!hasResult())
@@ -248,13 +276,19 @@ ImageBuffer* FilterEffect::openCLImageToImageBuffer()
     FilterContextOpenCL* context = FilterContextOpenCL::context();
     ASSERT(context);
 
+    if (context->inError())
+        return 0;
+
     size_t origin[3] = { 0, 0, 0 };
     size_t region[3] = { m_absolutePaintRect.width(), m_absolutePaintRect.height(), 1 };
 
     RefPtr<Uint8ClampedArray> destinationPixelArray = Uint8ClampedArray::create(m_absolutePaintRect.width() * m_absolutePaintRect.height() * 4);
 
-    clFinish(context->commandQueue());
-    clEnqueueReadImage(context->commandQueue(), m_openCLImageResult, CL_TRUE, origin, region, 0, 0, destinationPixelArray->data(), 0, 0, 0);
+    if (context->isFailed(clFinish(context->commandQueue())))
+        return 0;
+
+    if (context->isFailed(clEnqueueReadImage(context->commandQueue(), m_openCLImageResult, CL_TRUE, origin, region, 0, 0, destinationPixelArray->data(), 0, 0, 0)))
+        return 0;
 
     m_imageBufferResult = ImageBuffer::create(m_absolutePaintRect.size());
     IntRect destinationRect(IntPoint(), m_absolutePaintRect.size());
@@ -392,7 +426,7 @@ ImageBuffer* FilterEffect::createImageBufferResult()
     ASSERT(!hasResult());
     if (m_absolutePaintRect.isEmpty())
         return 0;
-    m_imageBufferResult = ImageBuffer::create(m_absolutePaintRect.size(), 1, m_colorSpace, m_filter->renderingMode());
+    m_imageBufferResult = ImageBuffer::create(m_absolutePaintRect.size(), 1, m_resultColorSpace, m_filter->renderingMode());
     if (!m_imageBufferResult)
         return 0;
     ASSERT(m_imageBufferResult->context());
@@ -426,15 +460,29 @@ Uint8ClampedArray* FilterEffect::createPremultipliedImageResult()
 #if ENABLE(OPENCL)
 OpenCLHandle FilterEffect::createOpenCLImageResult(uint8_t* source)
 {
+    FilterContextOpenCL* context = FilterContextOpenCL::context();
+    ASSERT(context);
+
+    if (context->inError())
+        return 0;
+
     ASSERT(!hasResult());
     cl_image_format clImageFormat;
     clImageFormat.image_channel_order = CL_RGBA;
     clImageFormat.image_channel_data_type = CL_UNORM_INT8;
 
-    FilterContextOpenCL* context = FilterContextOpenCL::context();
-    ASSERT(context);
+    int errorCode = 0;
+#ifdef CL_API_SUFFIX__VERSION_1_2
+    cl_image_desc imageDescriptor = { CL_MEM_OBJECT_IMAGE2D, m_absolutePaintRect.width(), m_absolutePaintRect.height(), 0, 0, 0, 0, 0, 0, 0};
+    m_openCLImageResult = clCreateImage(context->deviceContext(), CL_MEM_READ_WRITE | (source ? CL_MEM_COPY_HOST_PTR : 0),
+        &clImageFormat, &imageDescriptor, source, &errorCode);
+#else
     m_openCLImageResult = clCreateImage2D(context->deviceContext(), CL_MEM_READ_WRITE | (source ? CL_MEM_COPY_HOST_PTR : 0),
-        &clImageFormat, m_absolutePaintRect.width(), m_absolutePaintRect.height(), 0, source, 0);
+        &clImageFormat, m_absolutePaintRect.width(), m_absolutePaintRect.height(), 0, source, &errorCode);
+#endif
+    if (context->isFailed(errorCode))
+        return 0;
+
     return m_openCLImageResult;
 }
 #endif
@@ -450,17 +498,17 @@ void FilterEffect::transformResultColorSpace(ColorSpace dstColorSpace)
 
 #if ENABLE(OPENCL)
     if (openCLImage()) {
+        if (m_imageBufferResult)
+            m_imageBufferResult.clear();
         FilterContextOpenCL* context = FilterContextOpenCL::context();
         ASSERT(context);
         context->openCLTransformColorSpace(m_openCLImageResult, absolutePaintRect(), m_resultColorSpace, dstColorSpace);
-        if (m_imageBufferResult)
-            m_imageBufferResult.clear();
     } else {
 #endif
 
-    // FIXME: We can avoid this potentially unnecessary ImageBuffer conversion by adding
-    // color space transform support for the {pre,un}multiplied arrays.
-    asImageBuffer()->transformColorSpace(m_resultColorSpace, dstColorSpace);
+        // FIXME: We can avoid this potentially unnecessary ImageBuffer conversion by adding
+        // color space transform support for the {pre,un}multiplied arrays.
+        asImageBuffer()->transformColorSpace(m_resultColorSpace, dstColorSpace);
 
 #if ENABLE(OPENCL)
     }
