@@ -39,13 +39,14 @@
 #include "FileStream.h"
 #include "FileSystem.h"
 #include "HTTPParsers.h"
-#include "KURL.h"
-#include "NetworkingContext.h"
+#include "URL.h"
 #include "ResourceError.h"
 #include "ResourceHandleClient.h"
 #include "ResourceRequest.h"
 #include "ResourceResponse.h"
+#include "SharedBuffer.h"
 #include <wtf/MainThread.h>
+#include <wtf/Ref.h>
 
 namespace WebCore {
 
@@ -71,6 +72,7 @@ enum {
     securityError = 2,
     rangeError = 3,
     notReadableError = 4,
+    methodNotAllowed = 5
 };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -82,10 +84,10 @@ class BlobResourceSynchronousLoader : public ResourceHandleClient {
 public:
     BlobResourceSynchronousLoader(ResourceError&, ResourceResponse&, Vector<char>&);
 
-    virtual void didReceiveResponse(ResourceHandle*, const ResourceResponse&);
-    virtual void didReceiveData(ResourceHandle*, const char*, int, int /*encodedDataLength*/);
-    virtual void didFinishLoading(ResourceHandle*, double /*finishTime*/);
-    virtual void didFail(ResourceHandle*, const ResourceError&);
+    virtual void didReceiveResponse(ResourceHandle*, const ResourceResponse&) override;
+    virtual void didReceiveData(ResourceHandle*, const char*, unsigned, int /*encodedDataLength*/) override;
+    virtual void didFinishLoading(ResourceHandle*, double /*finishTime*/) override;
+    virtual void didFail(ResourceHandle*, const ResourceError&) override;
 
 private:
     ResourceError& m_error;
@@ -115,7 +117,7 @@ void BlobResourceSynchronousLoader::didReceiveResponse(ResourceHandle* handle, c
     static_cast<BlobResourceHandle*>(handle)->readSync(m_data.data(), static_cast<int>(m_data.size()));
 }
 
-void BlobResourceSynchronousLoader::didReceiveData(ResourceHandle*, const char*, int, int)
+void BlobResourceSynchronousLoader::didReceiveData(ResourceHandle*, const char*, unsigned, int)
 {
 }
 
@@ -133,16 +135,29 @@ void BlobResourceSynchronousLoader::didFail(ResourceHandle*, const ResourceError
 ///////////////////////////////////////////////////////////////////////////////
 // BlobResourceHandle
 
-// static
-void BlobResourceHandle::loadResourceSynchronously(PassRefPtr<BlobStorageData> blobData, const ResourceRequest& request, ResourceError& error, ResourceResponse& response, Vector<char>& data)
+PassRefPtr<BlobResourceHandle> BlobResourceHandle::createAsync(BlobStorageData* blobData, const ResourceRequest& request, ResourceHandleClient* client)
 {
+    // FIXME: Should probably call didFail() instead of blocking the load without explanation.
+    if (!equalIgnoringCase(request.httpMethod(), "GET"))
+        return 0;
+
+    return adoptRef(new BlobResourceHandle(blobData, request, client, true));
+}
+
+void BlobResourceHandle::loadResourceSynchronously(BlobStorageData* blobData, const ResourceRequest& request, ResourceError& error, ResourceResponse& response, Vector<char>& data)
+{
+    if (!equalIgnoringCase(request.httpMethod(), "GET")) {
+        error = ResourceError(webKitBlobResourceDomain, methodNotAllowed, response.url(), "Request method must be GET");
+        return;
+    }
+
     BlobResourceSynchronousLoader loader(error, response, data);
-    RefPtr<BlobResourceHandle> handle = BlobResourceHandle::create(blobData, request, &loader, false);
+    RefPtr<BlobResourceHandle> handle = adoptRef(new BlobResourceHandle(blobData, request, &loader, false));
     handle->start();
 }
 
 BlobResourceHandle::BlobResourceHandle(PassRefPtr<BlobStorageData> blobData, const ResourceRequest& request, ResourceHandleClient* client, bool async)
-    : ResourceHandle(request, client, false, false)
+    : ResourceHandle(0, request, client, false, false)
     , m_blobData(blobData)
     , m_async(async)
     , m_errorCode(0)
@@ -155,9 +170,9 @@ BlobResourceHandle::BlobResourceHandle(PassRefPtr<BlobStorageData> blobData, con
     , m_sizeItemCount(0)
     , m_readItemCount(0)
     , m_fileOpened(false)
-{    
+{
     if (m_async)
-        m_asyncStream = client->createAsyncFileStream(this);
+        m_asyncStream = AsyncFileStream::create(this);
     else
         m_stream = FileStream::create();
 }
@@ -231,7 +246,7 @@ void BlobResourceHandle::doStart()
     if (m_async)
         getSizeForNext();
     else {
-        RefPtr<BlobResourceHandle> protect(this); // getSizeForNext calls the client
+        Ref<BlobResourceHandle> protect(*this); // getSizeForNext calls the client
         for (size_t i = 0; i < m_blobData->items().size() && !m_aborted && !m_errorCode; ++i)
             getSizeForNext();
         notifyResponse();
@@ -246,7 +261,7 @@ void BlobResourceHandle::getSizeForNext()
 
         // Start reading if in asynchronous mode.
         if (m_async) {
-            RefPtr<BlobResourceHandle> protect(this);
+            Ref<BlobResourceHandle> protect(*this);
             notifyResponse();
             m_buffer.resize(bufferSize);
             readAsync();
@@ -331,7 +346,7 @@ void BlobResourceHandle::seek()
 int BlobResourceHandle::readSync(char* buf, int length)
 {
     ASSERT(!m_async);
-    RefPtr<BlobResourceHandle> protect(this);
+    Ref<BlobResourceHandle> protect(*this);
 
     int offset = 0;
     int remaining = length;
@@ -365,7 +380,9 @@ int BlobResourceHandle::readSync(char* buf, int length)
     else
         result = length - remaining;
 
-    notifyReceiveData(buf, result);
+    if (result > 0)
+        notifyReceiveData(buf, result);
+
     if (!result)
         notifyFinish();
 
@@ -451,7 +468,7 @@ void BlobResourceHandle::readAsync()
 void BlobResourceHandle::readDataAsync(const BlobDataItem& item)
 {
     ASSERT(m_async);
-    RefPtr<BlobResourceHandle> protect(this);
+    Ref<BlobResourceHandle> protect(*this);
 
     long long bytesToRead = item.length - m_currentItemReadSize;
     if (bytesToRead > m_totalRemainingSize)
@@ -503,7 +520,7 @@ void BlobResourceHandle::didRead(int bytesRead)
 void BlobResourceHandle::consumeData(const char* data, int bytesRead)
 {
     ASSERT(m_async);
-    RefPtr<BlobResourceHandle> protect(this);
+    Ref<BlobResourceHandle> protect(*this);
 
     m_totalRemainingSize -= bytesRead;
 
@@ -533,7 +550,7 @@ void BlobResourceHandle::consumeData(const char* data, int bytesRead)
 void BlobResourceHandle::failed(int errorCode)
 {
     ASSERT(m_async);
-    RefPtr<BlobResourceHandle> protect(this);
+    Ref<BlobResourceHandle> protect(*this);
 
     // Notify the client.
     notifyFail(errorCode);
@@ -551,7 +568,7 @@ void BlobResourceHandle::notifyResponse()
         return;
 
     if (m_errorCode) {
-        RefPtr<BlobResourceHandle> protect(this);
+        Ref<BlobResourceHandle> protect(*this);
         notifyResponseOnError();
         notifyFinish();
     } else
@@ -567,14 +584,21 @@ void BlobResourceHandle::notifyResponseOnSuccess()
     response.setHTTPStatusText(isRangeRequest ? httpPartialContentText : httpOKText);
     if (!m_blobData->contentDisposition().isEmpty())
         response.setHTTPHeaderField("Content-Disposition", m_blobData->contentDisposition());
-    client()->didReceiveResponse(this, response);
+
+    // BlobResourceHandle cannot be used with downloading, and doesn't even wait for continueDidReceiveResponse.
+    // It's currently client's responsibility to know that didReceiveResponseAsync cannot be used to convert a
+    // load into a download or blobs.
+    if (client()->usesAsyncCallbacks())
+        client()->didReceiveResponseAsync(this, response);
+    else
+        client()->didReceiveResponse(this, response);
 }
 
 void BlobResourceHandle::notifyResponseOnError()
 {
     ASSERT(m_errorCode);
 
-    ResourceResponse response(firstRequest().url(), String(), 0, String(), String());
+    ResourceResponse response(firstRequest().url(), "text/plain", 0, String(), String());
     switch (m_errorCode) {
     case rangeError:
         response.setHTTPStatusCode(httpRequestedRangeNotSatisfiable);
@@ -593,13 +617,19 @@ void BlobResourceHandle::notifyResponseOnError()
         response.setHTTPStatusText(httpInternalErrorText);
         break;
     }
-    client()->didReceiveResponse(this, response);
+
+    // Note that we don't wait for continueDidReceiveResponse when using didReceiveResponseAsync.
+    // This is not formally correct, but the client has to be a no-op anyway, because blobs can't be downloaded.
+    if (client()->usesAsyncCallbacks())
+        client()->didReceiveResponseAsync(this, response);
+    else
+        client()->didReceiveResponse(this, response);
 }
 
 void BlobResourceHandle::notifyReceiveData(const char* data, int bytesRead)
 {
     if (client())
-        client()->didReceiveData(this, data, bytesRead, bytesRead);
+        client()->didReceiveBuffer(this, SharedBuffer::create(data, bytesRead), bytesRead);
 }
 
 void BlobResourceHandle::notifyFail(int errorCode)
@@ -636,4 +666,3 @@ void BlobResourceHandle::notifyFinish()
 } // namespace WebCore
 
 #endif // ENABLE(BLOB)
-
