@@ -33,6 +33,7 @@
 #include "Timer.h"
 
 #if USE(CFNETWORK)
+#include "ResourceHandleCFURLConnectionDelegate.h"
 #include <CFNetwork/CFURLConnectionPriv.h>
 #endif
 
@@ -44,26 +45,16 @@
 #if USE(CURL)
 #include <curl/curl.h>
 #include "FormDataStreamCurl.h"
+#include "MultipartHandle.h"
 #endif
 
 #if USE(SOUP)
-#define LIBSOUP_USE_UNSTABLE_REQUEST_API
-#include <libsoup/soup-multipart-input-stream.h>
-#include <libsoup/soup-request.h>
+#include "GUniquePtrSoup.h"
 #include <libsoup/soup.h>
 #include <wtf/gobject/GRefPtr.h>
 class Frame;
 #endif
 
-#if PLATFORM(QT)
-QT_BEGIN_NAMESPACE
-class QWebNetworkJob;
-QT_END_NAMESPACE
-namespace WebCore {
-class QNetworkReplyHandler;
-}
-#endif
-
 #if PLATFORM(MAC)
 OBJC_CLASS NSURLAuthenticationChallenge;
 OBJC_CLASS NSURLConnection;
@@ -83,8 +74,9 @@ namespace WebCore {
     class ResourceHandleInternal {
         WTF_MAKE_NONCOPYABLE(ResourceHandleInternal); WTF_MAKE_FAST_ALLOCATED;
     public:
-        ResourceHandleInternal(ResourceHandle* loader, const ResourceRequest& request, ResourceHandleClient* c, bool defersLoading, bool shouldContentSniff)
-            : m_client(c)
+        ResourceHandleInternal(ResourceHandle* loader, NetworkingContext* context, const ResourceRequest& request, ResourceHandleClient* client, bool defersLoading, bool shouldContentSniff)
+            : m_context(context)
+            , m_client(client)
             , m_firstRequest(request)
             , m_lastHTTPMethod(request.httpMethod())
             , status(0)
@@ -92,6 +84,7 @@ namespace WebCore {
             , m_shouldContentSniff(shouldContentSniff)
 #if USE(CFNETWORK)
             , m_connection(0)
+            , m_currentRequest(request)
 #endif
 #if USE(WININET)
             , m_fileLoadTimer(loader, &ResourceHandle::fileLoadTimer)
@@ -108,18 +101,16 @@ namespace WebCore {
             , m_url(0)
             , m_customHeaders(0)
             , m_cancelled(false)
+            , m_authFailureCount(0)
             , m_formDataStream(loader)
+            , m_sslErrors(0)
 #endif
 #if USE(SOUP)
             , m_cancelled(false)
-            , m_buffer(0)
-            , m_bufferSize(0)
             , m_bodySize(0)
             , m_bodyDataSent(0)
             , m_redirectCount(0)
-#endif
-#if PLATFORM(QT)
-            , m_job(0)
+            , m_previousPosition(0)
 #endif
 #if PLATFORM(MAC)
             , m_startWhenScheduled(false)
@@ -127,9 +118,9 @@ namespace WebCore {
             , m_currentMacChallenge(nil)
 #endif
             , m_scheduledFailureType(ResourceHandle::NoFailure)
-            , m_failureTimer(loader, &ResourceHandle::fireFailure)
+            , m_failureTimer(loader, &ResourceHandle::failureTimerFired)
         {
-            const KURL& url = m_firstRequest.url();
+            const URL& url = m_firstRequest.url();
             m_user = url.user();
             m_pass = url.pass();
             m_firstRequest.removeCredentials();
@@ -138,8 +129,9 @@ namespace WebCore {
         ~ResourceHandleInternal();
 
         ResourceHandleClient* client() { return m_client; }
+
+        RefPtr<NetworkingContext> m_context;
         ResourceHandleClient* m_client;
-        
         ResourceRequest m_firstRequest;
         String m_lastHTTPMethod;
 
@@ -155,11 +147,12 @@ namespace WebCore {
         bool m_shouldContentSniff;
 #if USE(CFNETWORK)
         RetainPtr<CFURLConnectionRef> m_connection;
+        ResourceRequest m_currentRequest;
+        RefPtr<ResourceHandleCFURLConnectionDelegate> m_connectionDelegate;
 #endif
 #if PLATFORM(MAC) && !USE(CFNETWORK)
         RetainPtr<NSURLConnection> m_connection;
-        RetainPtr<WebCoreResourceHandleAsDelegate> m_delegate;
-        RetainPtr<id> m_proxy;
+        RetainPtr<id> m_delegate;
 #endif
 #if PLATFORM(MAC)
         bool m_startWhenScheduled;
@@ -186,9 +179,13 @@ namespace WebCore {
         struct curl_slist* m_customHeaders;
         ResourceResponse m_response;
         bool m_cancelled;
+        unsigned short m_authFailureCount;
 
         FormDataStream m_formDataStream;
+        unsigned m_sslErrors;
         Vector<char> m_postBytes;
+
+        OwnPtr<MultipartHandle> m_multipartHandle;
 #endif
 #if USE(SOUP)
         GRefPtr<SoupMessage> m_soupMessage;
@@ -200,13 +197,12 @@ namespace WebCore {
         GRefPtr<GCancellable> m_cancellable;
         GRefPtr<GAsyncResult> m_deferredResult;
         GRefPtr<GSource> m_timeoutSource;
-        char* m_buffer;
-        int m_bufferSize;
+        GUniquePtr<SoupBuffer> m_soupBuffer;
         unsigned long m_bodySize;
         unsigned long m_bodyDataSent;
-        RefPtr<NetworkingContext> m_context;
         SoupSession* soupSession();
         int m_redirectCount;
+        size_t m_previousPosition;
 #endif
 #if PLATFORM(GTK)
         struct {
@@ -214,10 +210,6 @@ namespace WebCore {
             AuthenticationChallenge challenge;
         } m_credentialDataToSaveInPersistentStore;
 #endif
-#if PLATFORM(QT)
-        QNetworkReplyHandler* m_job;
-        RefPtr<NetworkingContext> m_context;
-#endif
 
 #if PLATFORM(MAC)
         // We need to keep a reference to the original challenge to be able to cancel it.
@@ -225,13 +217,6 @@ namespace WebCore {
         NSURLAuthenticationChallenge *m_currentMacChallenge;
 #endif
         AuthenticationChallenge m_currentWebChallenge;
-#if PLATFORM(BLACKBERRY)
-        // We need to store the credentials for host and proxy separately for the platform
-        // networking layer. One of these will always be equal to m_currentWebChallenge.
-        AuthenticationChallenge m_hostWebChallenge;
-        AuthenticationChallenge m_proxyWebChallenge;
-#endif
-
         ResourceHandle::FailureType m_scheduledFailureType;
         Timer<ResourceHandle> m_failureTimer;
     };
