@@ -61,13 +61,14 @@
 #include "RenderBox.h"
 #include "RenderObject.h"
 #include "ScriptController.h"
-#include "ScriptValue.h"
 #include "SecurityOrigin.h"
 #include "Settings.h"
+#include "UserGestureIndicator.h"
 #include "WheelEvent.h"
 #include "c_instance.h"
 #include "npruntime_impl.h"
 #include "runtime_root.h"
+#include <bindings/ScriptValue.h>
 #include <runtime/JSCJSValue.h>
 #include <runtime/JSLock.h>
 #include <wtf/ASCIICType.h>
@@ -102,7 +103,7 @@ static InstanceMap& instanceMap()
     return map;
 }
 
-static String scriptStringIfJavaScriptURL(const KURL& url)
+static String scriptStringIfJavaScriptURL(const URL& url)
 {
     if (!protocolIsJavaScript(url))
         return String();
@@ -124,7 +125,7 @@ IntRect PluginView::windowClipRect() const
     IntRect clipRect(m_windowRect);
     
     // Take our element and get the clip rect from the enclosing layer and frame view.
-    FrameView* parentView = m_element->document()->view();
+    FrameView* parentView = m_element->document().view();
     clipRect.intersect(parentView->windowClipRectForFrameOwner(m_element, true));
 
     return clipRect;
@@ -132,7 +133,7 @@ IntRect PluginView::windowClipRect() const
 
 void PluginView::setFrameRect(const IntRect& rect)
 {
-    if (m_element->document()->printing())
+    if (m_element->document().printing())
         return;
 
     if (rect != frameRect())
@@ -173,7 +174,7 @@ void PluginView::handleEvent(Event* event)
     else if (event->isKeyboardEvent())
         handleKeyboardEvent(static_cast<KeyboardEvent*>(event));
 #if defined(XP_MACOSX)
-    else if (event->type() == eventNames().mousewheelEvent)
+    else if (event->type() == eventNames().wheelEvent || event->type() == eventNames().mousewheelEvent)
         handleWheelEvent(static_cast<WheelEvent*>(event));
 #endif
     else if (event->type() == eventNames().contextmenuEvent)
@@ -425,9 +426,11 @@ void PluginView::performRequest(PluginRequest* request)
     if (m_parentFrame->loader().documentLoader() != m_parentFrame->loader().activeDocumentLoader() && (targetFrameName.isNull() || m_parentFrame->tree().find(targetFrameName) != m_parentFrame))
         return;
 
-    KURL requestURL = request->frameLoadRequest().resourceRequest().url();
+    URL requestURL = request->frameLoadRequest().resourceRequest().url();
     String jsString = scriptStringIfJavaScriptURL(requestURL);
 
+    UserGestureIndicator gestureIndicator(request->shouldAllowPopups() ? DefinitelyProcessingUserGesture : PossiblyProcessingUserGesture);
+
     if (jsString.isNull()) {
         // if this is not a targeted request, create a stream for it. otherwise,
         // just pass it off to the loader
@@ -464,12 +467,12 @@ void PluginView::performRequest(PluginRequest* request)
     
     // Executing a script can cause the plugin view to be destroyed, so we keep a reference to it.
     RefPtr<PluginView> protector(this);
-    ScriptValue result = m_parentFrame->script().executeScript(jsString, request->shouldAllowPopups());
+    Deprecated::ScriptValue result = m_parentFrame->script().executeScript(jsString, request->shouldAllowPopups());
 
     if (targetFrameName.isNull()) {
         String resultString;
 
-        ScriptState* scriptState = m_parentFrame->script().globalObject(pluginWorld())->globalExec();
+        JSC::ExecState* scriptState = m_parentFrame->script().globalObject(pluginWorld())->globalExec();
         CString cstr;
         if (result.getString(scriptState, resultString))
             cstr = resultString.utf8();
@@ -509,7 +512,7 @@ NPError PluginView::load(const FrameLoadRequest& frameLoadRequest, bool sendNoti
 {
     ASSERT(frameLoadRequest.resourceRequest().httpMethod() == "GET" || frameLoadRequest.resourceRequest().httpMethod() == "POST");
 
-    KURL url = frameLoadRequest.resourceRequest().url();
+    URL url = frameLoadRequest.resourceRequest().url();
     
     if (url.isEmpty())
         return NPERR_INVALID_URL;
@@ -538,7 +541,7 @@ NPError PluginView::load(const FrameLoadRequest& frameLoadRequest, bool sendNoti
     return NPERR_NO_ERROR;
 }
 
-static KURL makeURL(const KURL& baseURL, const char* relativeURLString)
+static URL makeURL(const URL& baseURL, const char* relativeURLString)
 {
     String urlString = relativeURLString;
 
@@ -546,7 +549,7 @@ static KURL makeURL(const KURL& baseURL, const char* relativeURLString)
     urlString.replaceWithLiteral('\n', "");
     urlString.replaceWithLiteral('\r', "");
 
-    return KURL(baseURL, urlString);
+    return URL(baseURL, urlString);
 }
 
 NPError PluginView::getURLNotify(const char* url, const char* target, void* notifyData)
@@ -793,7 +796,7 @@ void PluginView::setParameters(const Vector<String>& paramNames, const Vector<St
     m_paramCount = paramCount;
 }
 
-PluginView::PluginView(Frame* parentFrame, const IntSize& size, PluginPackage* plugin, HTMLPlugInElement* element, const KURL& url, const Vector<String>& paramNames, const Vector<String>& paramValues, const String& mimeType, bool loadManually)
+PluginView::PluginView(Frame* parentFrame, const IntSize& size, PluginPackage* plugin, HTMLPlugInElement* element, const URL& url, const Vector<String>& paramNames, const Vector<String>& paramValues, const String& mimeType, bool loadManually)
     : m_parentFrame(parentFrame)
     , m_plugin(plugin)
     , m_element(element)
@@ -830,7 +833,7 @@ PluginView::PluginView(Frame* parentFrame, const IntSize& size, PluginPackage* p
     , m_wmPrintHDC(0)
     , m_haveUpdatedPluginWidget(false)
 #endif
-#if (PLATFORM(QT) && OS(WINDOWS)) || PLATFORM(EFL)
+#if PLATFORM(EFL)
     , m_window(0)
 #endif
 #if defined(XP_MACOSX)
@@ -842,9 +845,6 @@ PluginView::PluginView(Frame* parentFrame, const IntSize& size, PluginPackage* p
     , m_visual(0)
     , m_colormap(0)
     , m_pluginDisplay(0)
-#endif
-#if PLATFORM(QT) && defined(MOZ_PLATFORM_MAEMO) && (MOZ_PLATFORM_MAEMO >= 5)
-    , m_renderToImage(false)
 #endif
     , m_loadManually(loadManually)
     , m_manualStream(0)
@@ -945,7 +945,7 @@ bool PluginView::isCallingPlugin()
     return s_callingPlugin > 0;
 }
 
-PassRefPtr<PluginView> PluginView::create(Frame* parentFrame, const IntSize& size, HTMLPlugInElement* element, const KURL& url, const Vector<String>& paramNames, const Vector<String>& paramValues, const String& mimeType, bool loadManually)
+PassRefPtr<PluginView> PluginView::create(Frame* parentFrame, const IntSize& size, HTMLPlugInElement* element, const URL& url, const Vector<String>& paramNames, const Vector<String>& paramValues, const String& mimeType, bool loadManually)
 {
     // if we fail to find a plugin for this MIME type, findPlugin will search for
     // a plugin by the file extension and update the MIME type, so pass a mutable String
@@ -1368,11 +1368,7 @@ static Frame* getFrame(Frame* parentFrame, Element* element)
     if (parentFrame)
         return parentFrame;
     
-    Document* document = element->document();
-    if (document)
-        return document->frame();
-    
-    return 0;
+    return element->document().frame();
 }
 
 NPError PluginView::getValueForURL(NPNURLVariable variable, const char* url, char** value, uint32_t* len)
@@ -1383,7 +1379,7 @@ NPError PluginView::getValueForURL(NPNURLVariable variable, const char* url, cha
 
     switch (variable) {
     case NPNURLVCookie: {
-        KURL u(m_parentFrame->document()->baseURL(), url);
+        URL u(m_parentFrame->document()->baseURL(), url);
         if (u.isValid()) {
             Frame* frame = getFrame(parentFrame(), m_element);
             if (frame) {
@@ -1405,7 +1401,7 @@ NPError PluginView::getValueForURL(NPNURLVariable variable, const char* url, cha
         break;
     }
     case NPNURLVProxy: {
-        KURL u(m_parentFrame->document()->baseURL(), url);
+        URL u(m_parentFrame->document()->baseURL(), url);
         if (u.isValid()) {
             Frame* frame = getFrame(parentFrame(), m_element);
             const FrameLoader* frameLoader = frame ? &frame->loader() : 0;
@@ -1444,7 +1440,7 @@ NPError PluginView::setValueForURL(NPNURLVariable variable, const char* url, con
 
     switch (variable) {
     case NPNURLVCookie: {
-        KURL u(m_parentFrame->document()->baseURL(), url);
+        URL u(m_parentFrame->document()->baseURL(), url);
         if (u.isValid()) {
             const String cookieStr = String::fromUTF8(value, len);
             Frame* frame = getFrame(parentFrame(), m_element);
