@@ -31,27 +31,20 @@
 
 #include "ColorSpace.h"
 #include "Document.h"
+#include "ElementIterator.h"
 #include "FEColorMatrix.h"
 #include "FEComponentTransfer.h"
 #include "FEDropShadow.h"
 #include "FEGaussianBlur.h"
 #include "FEMerge.h"
 #include "FloatConversion.h"
+#include "Frame.h"
 #include "RenderLayer.h"
+#include "Settings.h"
 
 #include <algorithm>
 #include <wtf/MathExtras.h>
 
-#if ENABLE(CSS_SHADERS) && USE(3D_GRAPHICS)
-#include "CustomFilterGlobalContext.h"
-#include "CustomFilterOperation.h"
-#include "CustomFilterProgram.h"
-#include "CustomFilterValidatedProgram.h"
-#include "FECustomFilter.h"
-#include "RenderView.h"
-#include "ValidatedCustomFilterOperation.h"
-#endif
-
 #if ENABLE(SVG)
 #include "CachedSVGDocument.h"
 #include "CachedSVGDocumentReference.h"
@@ -85,32 +78,9 @@ inline bool isFilterSizeValid(FloatRect rect)
     return true;
 }
 
-#if ENABLE(CSS_SHADERS) && USE(3D_GRAPHICS)
-static PassRefPtr<FECustomFilter> createCustomFilterEffect(Filter* filter, Document* document, ValidatedCustomFilterOperation* operation)
-{
-    if (!document)
-        return 0;
-
-    CustomFilterGlobalContext* globalContext = document->renderView()->customFilterGlobalContext();
-    globalContext->prepareContextIfNeeded(document->view()->hostWindow());
-    if (!globalContext->context())
-        return 0;
-
-    return FECustomFilter::create(filter, globalContext->context(), operation->validatedProgram(), operation->parameters(),
-        operation->meshRows(), operation->meshColumns(),  operation->meshType());
-}
-#endif
-
 FilterEffectRenderer::FilterEffectRenderer()
-    : m_topOutset(0)
-    , m_rightOutset(0)
-    , m_bottomOutset(0)
-    , m_leftOutset(0)
-    , m_graphicsBufferAttached(false)
+    : m_graphicsBufferAttached(false)
     , m_hasFilterThatMovesPixels(false)
-#if ENABLE(CSS_SHADERS)
-    , m_hasCustomShaderFilter(false)
-#endif
 {
     setFilterResolution(FloatSize(1, 1));
     m_sourceGraphic = SourceGraphic::create(this);
@@ -125,14 +95,13 @@ GraphicsContext* FilterEffectRenderer::inputContext()
     return sourceImage() ? sourceImage()->context() : 0;
 }
 
-PassRefPtr<FilterEffect> FilterEffectRenderer::buildReferenceFilter(RenderObject* renderer, PassRefPtr<FilterEffect> previousEffect, ReferenceFilterOperation* filterOperation)
+PassRefPtr<FilterEffect> FilterEffectRenderer::buildReferenceFilter(RenderElement* renderer, PassRefPtr<FilterEffect> previousEffect, ReferenceFilterOperation* filterOperation)
 {
 #if ENABLE(SVG)
     if (!renderer)
         return 0;
 
-    Document* document = renderer->document();
-    ASSERT(document);
+    Document* document = &renderer->document();
 
     CachedSVGDocumentReference* cachedSVGDocumentReference = filterOperation->cachedSVGDocumentReference();
     CachedSVGDocument* cachedSVGDocument = cachedSVGDocumentReference ? cachedSVGDocumentReference->document() : 0;
@@ -149,7 +118,7 @@ PassRefPtr<FilterEffect> FilterEffectRenderer::buildReferenceFilter(RenderObject
     if (!filter) {
         // Although we did not find the referenced filter, it might exist later
         // in the document
-        document->accessSVGExtensions()->addPendingResource(filterOperation->fragment(), toElement(renderer->node()));
+        document->accessSVGExtensions()->addPendingResource(filterOperation->fragment(), renderer->element());
         return 0;
     }
 
@@ -160,24 +129,15 @@ PassRefPtr<FilterEffect> FilterEffectRenderer::buildReferenceFilter(RenderObject
     // wrong. We should probably be extracting the alpha from the 
     // previousEffect, but this requires some more processing.  
     // This may need a spec clarification.
-    RefPtr<SVGFilterBuilder> builder = SVGFilterBuilder::create(previousEffect, SourceAlpha::create(this));
+    auto builder = std::make_unique<SVGFilterBuilder>(previousEffect, SourceAlpha::create(this));
 
-    for (Node* node = filter->firstChild(); node; node = node->nextSibling()) {
-        if (!node->isSVGElement())
-            continue;
-
-        SVGElement* element = static_cast<SVGElement*>(node);
-        if (!element->isFilterEffect())
-            continue;
-
-        SVGFilterPrimitiveStandardAttributes* effectElement = static_cast<SVGFilterPrimitiveStandardAttributes*>(element);
-
-        effect = effectElement->build(builder.get(), this);
+    for (auto& effectElement : childrenOfType<SVGFilterPrimitiveStandardAttributes>(*filter)) {
+        effect = effectElement.build(builder.get(), this);
         if (!effect)
             continue;
 
-        effectElement->setStandardAttributes(effect.get());
-        builder->add(effectElement->result(), effect);
+        effectElement.setStandardAttributes(effect.get());
+        builder->add(effectElement.result(), effect);
         m_effects.append(effect);
     }
     return effect;
@@ -189,25 +149,19 @@ PassRefPtr<FilterEffect> FilterEffectRenderer::buildReferenceFilter(RenderObject
 #endif
 }
 
-bool FilterEffectRenderer::build(RenderObject* renderer, const FilterOperations& operations)
+bool FilterEffectRenderer::build(RenderElement* renderer, const FilterOperations& operations, FilterConsumer consumer)
 {
-#if ENABLE(CSS_SHADERS)
-    m_hasCustomShaderFilter = false;
-#endif
     m_hasFilterThatMovesPixels = operations.hasFilterThatMovesPixels();
     if (m_hasFilterThatMovesPixels)
-        operations.getOutsets(m_topOutset, m_rightOutset, m_bottomOutset, m_leftOutset);
-    
-    // Keep the old effects on the stack until we've created the new effects.
-    // New FECustomFilters can reuse cached resources from old FECustomFilters.
-    FilterEffectList oldEffects;
-    m_effects.swap(oldEffects);
+        m_outsets = operations.outsets();
+
+    m_effects.clear();
 
     RefPtr<FilterEffect> previousEffect = m_sourceGraphic;
     for (size_t i = 0; i < operations.operations().size(); ++i) {
         RefPtr<FilterEffect> effect;
         FilterOperation* filterOperation = operations.operations().at(i).get();
-        switch (filterOperation->getOperationType()) {
+        switch (filterOperation->type()) {
         case FilterOperation::REFERENCE: {
             ReferenceFilterOperation* referenceOperation = static_cast<ReferenceFilterOperation*>(filterOperation);
             effect = buildReferenceFilter(renderer, previousEffect, referenceOperation);
@@ -336,7 +290,7 @@ bool FilterEffectRenderer::build(RenderObject* renderer, const FilterOperations&
         case FilterOperation::BLUR: {
             BlurFilterOperation* blurOperation = static_cast<BlurFilterOperation*>(filterOperation);
             float stdDeviation = floatValueForLength(blurOperation->stdDeviation(), 0);
-            effect = FEGaussianBlur::create(this, stdDeviation, stdDeviation);
+            effect = FEGaussianBlur::create(this, stdDeviation, stdDeviation, consumer == FilterProperty ? EDGEMODE_NONE : EDGEMODE_DUPLICATE);
             break;
         }
         case FilterOperation::DROP_SHADOW: {
@@ -345,31 +299,17 @@ bool FilterEffectRenderer::build(RenderObject* renderer, const FilterOperations&
                                                 dropShadowOperation->x(), dropShadowOperation->y(), dropShadowOperation->color(), 1);
             break;
         }
-#if ENABLE(CSS_SHADERS) && USE(3D_GRAPHICS)
-        case FilterOperation::CUSTOM:
-            // CUSTOM operations are always converted to VALIDATED_CUSTOM before getting here.
-            // The conversion happens in RenderLayer::computeFilterOperations.
-            ASSERT_NOT_REACHED();
-            break;
-        case FilterOperation::VALIDATED_CUSTOM: {
-            ValidatedCustomFilterOperation* customFilterOperation = static_cast<ValidatedCustomFilterOperation*>(filterOperation);
-            Document* document = renderer ? renderer->document() : 0;
-            effect = createCustomFilterEffect(this, document, customFilterOperation);
-            if (effect)
-                m_hasCustomShaderFilter = true;
-            break;
-        }
-#endif
         default:
             break;
         }
 
         if (effect) {
-            // Unlike SVG, filters applied here should not clip to their primitive subregions.
-            effect->setClipsToBounds(false);
-            effect->setColorSpace(ColorSpaceDeviceRGB);
+            // Unlike SVG Filters and CSSFilterImages, filter functions on the filter
+            // property applied here should not clip to their primitive subregions.
+            effect->setClipsToBounds(consumer == FilterFunction);
+            effect->setOperatingColorSpace(ColorSpaceDeviceRGB);
             
-            if (filterOperation->getOperationType() != FilterOperation::REFERENCE) {
+            if (filterOperation->type() != FilterOperation::REFERENCE) {
                 effect->inputEffects().append(previousEffect);
                 m_effects.append(effect);
             }
@@ -427,20 +367,13 @@ void FilterEffectRenderer::apply()
 
 LayoutRect FilterEffectRenderer::computeSourceImageRectForDirtyRect(const LayoutRect& filterBoxRect, const LayoutRect& dirtyRect)
 {
-#if ENABLE(CSS_SHADERS)
-    if (hasCustomShaderFilter()) {
-        // When we have at least a custom shader in the chain, we need to compute the whole source image, because the shader can
-        // reference any pixel and we cannot control that.
-        return filterBoxRect;
-    }
-#endif
     // The result of this function is the area in the "filterBoxRect" that needs to be repainted, so that we fully cover the "dirtyRect".
     LayoutRect rectForRepaint = dirtyRect;
     if (hasFilterThatMovesPixels()) {
         // Note that the outsets are reversed here because we are going backwards -> we have the dirty rect and
         // need to find out what is the rectangle that might influence the result inside that dirty rect.
-        rectForRepaint.move(-m_rightOutset, -m_bottomOutset);
-        rectForRepaint.expand(m_leftOutset + m_rightOutset, m_topOutset + m_bottomOutset);
+        rectForRepaint.move(-m_outsets.right(), -m_outsets.bottom());
+        rectForRepaint.expand(m_outsets.left() + m_outsets.right(), m_outsets.top() + m_outsets.bottom());
     }
     rectForRepaint.intersect(filterBoxRect);
     return rectForRepaint;
@@ -473,8 +406,17 @@ bool FilterEffectRendererHelper::prepareFilterEffect(RenderLayer* renderLayer, c
     }
     return true;
 }
-   
-GraphicsContext* FilterEffectRendererHelper::beginFilterEffect(GraphicsContext* oldContext)
+
+GraphicsContext* FilterEffectRendererHelper::filterContext() const
+{
+    if (!m_haveFilterEffect)
+        return 0;
+
+    FilterEffectRenderer* filter = m_renderLayer->filterRenderer();
+    return filter->inputContext();
+}
+
+bool FilterEffectRendererHelper::beginFilterEffect()
 {
     ASSERT(m_renderLayer);
     
@@ -485,21 +427,20 @@ GraphicsContext* FilterEffectRendererHelper::beginFilterEffect(GraphicsContext*
     if (!sourceGraphicsContext || !isFilterSizeValid(filter->filterRegion())) {
         // Disable the filters and continue.
         m_haveFilterEffect = false;
-        return oldContext;
+        return false;
     }
     
-    m_savedGraphicsContext = oldContext;
-    
     // Translate the context so that the contents of the layer is captuterd in the offscreen memory buffer.
     sourceGraphicsContext->save();
     sourceGraphicsContext->translate(-m_paintOffset.x(), -m_paintOffset.y());
     sourceGraphicsContext->clearRect(m_repaintRect);
     sourceGraphicsContext->clip(m_repaintRect);
-    
-    return sourceGraphicsContext;
+
+    m_startedFilterEffect = true;
+    return true;
 }
 
-GraphicsContext* FilterEffectRendererHelper::applyFilterEffect()
+void FilterEffectRendererHelper::applyFilterEffect(GraphicsContext* destinationContext)
 {
     ASSERT(m_haveFilterEffect && m_renderLayer->filterRenderer());
     FilterEffectRenderer* filter = m_renderLayer->filterRenderer();
@@ -511,11 +452,9 @@ GraphicsContext* FilterEffectRendererHelper::applyFilterEffect()
     LayoutRect destRect = filter->outputRect();
     destRect.move(m_paintOffset.x(), m_paintOffset.y());
     
-    m_savedGraphicsContext->drawImageBuffer(filter->output(), m_renderLayer->renderer()->style()->colorSpace(), pixelSnappedIntRect(destRect), CompositeSourceOver);
+    destinationContext->drawImageBuffer(filter->output(), m_renderLayer->renderer().style().colorSpace(), pixelSnappedIntRect(destRect), CompositeSourceOver);
     
     filter->clearIntermediateResults();
-    
-    return m_savedGraphicsContext;
 }
 
 } // namespace WebCore
