@@ -53,9 +53,9 @@ typedef HashMap<OverlapTestRequestClient*, IntRect> OverlapTestRequestMap;
  * (tx|ty) is the calculated position of the parent
  */
 struct PaintInfo {
-    PaintInfo(GraphicsContext* newContext, const IntRect& newRect, PaintPhase newPhase, PaintBehavior newPaintBehavior,
-        RenderObject* newSubtreePaintRoot = 0, RenderRegion* region = 0, ListHashSet<RenderInline*>* newOutlineObjects = 0,
-        OverlapTestRequestMap* overlapTestRequests = 0, const RenderLayerModelObject* newPaintContainer = 0)
+    PaintInfo(GraphicsContext* newContext, const LayoutRect& newRect, PaintPhase newPhase, PaintBehavior newPaintBehavior,
+        RenderObject* newSubtreePaintRoot = nullptr, RenderRegion* region = nullptr, ListHashSet<RenderInline*>* newOutlineObjects = nullptr,
+        OverlapTestRequestMap* overlapTestRequests = nullptr, const RenderLayerModelObject* newPaintContainer = nullptr)
         : context(newContext)
         , rect(newRect)
         , phase(newPhase)
@@ -73,16 +73,16 @@ struct PaintInfo {
         if (!subtreePaintRoot)
             return;
 
-        // If we're the painting root, kids draw normally, and see root of 0.
+        // If we're the painting root, kids draw normally, and see root of nullptr.
         if (subtreePaintRoot == renderer) {
-            subtreePaintRoot = 0; 
+            subtreePaintRoot = nullptr;
             return;
         }
     }
 
-    bool shouldPaintWithinRoot(const RenderObject* renderer) const
+    bool shouldPaintWithinRoot(const RenderObject& renderer) const
     {
-        return !subtreePaintRoot || subtreePaintRoot == renderer;
+        return !subtreePaintRoot || subtreePaintRoot == &renderer;
     }
 
     bool forceBlackText() const { return paintBehavior & PaintBehaviorForceBlackText; }
@@ -98,17 +98,17 @@ struct PaintInfo {
 
         context->concatCTM(localToAncestorTransform);
 
-        if (rect == infiniteRect())
+        if (rect == LayoutRect::infiniteRect())
             return;
 
-        rect = localToAncestorTransform.inverse().mapRect(rect);
+        FloatRect tranformedRect(localToAncestorTransform.inverse().mapRect(rect));
+        rect.setLocation(LayoutPoint(tranformedRect.location()));
+        rect.setSize(LayoutSize(tranformedRect.size()));
     }
 #endif
 
-    static IntRect infiniteRect() { return IntRect(LayoutRect::infiniteRect()); }
-
     GraphicsContext* context;
-    IntRect rect;
+    LayoutRect rect;
     PaintPhase phase;
     PaintBehavior paintBehavior;
     RenderObject* subtreePaintRoot; // used to draw just one element and its visual children
