@@ -29,27 +29,28 @@ namespace WebCore {
 
 class CounterNode;
 
-class RenderCounter : public RenderText {
+class RenderCounter final : public RenderText {
 public:
-    RenderCounter(Document*, const CounterContent&);
+    RenderCounter(Document&, const CounterContent&);
     virtual ~RenderCounter();
 
     static void destroyCounterNodes(RenderObject*);
     static void destroyCounterNode(RenderObject*, const AtomicString& identifier);
     static void rendererSubtreeAttached(RenderObject*);
-    static void rendererRemovedFromTree(RenderObject*);
+    static void rendererRemovedFromTree(RenderObject&);
     static void rendererStyleChanged(RenderObject*, const RenderStyle* oldStyle, const RenderStyle* newStyle);
 
+    void updateCounter();
+
 protected:
     virtual void willBeDestroyed();
 
 private:
     virtual const char* renderName() const;
     virtual bool isCounter() const;
-    virtual PassRefPtr<StringImpl> originalText() const;
+    virtual String originalText() const;
     
-    virtual void updateText() OVERRIDE;
-    virtual void computePreferredLogicalWidths(float leadWidth) OVERRIDE;
+    virtual void computePreferredLogicalWidths(float leadWidth) override;
 
     // Removes the reference to the CounterNode associated with this renderer.
     // This is used to cause a counter display update when the CounterNode tree changes.
@@ -61,14 +62,8 @@ private:
     friend class CounterNode;
 };
 
-inline RenderCounter* toRenderCounter(RenderObject* object)
-{
-    ASSERT_WITH_SECURITY_IMPLICATION(!object || object->isCounter());
-    return static_cast<RenderCounter*>(object);
-}
-
-// This will catch anyone doing an unnecessary cast.
-void toRenderCounter(const RenderCounter*);
+template<> inline bool isRendererOfType<const RenderCounter>(const RenderObject& renderer) { return renderer.isCounter(); }
+RENDER_OBJECT_TYPE_CASTS(RenderCounter, isCounter())
 
 } // namespace WebCore
 
