@@ -30,7 +30,6 @@
 #include "FontSelector.h"
 #include "Pagination.h"
 #include "QuotesData.h"
-#include "RenderArena.h"
 #include "RenderObject.h"
 #include "ScaleTransformOperation.h"
 #include "ShadowData.h"
@@ -44,12 +43,14 @@
 #include <wtf/StdLibExtras.h>
 #include <algorithm>
 
+#if ENABLE(IOS_TEXT_AUTOSIZING)
+#include <wtf/text/StringHash.h>
+#endif
+
 #if ENABLE(TEXT_AUTOSIZING)
 #include "TextAutosizer.h"
 #endif
 
-using namespace std;
-
 namespace WebCore {
 
 struct SameSizeAsBorderValue {
@@ -76,46 +77,58 @@ struct SameSizeAsRenderStyle : public RefCounted<SameSizeAsRenderStyle> {
 
 COMPILE_ASSERT(sizeof(RenderStyle) == sizeof(SameSizeAsRenderStyle), RenderStyle_should_stay_small);
 
-inline RenderStyle* defaultStyle()
+inline RenderStyle& defaultStyle()
 {
-    static RenderStyle* s_defaultStyle = RenderStyle::createDefaultStyle().leakRef();
-    return s_defaultStyle;
+    static RenderStyle& style = RenderStyle::createDefaultStyle().leakRef();
+    return style;
 }
 
-PassRefPtr<RenderStyle> RenderStyle::create()
+PassRef<RenderStyle> RenderStyle::create()
 {
-    return adoptRef(new RenderStyle());
+    return adoptRef(*new RenderStyle());
 }
 
-PassRefPtr<RenderStyle> RenderStyle::createDefaultStyle()
+PassRef<RenderStyle> RenderStyle::createDefaultStyle()
 {
-    return adoptRef(new RenderStyle(true));
+    return adoptRef(*new RenderStyle(true));
 }
 
-PassRefPtr<RenderStyle> RenderStyle::createAnonymousStyleWithDisplay(const RenderStyle* parentStyle, EDisplay display)
+PassRef<RenderStyle> RenderStyle::createAnonymousStyleWithDisplay(const RenderStyle* parentStyle, EDisplay display)
 {
-    RefPtr<RenderStyle> newStyle = RenderStyle::create();
-    newStyle->inheritFrom(parentStyle);
-    newStyle->inheritUnicodeBidiFrom(parentStyle);
-    newStyle->setDisplay(display);
+    auto newStyle = RenderStyle::create();
+    newStyle.get().inheritFrom(parentStyle);
+    newStyle.get().inheritUnicodeBidiFrom(parentStyle);
+    newStyle.get().setDisplay(display);
     return newStyle;
 }
 
-PassRefPtr<RenderStyle> RenderStyle::clone(const RenderStyle* other)
+PassRef<RenderStyle> RenderStyle::clone(const RenderStyle* other)
 {
-    return adoptRef(new RenderStyle(*other));
+    return adoptRef(*new RenderStyle(*other));
+}
+
+PassRef<RenderStyle> RenderStyle::createStyleInheritingFromPseudoStyle(const RenderStyle& pseudoStyle)
+{
+    ASSERT(pseudoStyle.styleType() == BEFORE || pseudoStyle.styleType() == AFTER);
+
+    // Images are special and must inherit the pseudoStyle so the width and height of
+    // the pseudo element doesn't change the size of the image. In all other cases we
+    // can just share the style.
+    auto style = RenderStyle::create();
+    style.get().inheritFrom(&pseudoStyle);
+    return style;
 }
 
 ALWAYS_INLINE RenderStyle::RenderStyle()
-    : m_box(defaultStyle()->m_box)
-    , visual(defaultStyle()->visual)
-    , m_background(defaultStyle()->m_background)
-    , surround(defaultStyle()->surround)
-    , rareNonInheritedData(defaultStyle()->rareNonInheritedData)
-    , rareInheritedData(defaultStyle()->rareInheritedData)
-    , inherited(defaultStyle()->inherited)
+    : m_box(defaultStyle().m_box)
+    , visual(defaultStyle().visual)
+    , m_background(defaultStyle().m_background)
+    , surround(defaultStyle().surround)
+    , rareNonInheritedData(defaultStyle().rareNonInheritedData)
+    , rareInheritedData(defaultStyle().rareInheritedData)
+    , inherited(defaultStyle().inherited)
 #if ENABLE(SVG)
-    , m_svgStyle(defaultStyle()->m_svgStyle)
+    , m_svgStyle(defaultStyle().m_svgStyle)
 #endif
 {
     setBitDefaults(); // Would it be faster to copy this from the default style?
@@ -124,29 +137,18 @@ ALWAYS_INLINE RenderStyle::RenderStyle()
 }
 
 ALWAYS_INLINE RenderStyle::RenderStyle(bool)
+    : m_box(StyleBoxData::create())
+    , visual(StyleVisualData::create())
+    , m_background(StyleBackgroundData::create())
+    , surround(StyleSurroundData::create())
+    , rareNonInheritedData(StyleRareNonInheritedData::create())
+    , rareInheritedData(StyleRareInheritedData::create())
+    , inherited(StyleInheritedData::create())
+#if ENABLE(SVG)
+    , m_svgStyle(SVGRenderStyle::create())
+#endif
 {
     setBitDefaults();
-
-    m_box.init();
-    visual.init();
-    m_background.init();
-    surround.init();
-    rareNonInheritedData.init();
-    rareNonInheritedData.access()->m_deprecatedFlexibleBox.init();
-    rareNonInheritedData.access()->m_flexibleBox.init();
-    rareNonInheritedData.access()->m_marquee.init();
-    rareNonInheritedData.access()->m_multiCol.init();
-    rareNonInheritedData.access()->m_transform.init();
-#if ENABLE(CSS_FILTERS)
-    rareNonInheritedData.access()->m_filter.init();
-#endif
-    rareNonInheritedData.access()->m_grid.init();
-    rareNonInheritedData.access()->m_gridItem.init();
-    rareInheritedData.init();
-    inherited.init();
-#if ENABLE(SVG)
-    m_svgStyle.init();
-#endif
 }
 
 ALWAYS_INLINE RenderStyle::RenderStyle(const RenderStyle& o)
@@ -235,30 +237,6 @@ bool RenderStyle::isStyleAvailable() const
     return this != StyleResolver::styleNotYetAvailable();
 }
 
-static inline int pseudoBit(PseudoId pseudo)
-{
-    return 1 << (pseudo - 1);
-}
-
-bool RenderStyle::hasAnyPublicPseudoStyles() const
-{
-    return PUBLIC_PSEUDOID_MASK & noninherited_flags._pseudoBits;
-}
-
-bool RenderStyle::hasPseudoStyle(PseudoId pseudo) const
-{
-    ASSERT(pseudo > NOPSEUDO);
-    ASSERT(pseudo < FIRST_INTERNAL_PSEUDOID);
-    return pseudoBit(pseudo) & noninherited_flags._pseudoBits;
-}
-
-void RenderStyle::setHasPseudoStyle(PseudoId pseudo)
-{
-    ASSERT(pseudo > NOPSEUDO);
-    ASSERT(pseudo < FIRST_INTERNAL_PSEUDOID);
-    noninherited_flags._pseudoBits |= pseudoBit(pseudo);
-}
-
 bool RenderStyle::hasUniquePseudoStyle() const
 {
     if (!m_cachedPseudoStyles || styleType() != NOPSEUDO)
@@ -330,6 +308,64 @@ bool RenderStyle::inheritedNotEqual(const RenderStyle* other) const
            || rareInheritedData != other->rareInheritedData;
 }
 
+#if ENABLE(IOS_TEXT_AUTOSIZING)
+inline unsigned computeFontHash(const Font& font)
+{
+    unsigned hashCodes[2] = {
+        CaseFoldingHash::hash(font.fontDescription().firstFamily().impl()),
+        static_cast<unsigned>(font.fontDescription().specifiedSize())
+    };
+    return StringHasher::computeHash(reinterpret_cast<UChar*>(hashCodes), 2 * sizeof(unsigned) / sizeof(UChar));
+}
+
+uint32_t RenderStyle::hashForTextAutosizing() const
+{
+    // FIXME: Not a very smart hash. Could be improved upon. See <https://bugs.webkit.org/show_bug.cgi?id=121131>.
+    uint32_t hash = 0;
+    
+    hash ^= rareNonInheritedData->m_appearance;
+    hash ^= rareNonInheritedData->marginBeforeCollapse;
+    hash ^= rareNonInheritedData->marginAfterCollapse;
+    hash ^= rareNonInheritedData->lineClamp.value();
+    hash ^= rareInheritedData->overflowWrap;
+    hash ^= rareInheritedData->nbspMode;
+    hash ^= rareInheritedData->lineBreak;
+    hash ^= WTF::FloatHash<float>::hash(inherited->specifiedLineHeight.value());
+    hash ^= computeFontHash(inherited->font);
+    hash ^= inherited->horizontal_border_spacing;
+    hash ^= inherited->vertical_border_spacing;
+    hash ^= inherited_flags._box_direction;
+    hash ^= inherited_flags.m_rtlOrdering;
+    hash ^= noninherited_flags._position;
+    hash ^= noninherited_flags._floating;
+    hash ^= rareNonInheritedData->textOverflow;
+    hash ^= rareInheritedData->textSecurity;
+    return hash;
+}
+
+bool RenderStyle::equalForTextAutosizing(const RenderStyle* other) const
+{
+    return rareNonInheritedData->m_appearance == other->rareNonInheritedData->m_appearance
+        && rareNonInheritedData->marginBeforeCollapse == other->rareNonInheritedData->marginBeforeCollapse
+        && rareNonInheritedData->marginAfterCollapse == other->rareNonInheritedData->marginAfterCollapse
+        && rareNonInheritedData->lineClamp == other->rareNonInheritedData->lineClamp
+        && rareInheritedData->textSizeAdjust == other->rareInheritedData->textSizeAdjust
+        && rareInheritedData->overflowWrap == other->rareInheritedData->overflowWrap
+        && rareInheritedData->nbspMode == other->rareInheritedData->nbspMode
+        && rareInheritedData->lineBreak == other->rareInheritedData->lineBreak
+        && rareInheritedData->textSecurity == other->rareInheritedData->textSecurity
+        && inherited->specifiedLineHeight == other->inherited->specifiedLineHeight
+        && inherited->font.equalForTextAutoSizing(other->inherited->font)
+        && inherited->horizontal_border_spacing == other->inherited->horizontal_border_spacing
+        && inherited->vertical_border_spacing == other->inherited->vertical_border_spacing
+        && inherited_flags._box_direction == other->inherited_flags._box_direction
+        && inherited_flags.m_rtlOrdering == other->inherited_flags.m_rtlOrdering
+        && noninherited_flags._position == other->noninherited_flags._position
+        && noninherited_flags._floating == other->noninherited_flags._floating
+        && rareNonInheritedData->textOverflow == other->rareNonInheritedData->textOverflow;
+}
+#endif // ENABLE(IOS_TEXT_AUTOSIZING)
+
 bool RenderStyle::inheritedDataShared(const RenderStyle* other) const
 {
     // This is a fast check that only looks if the data structures are shared.
@@ -442,6 +478,7 @@ bool RenderStyle::changeRequiresLayout(const RenderStyle* other, unsigned& chang
             changedContextSensitiveProperties |= ContextSensitivePropertyTransform;
             // Don't return; keep looking for another change
 #else
+            UNUSED_PARAM(changedContextSensitiveProperties);
             return true;
 #endif
         }
@@ -477,9 +514,13 @@ bool RenderStyle::changeRequiresLayout(const RenderStyle* other, unsigned& chang
         if (rareInheritedData->highlight != other->rareInheritedData->highlight
             || rareInheritedData->indent != other->rareInheritedData->indent
 #if ENABLE(CSS3_TEXT)
+            || rareInheritedData->m_textAlignLast != other->rareInheritedData->m_textAlignLast
             || rareInheritedData->m_textIndentLine != other->rareInheritedData->m_textIndentLine
 #endif
             || rareInheritedData->m_effectiveZoom != other->rareInheritedData->m_effectiveZoom
+#if ENABLE(IOS_TEXT_AUTOSIZING)
+            || rareInheritedData->textSizeAdjust != other->rareInheritedData->textSizeAdjust
+#endif
             || rareInheritedData->wordBreak != other->rareInheritedData->wordBreak
             || rareInheritedData->overflowWrap != other->rareInheritedData->overflowWrap
             || rareInheritedData->nbspMode != other->rareInheritedData->nbspMode
@@ -498,6 +539,9 @@ bool RenderStyle::changeRequiresLayout(const RenderStyle* other, unsigned& chang
             || rareInheritedData->m_tabSize != other->rareInheritedData->m_tabSize
             || rareInheritedData->m_lineBoxContain != other->rareInheritedData->m_lineBoxContain
             || rareInheritedData->m_lineGrid != other->rareInheritedData->m_lineGrid
+#if ENABLE(CSS_IMAGE_ORIENTATION)
+            || rareInheritedData->m_imageOrientation != other->rareInheritedData->m_imageOrientation
+#endif
 #if ENABLE(CSS_IMAGE_RESOLUTION)
             || rareInheritedData->m_imageResolutionSource != other->rareInheritedData->m_imageResolutionSource
             || rareInheritedData->m_imageResolutionSnap != other->rareInheritedData->m_imageResolutionSnap
@@ -505,6 +549,9 @@ bool RenderStyle::changeRequiresLayout(const RenderStyle* other, unsigned& chang
 #endif
             || rareInheritedData->m_lineSnap != other->rareInheritedData->m_lineSnap
             || rareInheritedData->m_lineAlign != other->rareInheritedData->m_lineAlign
+#if ENABLE(ACCELERATED_OVERFLOW_SCROLLING)
+            || rareInheritedData->useTouchOverflowScrolling != other->rareInheritedData->useTouchOverflowScrolling
+#endif
             || rareInheritedData->listStyleImage != other->rareInheritedData->listStyleImage)
             return true;
 
@@ -521,6 +568,9 @@ bool RenderStyle::changeRequiresLayout(const RenderStyle* other, unsigned& chang
 #endif
 
     if (inherited->line_height != other->inherited->line_height
+#if ENABLE(IOS_TEXT_AUTOSIZING)
+        || inherited->specifiedLineHeight != other->inherited->specifiedLineHeight
+#endif
         || inherited->font != other->inherited->font
         || inherited->horizontal_border_spacing != other->inherited->horizontal_border_spacing
         || inherited->vertical_border_spacing != other->inherited->vertical_border_spacing
@@ -667,6 +717,7 @@ bool RenderStyle::changeRequiresLayerRepaint(const RenderStyle* other, unsigned&
         changedContextSensitiveProperties |= ContextSensitivePropertyOpacity;
         // Don't return; keep looking for another change.
 #else
+        UNUSED_PARAM(changedContextSensitiveProperties);
         return true;
 #endif
     }
@@ -701,6 +752,7 @@ bool RenderStyle::changeRequiresRepaint(const RenderStyle* other, unsigned&) con
         || rareInheritedData->userSelect != other->rareInheritedData->userSelect
         || rareNonInheritedData->userDrag != other->rareNonInheritedData->userDrag
         || rareNonInheritedData->m_borderFit != other->rareNonInheritedData->m_borderFit
+        || rareNonInheritedData->m_objectFit != other->rareNonInheritedData->m_objectFit
         || rareInheritedData->m_imageRendering != other->rareInheritedData->m_imageRendering)
         return true;
 
@@ -720,15 +772,14 @@ bool RenderStyle::changeRequiresRepaint(const RenderStyle* other, unsigned&) con
     return false;
 }
 
-bool RenderStyle::changeRequiresRepaintIfText(const RenderStyle* other, unsigned&) const
+bool RenderStyle::changeRequiresRepaintIfTextOrBorderOrOutline(const RenderStyle* other, unsigned&) const
 {
     if (inherited->color != other->inherited->color
         || inherited_flags._text_decorations != other->inherited_flags._text_decorations
         || visual->textDecoration != other->visual->textDecoration
-#if ENABLE(CSS3_TEXT)
         || rareNonInheritedData->m_textDecorationStyle != other->rareNonInheritedData->m_textDecorationStyle
         || rareNonInheritedData->m_textDecorationColor != other->rareNonInheritedData->m_textDecorationColor
-#endif // CSS3_TEXT
+        || rareInheritedData->m_textDecorationSkip != other->rareInheritedData->m_textDecorationSkip
         || rareInheritedData->textFillColor != other->rareInheritedData->textFillColor
         || rareInheritedData->textStrokeColor != other->rareInheritedData->textStrokeColor
         || rareInheritedData->textEmphasisColor != other->rareInheritedData->textEmphasisColor
@@ -749,6 +800,8 @@ bool RenderStyle::changeRequiresRecompositeLayer(const RenderStyle* other, unsig
             || rareNonInheritedData->m_perspectiveOriginY != other->rareNonInheritedData->m_perspectiveOriginY)
             return true;
     }
+#else
+    UNUSED_PARAM(other);
 #endif
     return false;
 }
@@ -792,8 +845,8 @@ StyleDifference RenderStyle::diff(const RenderStyle* other, unsigned& changedCon
         return StyleDifferenceRecompositeLayer;
 #endif
 
-    if (changeRequiresRepaintIfText(other, changedContextSensitiveProperties))
-        return StyleDifferenceRepaintIfText;
+    if (changeRequiresRepaintIfTextOrBorderOrOutline(other, changedContextSensitiveProperties))
+        return StyleDifferenceRepaintIfTextOrBorderOrOutline;
 
     // Cursors are not checked, since they will be set appropriately in response to mouse events,
     // so they don't need to cause any repaint or layout.
@@ -850,17 +903,17 @@ void RenderStyle::clearContent()
         rareNonInheritedData.access()->m_content = nullptr;
 }
 
-void RenderStyle::appendContent(PassOwnPtr<ContentData> contentData)
+void RenderStyle::appendContent(std::unique_ptr<ContentData> contentData)
 {
-    OwnPtr<ContentData>& content = rareNonInheritedData.access()->m_content;
+    auto& content = rareNonInheritedData.access()->m_content;
     ContentData* lastContent = content.get();
     while (lastContent && lastContent->next())
         lastContent = lastContent->next();
 
     if (lastContent)
-        lastContent->setNext(contentData);
+        lastContent->setNext(std::move(contentData));
     else
-        content = contentData;
+        content = std::move(contentData);
 }
 
 void RenderStyle::setContent(PassRefPtr<StyleImage> image, bool add)
@@ -869,16 +922,18 @@ void RenderStyle::setContent(PassRefPtr<StyleImage> image, bool add)
         return;
         
     if (add) {
-        appendContent(ContentData::create(image));
+        appendContent(std::make_unique<ImageContentData>(image));
         return;
     }
 
-    rareNonInheritedData.access()->m_content = ContentData::create(image);
+    rareNonInheritedData.access()->m_content = std::make_unique<ImageContentData>(image);
+    if (!rareNonInheritedData.access()->m_altText.isNull())
+        rareNonInheritedData.access()->m_content->setAltText(rareNonInheritedData.access()->m_altText);
 }
 
 void RenderStyle::setContent(const String& string, bool add)
 {
-    OwnPtr<ContentData>& content = rareNonInheritedData.access()->m_content;
+    auto& content = rareNonInheritedData.access()->m_content;
     if (add) {
         ContentData* lastContent = content.get();
         while (lastContent && lastContent->next())
@@ -890,39 +945,56 @@ void RenderStyle::setContent(const String& string, bool add)
                 TextContentData* textContent = static_cast<TextContentData*>(lastContent);
                 textContent->setText(textContent->text() + string);
             } else
-                lastContent->setNext(ContentData::create(string));
+                lastContent->setNext(std::make_unique<TextContentData>(string));
 
+            if (!rareNonInheritedData.access()->m_altText.isNull())
+                lastContent->setAltText(rareNonInheritedData.access()->m_altText);
             return;
         }
     }
 
-    content = ContentData::create(string);
+    content = std::make_unique<TextContentData>(string);
+    if (!rareNonInheritedData.access()->m_altText.isNull())
+        content->setAltText(rareNonInheritedData.access()->m_altText);
 }
 
-void RenderStyle::setContent(PassOwnPtr<CounterContent> counter, bool add)
+void RenderStyle::setContent(std::unique_ptr<CounterContent> counter, bool add)
 {
     if (!counter)
         return;
 
     if (add) {
-        appendContent(ContentData::create(counter));
+        appendContent(std::make_unique<CounterContentData>(std::move(counter)));
         return;
     }
 
-    rareNonInheritedData.access()->m_content = ContentData::create(counter);
+    rareNonInheritedData.access()->m_content = std::make_unique<CounterContentData>(std::move(counter));
 }
 
 void RenderStyle::setContent(QuoteType quote, bool add)
 {
     if (add) {
-        appendContent(ContentData::create(quote));
+        appendContent(std::make_unique<QuoteContentData>(quote));
         return;
     }
 
-    rareNonInheritedData.access()->m_content = ContentData::create(quote);
+    rareNonInheritedData.access()->m_content = std::make_unique<QuoteContentData>(quote);
 }
+
+void RenderStyle::setContentAltText(const String& string)
+{
+    rareNonInheritedData.access()->m_altText = string;
     
-inline bool requireTransformOrigin(const Vector<RefPtr<TransformOperation> >& transformOperations, RenderStyle::ApplyTransformOrigin applyOrigin)
+    if (rareNonInheritedData.access()->m_content)
+        rareNonInheritedData.access()->m_content->setAltText(string);
+}
+
+const String& RenderStyle::contentAltText() const
+{
+    return rareNonInheritedData->m_altText;
+}
+
+inline bool requireTransformOrigin(const Vector<RefPtr<TransformOperation>>& transformOperations, RenderStyle::ApplyTransformOrigin applyOrigin)
 {
     // transform-origin brackets the transform with translate operations.
     // Optimize for the case where the only transform is a translation, since the transform-origin is irrelevant
@@ -932,7 +1004,7 @@ inline bool requireTransformOrigin(const Vector<RefPtr<TransformOperation> >& tr
 
     unsigned size = transformOperations.size();
     for (unsigned i = 0; i < size; ++i) {
-        TransformOperation::OperationType type = transformOperations[i]->getOperationType();
+        TransformOperation::OperationType type = transformOperations[i]->type();
         if (type != TransformOperation::TRANSLATE_X
             && type != TransformOperation::TRANSLATE_Y
             && type != TransformOperation::TRANSLATE 
@@ -949,7 +1021,7 @@ void RenderStyle::applyTransform(TransformationMatrix& transform, const LayoutSi
     // FIXME: when subpixel layout is supported (bug 71143) the body of this function could be replaced by
     // applyTransform(transform, FloatRect(FloatPoint(), borderBoxSize), applyOrigin);
     
-    const Vector<RefPtr<TransformOperation> >& transformOperations = rareNonInheritedData->m_transform->m_operations.operations();
+    const Vector<RefPtr<TransformOperation>>& transformOperations = rareNonInheritedData->m_transform->m_operations.operations();
     bool applyTransformOrigin = requireTransformOrigin(transformOperations, applyOrigin);
 
     if (applyTransformOrigin)
@@ -965,7 +1037,7 @@ void RenderStyle::applyTransform(TransformationMatrix& transform, const LayoutSi
 
 void RenderStyle::applyTransform(TransformationMatrix& transform, const FloatRect& boundingBox, ApplyTransformOrigin applyOrigin) const
 {
-    const Vector<RefPtr<TransformOperation> >& transformOperations = rareNonInheritedData->m_transform->m_operations.operations();
+    const Vector<RefPtr<TransformOperation>>& transformOperations = rareNonInheritedData->m_transform->m_operations.operations();
     bool applyTransformOrigin = requireTransformOrigin(transformOperations, applyOrigin);
     
     float offsetX = transformOriginX().type() == Percent ? boundingBox.x() : 0;
@@ -1049,22 +1121,22 @@ static float calcConstraintScaleFor(const IntRect& rect, const RoundedRect::Radi
     // top
     radiiSum = static_cast<unsigned>(radii.topLeft().width()) + static_cast<unsigned>(radii.topRight().width()); // Casts to avoid integer overflow.
     if (radiiSum > static_cast<unsigned>(rect.width()))
-        factor = min(static_cast<float>(rect.width()) / radiiSum, factor);
+        factor = std::min(static_cast<float>(rect.width()) / radiiSum, factor);
 
     // bottom
     radiiSum = static_cast<unsigned>(radii.bottomLeft().width()) + static_cast<unsigned>(radii.bottomRight().width());
     if (radiiSum > static_cast<unsigned>(rect.width()))
-        factor = min(static_cast<float>(rect.width()) / radiiSum, factor);
+        factor = std::min(static_cast<float>(rect.width()) / radiiSum, factor);
     
     // left
     radiiSum = static_cast<unsigned>(radii.topLeft().height()) + static_cast<unsigned>(radii.bottomLeft().height());
     if (radiiSum > static_cast<unsigned>(rect.height()))
-        factor = min(static_cast<float>(rect.height()) / radiiSum, factor);
+        factor = std::min(static_cast<float>(rect.height()) / radiiSum, factor);
     
     // right
     radiiSum = static_cast<unsigned>(radii.topRight().height()) + static_cast<unsigned>(radii.bottomRight().height());
     if (radiiSum > static_cast<unsigned>(rect.height()))
-        factor = min(static_cast<float>(rect.height()) / radiiSum, factor);
+        factor = std::min(static_cast<float>(rect.height()) / radiiSum, factor);
     
     ASSERT(factor <= 1);
     return factor;
@@ -1254,7 +1326,7 @@ void RenderStyle::adjustAnimations()
 
     // Get rid of empty animations and anything beyond them
     for (size_t i = 0; i < animationList->size(); ++i) {
-        if (animationList->animation(i)->isEmpty()) {
+        if (animationList->animation(i).isEmpty()) {
             animationList->resize(i);
             break;
         }
@@ -1277,7 +1349,7 @@ void RenderStyle::adjustTransitions()
 
     // Get rid of empty transitions and anything beyond them
     for (size_t i = 0; i < transitionList->size(); ++i) {
-        if (transitionList->animation(i)->isEmpty()) {
+        if (transitionList->animation(i).isEmpty()) {
             transitionList->resize(i);
             break;
         }
@@ -1295,7 +1367,7 @@ void RenderStyle::adjustTransitions()
     // but the lists tend to be very short, so it is probably ok
     for (size_t i = 0; i < transitionList->size(); ++i) {
         for (size_t j = i+1; j < transitionList->size(); ++j) {
-            if (transitionList->animation(i)->property() == transitionList->animation(j)->property()) {
+            if (transitionList->animation(i).property() == transitionList->animation(j).property()) {
                 // toss i
                 transitionList->remove(i);
                 j = i;
@@ -1322,9 +1394,9 @@ const Animation* RenderStyle::transitionForProperty(CSSPropertyID property) cons
 {
     if (transitions()) {
         for (size_t i = 0; i < transitions()->size(); ++i) {
-            const Animation* p = transitions()->animation(i);
-            if (p->animationMode() == Animation::AnimateAll || p->property() == property) {
-                return p;
+            const Animation& p = transitions()->animation(i);
+            if (p.animationMode() == Animation::AnimateAll || p.property() == property) {
+                return &p;
             }
         }
     }
@@ -1338,8 +1410,8 @@ float RenderStyle::specifiedFontSize() const { return fontDescription().specifie
 float RenderStyle::computedFontSize() const { return fontDescription().computedSize(); }
 int RenderStyle::fontSize() const { return inherited->font.pixelSize(); }
 
-int RenderStyle::wordSpacing() const { return inherited->font.wordSpacing(); }
-int RenderStyle::letterSpacing() const { return inherited->font.letterSpacing(); }
+const Length& RenderStyle::wordSpacing() const { return rareInheritedData->wordSpacing; }
+float RenderStyle::letterSpacing() const { return inherited->font.letterSpacing(); }
 
 bool RenderStyle::setFontDescription(const FontDescription& v)
 {
@@ -1350,7 +1422,13 @@ bool RenderStyle::setFontDescription(const FontDescription& v)
     return false;
 }
 
-Length RenderStyle::specifiedLineHeight() const { return inherited->line_height; }
+#if ENABLE(IOS_TEXT_AUTOSIZING)
+const Length& RenderStyle::specifiedLineHeight() const { return inherited->specifiedLineHeight; }
+void RenderStyle::setSpecifiedLineHeight(Length v) { SET_VAR(inherited, specifiedLineHeight, v); }
+#else
+const Length& RenderStyle::specifiedLineHeight() const { return inherited->line_height; }
+#endif
+
 Length RenderStyle::lineHeight() const
 {
     const Length& lh = inherited->line_height;
@@ -1384,8 +1462,29 @@ int RenderStyle::computedLineHeight(RenderView* renderView) const
     return lh.value();
 }
 
-void RenderStyle::setWordSpacing(int v) { inherited.access()->font.setWordSpacing(v); }
-void RenderStyle::setLetterSpacing(int v) { inherited.access()->font.setLetterSpacing(v); }
+void RenderStyle::setWordSpacing(Length v)
+{
+    float fontWordSpacing;
+    switch (v.type()) {
+    case Auto:
+        fontWordSpacing = 0;
+        FALLTHROUGH;
+    case Percent:
+        fontWordSpacing = v.getFloatValue() * font().spaceWidth() / 100;
+        break;
+    case Fixed:
+        fontWordSpacing = v.getFloatValue();
+        break;
+    default:
+        ASSERT_NOT_REACHED();
+        fontWordSpacing = 0;
+        break;
+    }
+    inherited.access()->font.setWordSpacing(fontWordSpacing);
+    rareInheritedData.access()->wordSpacing = std::move(v);
+}
+
+void RenderStyle::setLetterSpacing(float v) { inherited.access()->font.setLetterSpacing(v); }
 
 void RenderStyle::setFontSize(float size)
 {
@@ -1396,7 +1495,7 @@ void RenderStyle::setFontSize(float size)
     if (!std::isfinite(size) || size < 0)
         size = 0;
     else
-        size = min(maximumAllowedFontSize, size);
+        size = std::min(maximumAllowedFontSize, size);
 
     FontSelector* currentFontSelector = font().fontSelector();
     FontDescription desc(fontDescription());
@@ -1427,10 +1526,10 @@ void RenderStyle::getShadowExtent(const ShadowData* shadow, LayoutUnit &top, Lay
             continue;
 
         int extentAndSpread = shadow->paintingExtent() + shadow->spread();
-        top = min<LayoutUnit>(top, shadow->y() - extentAndSpread);
-        right = max<LayoutUnit>(right, shadow->x() + extentAndSpread);
-        bottom = max<LayoutUnit>(bottom, shadow->y() + extentAndSpread);
-        left = min<LayoutUnit>(left, shadow->x() - extentAndSpread);
+        top = std::min<LayoutUnit>(top, shadow->y() - extentAndSpread);
+        right = std::max<LayoutUnit>(right, shadow->x() + extentAndSpread);
+        bottom = std::max<LayoutUnit>(bottom, shadow->y() + extentAndSpread);
+        left = std::min<LayoutUnit>(left, shadow->x() - extentAndSpread);
     }
 }
 
@@ -1446,10 +1545,10 @@ LayoutBoxExtent RenderStyle::getShadowInsetExtent(const ShadowData* shadow) cons
             continue;
 
         int extentAndSpread = shadow->paintingExtent() + shadow->spread();
-        top = max<LayoutUnit>(top, shadow->y() + extentAndSpread);
-        right = min<LayoutUnit>(right, shadow->x() - extentAndSpread);
-        bottom = min<LayoutUnit>(bottom, shadow->y() - extentAndSpread);
-        left = max<LayoutUnit>(left, shadow->x() + extentAndSpread);
+        top = std::max<LayoutUnit>(top, shadow->y() + extentAndSpread);
+        right = std::min<LayoutUnit>(right, shadow->x() - extentAndSpread);
+        bottom = std::min<LayoutUnit>(bottom, shadow->y() - extentAndSpread);
+        left = std::max<LayoutUnit>(left, shadow->x() + extentAndSpread);
     }
 
     return LayoutBoxExtent(top, right, bottom, left);
@@ -1465,8 +1564,8 @@ void RenderStyle::getShadowHorizontalExtent(const ShadowData* shadow, LayoutUnit
             continue;
 
         int extentAndSpread = shadow->paintingExtent() + shadow->spread();
-        left = min<LayoutUnit>(left, shadow->x() - extentAndSpread);
-        right = max<LayoutUnit>(right, shadow->x() + extentAndSpread);
+        left = std::min<LayoutUnit>(left, shadow->x() - extentAndSpread);
+        right = std::max<LayoutUnit>(right, shadow->x() + extentAndSpread);
     }
 }
 
@@ -1480,8 +1579,8 @@ void RenderStyle::getShadowVerticalExtent(const ShadowData* shadow, LayoutUnit &
             continue;
 
         int extentAndSpread = shadow->paintingExtent() + shadow->spread();
-        top = min<LayoutUnit>(top, shadow->y() - extentAndSpread);
-        bottom = max<LayoutUnit>(bottom, shadow->y() + extentAndSpread);
+        top = std::min<LayoutUnit>(top, shadow->y() - extentAndSpread);
+        bottom = std::max<LayoutUnit>(bottom, shadow->y() + extentAndSpread);
     }
 }
 
@@ -1517,11 +1616,9 @@ Color RenderStyle::colorIncludingFallback(int colorProperty, bool visitedLink) c
     case CSSPropertyWebkitColumnRuleColor:
         result = visitedLink ? visitedLinkColumnRuleColor() : columnRuleColor();
         break;
-#if ENABLE(CSS3_TEXT)
     case CSSPropertyWebkitTextDecorationColor:
         // Text decoration color fallback is handled in RenderObject::decorationColor.
         return visitedLink ? visitedLinkTextDecorationColor() : textDecorationColor();
-#endif // CSS3_TEXT
     case CSSPropertyWebkitTextEmphasisColor:
         result = visitedLink ? visitedLinkTextEmphasisColor() : textEmphasisColor();
         break;
@@ -1553,11 +1650,9 @@ Color RenderStyle::visitedDependentColor(int colorProperty) const
 
     Color visitedColor = colorIncludingFallback(colorProperty, true);
 
-#if ENABLE(CSS3_TEXT)
     // Text decoration color validity is preserved (checked in RenderObject::decorationColor).
     if (colorProperty == CSSPropertyWebkitTextDecorationColor)
         return visitedColor;
-#endif // CSS3_TEXT
 
     // FIXME: Technically someone could explicitly specify the color transparent, but for now we'll just
     // assume that if the background color is transparent that it wasn't set. Note that it's weird that
@@ -1787,14 +1882,6 @@ void RenderStyle::setBorderImageOutset(LengthBox outset)
     surround.access()->border.m_image.setOutset(outset);
 }
 
-#if ENABLE(CSS_SHAPES)
-ShapeValue* RenderStyle::initialShapeInside()
-{
-    static ShapeValue* outsideValue = ShapeValue::createOutsideValue().leakRef();
-    return outsideValue;
-}
-#endif
-
 void RenderStyle::setColumnStylesFromPaginationMode(const Pagination::Mode& paginationMode)
 {
     if (paginationMode == Pagination::Unpaginated)
