@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2012 Google Inc. All rights reserved.
+ * Copyright (C) 2013 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,10 +27,13 @@
 #ifndef Internals_h
 #define Internals_h
 
+#include "CSSComputedStyleDeclaration.h"
 #include "ContextDestructionObserver.h"
 #include "ExceptionCodePlaceholder.h"
 #include "NodeList.h"
-#include <wtf/ArrayBuffer.h>
+#include <bindings/ScriptValue.h>
+#include <runtime/ArrayBuffer.h>
+#include <runtime/Float32Array.h>
 #include <wtf/PassRefPtr.h>
 #include <wtf/RefCounted.h>
 #include <wtf/text/WTFString.h>
@@ -46,15 +50,15 @@ class Element;
 class Frame;
 class InspectorFrontendChannelDummy;
 class InternalSettings;
+class MemoryInfo;
 class Node;
 class Page;
-class PagePopupController;
 class Range;
 class ScriptExecutionContext;
 class ShadowRoot;
-class WebKitPoint;
 class MallocStatistics;
 class SerializedScriptValue;
+class TimeRanges;
 class TypeConversions;
 
 typedef int ExceptionCode;
@@ -74,7 +78,7 @@ public:
     bool isPreloaded(const String& url);
     bool isLoadingFromMemoryCache(const String& url);
 
-    size_t numberOfScopedHTMLStyleChildren(const Node*, ExceptionCode&) const;
+    PassRefPtr<CSSComputedStyleDeclaration> computedStyleIncludingVisitedInfo(Node*, ExceptionCode&) const;
 
 #if ENABLE(SHADOW_DOM)
     typedef ShadowRoot ShadowRootIfShadowDOMEnabledOrNode;
@@ -84,26 +88,26 @@ public:
     ShadowRootIfShadowDOMEnabledOrNode* ensureShadowRoot(Element* host, ExceptionCode&);
     ShadowRootIfShadowDOMEnabledOrNode* createShadowRoot(Element* host, ExceptionCode&);
     ShadowRootIfShadowDOMEnabledOrNode* shadowRoot(Element* host, ExceptionCode&);
-    ShadowRootIfShadowDOMEnabledOrNode* youngestShadowRoot(Element* host, ExceptionCode&);
-    ShadowRootIfShadowDOMEnabledOrNode* oldestShadowRoot(Element* host, ExceptionCode&);
-    ShadowRootIfShadowDOMEnabledOrNode* youngerShadowRoot(Node* shadow, ExceptionCode&);
-    ShadowRootIfShadowDOMEnabledOrNode* olderShadowRoot(Node* shadow, ExceptionCode&);
     String shadowRootType(const Node*, ExceptionCode&) const;
-    bool hasShadowInsertionPoint(const Node*, ExceptionCode&) const;
-    bool hasContentElement(const Node*, ExceptionCode&) const;
-    size_t countElementShadow(const Node*, ExceptionCode&) const;
     Element* includerFor(Node*, ExceptionCode&);
     String shadowPseudoId(Element*, ExceptionCode&);
     void setShadowPseudoId(Element*, const String&, ExceptionCode&);
 
+    // Spatial Navigation testing.
+    unsigned lastSpatialNavigationCandidateCount(ExceptionCode&) const;
+
+    // CSS Animation testing.
     unsigned numberOfActiveAnimations() const;
-    void suspendAnimations(Document*, ExceptionCode&) const;
-    void resumeAnimations(Document*, ExceptionCode&) const;
+    bool animationsAreSuspended(ExceptionCode&) const;
+    void suspendAnimations(ExceptionCode&) const;
+    void resumeAnimations(ExceptionCode&) const;
+    bool pauseAnimationAtTimeOnElement(const String& animationName, double pauseTime, Element*, ExceptionCode&);
     bool pauseAnimationAtTimeOnPseudoElement(const String& animationName, double pauseTime, Element*, const String& pseudoId, ExceptionCode&);
+
+    // CSS Transition testing.
+    bool pauseTransitionAtTimeOnElement(const String& propertyName, double pauseTime, Element*, ExceptionCode&);
     bool pauseTransitionAtTimeOnPseudoElement(const String& property, double pauseTime, Element*, const String& pseudoId, ExceptionCode&);
 
-    PassRefPtr<Element> createContentElement(ExceptionCode&);
-    bool isValidContentSelect(Element* insertionPoint, ExceptionCode&);
     Node* treeScopeRootNode(Node*, ExceptionCode&);
     Node* parentTreeScope(Node*, ExceptionCode&);
     bool hasSelectorForIdInShadow(Element* host, const String& idValue, ExceptionCode&);
@@ -113,125 +117,118 @@ public:
 
     bool attached(Node*, ExceptionCode&);
 
-    // FIXME: Rename these functions if walker is prefered.
-    Node* nextSiblingByWalker(Node*, ExceptionCode&);
-    Node* firstChildByWalker(Node*, ExceptionCode&);
-    Node* lastChildByWalker(Node*, ExceptionCode&);
-    Node* nextNodeByWalker(Node*, ExceptionCode&);
-    Node* previousNodeByWalker(Node*, ExceptionCode&);
-
     String visiblePlaceholder(Element*);
 #if ENABLE(INPUT_TYPE_COLOR)
     void selectColorInColorChooser(Element*, const String& colorValue);
 #endif
     Vector<String> formControlStateOfPreviousHistoryItem(ExceptionCode&);
     void setFormControlStateOfPreviousHistoryItem(const Vector<String>&, ExceptionCode&);
-    void setEnableMockPagePopup(bool, ExceptionCode&);
-#if ENABLE(PAGE_POPUP)
-    PassRefPtr<PagePopupController> pagePopupController();
-#endif
 
     PassRefPtr<ClientRect> absoluteCaretBounds(ExceptionCode&);
 
     PassRefPtr<ClientRect> boundingBox(Element*, ExceptionCode&);
 
-    PassRefPtr<ClientRectList> inspectorHighlightRects(Document*, ExceptionCode&);
-
-    void setBackgroundBlurOnNode(Node*, int blurLength, ExceptionCode&);
+    PassRefPtr<ClientRectList> inspectorHighlightRects(ExceptionCode&);
+    String inspectorHighlightObject(ExceptionCode&);
 
     unsigned markerCountForNode(Node*, const String&, ExceptionCode&);
     PassRefPtr<Range> markerRangeForNode(Node*, const String& markerType, unsigned index, ExceptionCode&);
     String markerDescriptionForNode(Node*, const String& markerType, unsigned index, ExceptionCode&);
     void addTextMatchMarker(const Range*, bool isActive);
 
-    void setScrollViewPosition(Document*, long x, long y, ExceptionCode&);
-    void setPagination(Document* document, const String& mode, int gap, ExceptionCode& ec) { setPagination(document, mode, gap, 0, ec); }
-    void setPagination(Document*, const String& mode, int gap, int pageLength, ExceptionCode&);
-    String configurationForViewport(Document*, float devicePixelRatio, int deviceWidth, int deviceHeight, int availableWidth, int availableHeight, ExceptionCode&);
+    void setScrollViewPosition(long x, long y, ExceptionCode&);
+    void setPagination(const String& mode, int gap, ExceptionCode& ec) { setPagination(mode, gap, 0, ec); }
+    void setPagination(const String& mode, int gap, int pageLength, ExceptionCode&);
+    String configurationForViewport(float devicePixelRatio, int deviceWidth, int deviceHeight, int availableWidth, int availableHeight, ExceptionCode&);
 
     bool wasLastChangeUserEdit(Element* textField, ExceptionCode&);
+    bool elementShouldAutoComplete(Element* inputElement, ExceptionCode&);
     String suggestedValue(Element* inputElement, ExceptionCode&);
     void setSuggestedValue(Element* inputElement, const String&, ExceptionCode&);
     void setEditingValue(Element* inputElement, const String&, ExceptionCode&);
+    void setAutofilled(Element*, bool enabled, ExceptionCode&);
     void scrollElementToRect(Element*, long x, long y, long w, long h, ExceptionCode&);
 
-    void paintControlTints(Document*, ExceptionCode&);
+    void paintControlTints(ExceptionCode&);
 
     PassRefPtr<Range> rangeFromLocationAndLength(Element* scope, int rangeLocation, int rangeLength, ExceptionCode&);
     unsigned locationFromRange(Element* scope, const Range*, ExceptionCode&);
     unsigned lengthFromRange(Element* scope, const Range*, ExceptionCode&);
     String rangeAsText(const Range*, ExceptionCode&);
 
-    void setDelegatesScrolling(bool enabled, Document*, ExceptionCode&);
-#if ENABLE(TOUCH_ADJUSTMENT)
-    PassRefPtr<WebKitPoint> touchPositionAdjustedToBestClickableNode(long x, long y, long width, long height, Document*, ExceptionCode&);
-    Node* touchNodeAdjustedToBestClickableNode(long x, long y, long width, long height, Document*, ExceptionCode&);
-    PassRefPtr<WebKitPoint> touchPositionAdjustedToBestContextMenuNode(long x, long y, long width, long height, Document*, ExceptionCode&);
-    Node* touchNodeAdjustedToBestContextMenuNode(long x, long y, long width, long height, Document*, ExceptionCode&);
-    PassRefPtr<ClientRect> bestZoomableAreaForTouchPoint(long x, long y, long width, long height, Document*, ExceptionCode&);
-#endif
+    void setDelegatesScrolling(bool enabled, ExceptionCode&);
 
-    int lastSpellCheckRequestSequence(Document*, ExceptionCode&);
-    int lastSpellCheckProcessedSequence(Document*, ExceptionCode&);
+    int lastSpellCheckRequestSequence(ExceptionCode&);
+    int lastSpellCheckProcessedSequence(ExceptionCode&);
 
     Vector<String> userPreferredLanguages() const;
     void setUserPreferredLanguages(const Vector<String>&);
 
-    unsigned wheelEventHandlerCount(Document*, ExceptionCode&);
-    unsigned touchEventHandlerCount(Document*, ExceptionCode&);
-#if ENABLE(TOUCH_EVENT_TRACKING)
-    PassRefPtr<ClientRectList> touchEventTargetClientRects(Document*, ExceptionCode&);
-#endif
+    unsigned wheelEventHandlerCount(ExceptionCode&);
+    unsigned touchEventHandlerCount(ExceptionCode&);
 
     PassRefPtr<NodeList> nodesFromRect(Document*, int x, int y, unsigned topPadding, unsigned rightPadding,
-        unsigned bottomPadding, unsigned leftPadding, bool ignoreClipping, bool allowShadowContent, ExceptionCode&) const;
+        unsigned bottomPadding, unsigned leftPadding, bool ignoreClipping, bool allowShadowContent, bool allowChildFrameContent, ExceptionCode&) const;
 
     void emitInspectorDidBeginFrame();
     void emitInspectorDidCancelFrame();
 
-    bool hasSpellingMarker(Document*, int from, int length, ExceptionCode&);
-    bool hasGrammarMarker(Document*, int from, int length, ExceptionCode&);
-    bool hasAutocorrectedMarker(Document*, int from, int length, ExceptionCode&);
+    String parserMetaData(Deprecated::ScriptValue = Deprecated::ScriptValue());
 
-    unsigned numberOfScrollableAreas(Document*, ExceptionCode&);
+    bool hasSpellingMarker(int from, int length, ExceptionCode&);
+    bool hasGrammarMarker(int from, int length, ExceptionCode&);
+    bool hasAutocorrectedMarker(int from, int length, ExceptionCode&);
+    void setContinuousSpellCheckingEnabled(bool enabled, ExceptionCode&);
+    void setAutomaticQuoteSubstitutionEnabled(bool enabled, ExceptionCode&);
+    void setAutomaticLinkDetectionEnabled(bool enabled, ExceptionCode&);
+    void setAutomaticDashSubstitutionEnabled(bool enabled, ExceptionCode&);
+    void setAutomaticTextReplacementEnabled(bool enabled, ExceptionCode&);
+    void setAutomaticSpellingCorrectionEnabled(bool enabled, ExceptionCode&);
 
-    bool isPageBoxVisible(Document*, int pageNumber, ExceptionCode&);
+    bool isOverwriteModeEnabled(ExceptionCode&);
+    void toggleOverwriteModeEnabled(ExceptionCode&);
+
+    unsigned numberOfScrollableAreas(ExceptionCode&);
+
+    bool isPageBoxVisible(int pageNumber, ExceptionCode&);
 
     static const char* internalsId;
 
     InternalSettings* settings() const;
+    unsigned workerThreadCount() const;
 
-    void setBatteryStatus(Document*, const String& eventType, bool charging, double chargingTime, double dischargingTime, double level, ExceptionCode&);
+    void setBatteryStatus(const String& eventType, bool charging, double chargingTime, double dischargingTime, double level, ExceptionCode&);
 
-    void setNetworkInformation(Document*, const String& eventType, double bandwidth, bool metered, ExceptionCode&);
+    void setNetworkInformation(const String& eventType, double bandwidth, bool metered, ExceptionCode&);
 
-    void setDeviceProximity(Document*, const String& eventType, double value, double min, double max, ExceptionCode&);
+    void setDeviceProximity(const String& eventType, double value, double min, double max, ExceptionCode&);
 
     enum {
         // Values need to be kept in sync with Internals.idl.
         LAYER_TREE_INCLUDES_VISIBLE_RECTS = 1,
         LAYER_TREE_INCLUDES_TILE_CACHES = 2,
-        LAYER_TREE_INCLUDES_REPAINT_RECTS = 4
-        
+        LAYER_TREE_INCLUDES_REPAINT_RECTS = 4,
+        LAYER_TREE_INCLUDES_PAINTING_PHASES = 8,
+        LAYER_TREE_INCLUDES_CONTENT_LAYERS = 16
     };
     String layerTreeAsText(Document*, unsigned flags, ExceptionCode&) const;
     String layerTreeAsText(Document*, ExceptionCode&) const;
-    String repaintRectsAsText(Document*, ExceptionCode&) const;
-    String scrollingStateTreeAsText(Document*, ExceptionCode&) const;
-    String mainThreadScrollingReasons(Document*, ExceptionCode&) const;
-    PassRefPtr<ClientRectList> nonFastScrollableRects(Document*, ExceptionCode&) const;
+    String repaintRectsAsText(ExceptionCode&) const;
+    String scrollingStateTreeAsText(ExceptionCode&) const;
+    String mainThreadScrollingReasons(ExceptionCode&) const;
+    PassRefPtr<ClientRectList> nonFastScrollableRects(ExceptionCode&) const;
 
-    void garbageCollectDocumentResources(Document*, ExceptionCode&) const;
+    void garbageCollectDocumentResources(ExceptionCode&) const;
 
     void allowRoundingHacks() const;
 
-    void insertAuthorCSS(Document*, const String&) const;
-    void insertUserCSS(Document*, const String&) const;
+    void insertAuthorCSS(const String&, ExceptionCode&) const;
+    void insertUserCSS(const String&, ExceptionCode&) const;
 
 #if ENABLE(INSPECTOR)
     unsigned numberOfLiveNodes() const;
     unsigned numberOfLiveDocuments() const;
-    Vector<String> consoleMessageArgumentCounts(Document*) const;
+    Vector<String> consoleMessageArgumentCounts() const;
     PassRefPtr<DOMWindow> openDummyInspectorFrontend(const String& url);
     void closeDummyInspectorFrontend();
     void setInspectorResourcesDataSizeLimits(int maximumResourcesContentSize, int maximumSingleResourceContentSize, ExceptionCode&);
@@ -241,7 +238,8 @@ public:
     String counterValue(Element*);
 
     int pageNumber(Element*, float pageWidth = 800, float pageHeight = 600);
-    Vector<String> iconURLs(Document*) const;
+    Vector<String> shortcutIconURLs() const;
+    Vector<String> allIconURLs() const;
 
     int numberOfPages(float pageWidthInPixels = 800, float pageHeightInPixels = 600);
     String pageProperty(String, int, ExceptionCode& = ASSERT_NO_EXCEPTION) const;
@@ -249,35 +247,92 @@ public:
 
     void setPageScaleFactor(float scaleFactor, int x, int y, ExceptionCode&);
 
+    void setHeaderHeight(float);
+    void setFooterHeight(float);
+
 #if ENABLE(FULLSCREEN_API)
-    void webkitWillEnterFullScreenForElement(Document*, Element*);
-    void webkitDidEnterFullScreenForElement(Document*, Element*);
-    void webkitWillExitFullScreenForElement(Document*, Element*);
-    void webkitDidExitFullScreenForElement(Document*, Element*);
+    void webkitWillEnterFullScreenForElement(Element*);
+    void webkitDidEnterFullScreenForElement(Element*);
+    void webkitWillExitFullScreenForElement(Element*);
+    void webkitDidExitFullScreenForElement(Element*);
 #endif
 
+    void setApplicationCacheOriginQuota(unsigned long long);
+
     void registerURLSchemeAsBypassingContentSecurityPolicy(const String& scheme);
     void removeURLSchemeRegisteredAsBypassingContentSecurityPolicy(const String& scheme);
 
     PassRefPtr<MallocStatistics> mallocStatistics() const;
     PassRefPtr<TypeConversions> typeConversions() const;
+    PassRefPtr<MemoryInfo> memoryInfo() const;
 
     Vector<String> getReferencedFilePaths() const;
 
-    void startTrackingRepaints(Document*, ExceptionCode&);
-    void stopTrackingRepaints(Document*, ExceptionCode&);
+    void startTrackingRepaints(ExceptionCode&);
+    void stopTrackingRepaints(ExceptionCode&);
 
     PassRefPtr<ArrayBuffer> serializeObject(PassRefPtr<SerializedScriptValue>) const;
     PassRefPtr<SerializedScriptValue> deserializeBuffer(PassRefPtr<ArrayBuffer>) const;
 
     void setUsesOverlayScrollbars(bool enabled);
 
-    String getCurrentCursorInfo(Document*, ExceptionCode&);
+    String getCurrentCursorInfo(ExceptionCode&);
+
+    String markerTextForListItem(Element*, ExceptionCode&);
+
+    void forceReload(bool endToEnd);
+
+#if ENABLE(ENCRYPTED_MEDIA_V2)
+    void initializeMockCDM();
+#endif
+
+#if ENABLE(SPEECH_SYNTHESIS)
+    void enableMockSpeechSynthesizer();
+#endif
+
+#if ENABLE(MEDIA_STREAM)
+    void enableMockRTCPeerConnectionHandler();
+#endif
+
+    String getImageSourceURL(Element*, ExceptionCode&);
+
+#if ENABLE(VIDEO)
+    void simulateAudioInterruption(Node*);
+#endif
+
+    bool isSelectPopupVisible(Node*);
+
+    String captionsStyleSheetOverride(ExceptionCode&);
+    void setCaptionsStyleSheetOverride(const String&, ExceptionCode&);
+    void setPrimaryAudioTrackLanguageOverride(const String&, ExceptionCode&);
+    void setCaptionDisplayMode(const String&, ExceptionCode&);
+
+#if ENABLE(VIDEO)
+    PassRefPtr<TimeRanges> createTimeRanges(Float32Array* startTimes, Float32Array* endTimes);
+    double closestTimeToTimeRanges(double time, TimeRanges*);
+#endif
+
+    PassRefPtr<ClientRect> selectionBounds(ExceptionCode&);
+
+#if ENABLE(VIBRATION)
+    bool isVibrating();
+#endif
+
+    bool isPluginUnavailabilityIndicatorObscured(Element*, ExceptionCode&);
+
+#if ENABLE(MEDIA_SOURCE)
+    void initializeMockMediaSource();
+#endif
+
+    void beginMediaSessionInterruption();
+    void endMediaSessionInterruption(const String&);
+    void setMediaSessionRestrictions(const String& mediaType, const String& restrictions, ExceptionCode& ec);
 
 private:
     explicit Internals(Document*);
     Document* contextDocument() const;
     Frame* frame() const;
+    Vector<String> iconURLs(Document*, int iconTypesMask) const;
 
     DocumentMarker* markerAt(Node*, const String& markerType, unsigned index, ExceptionCode&);
 #if ENABLE(INSPECTOR)
