@@ -45,7 +45,7 @@
 #include "Icon.h"
 #include "InspectorController.h"
 #include "IntRect.h"
-#include "KURL.h"
+#include "MainFrame.h"
 #include "NavigationAction.h"
 #include "NotImplemented.h"
 #include "PopupMenuClient.h"
@@ -53,6 +53,7 @@
 #include "RefPtrCairo.h"
 #include "SearchPopupMenuGtk.h"
 #include "SecurityOrigin.h"
+#include "URL.h"
 #include "WebKitDOMHTMLElementPrivate.h"
 #include "WindowFeatures.h"
 #include "webkitfilechooserrequestprivate.h"
@@ -75,7 +76,7 @@
 #include <wtf/text/CString.h>
 #include <wtf/text/WTFString.h>
 
-#ifdef GDK_WINDOWING_X11
+#if PLATFORM(X11) && defined(GDK_WINDOWING_X11)
 #define Font XFont
 #define Cursor XCursor
 #define Region XRegion
@@ -91,11 +92,7 @@
 #include "DatabaseManager.h"
 #endif
 
-#if ENABLE(VIDEO) && USE(NATIVE_FULLSCREEN_VIDEO)
-#include "HTMLMediaElement.h"
-#endif
-
-#ifdef GDK_WINDOWING_X11
+#if PLATFORM(X11) && defined(GDK_WINDOWING_X11)
 #include "WidgetBackingStoreGtkX11.h"
 #endif
 #include "WidgetBackingStoreCairo.h"
@@ -106,7 +103,7 @@ namespace WebKit {
 
 static PassOwnPtr<WidgetBackingStore> createBackingStore(GtkWidget* widget, const IntSize& size)
 {
-#ifdef GDK_WINDOWING_X11
+#if PLATFORM(X11) && defined(GDK_WINDOWING_X11)
     GdkDisplay* display = gdk_display_manager_get_default_display(gdk_display_manager_get());
     if (GDK_IS_X11_DISPLAY(display))
         return WebCore::WidgetBackingStoreGtkX11::create(widget, size);
@@ -203,6 +200,11 @@ Page* ChromeClient::createWindow(Frame* frame, const FrameLoadRequest& frameLoad
 {
     WebKitWebView* webView = 0;
 
+#if ENABLE(FULLSCREEN_API)
+    if (frame->document() && frame->document()->webkitCurrentFullScreenElement())
+        frame->document()->webkitCancelFullScreen();
+#endif
+
     g_signal_emit_by_name(m_webView, "create-web-view", kit(frame), &webView);
 
     if (!webView)
@@ -329,7 +331,7 @@ void ChromeClient::closeWindowSoon()
     // onload handler, it will need to call FrameLoaderClient::dispatchDidHandleOnloadEvents.
     // Instead of firing the close-web-view signal now, fire it after the caller finishes.
     // This seems to match the Mac/Windows port behavior.
-    m_closeSoonTimer = g_timeout_add(0, reinterpret_cast<GSourceFunc>(emitCloseWebViewSignalLater), m_webView);
+    m_closeSoonTimer = g_idle_add_full(G_PRIORITY_DEFAULT, reinterpret_cast<GSourceFunc>(emitCloseWebViewSignalLater), m_webView, 0);
 }
 
 bool ChromeClient::canTakeFocus(FocusDirection)
@@ -513,7 +515,7 @@ void ChromeClient::widgetSizeChanged(const IntSize& oldWidgetSize, IntSize newSi
     // WebCore timers by default have a lower priority which leads to more artifacts when opaque
     // resize is on, thus we use g_timeout_add here to force a higher timeout priority.
     if (!m_repaintSoonSourceId)
-        m_repaintSoonSourceId = g_timeout_add(0, reinterpret_cast<GSourceFunc>(repaintEverythingSoonTimeout), this);
+        m_repaintSoonSourceId = g_idle_add_full(G_PRIORITY_DEFAULT, reinterpret_cast<GSourceFunc>(repaintEverythingSoonTimeout), this, 0);
 }
 
 static void coalesceRectsIfPossible(const IntRect& clipRect, Vector<IntRect>& rects)
@@ -565,7 +567,7 @@ static void paintWebView(WebKitWebView* webView, Frame* frame, const Region& dir
 
     gc.save();
     gc.clip(dirtyRegion.bounds());
-    frame->page()->inspectorController()->drawHighlight(gc);
+    frame->page()->inspectorController().drawHighlight(gc);
     gc.restore();
 }
 
@@ -623,8 +625,8 @@ void ChromeClient::paint(WebCore::Timer<ChromeClient>*)
     // We update the IM context window location here, because we want it to be
     // synced with cursor movement. For instance, a text field can move without
     // the selection changing.
-    Frame* focusedFrame = core(m_webView)->focusController().focusedOrMainFrame();
-    if (focusedFrame && focusedFrame->editor().canEdit())
+    Frame& focusedFrame = core(m_webView)->focusController().focusedOrMainFrame();
+    if (focusedFrame.editor().canEdit())
         m_webView->priv->imFilter.setCursorRect(frame.selection().absoluteCaretBounds());
 }
 
@@ -798,7 +800,7 @@ void ChromeClient::mouseDidMoveOverElement(const HitTestResult& hit, unsigned mo
     // check if the element is a link...
     bool isLink = hit.isLiveLink();
     if (isLink) {
-        KURL url = hit.absoluteLinkURL();
+        URL url = hit.absoluteLinkURL();
         if (!url.isEmpty() && url != m_hoveredLinkURL) {
             TextDirection dir;
             CString titleString = hit.title(dir).utf8();
@@ -808,11 +810,11 @@ void ChromeClient::mouseDidMoveOverElement(const HitTestResult& hit, unsigned mo
         }
     } else if (!isLink && !m_hoveredLinkURL.isEmpty()) {
         g_signal_emit_by_name(m_webView, "hovering-over-link", 0, 0);
-        m_hoveredLinkURL = KURL();
+        m_hoveredLinkURL = URL();
     }
 
     if (Node* node = hit.innerNonSharedNode()) {
-        Frame* frame = node->document()->frame();
+        Frame* frame = node->document().frame();
         FrameView* view = frame ? frame->view() : 0;
         m_webView->priv->tooltipArea = view ? view->contentsToWindow(node->pixelSnappedBoundingBox()) : IntRect();
     } else
@@ -924,39 +926,6 @@ PassRefPtr<WebCore::SearchPopupMenu> ChromeClient::createSearchPopupMenu(WebCore
     return adoptRef(new SearchPopupMenuGtk(client));
 }
 
-#if ENABLE(VIDEO) && USE(NATIVE_FULLSCREEN_VIDEO)
-bool ChromeClient::supportsFullscreenForNode(const Node* node)
-{
-    return node->hasTagName(HTMLNames::videoTag);
-}
-
-void ChromeClient::enterFullscreenForNode(Node* node)
-{
-    if (!node)
-        return;
-
-    HTMLElement* element = static_cast<HTMLElement*>(node);
-    if (element && element->isMediaElement()) {
-        HTMLMediaElement* mediaElement = toHTMLMediaElement(element);
-        if (mediaElement->player() && mediaElement->player()->canEnterFullscreen())
-            mediaElement->player()->enterFullscreen();
-    }
-}
-
-void ChromeClient::exitFullscreenForNode(Node* node)
-{
-    if (!node)
-        return;
-
-    HTMLElement* element = static_cast<HTMLElement*>(node);
-    if (element && element->isMediaElement()) {
-        HTMLMediaElement* mediaElement = toHTMLMediaElement(element);
-        if (mediaElement->player())
-            mediaElement->player()->exitFullscreen();
-    }
-}
-#endif
-
 #if ENABLE(FULLSCREEN_API)
 bool ChromeClient::supportsFullScreenForElement(const WebCore::Element* element, bool withKeyboard)
 {
@@ -981,7 +950,7 @@ static gboolean onFullscreenGtkKeyPressEvent(GtkWidget* widget, GdkEventKey* eve
 void ChromeClient::cancelFullScreen()
 {
     ASSERT(m_fullScreenElement);
-    m_fullScreenElement->document()->webkitCancelFullScreen();
+    m_fullScreenElement->document().webkitCancelFullScreen();
 }
 
 void ChromeClient::enterFullScreenForElement(WebCore::Element* element)
@@ -992,19 +961,6 @@ void ChromeClient::enterFullScreenForElement(WebCore::Element* element)
     if (returnValue)
         return;
 
-#if ENABLE(VIDEO) && USE(NATIVE_FULLSCREEN_VIDEO)
-    if (element && element->isMediaElement()) {
-        HTMLMediaElement* mediaElement = toHTMLMediaElement(element);
-        if (mediaElement->player() && mediaElement->player()->canEnterFullscreen()) {
-            element->document()->webkitWillEnterFullScreenForElement(element);
-            mediaElement->player()->enterFullscreen();
-            m_fullScreenElement = element;
-            element->document()->webkitDidEnterFullScreenForElement(element);
-        }
-        return;
-    }
-#endif
-
     GtkWidget* window = gtk_widget_get_toplevel(GTK_WIDGET(m_webView));
     if (!widgetIsOnscreenToplevelWindow(window))
         return;
@@ -1013,10 +969,10 @@ void ChromeClient::enterFullScreenForElement(WebCore::Element* element)
 
     m_fullScreenElement = element;
 
-    element->document()->webkitWillEnterFullScreenForElement(element);
+    element->document().webkitWillEnterFullScreenForElement(element);
     m_adjustmentWatcher.disableAllScrollbars();
     gtk_window_fullscreen(GTK_WINDOW(window));
-    element->document()->webkitDidEnterFullScreenForElement(element);
+    element->document().webkitDidEnterFullScreenForElement(element);
 }
 
 void ChromeClient::exitFullScreenForElement(WebCore::Element*)
@@ -1032,27 +988,14 @@ void ChromeClient::exitFullScreenForElement(WebCore::Element*)
     if (returnValue)
         return;
 
-#if ENABLE(VIDEO) && USE(NATIVE_FULLSCREEN_VIDEO)
-    if (m_fullScreenElement && m_fullScreenElement->isMediaElement()) {
-        m_fullScreenElement->document()->webkitWillExitFullScreenForElement(m_fullScreenElement.get());
-        HTMLMediaElement* mediaElement = toHTMLMediaElement(m_fullScreenElement.get());
-        if (mediaElement->player()) {
-            mediaElement->player()->exitFullscreen();
-            m_fullScreenElement->document()->webkitDidExitFullScreenForElement(m_fullScreenElement.get());
-            m_fullScreenElement.clear();
-        }
-        return;
-    }
-#endif
-
     GtkWidget* window = gtk_widget_get_toplevel(GTK_WIDGET(m_webView));
     ASSERT(widgetIsOnscreenToplevelWindow(window));
     g_signal_handlers_disconnect_by_func(window, reinterpret_cast<void*>(onFullscreenGtkKeyPressEvent), this);
 
-    m_fullScreenElement->document()->webkitWillExitFullScreenForElement(m_fullScreenElement.get());
+    m_fullScreenElement->document().webkitWillExitFullScreenForElement(m_fullScreenElement.get());
     gtk_window_unfullscreen(GTK_WINDOW(window));
     m_adjustmentWatcher.enableAllScrollbars();
-    m_fullScreenElement->document()->webkitDidExitFullScreenForElement(m_fullScreenElement.get());
+    m_fullScreenElement->document().webkitDidExitFullScreenForElement(m_fullScreenElement.get());
     m_fullScreenElement.clear();
 }
 #endif
