@@ -351,12 +351,8 @@ static void webkit_web_inspector_set_property(GObject* object, guint prop_id, co
 
     switch(prop_id) {
     case PROP_JAVASCRIPT_PROFILING_ENABLED: {
-#if ENABLE(JAVASCRIPT_DEBUGGER)
         bool enabled = g_value_get_boolean(value);
-        priv->page->inspectorController()->setProfilerEnabled(enabled);
-#else
-        g_message("PROP_JAVASCRIPT_PROFILING_ENABLED is not work because of the javascript debugger is disabled\n");
-#endif
+        priv->page->inspectorController().setProfilerEnabled(enabled);
         break;
     }
     case PROP_TIMELINE_PROFILING_ENABLED: {
@@ -382,11 +378,7 @@ static void webkit_web_inspector_get_property(GObject* object, guint prop_id, GV
         g_value_set_string(value, priv->inspected_uri);
         break;
     case PROP_JAVASCRIPT_PROFILING_ENABLED:
-#if ENABLE(JAVASCRIPT_DEBUGGER)
-        g_value_set_boolean(value, priv->page->inspectorController()->profilerEnabled());
-#else
-        g_message("PROP_JAVASCRIPT_PROFILING_ENABLED is not work because of the javascript debugger is disabled\n");
-#endif
+        g_value_set_boolean(value, priv->page->inspectorController().profilerEnabled());
         break;
     case PROP_TIMELINE_PROFILING_ENABLED:
         g_message("PROP_TIMELINE_PROFILING_ENABLED has been deprecated\n");
@@ -485,13 +477,13 @@ void webkit_web_inspector_show(WebKitWebInspector* webInspector)
 
     WebKitWebInspectorPrivate* priv = webInspector->priv;
 
-    Frame* frame = priv->page->focusController().focusedOrMainFrame();
-    FrameView* view = frame->view();
+    Frame& frame = priv->page->focusController().focusedOrMainFrame();
+    FrameView* view = frame.view();
 
     if (!view)
         return;
 
-    priv->page->inspectorController()->show();
+    priv->page->inspectorController().show();
 }
 
 /**
@@ -508,7 +500,7 @@ void webkit_web_inspector_inspect_node(WebKitWebInspector* webInspector, WebKitD
     g_return_if_fail(WEBKIT_IS_WEB_INSPECTOR(webInspector));
     g_return_if_fail(WEBKIT_DOM_IS_NODE(node));
 
-    webInspector->priv->page->inspectorController()->inspect(core(node));
+    webInspector->priv->page->inspectorController().inspect(core(node));
 }
 
 /**
@@ -535,8 +527,8 @@ void webkit_web_inspector_inspect_coordinates(WebKitWebInspector* webInspector,
 
     WebKitWebInspectorPrivate* priv = webInspector->priv;
 
-    Frame* frame = priv->page->focusController().focusedOrMainFrame();
-    FrameView* view = frame->view();
+    Frame& frame = priv->page->focusController().focusedOrMainFrame();
+    FrameView* view = frame.view();
 
     if (!view)
         return;
@@ -545,8 +537,8 @@ void webkit_web_inspector_inspect_coordinates(WebKitWebInspector* webInspector,
     IntPoint documentPoint = view->windowToContents(IntPoint(static_cast<int>(x), static_cast<int>(y)));
     HitTestResult result(documentPoint);
 
-    frame->contentRenderer()->layer()->hitTest(request, result);
-    priv->page->inspectorController()->inspect(result.innerNonSharedNode());
+    frame.contentRenderer()->layer()->hitTest(request, result);
+    priv->page->inspectorController().inspect(result.innerNonSharedNode());
 }
 
 /**
@@ -562,7 +554,7 @@ void webkit_web_inspector_close(WebKitWebInspector* webInspector)
     g_return_if_fail(WEBKIT_IS_WEB_INSPECTOR(webInspector));
 
     WebKitWebInspectorPrivate* priv = webInspector->priv;
-    priv->page->inspectorController()->close();
+    priv->page->inspectorController().close();
 }
 
 void webkit_web_inspector_execute_script(WebKitWebInspector* webInspector, long callId, const gchar* script)
@@ -571,5 +563,5 @@ void webkit_web_inspector_execute_script(WebKitWebInspector* webInspector, long
     g_return_if_fail(script);
 
     WebKitWebInspectorPrivate* priv = webInspector->priv;
-    priv->page->inspectorController()->evaluateForTestInFrontend(callId, script);
+    priv->page->inspectorController().evaluateForTestInFrontend(callId, script);
 }
