@@ -28,30 +28,25 @@
 
 #include "EditingBehavior.h"
 #include "FileSystem.h"
-#include "KURL.h"
+#include "URL.h"
 #include "PluginDatabase.h"
+#include "UserAgentGtk.h"
 #include "webkitenumtypes.h"
 #include "webkitglobalsprivate.h"
 #include "webkitversion.h"
 #include "webkitwebsettingsprivate.h"
-#include <wtf/gobject/GOwnPtr.h>
+#include <wtf/gobject/GUniquePtr.h>
 #include <wtf/text/CString.h>
 #include <wtf/text/StringConcatenate.h>
 #include <glib/gi18n-lib.h>
 
-#if OS(UNIX)
-#include <sys/utsname.h>
-#elif OS(WINDOWS)
-#include "SystemInfo.h"
-#endif
-
 /**
  * SECTION:webkitwebsettings
  * @short_description: Control the behaviour of a #WebKitWebView
  *
- * #WebKitWebSettings can be applied to a #WebKitWebView to control text encoding, 
- * color, font sizes, printing mode, script support, loading of images and various other things. 
- * After creation, a #WebKitWebSettings object contains default settings. 
+ * #WebKitWebSettings can be applied to a #WebKitWebView to control text encoding,
+ * color, font sizes, printing mode, script support, loading of images and various other things.
+ * After creation, a #WebKitWebSettings object contains default settings.
  *
  * <informalexample><programlisting>
  * /<!-- -->* Create a new websettings and disable java script *<!-- -->/
@@ -84,6 +79,7 @@ enum {
     PROP_ENFORCE_96_DPI,
     PROP_AUTO_LOAD_IMAGES,
     PROP_AUTO_SHRINK_IMAGES,
+    PROP_RESPECT_IMAGE_ORIENTATION,
     PROP_PRINT_BACKGROUNDS,
     PROP_ENABLE_SCRIPTS,
     PROP_ENABLE_PLUGINS,
@@ -119,78 +115,17 @@ enum {
     PROP_ENABLE_FULLSCREEN,
     PROP_ENABLE_DNS_PREFETCHING,
     PROP_ENABLE_WEBGL,
+    PROP_ENABLE_MEDIA_STREAM,
     PROP_ENABLE_WEB_AUDIO,
     PROP_ENABLE_ACCELERATED_COMPOSITING,
     PROP_ENABLE_SMOOTH_SCROLLING,
     PROP_MEDIA_PLAYBACK_REQUIRES_USER_GESTURE,
-    PROP_MEDIA_PLAYBACK_ALLOWS_INLINE
+    PROP_MEDIA_PLAYBACK_ALLOWS_INLINE,
+    PROP_ENABLE_RUNNING_OF_INSECURE_CONTENT,
+    PROP_ENABLE_DISPLAY_OF_INSECURE_CONTENT,
+    PROP_ENABLE_MEDIA_SOURCE
 };
 
-// Create a default user agent string
-// This is a liberal interpretation of http://www.mozilla.org/build/revised-user-agent-strings.html
-// See also http://developer.apple.com/internet/safari/faq.html#anchor2
-static String webkitPlatform()
-{
-#if PLATFORM(X11)
-    DEFINE_STATIC_LOCAL(const String, uaPlatform, (String("X11; ")));
-#elif OS(WINDOWS)
-    DEFINE_STATIC_LOCAL(const String, uaPlatform, (String("")));
-#elif PLATFORM(MAC)
-    DEFINE_STATIC_LOCAL(const String, uaPlatform, (String("Macintosh; ")));
-#elif defined(GDK_WINDOWING_DIRECTFB)
-    DEFINE_STATIC_LOCAL(const String, uaPlatform, (String("DirectFB; ")));
-#else
-    DEFINE_STATIC_LOCAL(const String, uaPlatform, (String("Unknown; ")));
-#endif
-
-    return uaPlatform;
-}
-
-static String webkitOSVersion()
-{
-   // FIXME: platform/version detection can be shared.
-#if OS(DARWIN)
-
-#if CPU(X86)
-    DEFINE_STATIC_LOCAL(const String, uaOSVersion, (String("Intel Mac OS X")));
-#else
-    DEFINE_STATIC_LOCAL(const String, uaOSVersion, (String("PPC Mac OS X")));
-#endif
-
-#elif OS(UNIX)
-    DEFINE_STATIC_LOCAL(String, uaOSVersion, (String()));
-
-    if (!uaOSVersion.isEmpty())
-        return uaOSVersion;
-
-    struct utsname name;
-    if (uname(&name) != -1)
-        uaOSVersion = makeString(name.sysname, ' ', name.machine);
-    else
-        uaOSVersion = String("Unknown");
-#elif OS(WINDOWS)
-    DEFINE_STATIC_LOCAL(const String, uaOSVersion, (windowsVersionForUAString()));
-#else
-    DEFINE_STATIC_LOCAL(const String, uaOSVersion, (String("Unknown")));
-#endif
-
-    return uaOSVersion;
-}
-
-static String chromeUserAgent()
-{
-    // We mention Safari since many broken sites check for it (OmniWeb does this too)
-    // We re-use the WebKit version, though it doesn't seem to matter much in practice
-    // We claim to be Chrome as well, which prevents sites that look for Safari and assume
-    // that since we are not OS X, that we are the mobile version of Safari.
-
-    DEFINE_STATIC_LOCAL(const String, uaVersion, (makeString(String::number(WEBKIT_USER_AGENT_MAJOR_VERSION), '.', String::number(WEBKIT_USER_AGENT_MINOR_VERSION), '+')));
-    DEFINE_STATIC_LOCAL(const String, staticUA, (makeString("Mozilla/5.0 (", webkitPlatform(), webkitOSVersion(), ") AppleWebKit/", uaVersion) +
-                                                 makeString(" (KHTML, like Gecko) Chromium/23.0.1271.95 Chrome/23.0.1271.95 Safari/", uaVersion)));
-
-    return staticUA;
-}
-
 static void webkit_web_settings_finalize(GObject* object);
 
 static void webkit_web_settings_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* pspec);
@@ -345,6 +280,15 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                     TRUE,
                                     flags));
 
+    g_object_class_install_property(gobject_class,
+                                    PROP_RESPECT_IMAGE_ORIENTATION,
+                                    g_param_spec_boolean(
+                                    "respect-image-orientation",
+                                    _("Respect Image Orientation"),
+                                    _("Whether WebKit should respect image orientation."),
+                                    FALSE,
+                                    flags));
+
     g_object_class_install_property(gobject_class,
                                     PROP_PRINT_BACKGROUNDS,
                                     g_param_spec_boolean(
@@ -539,7 +483,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
     *
     * Since: 1.5.2
     */
-    GOwnPtr<gchar> localStoragePath(g_build_filename(g_get_user_data_dir(), "webkit", "databases", NULL));
+    GUniquePtr<gchar> localStoragePath(g_build_filename(g_get_user_data_dir(), "webkit", "databases", NULL));
     g_object_class_install_property(gobject_class,
                                     PROP_HTML5_LOCAL_STORAGE_DATABASE_PATH,
                                     g_param_spec_string("html5-local-storage-database-path",
@@ -548,7 +492,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                         localStoragePath.get(),
                                                         flags));
     /**
-    * WebKitWebSettings:enable-xss-auditor
+    * WebKitWebSettings:enable-xss-auditor:
     *
     * Whether to enable the XSS Auditor. This feature filters some kinds of
     * reflective XSS attacks on vulnerable web sites.
@@ -563,7 +507,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                          TRUE,
                                                          flags));
     /**
-    * WebKitWebSettings:enable-spatial-navigation
+    * WebKitWebSettings:enable-spatial-navigation:
     *
     * Whether to enable the Spatial Navigation. This feature consists in the ability
     * to navigate between focusable elements in a Web page, such as hyperlinks and
@@ -582,7 +526,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                          FALSE,
                                                          flags));
     /**
-    * WebKitWebSettings:enable-frame-flattening
+    * WebKitWebSettings:enable-frame-flattening:
     *
     * Whether to enable the Frame Flattening. With this setting each subframe is expanded
     * to its contents, which will flatten all the frames to become one scrollable page.
@@ -620,7 +564,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                         flags));
 
     /**
-    * WebKitWebSettings:javascript-can-open-windows-automatically
+    * WebKitWebSettings:javascript-can-open-windows-automatically:
     *
     * Whether JavaScript can open popup windows automatically without user
     * intervention.
@@ -636,7 +580,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                          flags));
 
     /**
-    * WebKitWebSettings:javascript-can-access-clipboard
+    * WebKitWebSettings:javascript-can-access-clipboard:
     *
     * Whether JavaScript can access Clipboard.
     *
@@ -651,7 +595,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                          flags));
 
     /**
-    * WebKitWebSettings:enable-offline-web-application-cache
+    * WebKitWebSettings:enable-offline-web-application-cache:
     *
     * Whether to enable HTML5 offline web application cache support. Offline
     * Web Application Cache ensures web applications are available even when
@@ -669,12 +613,12 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
 
 
     /**
-    * WebKitWebSettings:editing-behavior
+    * WebKitWebSettings:editing-behavior:
     *
     * This setting controls various editing behaviors that differ
     * between platforms and that have been combined in two groups,
     * 'Mac' and 'Windows'. Some examples:
-    * 
+    *
     *  1) Clicking below the last line of an editable area puts the
     * caret at the end of the last line on Mac, but in the middle of
     * the last line on Windows.
@@ -695,7 +639,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                       flags));
 
     /**
-     * WebKitWebSettings:enable-universal-access-from-file-uris
+     * WebKitWebSettings:enable-universal-access-from-file-uris:
      *
      * Whether to allow files loaded through file:// URIs universal access to
      * all pages.
@@ -711,7 +655,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                          flags));
 
     /**
-     * WebKitWebSettings:enable-dom-paste
+     * WebKitWebSettings:enable-dom-paste:
      *
      * Whether to enable DOM paste. If set to %TRUE, document.execCommand("Paste")
      * will correctly execute and paste content of the clipboard.
@@ -769,7 +713,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                     flags));
 
     /**
-     * WebKitWebSettings::enable-site-specific-quirks
+     * WebKitWebSettings::enable-site-specific-quirks:
      *
      * Whether to turn on site-specific hacks.  Turning this on will
      * tell WebKitGTK+ to use some site-specific workarounds for
@@ -822,7 +766,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
     * you'd like it to do so. If you wish to handle this manually, you
     * can connect to the notify signal for the
     * #WebKitWebWindowFeatures of your #WebKitWebView.
-    * 
+    *
     * Since: 1.1.22
     */
     g_object_class_install_property(gobject_class,
@@ -881,14 +825,25 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                          FALSE,
                                                          flags));
 
-    /* Undocumented for now */
+    /**
+    * WebKitWebSettings:enable-fullscreen:
+    *
+    *
+    * Whether to enable the Javascript Fullscreen API. The API
+    * allows any HTML element to request fullscreen display. See also
+    * the current draft of the spec:
+    * http://www.w3.org/TR/fullscreen/
+    *
+    * Since: 2.4
+    */
     g_object_class_install_property(gobject_class,
-                                    PROP_ENABLE_FULLSCREEN,
-                                    g_param_spec_boolean("enable-fullscreen",
-                                                         _("Enable Fullscreen"),
-                                                         _("Whether the Mozilla style API should be enabled."),
-                                                         FALSE,
-                                                         flags));
+        PROP_ENABLE_FULLSCREEN,
+        g_param_spec_boolean("enable-fullscreen",
+            _("Enable Fullscreen"),
+            _("Whether to enable the Javascript Fullscreen API"),
+            TRUE,
+            flags));
+
     /**
     * WebKitWebSettings:enable-webgl:
     *
@@ -944,7 +899,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
 
 
     /**
-    * WebKitWebSettings:enable-dns-prefetching
+    * WebKitWebSettings:enable-dns-prefetching:
     *
     * Whether webkit prefetches domain names.  This is a separate knob from private browsing.
     * Whether private browsing should set this or not is up for debate, for now it doesn't.
@@ -958,9 +913,28 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                          _("Whether WebKit prefetches domain names"),
                                                          TRUE,
                                                          flags));
+    /**
+    * WebKitWebSettings:enable-media-stream:
+    *
+    * Enable or disable support for Media Stream on pages. Media Stream is
+    * an experimental proposal for allowing web pages to access local video and
+    * audio input devices.  The standard is currently a work-in-progress as part
+    * of the Web Applications 1.0 specification from WHATWG.
+    *
+    * See also http://www.w3.org/TR/mediacapture-streams/
+    *
+    * Since: 1.10.0
+    */
+    g_object_class_install_property(gobject_class,
+                                    PROP_ENABLE_MEDIA_STREAM,
+                                    g_param_spec_boolean("enable-media-stream",
+                                                         _("Enable Media Stream"),
+                                                         _("Whether Media Stream should be enabled"),
+                                                         FALSE,
+                                                         flags));
 
     /**
-    * WebKitWebSettings:enable-smooth-scrolling
+    * WebKitWebSettings:enable-smooth-scrolling:
     *
     * Enable or disable support for smooth scrolling. The current implementation relies upon
     * CPU work to produce a smooth scrolling experience.
@@ -976,7 +950,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                          flags));
 
     /**
-    * WebKitWebSettings:media-playback-requires-user-gesture
+    * WebKitWebSettings:media-playback-requires-user-gesture:
     *
     * Whether a user gesture (such as clicking the play button) would
     * be required to start media playback or load media. This is off
@@ -995,7 +969,7 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                          flags));
 
     /**
-    * WebKitWebSettings:media-playback-allows-inline
+    * WebKitWebSettings:media-playback-allows-inline:
     *
     * Whether media playback is full-screen only or inline playback is allowed.
     * This is true by default, so media playback can be inline. Setting it to
@@ -1011,6 +985,57 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                          TRUE,
                                                          flags));
 
+    /**
+    * WebKitWebSettings:enable-display-of-insecure-content:
+    *
+    * Whether pages loaded via HTTPS should load subresources such as
+    * images and frames from non-HTTPS URLs.
+    *
+    * Since: 2.0
+    */
+    g_object_class_install_property(gobject_class,
+        PROP_ENABLE_DISPLAY_OF_INSECURE_CONTENT,
+        g_param_spec_boolean("enable-display-of-insecure-content",
+            _("Enable display of insecure content"),
+            _("Whether non-HTTPS resources can display on HTTPS pages."),
+            TRUE,
+            flags));
+
+    /**
+    * WebKitWebSettings:enable-running-of-insecure-content:
+    *
+    * Whether pages loaded via HTTPS should run subresources such as
+    * CSS, scripts, and plugins from non-HTTPS URLs.
+    *
+    * Since: 2.0
+    */
+    g_object_class_install_property(gobject_class,
+        PROP_ENABLE_RUNNING_OF_INSECURE_CONTENT,
+        g_param_spec_boolean("enable-running-of-insecure-content",
+            _("Enable running of insecure content"),
+            _("Whether non-HTTPS resources can run on HTTPS pages."),
+            TRUE,
+            flags));
+
+    /**
+    * WebKitWebSettings:enable-mediasource:
+    *
+    * Enable or disable support for MediaSource on pages. MediaSource is an
+    * experimental proposal which extends HTMLMediaElement to allow
+    * JavaScript to generate media streams for playback.  The standard is
+    * currently a work-in-progress by the W3C HTML Media Task Force.
+    *
+    * See also http://www.w3.org/TR/media-source/
+    *
+    * Since: 2.4
+    */
+    g_object_class_install_property(gobject_class,
+        PROP_ENABLE_MEDIA_SOURCE,
+        g_param_spec_boolean("enable-mediasource",
+            _("Enable MediaSource"),
+            _("Whether MediaSource should be enabled."),
+            FALSE,
+            flags));
 }
 
 static void webkit_web_settings_init(WebKitWebSettings* web_settings)
@@ -1072,6 +1097,9 @@ static void webkit_web_settings_set_property(GObject* object, guint prop_id, con
     case PROP_AUTO_SHRINK_IMAGES:
         priv->autoShrinkImages = g_value_get_boolean(value);
         break;
+    case PROP_RESPECT_IMAGE_ORIENTATION:
+        priv->respectImageOrientation = g_value_get_boolean(value);
+        break;
     case PROP_PRINT_BACKGROUNDS:
         priv->printBackgrounds = g_value_get_boolean(value);
         break;
@@ -1125,7 +1153,7 @@ static void webkit_web_settings_set_property(GObject* object, guint prop_id, con
         break;
     case PROP_USER_AGENT:
         if (!g_value_get_string(value) || !strlen(g_value_get_string(value)))
-            priv->userAgent = chromeUserAgent().utf8();
+            priv->userAgent = standardUserAgent().utf8();
         else
             priv->userAgent = g_value_get_string(value);
         break;
@@ -1180,6 +1208,9 @@ static void webkit_web_settings_set_property(GObject* object, guint prop_id, con
     case PROP_ENABLE_WEBGL:
         priv->enableWebgl = g_value_get_boolean(value);
         break;
+    case PROP_ENABLE_MEDIA_STREAM:
+        priv->enableMediaStream = g_value_get_boolean(value);
+        break;
     case PROP_ENABLE_WEB_AUDIO:
         priv->enableWebAudio = g_value_get_boolean(value);
         break;
@@ -1195,6 +1226,15 @@ static void webkit_web_settings_set_property(GObject* object, guint prop_id, con
     case PROP_MEDIA_PLAYBACK_ALLOWS_INLINE:
         priv->mediaPlaybackAllowsInline = g_value_get_boolean(value);
         break;
+    case PROP_ENABLE_DISPLAY_OF_INSECURE_CONTENT:
+        priv->enableDisplayOfInsecureContent = g_value_get_boolean(value);
+        break;
+    case PROP_ENABLE_RUNNING_OF_INSECURE_CONTENT:
+        priv->enableRunningOfInsecureContent = g_value_get_boolean(value);
+        break;
+    case PROP_ENABLE_MEDIA_SOURCE:
+        priv->enableMediaSource = g_value_get_boolean(value);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -1249,6 +1289,9 @@ static void webkit_web_settings_get_property(GObject* object, guint prop_id, GVa
     case PROP_AUTO_SHRINK_IMAGES:
         g_value_set_boolean(value, priv->autoShrinkImages);
         break;
+    case PROP_RESPECT_IMAGE_ORIENTATION:
+        g_value_set_boolean(value, priv->respectImageOrientation);
+        break;
     case PROP_PRINT_BACKGROUNDS:
         g_value_set_boolean(value, priv->printBackgrounds);
         break;
@@ -1354,6 +1397,9 @@ static void webkit_web_settings_get_property(GObject* object, guint prop_id, GVa
     case PROP_ENABLE_WEBGL:
         g_value_set_boolean(value, priv->enableWebgl);
         break;
+    case PROP_ENABLE_MEDIA_STREAM:
+        g_value_set_boolean(value, priv->enableMediaStream);
+        break;
     case PROP_ENABLE_WEB_AUDIO:
         g_value_set_boolean(value, priv->enableWebAudio);
         break;
@@ -1369,6 +1415,15 @@ static void webkit_web_settings_get_property(GObject* object, guint prop_id, GVa
     case PROP_MEDIA_PLAYBACK_ALLOWS_INLINE:
         g_value_set_boolean(value, priv->mediaPlaybackAllowsInline);
         break;
+    case PROP_ENABLE_DISPLAY_OF_INSECURE_CONTENT:
+        g_value_set_boolean(value, priv->enableDisplayOfInsecureContent);
+        break;
+    case PROP_ENABLE_RUNNING_OF_INSECURE_CONTENT:
+        g_value_set_boolean(value, priv->enableRunningOfInsecureContent);
+        break;
+    case PROP_ENABLE_MEDIA_SOURCE:
+        g_value_set_boolean(value, priv->enableMediaSource);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -1399,9 +1454,9 @@ WebKitWebSettings* webkit_web_settings_new()
 WebKitWebSettings* webkit_web_settings_copy(WebKitWebSettings* original)
 {
     unsigned numberOfProperties = 0;
-    GOwnPtr<GParamSpec*> properties(g_object_class_list_properties(
+    GUniquePtr<GParamSpec*> properties(g_object_class_list_properties(
         G_OBJECT_CLASS(WEBKIT_WEB_SETTINGS_GET_CLASS(original)), &numberOfProperties));
-    GOwnPtr<GParameter> parameters(g_new0(GParameter, numberOfProperties));
+    GUniquePtr<GParameter> parameters(g_new0(GParameter, numberOfProperties));
 
     for (size_t i = 0; i < numberOfProperties; i++) {
         GParamSpec* property = properties.get()[i];
@@ -1554,12 +1609,12 @@ static bool isGoogleDomain(String host)
     return false;
 }
 
-static String userAgentForURL(const KURL& url)
+static String userAgentForURL(const URL& url)
 {
     // For Google domains, drop the browser's custom User Agent string, and use the
     // standard Chrome one, so they don't give us a broken experience.
     if (isGoogleDomain(url.host()))
-        return chromeUserAgent();
+        return standardUserAgent();
 
     return String();
 }
@@ -1592,7 +1647,7 @@ static String userAgentForURL(const KURL& url)
 char* webkitWebSettingsUserAgentForURI(WebKitWebSettings* webSettings, const char* uri)
 {
     if (webSettings->priv->enableSiteSpecificQuirks) {
-        String userAgentString = userAgentForURL(WebCore::KURL(WebCore::KURL(), String::fromUTF8(uri)));
+        String userAgentString = userAgentForURL(WebCore::URL(WebCore::URL(), String::fromUTF8(uri)));
         if (!userAgentString.isEmpty())
             return g_strdup(userAgentString.utf8().data());
     }
