@@ -22,11 +22,12 @@
 
 import collections
 import re
+import sys
 from webkit2 import parser
 
+WANTS_CONNECTION_ATTRIBUTE = 'WantsConnection'
 LEGACY_RECEIVER_ATTRIBUTE = 'LegacyReceiver'
 DELAYED_ATTRIBUTE = 'Delayed'
-DISPATCH_ON_CONNECTION_QUEUE_ATTRIBUTE = 'DispatchOnConnectionQueue'
 VARIADIC_ATTRIBUTE = 'Variadic'
 
 _license_header = """/*
@@ -92,62 +93,75 @@ def reply_parameter_type(type):
     return '%s&' % type
 
 
-def arguments_type(parameters, parameter_type_function):
-    arguments_type = 'CoreIPC::Arguments%d' % len(parameters)
+def arguments_type_old(parameters, parameter_type_function):
+    arguments_type = 'IPC::Arguments%d' % len(parameters)
     if len(parameters):
         arguments_type = '%s<%s>' % (arguments_type, ', '.join(parameter_type_function(parameter.type) for parameter in parameters))
     return arguments_type
 
 
+def arguments_type(message):
+    return 'std::tuple<%s>' % ', '.join(function_parameter_type(parameter.type) for parameter in message.parameters)
+
 def base_class(message):
     return arguments_type(message.parameters, function_parameter_type)
 
 
 def reply_type(message):
-    return arguments_type(message.reply_parameters, reply_parameter_type)
+    return arguments_type_old(message.reply_parameters, reply_parameter_type)
 
 
 def decode_type(message):
-    if message.has_attribute(VARIADIC_ATTRIBUTE):
-        return arguments_type(message.parameters[:-1], reply_parameter_type)
-    return base_class(message)
+    parameters = message.parameters
 
+    if message.has_attribute(VARIADIC_ATTRIBUTE):
+        parameters = parameters[:-1]
+
+    return 'std::tuple<%s>' % ', '.join(parameter.type for parameter in parameters)
 
 def delayed_reply_type(message):
-    return arguments_type(message.reply_parameters, function_parameter_type)
+    return arguments_type_old(message.reply_parameters, function_parameter_type)
 
 
 def message_to_struct_declaration(message):
     result = []
     function_parameters = [(function_parameter_type(x.type), x.name) for x in message.parameters]
-    result.append('struct %s : %s' % (message.name, base_class(message)))
-    result.append(' {\n')
-    result.append('    static CoreIPC::StringReference receiverName() { return messageReceiverName(); }\n')
-    result.append('    static CoreIPC::StringReference name() { return CoreIPC::StringReference("%s"); }\n' % message.name)
+    result.append('class %s {\n' % message.name)
+    result.append('public:\n')
+    result.append('    typedef %s DecodeType;\n' % decode_type(message))
+    result.append('\n')
+    result.append('    static IPC::StringReference receiverName() { return messageReceiverName(); }\n')
+    result.append('    static IPC::StringReference name() { return IPC::StringReference("%s"); }\n' % message.name)
     result.append('    static const bool isSync = %s;\n' % ('false', 'true')[message.reply_parameters != None])
     result.append('\n')
     if message.reply_parameters != None:
         if message.has_attribute(DELAYED_ATTRIBUTE):
             send_parameters = [(function_parameter_type(x.type), x.name) for x in message.reply_parameters]
             result.append('    struct DelayedReply : public ThreadSafeRefCounted<DelayedReply> {\n')
-            result.append('        DelayedReply(PassRefPtr<CoreIPC::Connection>, PassOwnPtr<CoreIPC::MessageEncoder>);\n')
+            result.append('        DelayedReply(PassRefPtr<IPC::Connection>, std::unique_ptr<IPC::MessageEncoder>);\n')
             result.append('        ~DelayedReply();\n')
             result.append('\n')
             result.append('        bool send(%s);\n' % ', '.join([' '.join(x) for x in send_parameters]))
             result.append('\n')
             result.append('    private:\n')
-            result.append('        RefPtr<CoreIPC::Connection> m_connection;\n')
-            result.append('        OwnPtr<CoreIPC::MessageEncoder> m_encoder;\n')
+            result.append('        RefPtr<IPC::Connection> m_connection;\n')
+            result.append('        std::unique_ptr<IPC::MessageEncoder> m_encoder;\n')
             result.append('    };\n\n')
 
         result.append('    typedef %s Reply;\n' % reply_type(message))
 
-    result.append('    typedef %s DecodeType;\n' % decode_type(message))
     if len(function_parameters):
         result.append('    %s%s(%s)' % (len(function_parameters) == 1 and 'explicit ' or '', message.name, ', '.join([' '.join(x) for x in function_parameters])))
-        result.append('\n        : %s(%s)\n' % (base_class(message), ', '.join([x[1] for x in function_parameters])))
+        result.append('\n        : m_arguments(%s)\n' % ', '.join([x[1] for x in function_parameters]))
         result.append('    {\n')
-        result.append('    }\n')
+        result.append('    }\n\n')
+    result.append('    const %s arguments() const\n' % arguments_type(message))
+    result.append('    {\n')
+    result.append('        return m_arguments;\n')
+    result.append('    }\n')
+    result.append('\n')
+    result.append('private:\n')
+    result.append('    %s m_arguments;\n' % arguments_type(message))
     result.append('};\n')
     return surround_in_condition(''.join(result), message.condition)
 
@@ -162,12 +176,20 @@ def struct_or_class(namespace, type):
         'WebCore::FloatPoint3D',
         'WebCore::FileChooserSettings',
         'WebCore::GrammarDetail',
+        'WebCore::IDBDatabaseMetadata',
+        'WebCore::IDBGetResult',
+        'WebCore::IDBIndexMetadata',
+        'WebCore::IDBKeyData',
+        'WebCore::IDBKeyRangeData',
+        'WebCore::IDBObjectStoreMetadata',
         'WebCore::IdentityTransformOperation',
         'WebCore::KeypressCommand',
         'WebCore::Length',
         'WebCore::MatrixTransformOperation',
         'WebCore::Matrix3DTransformOperation',
         'WebCore::NotificationContents',
+        'WebCore::PasteboardImage',
+        'WebCore::PasteboardWebContent',
         'WebCore::PerspectiveTransformOperation',
         'WebCore::PluginInfo',
         'WebCore::PrintInfo',
@@ -179,21 +201,24 @@ def struct_or_class(namespace, type):
         'WebCore::TransformOperation',
         'WebCore::TransformOperations',
         'WebCore::TranslateTransformOperation',
+        'WebCore::ViewportArguments',
         'WebCore::ViewportAttributes',
         'WebCore::WindowFeatures',
         'WebKit::AttributedString',
         'WebKit::ColorSpaceData',
         'WebKit::ContextMenuState',
+        'WebKit::DatabaseProcessCreationParameters',
         'WebKit::DictionaryPopupInfo',
         'WebKit::DrawingAreaInfo',
         'WebKit::EditorState',
+        'WebKit::InteractionInformationAtPosition',
+        'WebKit::NavigationActionData',
         'WebKit::NetworkProcessCreationParameters',
         'WebKit::PlatformPopupMenuData',
         'WebKit::PluginCreationParameters',
         'WebKit::PluginProcessCreationParameters',
         'WebKit::PrintInfo',
         'WebKit::SecurityOriginData',
-        'WebKit::SharedWorkerProcessCreationParameters',
         'WebKit::StatisticsData',
         'WebKit::TextCheckerState',
         'WebKit::WebNavigationDataStore',
@@ -226,10 +251,14 @@ def forward_declarations_and_headers(receiver):
         '"StringReference.h"',
     ])
 
+    non_template_wtf_types = frozenset([
+        'String',
+    ])
+
     for message in receiver.messages:
         if message.reply_parameters != None and message.has_attribute(DELAYED_ATTRIBUTE):
             headers.add('<wtf/ThreadSafeRefCounted.h>')
-            types_by_namespace['CoreIPC'].update(['Connection'])
+            types_by_namespace['IPC'].update(['Connection'])
 
     for parameter in receiver.iterparameters():
         type = parameter.type
@@ -241,6 +270,10 @@ def forward_declarations_and_headers(receiver):
 
         split = type.split('::')
 
+        # Handle WTF types even if the WTF:: prefix is not given
+        if split[0] in non_template_wtf_types:
+            split.insert(0, 'WTF')
+
         if len(split) == 2:
             namespace = split[0]
             inner_type = split[1]
@@ -279,9 +312,9 @@ def generate_messages_header(file):
 
     result.append('namespace Messages {\nnamespace %s {\n' % receiver.name)
     result.append('\n')
-    result.append('static inline CoreIPC::StringReference messageReceiverName()\n')
+    result.append('static inline IPC::StringReference messageReceiverName()\n')
     result.append('{\n')
-    result.append('    return CoreIPC::StringReference("%s");\n' % receiver.name)
+    result.append('    return IPC::StringReference("%s");\n' % receiver.name)
     result.append('}\n')
     result.append('\n')
     result.append('\n'.join([message_to_struct_declaration(x) for x in receiver.messages]))
@@ -302,25 +335,19 @@ def handler_function(receiver, message):
     return '%s::%s' % (receiver.name, message.name[0].lower() + message.name[1:])
 
 
-def connection_work_queue_message_statement(receiver, message):
-    dispatch_function_args = ['connection', '*decoder', 'this', '&%s' % handler_function(receiver, message)]
-    result = []
-    result.append('    if (decoder->messageName() == Messages::%s::%s::name()) {\n' % (receiver.name, message.name))
-    result.append('        CoreIPC::handleMessageOnConnectionQueue<Messages::%s::%s>(%s);\n' % (receiver.name, message.name, ', '.join(dispatch_function_args)))
-    result.append('        decoder = nullptr;\n')
-    result.append('        return;\n')
-    result.append('    }\n')
-    return surround_in_condition(''.join(result), message.condition)
-
 def async_message_statement(receiver, message):
     dispatch_function_args = ['decoder', 'this', '&%s' % handler_function(receiver, message)]
+
     dispatch_function = 'handleMessage'
     if message.has_attribute(VARIADIC_ATTRIBUTE):
         dispatch_function += 'Variadic'
 
+    if message.has_attribute(WANTS_CONNECTION_ATTRIBUTE):
+        dispatch_function_args.insert(0, 'connection')
+
     result = []
     result.append('    if (decoder.messageName() == Messages::%s::%s::name()) {\n' % (receiver.name, message.name))
-    result.append('        CoreIPC::%s<Messages::%s::%s>(%s);\n' % (dispatch_function, receiver.name, message.name, ', '.join(dispatch_function_args)))
+    result.append('        IPC::%s<Messages::%s::%s>(%s);\n' % (dispatch_function, receiver.name, message.name, ', '.join(dispatch_function_args)))
     result.append('        return;\n')
     result.append('    }\n')
     return surround_in_condition(''.join(result), message.condition)
@@ -333,56 +360,87 @@ def sync_message_statement(receiver, message):
     if message.has_attribute(VARIADIC_ATTRIBUTE):
         dispatch_function += 'Variadic'
 
+    wants_connection = message.has_attribute(DELAYED_ATTRIBUTE) or message.has_attribute(WANTS_CONNECTION_ATTRIBUTE)
+
     result = []
     result.append('    if (decoder.messageName() == Messages::%s::%s::name()) {\n' % (receiver.name, message.name))
-    result.append('        CoreIPC::%s<Messages::%s::%s>(%sdecoder, %sreplyEncoder, this, &%s);\n' % (dispatch_function, receiver.name, message.name, 'connection, ' if message.has_attribute(DELAYED_ATTRIBUTE) else '', '' if message.has_attribute(DELAYED_ATTRIBUTE) else '*', handler_function(receiver, message)))
+    result.append('        IPC::%s<Messages::%s::%s>(%sdecoder, %sreplyEncoder, this, &%s);\n' % (dispatch_function, receiver.name, message.name, 'connection, ' if wants_connection else '', '' if message.has_attribute(DELAYED_ATTRIBUTE) else '*', handler_function(receiver, message)))
     result.append('        return;\n')
     result.append('    }\n')
     return surround_in_condition(''.join(result), message.condition)
 
 
+def class_template_headers(template_string):
+    template_string = template_string.strip()
+
+    class_template_types = {
+        'Vector': {'headers': ['<wtf/Vector.h>'], 'argument_coder_headers': ['"ArgumentCoders.h"']},
+        'HashMap': {'headers': ['<wtf/HashMap.h>'], 'argument_coder_headers': ['"ArgumentCoders.h"']},
+        'std::pair': {'headers': ['<utility>'], 'argument_coder_headers': ['"ArgumentCoders.h"']},
+    }
+
+    match = re.match('(?P<template_name>.+?)<(?P<parameter_string>.+)>', template_string)
+    if not match:
+        return {'header_infos':[], 'types':[template_string]}
+
+    template_name = match.groupdict()['template_name']
+    if template_name not in class_template_types:
+        sys.stderr.write("Error: no class template type is defined for '%s'\n" % (template_string))
+        sys.exit(1)
+
+    header_infos = [class_template_types[template_name]]
+    types = []
+
+    for parameter in parser.split_parameters_string(match.groupdict()['parameter_string']):
+        parameter_header_infos_and_types = class_template_headers(parameter)
+
+        header_infos += parameter_header_infos_and_types['header_infos'];
+        types += parameter_header_infos_and_types['types']
+
+    return {'header_infos':header_infos, 'types':types}
+
+
 def argument_coder_headers_for_type(type):
-    # Check for Vector.
-    match = re.search(r'Vector<(.+)>', type)
-    if match:
-        element_type = match.groups()[0].strip()
-        return ['"ArgumentCoders.h"'] + argument_coder_headers_for_type(element_type)
+    header_infos_and_types = class_template_headers(type)
 
     special_cases = {
-        'WTF::String': '"ArgumentCoders.h"',
+        'String': '"ArgumentCoders.h"',
         'WebKit::InjectedBundleUserMessageEncoder': '"InjectedBundleUserMessageCoders.h"',
         'WebKit::WebContextUserMessageEncoder': '"WebContextUserMessageCoders.h"',
     }
 
-    if type in special_cases:
-        return [special_cases[type]]
+    headers = []
+    for header_info in header_infos_and_types['header_infos']:
+        headers += header_info['argument_coder_headers']
 
-    split = type.split('::')
-    if len(split) < 2:
-        return []
-    if split[0] == 'WebCore':
-        return ['"WebCoreArgumentCoders.h"']
+    for type in header_infos_and_types['types']:
+        if type in special_cases:
+            headers.append(special_cases[type])
+            continue
 
-    return []
+        split = type.split('::')
+        if len(split) < 2:
+            continue
+        if split[0] == 'WebCore':
+            headers.append('"WebCoreArgumentCoders.h"')
 
+    return headers
 
 def headers_for_type(type):
-    # Check for Vector.
-    match = re.search(r'Vector<(.+)>', type)
-    if match:
-        element_type = match.groups()[0].strip()
-        return ['<wtf/Vector.h>'] + headers_for_type(element_type)
+    header_infos_and_types = class_template_headers(type)
 
     special_cases = {
-        'WTF::String': ['<wtf/text/WTFString.h>'],
+        'String': ['<wtf/text/WTFString.h>'],
         'WebCore::CompositionUnderline': ['<WebCore/Editor.h>'],
-        'WebCore::CoordinatedLayerID': ['<WebCore/CoordinatedLayerInfo.h>'],
         'WebCore::GrammarDetail': ['<WebCore/TextCheckerClient.h>'],
         'WebCore::GraphicsLayerAnimations': ['<WebCore/GraphicsLayerAnimation.h>'],
         'WebCore::KeyframeValueList': ['<WebCore/GraphicsLayer.h>'],
         'WebCore::KeypressCommand': ['<WebCore/KeyboardEvent.h>'],
         'WebCore::FileChooserSettings': ['<WebCore/FileChooser.h>'],
         'WebCore::PluginInfo': ['<WebCore/PluginData.h>'],
+        'WebCore::PasteboardImage': ['<WebCore/Pasteboard.h>'],
+        'WebCore::PasteboardWebContent': ['<WebCore/Pasteboard.h>'],
+        'WebCore::TextCheckingRequestData': ['<WebCore/TextChecking.h>'],
         'WebCore::TextCheckingResult': ['<WebCore/TextCheckerClient.h>'],
         'WebCore::ViewportAttributes': ['<WebCore/ViewportArguments.h>'],
         'WebKit::InjectedBundleUserMessageEncoder': [],
@@ -393,22 +451,32 @@ def headers_for_type(type):
         'WebKit::WebTouchEvent': ['"WebEvent.h"'],
         'WebKit::WebWheelEvent': ['"WebEvent.h"'],
     }
-    if type in special_cases:
-        return special_cases[type]
 
-    # We assume that we must include a header for a type iff it has a scope
-    # resolution operator (::).
-    split = type.split('::')
-    if len(split) < 2:
-        return []
-    if split[0] == 'WebKit' or split[0] == 'CoreIPC':
-        return ['"%s.h"' % split[1]]
-    return ['<%s/%s.h>' % tuple(split)]
+    headers = []
+    for header_info in header_infos_and_types['header_infos']:
+        headers += header_info['headers']
 
+    for type in header_infos_and_types['types']:
+        if type in special_cases:
+            headers += special_cases[type]
+            continue
+
+        # We assume that we must include a header for a type iff it has a scope
+        # resolution operator (::).
+        split = type.split('::')
+        if len(split) < 2:
+            continue
+
+        if split[0] == 'WebKit' or split[0] == 'IPC':
+            headers.append('"%s.h"' % split[1])
+        else:
+            headers.append('<%s/%s.h>' % tuple(split))
+
+    return headers
 
 def generate_message_handler(file):
     receiver = parser.parse(file)
-    headers = {
+    header_conditions = {
         '"%s"' % messages_header_filename(receiver): [None],
         '"HandleMessage.h"': [None],
         '"MessageDecoder.h"': [None],
@@ -429,15 +497,15 @@ def generate_message_handler(file):
         argument_encoder_headers = argument_coder_headers_for_type(type)
         if argument_encoder_headers:
             for header in argument_encoder_headers:
-                if header not in headers:
-                    headers[header] = []
-                headers[header].extend(conditions)
+                if header not in header_conditions:
+                    header_conditions[header] = []
+                header_conditions[header].extend(conditions)
 
         type_headers = headers_for_type(type)
         for header in type_headers:
-            if header not in headers:
-                headers[header] = []
-            headers[header].extend(conditions)
+            if header not in header_conditions:
+                header_conditions[header] = []
+            header_conditions[header].extend(conditions)
 
     for message in receiver.messages:
         if message.reply_parameters is not None:
@@ -446,15 +514,15 @@ def generate_message_handler(file):
                 argument_encoder_headers = argument_coder_headers_for_type(type)
                 if argument_encoder_headers:
                     for header in argument_encoder_headers:
-                        if header not in headers:
-                            headers[header] = []
-                        headers[header].append(message.condition)
+                        if header not in header_conditions:
+                            header_conditions[header] = []
+                        header_conditions[header].append(message.condition)
 
                 type_headers = headers_for_type(type)
                 for header in type_headers:
-                    if header not in headers:
-                        headers[header] = []
-                    headers[header].append(message.condition)
+                    if header not in header_conditions:
+                        header_conditions[header] = []
+                    header_conditions[header].append(message.condition)
 
 
     result = []
@@ -467,13 +535,13 @@ def generate_message_handler(file):
         result.append('#if %s\n\n' % receiver.condition)
 
     result.append('#include "%s.h"\n\n' % receiver.name)
-    for headercondition in sorted(headers):
-        if headers[headercondition] and not None in headers[headercondition]:
-            result.append('#if %s\n' % ' || '.join(set(headers[headercondition])))
-            result += ['#include %s\n' % headercondition]
+    for header in sorted(header_conditions):
+        if header_conditions[header] and not None in header_conditions[header]:
+            result.append('#if %s\n' % ' || '.join(set(header_conditions[header])))
+            result += ['#include %s\n' % header]
             result.append('#endif\n')
         else:
-            result += ['#include %s\n' % headercondition]
+            result += ['#include %s\n' % header]
     result.append('\n')
 
     sync_delayed_messages = []
@@ -489,10 +557,10 @@ def generate_message_handler(file):
 
             if message.condition:
                 result.append('#if %s\n\n' % message.condition)
-            
-            result.append('%s::DelayedReply::DelayedReply(PassRefPtr<CoreIPC::Connection> connection, PassOwnPtr<CoreIPC::MessageEncoder> encoder)\n' % message.name)
+
+            result.append('%s::DelayedReply::DelayedReply(PassRefPtr<IPC::Connection> connection, std::unique_ptr<IPC::MessageEncoder> encoder)\n' % message.name)
             result.append('    : m_connection(connection)\n')
-            result.append('    , m_encoder(encoder)\n')
+            result.append('    , m_encoder(std::move(encoder))\n')
             result.append('{\n')
             result.append('}\n')
             result.append('\n')
@@ -504,8 +572,8 @@ def generate_message_handler(file):
             result.append('bool %s::DelayedReply::send(%s)\n' % (message.name, ', '.join([' '.join(x) for x in send_parameters])))
             result.append('{\n')
             result.append('    ASSERT(m_encoder);\n')
-            result += ['    m_encoder->encode(%s);\n' % x.name for x in message.reply_parameters]
-            result.append('    bool result = m_connection->sendSyncReply(m_encoder.release());\n')
+            result += ['    *m_encoder << %s;\n' % x.name for x in message.reply_parameters]
+            result.append('    bool result = m_connection->sendSyncReply(std::move(m_encoder));\n')
             result.append('    m_connection = nullptr;\n')
             result.append('    return result;\n')
             result.append('}\n')
@@ -518,54 +586,40 @@ def generate_message_handler(file):
 
     result.append('namespace WebKit {\n\n')
 
-    async_dispatch_on_connection_queue_messages = []
-    sync_dispatch_on_connection_queue_messages = []
     async_messages = []
     sync_messages = []
     for message in receiver.messages:
         if message.reply_parameters is not None:
-            if message.has_attribute(DISPATCH_ON_CONNECTION_QUEUE_ATTRIBUTE):
-                sync_dispatch_on_connection_queue_messages.append(message)
-            else:
-                sync_messages.append(message)
+            sync_messages.append(message)
         else:
-            if message.has_attribute(DISPATCH_ON_CONNECTION_QUEUE_ATTRIBUTE):
-                async_dispatch_on_connection_queue_messages.append(message)
-            else:
-                async_messages.append(message)
-
-    if async_dispatch_on_connection_queue_messages:
-        result.append('void %s::didReceive%sMessageOnConnectionWorkQueue(CoreIPC::Connection* connection, OwnPtr<CoreIPC::MessageDecoder>& decoder)\n' % (receiver.name, receiver.name))
-        result.append('{\n')
-        result.append('#if COMPILER(MSVC)\n')
-        result.append('#pragma warning(push)\n')
-        result.append('#pragma warning(disable: 4065)\n')
-        result.append('#endif\n')
-        result += [connection_work_queue_message_statement(receiver, message) for message in async_dispatch_on_connection_queue_messages]
-        result.append('#if COMPILER(MSVC)\n')
-        result.append('#pragma warning(pop)\n')
-        result.append('#endif\n')
-        result.append('}\n\n')
+            async_messages.append(message)
 
     if async_messages:
         if receiver.has_attribute(LEGACY_RECEIVER_ATTRIBUTE):
-            result.append('void %s::didReceive%sMessage(CoreIPC::Connection*, CoreIPC::MessageDecoder& decoder)\n' % (receiver.name, receiver.name))
+            result.append('void %s::didReceive%sMessage(IPC::Connection*, IPC::MessageDecoder& decoder)\n' % (receiver.name, receiver.name))
         else:
-            result.append('void %s::didReceiveMessage(CoreIPC::Connection*, CoreIPC::MessageDecoder& decoder)\n' % (receiver.name))
+            result.append('void %s::didReceiveMessage(IPC::Connection* connection, IPC::MessageDecoder& decoder)\n' % (receiver.name))
 
         result.append('{\n')
         result += [async_message_statement(receiver, message) for message in async_messages]
-        result.append('    ASSERT_NOT_REACHED();\n')
+        if (receiver.superclass):
+            result.append('    %s::didReceiveMessage(connection, decoder);\n' % (receiver.superclass))
+        else:
+            if not receiver.has_attribute(LEGACY_RECEIVER_ATTRIBUTE):
+                result.append('    UNUSED_PARAM(connection);\n')
+            result.append('    ASSERT_NOT_REACHED();\n')
         result.append('}\n')
 
     if sync_messages:
         result.append('\n')
         if receiver.has_attribute(LEGACY_RECEIVER_ATTRIBUTE):
-            result.append('void %s::didReceiveSync%sMessage(CoreIPC::Connection*%s, CoreIPC::MessageDecoder& decoder, OwnPtr<CoreIPC::MessageEncoder>& replyEncoder)\n' % (receiver.name, receiver.name, ' connection' if sync_delayed_messages else ''))
+            result.append('void %s::didReceiveSync%sMessage(IPC::Connection*%s, IPC::MessageDecoder& decoder, std::unique_ptr<IPC::MessageEncoder>& replyEncoder)\n' % (receiver.name, receiver.name, ' connection' if sync_delayed_messages else ''))
         else:
-            result.append('void %s::didReceiveSyncMessage(CoreIPC::Connection*%s, CoreIPC::MessageDecoder& decoder, OwnPtr<CoreIPC::MessageEncoder>& replyEncoder)\n' % (receiver.name, ' connection' if sync_delayed_messages else ''))
+            result.append('void %s::didReceiveSyncMessage(IPC::Connection* connection, IPC::MessageDecoder& decoder, std::unique_ptr<IPC::MessageEncoder>& replyEncoder)\n' % (receiver.name))
         result.append('{\n')
         result += [sync_message_statement(receiver, message) for message in sync_messages]
+        if not receiver.has_attribute(LEGACY_RECEIVER_ATTRIBUTE):
+            result.append('    UNUSED_PARAM(connection);\n')
         result.append('    ASSERT_NOT_REACHED();\n')
         result.append('}\n')
 
