@@ -32,8 +32,8 @@
 #include <WebCore/NotImplemented.h>
 #include <WebCore/ResourceHandleInternal.h>
 #include <gio/gio.h>
-#include <wtf/gobject/GOwnPtr.h>
 #include <wtf/gobject/GRefPtr.h>
+#include <wtf/gobject/GUniquePtr.h>
 #include <wtf/text/CString.h>
 
 #if PLATFORM(GTK)
@@ -69,7 +69,7 @@ public:
     void downloadFailed(const ResourceError& error)
     {
         deleteIntermediateFileInNeeded();
-        m_download->didFail(error, CoreIPC::DataReference());
+        m_download->didFail(error, IPC::DataReference());
     }
 
     void didReceiveResponse(ResourceHandle*, const ResourceResponse& response)
@@ -84,7 +84,7 @@ public:
 
         String suggestedFilename = response.suggestedFilename();
         if (suggestedFilename.isEmpty()) {
-            KURL url = response.url();
+            URL url = response.url();
             url.setQuery(String());
             url.removeFragmentIdentifier();
             suggestedFilename = decodeURLEscapeSequences(url.lastPathComponent());
@@ -94,7 +94,7 @@ public:
         m_destinationURI = m_download->decideDestinationWithSuggestedFilename(suggestedFilename, overwrite);
         if (m_destinationURI.isEmpty()) {
 #if PLATFORM(GTK)
-            GOwnPtr<char> buffer(g_strdup_printf(_("Cannot determine destination URI for download with suggested filename %s"), suggestedFilename.utf8().data()));
+            GUniquePtr<char> buffer(g_strdup_printf(_("Cannot determine destination URI for download with suggested filename %s"), suggestedFilename.utf8().data()));
             String errorMessage = String::fromUTF8(buffer.get());
 #else
             String errorMessage = makeString("Cannot determine destination URI for download with suggested filename ", suggestedFilename);
@@ -105,7 +105,7 @@ public:
 
         String intermediateURI = m_destinationURI + ".wkdownload";
         m_intermediateFile = adoptGRef(g_file_new_for_uri(intermediateURI.utf8().data()));
-        GOwnPtr<GError> error;
+        GUniqueOutPtr<GError> error;
         m_outputStream = adoptGRef(g_file_replace(m_intermediateFile.get(), 0, TRUE, G_FILE_CREATE_NONE, 0, &error.outPtr()));
         if (!m_outputStream) {
             downloadFailed(platformDownloadDestinationError(response, error->message));
@@ -115,7 +115,7 @@ public:
         m_download->didCreateDestination(m_destinationURI);
     }
 
-    void didReceiveData(ResourceHandle*, const char* data, int length, int /*encodedDataLength*/)
+    void didReceiveData(ResourceHandle*, const char* data, unsigned length, int /*encodedDataLength*/)
     {
         if (m_handleResponseLaterID) {
             g_source_remove(m_handleResponseLaterID);
@@ -123,7 +123,7 @@ public:
         }
 
         gsize bytesWritten;
-        GOwnPtr<GError> error;
+        GUniqueOutPtr<GError> error;
         g_output_stream_write_all(G_OUTPUT_STREAM(m_outputStream.get()), data, length, &bytesWritten, 0, &error.outPtr());
         if (error) {
             downloadFailed(platformDownloadDestinationError(m_response, error->message));
@@ -138,7 +138,7 @@ public:
 
         ASSERT(m_intermediateFile);
         GRefPtr<GFile> destinationFile = adoptGRef(g_file_new_for_uri(m_destinationURI.utf8().data()));
-        GOwnPtr<GError> error;
+        GUniqueOutPtr<GError> error;
         if (!g_file_move(m_intermediateFile.get(), destinationFile.get(), G_FILE_COPY_NONE, nullptr, nullptr, nullptr, &error.outPtr())) {
             downloadFailed(platformDownloadDestinationError(m_response, error->message));
             return;
@@ -172,7 +172,7 @@ public:
     {
         handle->cancel();
         deleteIntermediateFileInNeeded();
-        m_download->didCancel(CoreIPC::DataReference());
+        m_download->didCancel(IPC::DataReference());
     }
 
     void handleResponse()
