@@ -33,16 +33,16 @@
 #include <WebCore/FileSystem.h>
 #include <WebCore/NetworkingContext.h>
 #include <WebCore/ResourceHandle.h>
-#include <WebCore/RunLoop.h>
 #include <errno.h>
+#include <glib.h>
 #include <locale.h>
+#include <wtf/RunLoop.h>
 #include <wtf/text/CString.h>
 #include <wtf/text/WTFString.h>
-#include <wtf/gobject/GOwnPtr.h>
+#include <wtf/gobject/GUniquePtr.h>
 #include <wtf/gobject/GlibUtilities.h>
 
-#if OS(LINUX)
-#include <sys/prctl.h>
+#if OS(UNIX)
 #include <sys/socket.h>
 #endif
 
@@ -62,14 +62,6 @@ static void childSetupFunction(gpointer userData)
     close(socket);
 }
 
-static void childFinishedFunction(GPid, gint status, gpointer userData)
-{
-    if (WIFEXITED(status) && !WEXITSTATUS(status))
-        return;
-
-    close(GPOINTER_TO_INT(userData));
-}
-
 void ProcessLauncher::launchProcess()
 {
     GPid pid = 0;
@@ -83,25 +75,55 @@ void ProcessLauncher::launchProcess()
 
     String executablePath, pluginPath;
     CString realExecutablePath, realPluginPath;
-    if (m_launchOptions.processType == WebProcess)
+    switch (m_launchOptions.processType) {
+    case WebProcess:
         executablePath = executablePathOfWebProcess();
-    else {
+        break;
+    case PluginProcess:
         executablePath = executablePathOfPluginProcess();
         pluginPath = m_launchOptions.extraInitializationData.get("plugin-path");
         realPluginPath = fileSystemRepresentation(pluginPath);
+        break;
+#if ENABLE(NETWORK_PROCESS)
+    case NetworkProcess:
+        executablePath = executablePathOfNetworkProcess();
+        break;
+#endif
+    default:
+        ASSERT_NOT_REACHED();
+        return;
     }
 
     realExecutablePath = fileSystemRepresentation(executablePath);
-    GOwnPtr<gchar> socket(g_strdup_printf("%d", sockets[0]));
-    char* argv[4];
-    argv[0] = const_cast<char*>(realExecutablePath.data());
-    argv[1] = socket.get();
-    argv[2] = const_cast<char*>(realPluginPath.data());
-    argv[3] = 0;
+    GUniquePtr<gchar> socket(g_strdup_printf("%d", sockets[0]));
 
-    GOwnPtr<GError> error;
-    int spawnFlags = G_SPAWN_LEAVE_DESCRIPTORS_OPEN | G_SPAWN_DO_NOT_REAP_CHILD;
-    if (!g_spawn_async(0, argv, 0, static_cast<GSpawnFlags>(spawnFlags), childSetupFunction, GINT_TO_POINTER(sockets[1]), &pid, &error.outPtr())) {
+    unsigned nargs = 4; // size of the argv array for g_spawn_async()
+
+#ifndef NDEBUG
+    Vector<CString> prefixArgs;
+    if (!m_launchOptions.processCmdPrefix.isNull()) {
+        Vector<String> splitArgs;
+        m_launchOptions.processCmdPrefix.split(' ', splitArgs);
+        for (auto it = splitArgs.begin(); it != splitArgs.end(); it++)
+            prefixArgs.append(it->utf8());
+        nargs += prefixArgs.size();
+    }
+#endif
+
+    char** argv = g_newa(char*, nargs);
+    unsigned i = 0;
+#ifndef NDEBUG
+    // If there's a prefix command, put it before the rest of the args.
+    for (auto it = prefixArgs.begin(); it != prefixArgs.end(); it++)
+        argv[i++] = const_cast<char*>(it->data());
+#endif
+    argv[i++] = const_cast<char*>(realExecutablePath.data());
+    argv[i++] = socket.get();
+    argv[i++] = const_cast<char*>(realPluginPath.data());
+    argv[i++] = 0;
+
+    GUniqueOutPtr<GError> error;
+    if (!g_spawn_async(0, argv, 0, G_SPAWN_LEAVE_DESCRIPTORS_OPEN, childSetupFunction, GINT_TO_POINTER(sockets[1]), &pid, &error.outPtr())) {
         g_printerr("Unable to fork a new WebProcess: %s.\n", error->message);
         ASSERT_NOT_REACHED();
     }
@@ -109,20 +131,22 @@ void ProcessLauncher::launchProcess()
     close(sockets[0]);
     m_processIdentifier = pid;
 
-    // Monitor the child process, it calls waitpid to prevent the child process from becomming a zombie,
-    // and it allows us to close the socket when the child process crashes.
-    g_child_watch_add(m_processIdentifier, childFinishedFunction, GINT_TO_POINTER(sockets[1]));
-
     // We've finished launching the process, message back to the main run loop.
     RunLoop::main()->dispatch(bind(&ProcessLauncher::didFinishLaunchingProcess, this, m_processIdentifier, sockets[1]));
 }
 
 void ProcessLauncher::terminateProcess()
-{   
+{
+    if (m_isLaunching) {
+        invalidate();
+        return;
+    }
+
     if (!m_processIdentifier)
         return;
 
     kill(m_processIdentifier, SIGKILL);
+    m_processIdentifier = 0;
 }
 
 void ProcessLauncher::platformInvalidate()
