@@ -26,12 +26,15 @@
 #include "config.h"
 #include "WebProcessProxy.h"
 
+#include "APIFrameHandle.h"
+#include "CustomProtocolManagerProxyMessages.h"
 #include "DataReference.h"
 #include "DownloadProxyMap.h"
 #include "PluginInfoStore.h"
 #include "PluginProcessManager.h"
 #include "TextChecker.h"
 #include "TextCheckerState.h"
+#include "UserData.h"
 #include "WebBackForwardListItem.h"
 #include "WebContext.h"
 #include "WebNavigationDataStore.h"
@@ -40,29 +43,23 @@
 #include "WebPluginSiteDataManager.h"
 #include "WebProcessMessages.h"
 #include "WebProcessProxyMessages.h"
-#include <WebCore/KURL.h>
+#include <WebCore/SuddenTermination.h>
+#include <WebCore/URL.h>
 #include <stdio.h>
-#include <wtf/MainThread.h>
+#include <wtf/NeverDestroyed.h>
+#include <wtf/RunLoop.h>
 #include <wtf/text/CString.h>
 #include <wtf/text/WTFString.h>
 
 #if PLATFORM(MAC)
-#include "SimplePDFPlugin.h"
-#if ENABLE(PDFKIT_PLUGIN)
 #include "PDFPlugin.h"
 #endif
-#endif
 
-#if ENABLE(CUSTOM_PROTOCOLS)
-#include "CustomProtocolManagerProxyMessages.h"
-#endif
-
-#if USE(SECURITY_FRAMEWORK)
+#if ENABLE(SEC_ITEM_SHIM)
 #include "SecItemShimProxy.h"
 #endif
 
 using namespace WebCore;
-using namespace std;
 
 #define MESSAGE_CHECK(assertion) MESSAGE_CHECK_BASE(assertion, connection())
 #define MESSAGE_CHECK_URL(url) MESSAGE_CHECK_BASE(checkURLReceivedFromWebProcess(url), connection())
@@ -77,26 +74,27 @@ static uint64_t generatePageID()
 
 static WebProcessProxy::WebPageProxyMap& globalPageMap()
 {
-    ASSERT(isMainThread());
-    DEFINE_STATIC_LOCAL(WebProcessProxy::WebPageProxyMap, pageMap, ());
+    ASSERT(RunLoop::isMain());
+    static NeverDestroyed<WebProcessProxy::WebPageProxyMap> pageMap;
     return pageMap;
 }
 
-PassRefPtr<WebProcessProxy> WebProcessProxy::create(PassRefPtr<WebContext> context)
+PassRefPtr<WebProcessProxy> WebProcessProxy::create(WebContext& context)
 {
     return adoptRef(new WebProcessProxy(context));
 }
 
-WebProcessProxy::WebProcessProxy(PassRefPtr<WebContext> context)
+WebProcessProxy::WebProcessProxy(WebContext& context)
     : m_responsivenessTimer(this)
     , m_context(context)
     , m_mayHaveUniversalFileReadSandboxExtension(false)
 #if ENABLE(CUSTOM_PROTOCOLS)
-    , m_customProtocolManagerProxy(this)
+    , m_customProtocolManagerProxy(this, context)
 #endif
 #if PLATFORM(MAC)
     , m_processSuppressionEnabled(false)
 #endif
+    , m_numberOfTimesSuddenTerminationWasDisabled(0)
 {
     connect();
 }
@@ -105,6 +103,9 @@ WebProcessProxy::~WebProcessProxy()
 {
     if (m_webConnection)
         m_webConnection->invalidate();
+
+    while (m_numberOfTimesSuddenTerminationWasDisabled-- > 0)
+        WebCore::enableSuddenTermination();
 }
 
 void WebProcessProxy::getLaunchOptions(ProcessLauncher::LaunchOptions& launchOptions)
@@ -113,21 +114,27 @@ void WebProcessProxy::getLaunchOptions(ProcessLauncher::LaunchOptions& launchOpt
     platformGetLaunchOptions(launchOptions);
 }
 
-void WebProcessProxy::connectionWillOpen(CoreIPC::Connection* connection)
+void WebProcessProxy::connectionWillOpen(IPC::Connection* connection)
 {
     ASSERT(this->connection() == connection);
 
-#if USE(SECURITY_FRAMEWORK)
+#if ENABLE(SEC_ITEM_SHIM)
     SecItemShimProxy::shared().initializeConnection(connection);
 #endif
 
+    for (WebPageProxyMap::iterator it = m_pageMap.begin(), end = m_pageMap.end(); it != end; ++it)
+        it->value->connectionWillOpen(connection);
+
     m_context->processWillOpenConnection(this);
 }
 
-void WebProcessProxy::connectionWillClose(CoreIPC::Connection* connection)
+void WebProcessProxy::connectionWillClose(IPC::Connection* connection)
 {
     ASSERT(this->connection() == connection);
 
+    for (WebPageProxyMap::iterator it = m_pageMap.begin(), end = m_pageMap.end(); it != end; ++it)
+        it->value->connectionWillClose(connection);
+
     m_context->processWillCloseConnection(this);
 }
 
@@ -140,9 +147,9 @@ void WebProcessProxy::disconnect()
         m_webConnection = nullptr;
     }
 
-    m_responsivenessTimer.stop();
+    m_responsivenessTimer.invalidate();
 
-    Vector<RefPtr<WebFrameProxy> > frames;
+    Vector<RefPtr<WebFrameProxy>> frames;
     copyValuesToVector(m_frameMap, frames);
 
     for (size_t i = 0, size = frames.size(); i < size; ++i)
@@ -160,14 +167,14 @@ WebPageProxy* WebProcessProxy::webPage(uint64_t pageID)
     return globalPageMap().get(pageID);
 }
 
-PassRefPtr<WebPageProxy> WebProcessProxy::createWebPage(PageClient* pageClient, WebContext*, WebPageGroup* pageGroup)
+PassRefPtr<WebPageProxy> WebProcessProxy::createWebPage(PageClient& pageClient, const WebPageConfiguration& configuration)
 {
     uint64_t pageID = generatePageID();
-    RefPtr<WebPageProxy> webPage = WebPageProxy::create(pageClient, this, pageGroup, pageID);
+    RefPtr<WebPageProxy> webPage = WebPageProxy::create(pageClient, *this, pageID, configuration);
     m_pageMap.set(pageID, webPage.get());
     globalPageMap().set(pageID, webPage.get());
 #if PLATFORM(MAC)
-    if (pageIsProcessSuppressible(webPage.get()))
+    if (webPage->isProcessSuppressible())
         m_processSuppressiblePages.add(pageID);
     updateProcessSuppressionState();
 #endif
@@ -179,7 +186,7 @@ void WebProcessProxy::addExistingWebPage(WebPageProxy* webPage, uint64_t pageID)
     m_pageMap.set(pageID, webPage);
     globalPageMap().set(pageID, webPage);
 #if PLATFORM(MAC)
-    if (pageIsProcessSuppressible(webPage))
+    if (webPage->isProcessSuppressible())
         m_processSuppressiblePages.add(pageID);
     updateProcessSuppressionState();
 #endif
@@ -193,6 +200,13 @@ void WebProcessProxy::removeWebPage(uint64_t pageID)
     m_processSuppressiblePages.remove(pageID);
     updateProcessSuppressionState();
 #endif
+
+    // If this was the last WebPage open in that web process, and we have no other reason to keep it alive, let it go.
+    // We only allow this when using a network process, as otherwise the WebProcess needs to preserve its session state.
+    if (m_context->usesNetworkProcess() && canTerminateChildProcess()) {
+        abortProcessLaunchIfNeeded();
+        disconnect();
+    }
 }
 
 Vector<WebPageProxy*> WebProcessProxy::pages() const
@@ -204,7 +218,7 @@ Vector<WebPageProxy*> WebProcessProxy::pages() const
 
 WebBackForwardListItem* WebProcessProxy::webBackForwardItem(uint64_t itemID) const
 {
-    return m_backForwardListItemMap.get(itemID).get();
+    return m_backForwardListItemMap.get(itemID);
 }
 
 void WebProcessProxy::registerNewWebBackForwardListItem(WebBackForwardListItem* item)
@@ -218,13 +232,13 @@ void WebProcessProxy::registerNewWebBackForwardListItem(WebBackForwardListItem*
 
 void WebProcessProxy::assumeReadAccessToBaseURL(const String& urlString)
 {
-    KURL url(KURL(), urlString);
+    URL url(URL(), urlString);
     if (!url.isLocalFile())
         return;
 
     // There's a chance that urlString does not point to a directory.
     // Get url's base URL to add to m_localPathsWithAssumedReadAccess.
-    KURL baseURL(KURL(), url.baseAsString());
+    URL baseURL(URL(), url.baseAsString());
     
     // Client loads an alternate string. This doesn't grant universal file read, but the web process is assumed
     // to have read access to this directory already.
@@ -233,10 +247,10 @@ void WebProcessProxy::assumeReadAccessToBaseURL(const String& urlString)
 
 bool WebProcessProxy::checkURLReceivedFromWebProcess(const String& urlString)
 {
-    return checkURLReceivedFromWebProcess(KURL(KURL(), urlString));
+    return checkURLReceivedFromWebProcess(URL(URL(), urlString));
 }
 
-bool WebProcessProxy::checkURLReceivedFromWebProcess(const KURL& url)
+bool WebProcessProxy::checkURLReceivedFromWebProcess(const URL& url)
 {
     // FIXME: Consider checking that the URL is valid. Currently, WebProcess sends invalid URLs in many cases, but it probably doesn't have good reasons to do that.
 
@@ -249,7 +263,7 @@ bool WebProcessProxy::checkURLReceivedFromWebProcess(const KURL& url)
         return true;
 
     // If we loaded a string with a file base URL before, loading resources from that subdirectory is fine.
-    // There are no ".." components, because all URLs received from WebProcess are parsed with KURL, which removes those.
+    // There are no ".." components, because all URLs received from WebProcess are parsed with URL, which removes those.
     String path = url.fileSystemPath();
     for (HashSet<String>::const_iterator iter = m_localPathsWithAssumedReadAccess.begin(); iter != m_localPathsWithAssumedReadAccess.end(); ++iter) {
         if (path.startsWith(*iter))
@@ -259,9 +273,9 @@ bool WebProcessProxy::checkURLReceivedFromWebProcess(const KURL& url)
     // Items in back/forward list have been already checked.
     // One case where we don't have sandbox extensions for file URLs in b/f list is if the list has been reinstated after a crash or a browser restart.
     for (WebBackForwardListItemMap::iterator iter = m_backForwardListItemMap.begin(), end = m_backForwardListItemMap.end(); iter != end; ++iter) {
-        if (KURL(KURL(), iter->value->url()).fileSystemPath() == path)
+        if (URL(URL(), iter->value->url()).fileSystemPath() == path)
             return true;
-        if (KURL(KURL(), iter->value->originalURL()).fileSystemPath() == path)
+        if (URL(URL(), iter->value->originalURL()).fileSystemPath() == path)
             return true;
     }
 
@@ -277,12 +291,12 @@ bool WebProcessProxy::fullKeyboardAccessEnabled()
 }
 #endif
 
-void WebProcessProxy::addBackForwardItem(uint64_t itemID, const String& originalURL, const String& url, const String& title, const CoreIPC::DataReference& backForwardData)
+void WebProcessProxy::addBackForwardItem(uint64_t itemID, const String& originalURL, const String& url, const String& title, const IPC::DataReference& backForwardData)
 {
     MESSAGE_CHECK_URL(originalURL);
     MESSAGE_CHECK_URL(url);
 
-    WebBackForwardListItemMap::AddResult result = m_backForwardListItemMap.add(itemID, 0);
+    WebBackForwardListItemMap::AddResult result = m_backForwardListItemMap.add(itemID, nullptr);
     if (result.isNewEntry) {
         result.iterator->value = WebBackForwardListItem::create(originalURL, url, title, backForwardData.data(), backForwardData.size(), itemID);
         return;
@@ -296,7 +310,7 @@ void WebProcessProxy::addBackForwardItem(uint64_t itemID, const String& original
 }
 
 #if ENABLE(NETSCAPE_PLUGIN_API)
-void WebProcessProxy::getPlugins(bool refresh, Vector<PluginInfo>& plugins)
+void WebProcessProxy::getPlugins(bool refresh, Vector<PluginInfo>& plugins, Vector<PluginInfo>& applicationPlugins)
 {
     if (refresh)
         m_context->pluginInfoStore().refresh();
@@ -305,45 +319,25 @@ void WebProcessProxy::getPlugins(bool refresh, Vector<PluginInfo>& plugins)
     for (size_t i = 0; i < pluginModules.size(); ++i)
         plugins.append(pluginModules[i].info);
 
-#if PLATFORM(MAC)
+#if ENABLE(PDFKIT_PLUGIN)
     // Add built-in PDF last, so that it's not used when a real plug-in is installed.
     if (!m_context->omitPDFSupport()) {
-#if ENABLE(PDFKIT_PLUGIN)
         plugins.append(PDFPlugin::pluginInfo());
-#endif
-        plugins.append(SimplePDFPlugin::pluginInfo());
+        applicationPlugins.append(PDFPlugin::pluginInfo());
     }
+#else
+    UNUSED_PARAM(applicationPlugins);
 #endif
 }
 #endif // ENABLE(NETSCAPE_PLUGIN_API)
 
-#if ENABLE(PLUGIN_PROCESS)
-void WebProcessProxy::getPluginProcessConnection(const String& pluginPath, uint32_t processType, PassRefPtr<Messages::WebProcessProxy::GetPluginProcessConnection::DelayedReply> reply)
+#if ENABLE(NETSCAPE_PLUGIN_API)
+void WebProcessProxy::getPluginProcessConnection(uint64_t pluginProcessToken, PassRefPtr<Messages::WebProcessProxy::GetPluginProcessConnection::DelayedReply> reply)
 {
-    PluginProcessManager::shared().getPluginProcessConnection(m_context->pluginInfoStore(), pluginPath, static_cast<PluginProcess::Type>(processType), reply);
+    PluginProcessManager::shared().getPluginProcessConnection(pluginProcessToken, reply);
 }
-
-#elif ENABLE(NETSCAPE_PLUGIN_API)
-
-void WebProcessProxy::didGetSitesWithPluginData(const Vector<String>& sites, uint64_t callbackID)
-{
-    m_context->pluginSiteDataManager()->didGetSitesWithData(sites, callbackID);
-}
-
-void WebProcessProxy::didClearPluginSiteData(uint64_t callbackID)
-{
-    m_context->pluginSiteDataManager()->didClearSiteData(callbackID);
-}
-
 #endif
 
-#if ENABLE(SHARED_WORKER_PROCESS)
-void WebProcessProxy::getSharedWorkerProcessConnection(const String& /* url */, const String& /* name */, PassRefPtr<Messages::WebProcessProxy::GetSharedWorkerProcessConnection::DelayedReply>)
-{
-    // FIXME: Implement
-}
-#endif // ENABLE(SHARED_WORKER_PROCESS)
-
 #if ENABLE(NETWORK_PROCESS)
 void WebProcessProxy::getNetworkProcessConnection(PassRefPtr<Messages::WebProcessProxy::GetNetworkProcessConnection::DelayedReply> reply)
 {
@@ -351,7 +345,14 @@ void WebProcessProxy::getNetworkProcessConnection(PassRefPtr<Messages::WebProces
 }
 #endif // ENABLE(NETWORK_PROCESS)
 
-void WebProcessProxy::didReceiveMessage(CoreIPC::Connection* connection, CoreIPC::MessageDecoder& decoder)
+#if ENABLE(DATABASE_PROCESS)
+void WebProcessProxy::getDatabaseProcessConnection(PassRefPtr<Messages::WebProcessProxy::GetDatabaseProcessConnection::DelayedReply> reply)
+{
+    m_context->getDatabaseProcessConnection(reply);
+}
+#endif // ENABLE(DATABASE_PROCESS)
+
+void WebProcessProxy::didReceiveMessage(IPC::Connection* connection, IPC::MessageDecoder& decoder)
 {
     if (dispatchMessage(connection, decoder))
         return;
@@ -367,7 +368,7 @@ void WebProcessProxy::didReceiveMessage(CoreIPC::Connection* connection, CoreIPC
     // FIXME: Add unhandled message logging.
 }
 
-void WebProcessProxy::didReceiveSyncMessage(CoreIPC::Connection* connection, CoreIPC::MessageDecoder& decoder, OwnPtr<CoreIPC::MessageEncoder>& replyEncoder)
+void WebProcessProxy::didReceiveSyncMessage(IPC::Connection* connection, IPC::MessageDecoder& decoder, std::unique_ptr<IPC::MessageEncoder>& replyEncoder)
 {
     if (dispatchSyncMessage(connection, decoder, replyEncoder))
         return;
@@ -383,15 +384,15 @@ void WebProcessProxy::didReceiveSyncMessage(CoreIPC::Connection* connection, Cor
     // FIXME: Add unhandled message logging.
 }
 
-void WebProcessProxy::didClose(CoreIPC::Connection*)
+void WebProcessProxy::didClose(IPC::Connection*)
 {
     // Protect ourselves, as the call to disconnect() below may otherwise cause us
     // to be deleted before we can finish our work.
-    RefPtr<WebProcessProxy> protect(this);
+    Ref<WebProcessProxy> protect(*this);
 
     webConnection()->didClose();
 
-    Vector<RefPtr<WebPageProxy> > pages;
+    Vector<RefPtr<WebPageProxy>> pages;
     copyValuesToVector(m_pageMap, pages);
 
     disconnect();
@@ -401,21 +402,23 @@ void WebProcessProxy::didClose(CoreIPC::Connection*)
 
 }
 
-void WebProcessProxy::didReceiveInvalidMessage(CoreIPC::Connection* connection, CoreIPC::StringReference messageReceiverName, CoreIPC::StringReference messageName)
+void WebProcessProxy::didReceiveInvalidMessage(IPC::Connection* connection, IPC::StringReference messageReceiverName, IPC::StringReference messageName)
 {
     WTFLogAlways("Received an invalid message \"%s.%s\" from the web process.\n", messageReceiverName.toString().data(), messageName.toString().data());
 
-    // Terminate the WebProcesses.
+    WebContext::didReceiveInvalidMessage(messageReceiverName, messageName);
+
+    // Terminate the WebProcess.
     terminate();
 
-    // Since we've invalidated the connection we'll never get a CoreIPC::Connection::Client::didClose
+    // Since we've invalidated the connection we'll never get a IPC::Connection::Client::didClose
     // callback so we'll explicitly call it here instead.
     didClose(connection);
 }
 
 void WebProcessProxy::didBecomeUnresponsive(ResponsivenessTimer*)
 {
-    Vector<RefPtr<WebPageProxy> > pages;
+    Vector<RefPtr<WebPageProxy>> pages;
     copyValuesToVector(m_pageMap, pages);
     for (size_t i = 0, size = pages.size(); i < size; ++i)
         pages[i]->processDidBecomeUnresponsive();
@@ -423,7 +426,7 @@ void WebProcessProxy::didBecomeUnresponsive(ResponsivenessTimer*)
 
 void WebProcessProxy::interactionOccurredWhileUnresponsive(ResponsivenessTimer*)
 {
-    Vector<RefPtr<WebPageProxy> > pages;
+    Vector<RefPtr<WebPageProxy>> pages;
     copyValuesToVector(m_pageMap, pages);
     for (size_t i = 0, size = pages.size(); i < size; ++i)
         pages[i]->interactionOccurredWhileProcessUnresponsive();
@@ -431,13 +434,13 @@ void WebProcessProxy::interactionOccurredWhileUnresponsive(ResponsivenessTimer*)
 
 void WebProcessProxy::didBecomeResponsive(ResponsivenessTimer*)
 {
-    Vector<RefPtr<WebPageProxy> > pages;
+    Vector<RefPtr<WebPageProxy>> pages;
     copyValuesToVector(m_pageMap, pages);
     for (size_t i = 0, size = pages.size(); i < size; ++i)
         pages[i]->processDidBecomeResponsive();
 }
 
-void WebProcessProxy::didFinishLaunching(ProcessLauncher* launcher, CoreIPC::Connection::Identifier connectionIdentifier)
+void WebProcessProxy::didFinishLaunching(ProcessLauncher* launcher, IPC::Connection::Identifier connectionIdentifier)
 {
     ChildProcessProxy::didFinishLaunching(launcher, connectionIdentifier);
 
@@ -455,7 +458,7 @@ WebFrameProxy* WebProcessProxy::webFrame(uint64_t frameID) const
     if (!WebFrameProxyMap::isValidKey(frameID))
         return 0;
 
-    return m_frameMap.get(frameID).get();
+    return m_frameMap.get(frameID);
 }
 
 bool WebProcessProxy::canCreateFrame(uint64_t frameID) const
@@ -480,7 +483,7 @@ void WebProcessProxy::didDestroyFrame(uint64_t frameID)
 
 void WebProcessProxy::disconnectFramesFromPage(WebPageProxy* page)
 {
-    Vector<RefPtr<WebFrameProxy> > frames;
+    Vector<RefPtr<WebFrameProxy>> frames;
     copyValuesToVector(m_frameMap, frames);
     for (size_t i = 0, size = frames.size(); i < size; ++i) {
         if (frames[i]->page() == page)
@@ -491,24 +494,34 @@ void WebProcessProxy::disconnectFramesFromPage(WebPageProxy* page)
 size_t WebProcessProxy::frameCountInPage(WebPageProxy* page) const
 {
     size_t result = 0;
-    for (HashMap<uint64_t, RefPtr<WebFrameProxy> >::const_iterator iter = m_frameMap.begin(); iter != m_frameMap.end(); ++iter) {
+    for (HashMap<uint64_t, RefPtr<WebFrameProxy>>::const_iterator iter = m_frameMap.begin(); iter != m_frameMap.end(); ++iter) {
         if (iter->value->page() == page)
             ++result;
     }
     return result;
 }
 
+bool WebProcessProxy::canTerminateChildProcess()
+{
+    if (!m_pageMap.isEmpty())
+        return false;
+
+    if (m_downloadProxyMap && !m_downloadProxyMap->isEmpty())
+        return false;
+
+    if (!m_context->shouldTerminate(this))
+        return false;
+
+    return true;
+}
+
 void WebProcessProxy::shouldTerminate(bool& shouldTerminate)
 {
-    if (!m_pageMap.isEmpty() || (m_downloadProxyMap && !m_downloadProxyMap->isEmpty()) || !m_context->shouldTerminate(this)) {
-        shouldTerminate = false;
-        return;
+    shouldTerminate = canTerminateChildProcess();
+    if (shouldTerminate) {
+        // We know that the web process is going to terminate so disconnect it from the context.
+        disconnect();
     }
-
-    shouldTerminate = true;
-
-    // We know that the web process is going to terminate so disconnect it from the context.
-    disconnect();
 }
 
 void WebProcessProxy::updateTextCheckerState()
@@ -539,7 +552,7 @@ void WebProcessProxy::didNavigateWithNavigationData(uint64_t pageID, const WebNa
     MESSAGE_CHECK(frame);
     MESSAGE_CHECK(frame->page() == page);
     
-    m_context->historyClient().didNavigateWithNavigationData(m_context.get(), page, store, frame);
+    m_context->historyClient().didNavigateWithNavigationData(&m_context.get(), page, store, frame);
 }
 
 void WebProcessProxy::didPerformClientRedirect(uint64_t pageID, const String& sourceURLString, const String& destinationURLString, uint64_t frameID)
@@ -557,7 +570,7 @@ void WebProcessProxy::didPerformClientRedirect(uint64_t pageID, const String& so
     MESSAGE_CHECK_URL(sourceURLString);
     MESSAGE_CHECK_URL(destinationURLString);
 
-    m_context->historyClient().didPerformClientRedirect(m_context.get(), page, sourceURLString, destinationURLString, frame);
+    m_context->historyClient().didPerformClientRedirect(&m_context.get(), page, sourceURLString, destinationURLString, frame);
 }
 
 void WebProcessProxy::didPerformServerRedirect(uint64_t pageID, const String& sourceURLString, const String& destinationURLString, uint64_t frameID)
@@ -575,7 +588,7 @@ void WebProcessProxy::didPerformServerRedirect(uint64_t pageID, const String& so
     MESSAGE_CHECK_URL(sourceURLString);
     MESSAGE_CHECK_URL(destinationURLString);
 
-    m_context->historyClient().didPerformServerRedirect(m_context.get(), page, sourceURLString, destinationURLString, frame);
+    m_context->historyClient().didPerformServerRedirect(&m_context.get(), page, sourceURLString, destinationURLString, frame);
 }
 
 void WebProcessProxy::didUpdateHistoryTitle(uint64_t pageID, const String& title, const String& url, uint64_t frameID)
@@ -589,13 +602,13 @@ void WebProcessProxy::didUpdateHistoryTitle(uint64_t pageID, const String& title
     MESSAGE_CHECK(frame->page() == page);
     MESSAGE_CHECK_URL(url);
 
-    m_context->historyClient().didUpdateHistoryTitle(m_context.get(), page, title, url, frame);
+    m_context->historyClient().didUpdateHistoryTitle(&m_context.get(), page, title, url, frame);
 }
 
-void WebProcessProxy::pageVisibilityChanged(WebKit::WebPageProxy *page)
+void WebProcessProxy::pageSuppressibilityChanged(WebKit::WebPageProxy *page)
 {
 #if PLATFORM(MAC)
-    if (pageIsProcessSuppressible(page))
+    if (page->isProcessSuppressible())
         m_processSuppressiblePages.add(page->pageID());
     else
         m_processSuppressiblePages.remove(page->pageID());
@@ -608,7 +621,7 @@ void WebProcessProxy::pageVisibilityChanged(WebKit::WebPageProxy *page)
 void WebProcessProxy::pagePreferencesChanged(WebKit::WebPageProxy *page)
 {
 #if PLATFORM(MAC)
-    if (pageIsProcessSuppressible(page))
+    if (page->isProcessSuppressible())
         m_processSuppressiblePages.add(page->pageID());
     else
         m_processSuppressiblePages.remove(page->pageID());
@@ -618,4 +631,68 @@ void WebProcessProxy::pagePreferencesChanged(WebKit::WebPageProxy *page)
 #endif
 }
 
+void WebProcessProxy::didSaveToPageCache()
+{
+    m_context->processDidCachePage(this);
+}
+
+void WebProcessProxy::releasePageCache()
+{
+    if (canSendMessage())
+        send(Messages::WebProcess::ReleasePageCache(), 0);
+}
+
+void WebProcessProxy::windowServerConnectionStateChanged()
+{
+    for (const auto& page : m_pageMap.values())
+        page->viewStateDidChange(ViewState::IsVisuallyIdle);
+}
+
+void WebProcessProxy::requestTermination()
+{
+    if (!isValid())
+        return;
+
+    ChildProcessProxy::terminate();
+
+    if (webConnection())
+        webConnection()->didClose();
+
+    disconnect();
+}
+
+void WebProcessProxy::enableSuddenTermination()
+{
+    if (!isValid())
+        return;
+
+    ASSERT(m_numberOfTimesSuddenTerminationWasDisabled);
+    WebCore::enableSuddenTermination();
+    --m_numberOfTimesSuddenTerminationWasDisabled;
+}
+
+void WebProcessProxy::disableSuddenTermination()
+{
+    if (!isValid())
+        return;
+
+    WebCore::disableSuddenTermination();
+    ++m_numberOfTimesSuddenTerminationWasDisabled;
+}
+
+RefPtr<API::Object> WebProcessProxy::apiObjectByConvertingToHandles(API::Object* object)
+{
+    return UserData::transform(object, [](const API::Object& object) -> RefPtr<API::Object> {
+        switch (object.type()) {
+        case API::Object::Type::Frame: {
+            auto& frame = static_cast<const WebFrameProxy&>(object);
+            return API::FrameHandle::create(frame.frameID());
+        }
+
+        default:
+            return nullptr;
+        }
+    });
+}
+
 } // namespace WebKit
