@@ -34,9 +34,7 @@
 #include <WebCore/GraphicsContext.h>
 #include <WebCore/NotImplemented.h>
 
-#if PLATFORM(QT)
-#include <WebCore/QtX11ImageConversion.h>
-#elif PLATFORM(GTK)
+#if PLATFORM(GTK)
 #include <gtk/gtk.h>
 #ifndef GTK_API_VERSION_2
 #include <gtk/gtkx.h>
@@ -59,33 +57,7 @@ namespace WebKit {
 
 static Display* getPluginDisplay()
 {
-#if PLATFORM(QT)
-    // At the moment, we only support gdk based plugins (like Flash) that use a different X connection.
-    // The code below has the same effect as this one:
-    // Display *gdkDisplay = gdk_x11_display_get_xdisplay(gdk_display_get_default());
-
-    QLibrary library(QLatin1String("libgdk-x11-2.0"), 0);
-    if (!library.load())
-        return 0;
-
-    typedef void *(*gdk_init_check_ptr)(void*, void*);
-    gdk_init_check_ptr gdk_init_check = (gdk_init_check_ptr)library.resolve("gdk_init_check");
-    if (!gdk_init_check)
-        return 0;
-
-    typedef void *(*gdk_display_get_default_ptr)();
-    gdk_display_get_default_ptr gdk_display_get_default = (gdk_display_get_default_ptr)library.resolve("gdk_display_get_default");
-    if (!gdk_display_get_default)
-        return 0;
-
-    typedef void *(*gdk_x11_display_get_xdisplay_ptr)(void *);
-    gdk_x11_display_get_xdisplay_ptr gdk_x11_display_get_xdisplay = (gdk_x11_display_get_xdisplay_ptr)library.resolve("gdk_x11_display_get_xdisplay");
-    if (!gdk_x11_display_get_xdisplay)
-        return 0;
-
-    gdk_init_check(0, 0);
-    return (Display*)gdk_x11_display_get_xdisplay(gdk_display_get_default());
-#elif PLATFORM(GTK)
+#if PLATFORM(GTK)
     // Since we're a gdk/gtk app, we'll (probably?) have the same X connection as any gdk-based
     // plugins, so we can return that. We might want to add other implementations here later.
     return GDK_DISPLAY_XDISPLAY(gdk_display_get_default());
@@ -98,9 +70,7 @@ static Display* getPluginDisplay()
 
 static inline int x11Screen()
 {
-#if PLATFORM(QT)
-    return XDefaultScreen(NetscapePlugin::x11HostDisplay());
-#elif PLATFORM(GTK)
+#if PLATFORM(GTK)
     return gdk_screen_get_number(gdk_screen_get_default());
 #elif PLATFORM(EFL) && defined(HAVE_ECORE_X)
     return ecore_x_screen_index_get(ecore_x_default_screen_get());
@@ -111,9 +81,7 @@ static inline int x11Screen()
 
 static inline int displayDepth()
 {
-#if PLATFORM(QT)
-    return XDefaultDepth(NetscapePlugin::x11HostDisplay(), x11Screen());
-#elif PLATFORM(GTK)
+#if PLATFORM(GTK)
     return gdk_visual_get_depth(gdk_screen_get_system_visual(gdk_screen_get_default()));
 #elif PLATFORM(EFL) && defined(HAVE_ECORE_X)
     return ecore_x_default_depth_get(NetscapePlugin::x11HostDisplay(), ecore_x_default_screen_get());
@@ -124,9 +92,7 @@ static inline int displayDepth()
 
 static inline unsigned long rootWindowID()
 {
-#if PLATFORM(QT)
-    return XDefaultRootWindow(NetscapePlugin::x11HostDisplay());
-#elif PLATFORM(GTK)
+#if PLATFORM(GTK)
     return GDK_ROOT_WINDOW();
 #elif PLATFORM(EFL) && defined(HAVE_ECORE_X)
     return ecore_x_window_root_first_get();
@@ -148,14 +114,7 @@ static bool moduleMixesGtkSymbols(Module* module)
 
 Display* NetscapePlugin::x11HostDisplay()
 {
-#if PLATFORM(QT)
-    static Display* dedicatedDisplay = 0;
-    if (!dedicatedDisplay)
-        dedicatedDisplay = XOpenDisplay(0);
-
-    ASSERT(dedicatedDisplay);
-    return dedicatedDisplay;
-#elif PLATFORM(GTK)
+#if PLATFORM(GTK)
     return GDK_DISPLAY_XDISPLAY(gdk_display_get_default());
 #elif PLATFORM(EFL) && defined(HAVE_ECORE_X)
     return static_cast<Display*>(ecore_x_display_get());
@@ -221,9 +180,7 @@ bool NetscapePlugin::platformPostInitializeWindowless()
     m_npWindow.window = 0;
 
     int depth = displayDepth();
-#if PLATFORM(QT)
-    ASSERT(depth == 16 || depth == 24 || depth == 32);
-#endif
+
     NPSetWindowCallbackStruct* callbackStruct = static_cast<NPSetWindowCallbackStruct*>(m_npWindow.ws_info);
     callbackStruct->display = display;
     callbackStruct->depth = depth;
@@ -376,14 +333,7 @@ void NetscapePlugin::platformPaint(GraphicsContext* context, const IntRect& dirt
     if (m_pluginDisplay != x11HostDisplay())
         XSync(m_pluginDisplay, false);
 
-#if PLATFORM(QT)
-    XImage* xImage = XGetImage(NetscapePlugin::x11HostDisplay(), m_drawable, exposedRect.x(), exposedRect.y(),
-                               exposedRect.width(), exposedRect.height(), ULONG_MAX, ZPixmap);
-    QPainter* painter = context->platformContext();
-    painter->drawImage(QPoint(exposedRect.x(), exposedRect.y()), qimageFromXImage(xImage), exposedRect);
-
-    XDestroyImage(xImage);
-#elif PLATFORM(GTK) || (PLATFORM(EFL) && USE(CAIRO))
+#if PLATFORM(GTK) || (PLATFORM(EFL) && USE(CAIRO))
     RefPtr<cairo_surface_t> drawableSurface = adoptRef(cairo_xlib_surface_create(m_pluginDisplay,
                                                                                  m_drawable,
                                                                                  static_cast<NPSetWindowCallbackStruct*>(m_npWindow.ws_info)->visual,
