@@ -27,26 +27,30 @@
 #define DrawingArea_h
 
 #include "DrawingAreaInfo.h"
-#include <WebCore/FloatPoint.h>
+#include "LayerTreeContext.h"
+#include <WebCore/FloatRect.h>
 #include <WebCore/IntRect.h>
+#include <WebCore/ViewState.h>
+#include <functional>
 #include <wtf/Forward.h>
 #include <wtf/Noncopyable.h>
-#include <wtf/PassOwnPtr.h>
 
-namespace CoreIPC {
-    class Connection;
-    class MessageDecoder;
+namespace IPC {
+class Connection;
+class MessageDecoder;
 }
 
 namespace WebCore {
-    class GraphicsLayer;
-    class GraphicsLayerFactory;
+class FrameView;
+class GraphicsLayer;
+class GraphicsLayerFactory;
 }
 
 namespace WebKit {
 
 struct ColorSpaceData;
 class LayerTreeHost;
+class PageOverlay;
 class WebPage;
 struct WebPageCreationParameters;
 struct WebPreferencesStore;
@@ -55,13 +59,16 @@ class DrawingArea {
     WTF_MAKE_NONCOPYABLE(DrawingArea);
 
 public:
-    static PassOwnPtr<DrawingArea> create(WebPage*, const WebPageCreationParameters&);
+    static std::unique_ptr<DrawingArea> create(WebPage*, const WebPageCreationParameters&);
     virtual ~DrawingArea();
     
-    void didReceiveDrawingAreaMessage(CoreIPC::Connection*, CoreIPC::MessageDecoder&);
+    DrawingAreaType type() const { return m_type; }
+    
+    void didReceiveDrawingAreaMessage(IPC::Connection*, IPC::MessageDecoder&);
 
-    virtual void setNeedsDisplay(const WebCore::IntRect&) = 0;
-    virtual void scroll(const WebCore::IntRect& scrollRect, const WebCore::IntSize& scrollOffset) = 0;
+    virtual void setNeedsDisplay() = 0;
+    virtual void setNeedsDisplayInRect(const WebCore::IntRect&) = 0;
+    virtual void scroll(const WebCore::IntRect& scrollRect, const WebCore::IntSize& scrollDelta) = 0;
 
     // FIXME: These should be pure virtual.
     virtual void pageBackgroundTransparencyChanged() { }
@@ -71,21 +78,29 @@ public:
     virtual bool layerTreeStateIsFrozen() const { return false; }
     virtual LayerTreeHost* layerTreeHost() const { return 0; }
 
-    virtual void didInstallPageOverlay() { }
-    virtual void didUninstallPageOverlay() { }
-    virtual void setPageOverlayNeedsDisplay(const WebCore::IntRect&) { }
-    virtual void setPageOverlayOpacity(float) { }
-    // If this function returns false, PageOverlay should apply opacity when painting.
-    virtual bool pageOverlayShouldApplyFadeWhenPainting() const { return true; }
-    virtual void pageCustomRepresentationChanged() { }
+    virtual void didInstallPageOverlay(PageOverlay*) { }
+    virtual void didUninstallPageOverlay(PageOverlay*) { }
+    virtual void setPageOverlayNeedsDisplay(PageOverlay*, const WebCore::IntRect&) { }
+    virtual void setPageOverlayOpacity(PageOverlay*, float) { }
+    virtual void clearPageOverlay(PageOverlay*) { }
 
     virtual void setPaintingEnabled(bool) { }
     virtual void updatePreferences(const WebPreferencesStore&) { }
     virtual void mainFrameContentSizeChanged(const WebCore::IntSize&) { }
 
-    virtual void setExposedRect(const WebCore::IntRect&) { }
+#if PLATFORM(MAC)
+    virtual void setExposedRect(const WebCore::FloatRect&) = 0;
+    virtual WebCore::FloatRect exposedRect() const = 0;
+    virtual void setCustomFixedPositionRect(const WebCore::FloatRect&) = 0;
+#endif
     virtual void mainFrameScrollabilityChanged(bool) { }
 
+    virtual bool supportsAsyncScrolling() { return false; }
+
+    virtual void didChangeScrollOffsetForAnyFrame() { }
+
+    virtual bool shouldUseTiledBackingForFrameView(const WebCore::FrameView*) { return false; }
+
 #if USE(ACCELERATED_COMPOSITING)
     virtual WebCore::GraphicsLayerFactory* graphicsLayerFactory() { return 0; }
     virtual void setRootCompositingLayer(WebCore::GraphicsLayer*) = 0;
@@ -93,10 +108,13 @@ public:
 #endif
 
 #if USE(COORDINATED_GRAPHICS)
-    virtual void didReceiveCoordinatedLayerTreeHostMessage(CoreIPC::Connection*, CoreIPC::MessageDecoder&) = 0;
+    virtual void didReceiveCoordinatedLayerTreeHostMessage(IPC::Connection*, IPC::MessageDecoder&) = 0;
 #endif
 
-    virtual void dispatchAfterEnsuringUpdatedScrollPosition(const Function<void ()>&);
+    virtual void dispatchAfterEnsuringUpdatedScrollPosition(std::function<void ()>);
+
+    virtual void viewStateDidChange(WebCore::ViewState::Flags) { }
+    virtual void setLayerHostingMode(LayerHostingMode) { }
 
 protected:
     DrawingArea(DrawingAreaType, WebPage*);
@@ -105,23 +123,26 @@ protected:
     WebPage* m_webPage;
 
 private:
-    // CoreIPC message handlers.
+    // Message handlers.
     // FIXME: These should be pure virtual.
     virtual void updateBackingStoreState(uint64_t /*backingStoreStateID*/, bool /*respondImmediately*/, float /*deviceScaleFactor*/, const WebCore::IntSize& /*size*/, 
                                          const WebCore::IntSize& /*scrollOffset*/) { }
     virtual void didUpdate() { }
-    virtual void suspendPainting() { }
-    virtual void resumePainting() { }
-    virtual void setLayerHostingMode(uint32_t) { }
 
 #if PLATFORM(MAC)
     // Used by TiledCoreAnimationDrawingArea.
-    virtual void updateGeometry(const WebCore::IntSize& viewSize) { }
+    virtual void updateGeometry(const WebCore::IntSize& viewSize, const WebCore::IntSize& layerPosition) { }
     virtual void setDeviceScaleFactor(float) { }
     virtual void setColorSpace(const ColorSpaceData&) { }
+
+    virtual void adjustTransientZoom(double scale, WebCore::FloatPoint origin) { }
+    virtual void commitTransientZoom(double scale, WebCore::FloatPoint origin) { }
 #endif
 };
 
+#define DRAWING_AREA_TYPE_CASTS(ToValueTypeName, predicate) \
+    TYPE_CASTS_BASE(ToValueTypeName, DrawingArea, value, value->predicate, value.predicate)
+
 } // namespace WebKit
 
 #endif // DrawingArea_h
