@@ -48,8 +48,8 @@
 #include <wtf/text/CString.h>
 #include <wtf/text/StringBuilder.h>
 
-#if PLATFORM(QT)
-#include "DumpRenderTreeSupportQt.h"
+#if USE(CF)
+#include "WebArchiveDumpSupport.h"
 #endif
 
 using namespace std;
@@ -234,44 +234,24 @@ static const char divider = '/';
 static inline WTF::String pathSuitableForTestResult(WKURLRef fileUrl)
 {
     if (!fileUrl)
-        return String();
+        return "(null)";
 
     WKRetainPtr<WKStringRef> schemeString = adoptWK(WKURLCopyScheme(fileUrl));
     if (!isLocalFileScheme(schemeString.get()))
         return toWTFString(adoptWK(WKURLCopyString(fileUrl)));
 
+    WKBundleFrameRef mainFrame = WKBundlePageGetMainFrame(InjectedBundle::shared().page()->page());
+    WKRetainPtr<WKURLRef> mainFrameURL = adoptWK(WKBundleFrameCopyURL(mainFrame));
+    if (!mainFrameURL)
+        mainFrameURL = adoptWK(WKBundleFrameCopyProvisionalURL(mainFrame));
+
     String pathString = toWTFString(adoptWK(WKURLCopyPath(fileUrl)));
-    StringBuilder stringBuilder;
-
-    // Remove the leading path from file urls.
-    const size_t indexBaseName = pathString.reverseFind(divider);
-    if (indexBaseName != notFound) {
-        const size_t indexDirName = pathString.reverseFind(divider, indexBaseName - 1);
-        if (indexDirName != notFound)
-            stringBuilder.append(pathString.substring(indexDirName + 1, indexBaseName - indexDirName - 1));
-        stringBuilder.append(divider);
-        stringBuilder.append(pathString.substring(indexBaseName + 1)); // Filename.
-    } else {
-        stringBuilder.append(divider);
-        stringBuilder.append(pathString); // Return "/pathString".
-    }
-
-    return stringBuilder.toString();
-}
-
-static inline WTF::String urlSuitableForTestResult(WKURLRef fileUrl)
-{
-    if (!fileUrl)
-        return String();
-
-    WKRetainPtr<WKStringRef> schemeString = adoptWK(WKURLCopyScheme(fileUrl));
-    if (!isLocalFileScheme(schemeString.get()))
-        return toWTFString(adoptWK(WKURLCopyString(fileUrl)));
-
-    WTF::String urlString = toWTFString(adoptWK(WKURLCopyString(fileUrl)));
-    const size_t indexBaseName = urlString.reverseFind(divider);
-
-    return (indexBaseName == notFound) ? urlString : urlString.substring(indexBaseName + 1);
+    String mainFrameURLPathString = toWTFString(adoptWK(WKURLCopyPath(mainFrameURL.get())));
+    String basePath = mainFrameURLPathString.substring(0, mainFrameURLPathString.reverseFind(divider) + 1);
+    
+    if (!basePath.isEmpty() && pathString.startsWith(basePath))
+        return pathString.substring(basePath.length());
+    return toWTFString(adoptWK(WKURLCopyLastPathComponent(fileUrl))); // We lose some information here, but it's better than exposing a full path, which is always machine specific.
 }
 
 static HashMap<uint64_t, String> assignedUrlsCache;
@@ -288,9 +268,8 @@ InjectedBundlePage::InjectedBundlePage(WKBundlePageRef page)
     : m_page(page)
     , m_world(AdoptWK, WKBundleScriptWorldCreateWorld())
 {
-    WKBundlePageLoaderClient loaderClient = {
-        kWKBundlePageLoaderClientCurrentVersion,
-        this,
+    WKBundlePageLoaderClientV7 loaderClient = {
+        { 7, this },
         didStartProvisionalLoadForFrame,
         didReceiveServerRedirectForProvisionalLoadForFrame,
         didFailProvisionalLoadWithErrorForFrame,
@@ -310,7 +289,7 @@ InjectedBundlePage::InjectedBundlePage(WKBundlePageRef page)
         willPerformClientRedirectForFrame,
         didHandleOnloadEventsForFrame,
         0, // didLayoutForFrame
-        0, // didNewFirstVisuallyNonEmptyLayoutForFrame
+        0, // didNewFirstVisuallyNonEmptyLayout_unavailable
         didDetectXSSForFrame,
         0, // shouldGoToBackForwardListItem
         0, // didCreateGlobalObjectForFrame
@@ -322,12 +301,15 @@ InjectedBundlePage::InjectedBundlePage(WKBundlePageRef page)
         0, // didReceiveIntentForFrame
         0, // registerIntentServiceForFrame
         0, // didLayout
+        0, // featuresUsedInPage
+        0, // willLoadURLRequest
+        0, // willLoadDataRequest
+        0, // willDestroyFrame
     };
-    WKBundlePageSetPageLoaderClient(m_page, &loaderClient);
+    WKBundlePageSetPageLoaderClient(m_page, &loaderClient.base);
 
-    WKBundlePageResourceLoadClient resourceLoadClient = {
-        kWKBundlePageResourceLoadClientCurrentVersion,
-        this,
+    WKBundlePageResourceLoadClientV1 resourceLoadClient = {
+        { 1, this },
         didInitiateLoadForResource,
         willSendRequestForFrame,
         didReceiveResponseForResource,
@@ -337,21 +319,19 @@ InjectedBundlePage::InjectedBundlePage(WKBundlePageRef page)
         shouldCacheResponse,
         0 // shouldUseCredentialStorage
     };
-    WKBundlePageSetResourceLoadClient(m_page, &resourceLoadClient);
+    WKBundlePageSetResourceLoadClient(m_page, &resourceLoadClient.base);
 
-    WKBundlePagePolicyClient policyClient = {
-        kWKBundlePagePolicyClientCurrentVersion,
-        this,
+    WKBundlePagePolicyClientV0 policyClient = {
+        { 0, this },
         decidePolicyForNavigationAction,
         decidePolicyForNewWindowAction,
         decidePolicyForResponse,
         unableToImplementPolicy
     };
-    WKBundlePageSetPolicyClient(m_page, &policyClient);
+    WKBundlePageSetPolicyClient(m_page, &policyClient.base);
 
-    WKBundlePageUIClient uiClient = {
-        kWKBundlePageUIClientCurrentVersion,
-        this,
+    WKBundlePageUIClientV2 uiClient = {
+        { 2, this },
         willAddMessageToConsole,
         willSetStatusbarText,
         willRunJavaScriptAlert,
@@ -368,16 +348,15 @@ InjectedBundlePage::InjectedBundlePage(WKBundlePageRef page)
         0, /*toolbarsAreVisible*/
         didReachApplicationCacheOriginQuota,
         didExceedDatabaseQuota,
-        0, /*plugInStartLabelImage*/
         0, /*plugInStartLabelTitle*/
         0, /*plugInStartLabelSubtitle*/
         0, /*plugInExtraStyleSheet*/
+        0, /*plugInExtraScript*/
     };
-    WKBundlePageSetUIClient(m_page, &uiClient);
+    WKBundlePageSetUIClient(m_page, &uiClient.base);
 
-    WKBundlePageEditorClient editorClient = {
-        kWKBundlePageEditorClientCurrentVersion,
-        this,
+    WKBundlePageEditorClientV1 editorClient = {
+        { 1, this },
         shouldBeginEditing,
         shouldEndEditing,
         shouldInsertNode,
@@ -393,12 +372,11 @@ InjectedBundlePage::InjectedBundlePage(WKBundlePageRef page)
         0, /* getPasteboardDataForRange */
         0  /* didWriteToPasteboard */
     };
-    WKBundlePageSetEditorClient(m_page, &editorClient);
+    WKBundlePageSetEditorClient(m_page, &editorClient.base);
 
 #if ENABLE(FULLSCREEN_API)
-    WKBundlePageFullScreenClient fullScreenClient = {
-        kWKBundlePageFullScreenClientCurrentVersion,
-        this,
+    WKBundlePageFullScreenClientV1 fullScreenClient = {
+        { 1, this },
         supportsFullScreen,
         enterFullScreenForElement,
         exitFullScreenForElement,
@@ -406,7 +384,7 @@ InjectedBundlePage::InjectedBundlePage(WKBundlePageRef page)
         beganExitFullScreen,
         closeFullScreen,
     };
-    WKBundlePageSetFullScreenClient(m_page, &fullScreenClient);
+    WKBundlePageSetFullScreenClient(m_page, &fullScreenClient.base);
 #endif
 }
 
@@ -440,11 +418,7 @@ void InjectedBundlePage::resetAfterTest()
 {
     WKBundleFrameRef frame = WKBundlePageGetMainFrame(m_page);
     JSGlobalContextRef context = WKBundleFrameGetJavaScriptContext(frame);
-#if PLATFORM(QT)
-    DumpRenderTreeSupportQt::resetInternalsObject(context);
-#else
     WebCoreTestSupport::resetInternalsObject(context);
-#endif
     assignedUrlsCache.clear();
 }
 
@@ -495,7 +469,7 @@ static inline void dumpRequestDescriptionSuitableForTestResult(WKURLRequestRef r
     stringBuilder.appendLiteral("<NSURLRequest URL ");
     stringBuilder.append(pathSuitableForTestResult(url.get()));
     stringBuilder.appendLiteral(", main document URL ");
-    stringBuilder.append(urlSuitableForTestResult(firstParty.get()));
+    stringBuilder.append(pathSuitableForTestResult(firstParty.get()));
     stringBuilder.appendLiteral(", http method ");
 
     if (WKStringIsEmpty(httpMethod.get()))
@@ -607,7 +581,7 @@ void InjectedBundlePage::didCancelClientRedirectForFrame(WKBundlePageRef page, W
 
 void InjectedBundlePage::willPerformClientRedirectForFrame(WKBundlePageRef page, WKBundleFrameRef frame, WKURLRef url, double delay, double date, const void* clientInfo)
 {
-    static_cast<InjectedBundlePage*>(const_cast<void*>(clientInfo))->willPerformClientRedirectForFrame(frame, url, delay, date);
+    static_cast<InjectedBundlePage*>(const_cast<void*>(clientInfo))->willPerformClientRedirectForFrame(page, frame, url, delay, date);
 }
 
 void InjectedBundlePage::didSameDocumentNavigationForFrame(WKBundlePageRef page, WKBundleFrameRef frame, WKSameDocumentNavigationType type, WKTypeRef*, const void* clientInfo)
@@ -816,6 +790,10 @@ static void dumpDescendantFramesText(WKBundleFrameRef frame, StringBuilder& stri
         WKBundleFrameRef subframe = static_cast<WKBundleFrameRef>(WKArrayGetItemAtIndex(childFrames.get(), i));
         WKRetainPtr<WKStringRef> subframeName(AdoptWK, WKBundleFrameCopyName(subframe));
 
+        // DumpRenderTree ignores empty frames, so do the same thing here.
+        if (!hasDocumentElement(subframe))
+            continue;
+
         stringBuilder.appendLiteral("\n--------\nFrame: '");
         stringBuilder.append(toWTFString(subframeName));
         stringBuilder.appendLiteral("'\n--------\n");
@@ -832,6 +810,17 @@ void InjectedBundlePage::dumpAllFramesText(StringBuilder& stringBuilder)
     dumpDescendantFramesText(frame, stringBuilder);
 }
 
+
+void InjectedBundlePage::dumpDOMAsWebArchive(WKBundleFrameRef frame, StringBuilder& stringBuilder)
+{
+#if USE(CF)
+    WKRetainPtr<WKDataRef> wkData = adoptWK(WKBundleFrameCopyWebArchive(frame));
+    RetainPtr<CFDataRef> cfData = adoptCF(CFDataCreate(0, WKDataGetBytes(wkData.get()), WKDataGetSize(wkData.get())));
+    RetainPtr<CFStringRef> cfString = adoptCF(createXMLStringFromWebArchiveData(cfData.get()));
+    stringBuilder.append(cfString.get());
+#endif
+}
+
 void InjectedBundlePage::dump()
 {
     ASSERT(InjectedBundle::shared().isTestRunning());
@@ -843,16 +832,20 @@ void InjectedBundlePage::dump()
     WKBundlePageForceRepaint(m_page);
 
     WKBundleFrameRef frame = WKBundlePageGetMainFrame(m_page);
-    WTF::String url = toWTFString(adoptWK(WKURLCopyString(adoptWK(WKBundleFrameCopyURL(frame)).get())));
-    if (url.find("dumpAsText/") != WTF::notFound)
+    WKRetainPtr<WKURLRef> urlRef = adoptWK(WKBundleFrameCopyURL(frame));
+    String url = toWTFString(adoptWK(WKURLCopyString(urlRef.get())));
+    WKRetainPtr<WKStringRef> mimeType = adoptWK(WKBundleFrameCopyMIMETypeForResourceWithURL(frame, urlRef.get()));
+    if (url.find("dumpAsText/") != notFound || WKStringIsEqualToUTF8CString(mimeType.get(), "text/plain"))
         InjectedBundle::shared().testRunner()->dumpAsText(false);
 
     StringBuilder stringBuilder;
 
     switch (InjectedBundle::shared().testRunner()->whatToDump()) {
     case TestRunner::RenderTree: {
-        WKRetainPtr<WKStringRef> text(AdoptWK, WKBundlePageCopyRenderTreeExternalRepresentation(m_page));
-        stringBuilder.append(toWTFString(text));
+        if (InjectedBundle::shared().testRunner()->isPrinting())
+            stringBuilder.append(toWTFString(adoptWK(WKBundlePageCopyRenderTreeExternalRepresentationForPrinting(m_page)).get()));
+        else
+            stringBuilder.append(toWTFString(adoptWK(WKBundlePageCopyRenderTreeExternalRepresentation(m_page)).get()));
         break;
     }
     case TestRunner::MainFrameText:
@@ -863,6 +856,9 @@ void InjectedBundlePage::dump()
         break;
     case TestRunner::Audio:
         break;
+    case TestRunner::DOMAsWebArchive:
+        dumpDOMAsWebArchive(frame, stringBuilder);
+        break;
     }
 
     if (InjectedBundle::shared().testRunner()->shouldDumpAllFrameScrollPositions())
@@ -951,11 +947,7 @@ void InjectedBundlePage::didClearWindowForFrame(WKBundleFrameRef frame, WKBundle
     InjectedBundle::shared().textInputController()->makeWindowObject(context, window, &exception);
     InjectedBundle::shared().accessibilityController()->makeWindowObject(context, window, &exception);
 
-#if PLATFORM(QT)
-    DumpRenderTreeSupportQt::injectInternalsObject(context);
-#else
     WebCoreTestSupport::injectInternalsObject(context);
-#endif
 }
 
 void InjectedBundlePage::didCancelClientRedirectForFrame(WKBundleFrameRef frame)
@@ -969,7 +961,7 @@ void InjectedBundlePage::didCancelClientRedirectForFrame(WKBundleFrameRef frame)
     dumpLoadEvent(frame, "didCancelClientRedirectForFrame");
 }
 
-void InjectedBundlePage::willPerformClientRedirectForFrame(WKBundleFrameRef frame, WKURLRef url, double delay, double date)
+void InjectedBundlePage::willPerformClientRedirectForFrame(WKBundlePageRef, WKBundleFrameRef frame, WKURLRef url, double delay, double date)
 {
     if (!InjectedBundle::shared().isTestRunning())
         return;
@@ -1035,14 +1027,11 @@ void InjectedBundlePage::didDetectXSSForFrame(WKBundleFrameRef frame)
         InjectedBundle::shared().outputText("didDetectXSS\n");
 }
 
-void InjectedBundlePage::didInitiateLoadForResource(WKBundlePageRef, WKBundleFrameRef, uint64_t identifier, WKURLRequestRef request, bool)
+void InjectedBundlePage::didInitiateLoadForResource(WKBundlePageRef page, WKBundleFrameRef, uint64_t identifier, WKURLRequestRef request, bool)
 {
     if (!InjectedBundle::shared().isTestRunning())
         return;
 
-    if (!InjectedBundle::shared().testRunner()->shouldDumpResourceLoadCallbacks())
-        return;
-
     WKRetainPtr<WKURLRef> url = adoptWK(WKURLRequestCopyURL(request));
     assignedUrlsCache.add(identifier, pathSuitableForTestResult(url.get()));
 }
@@ -1059,7 +1048,7 @@ static inline bool isHTTPOrHTTPSScheme(WKStringRef scheme)
     return WKStringIsEqualToUTF8CStringIgnoringCase(scheme, "http") || WKStringIsEqualToUTF8CStringIgnoringCase(scheme, "https");
 }
 
-WKURLRequestRef InjectedBundlePage::willSendRequestForFrame(WKBundlePageRef, WKBundleFrameRef frame, uint64_t identifier, WKURLRequestRef request, WKURLResponseRef response)
+WKURLRequestRef InjectedBundlePage::willSendRequestForFrame(WKBundlePageRef page, WKBundleFrameRef frame, uint64_t identifier, WKURLRequestRef request, WKURLResponseRef response)
 {
     if (InjectedBundle::shared().isTestRunning()
         && InjectedBundle::shared().testRunner()->shouldDumpResourceLoadCallbacks()) {
@@ -1115,7 +1104,7 @@ WKURLRequestRef InjectedBundlePage::willSendRequestForFrame(WKBundlePageRef, WKB
     return request;
 }
 
-void InjectedBundlePage::didReceiveResponseForResource(WKBundlePageRef, WKBundleFrameRef, uint64_t identifier, WKURLResponseRef response)
+void InjectedBundlePage::didReceiveResponseForResource(WKBundlePageRef page, WKBundleFrameRef, uint64_t identifier, WKURLResponseRef response)
 {
     if (!InjectedBundle::shared().isTestRunning())
         return;
@@ -1453,8 +1442,26 @@ uint64_t InjectedBundlePage::didExceedDatabaseQuota(WKSecurityOriginRef origin,
         InjectedBundle::shared().outputText(stringBuilder.toString());
     }
 
-    static const uint64_t defaultQuota = 5 * 1024 * 1024;
-    return defaultQuota;
+    uint64_t defaultQuota = 5 * 1024 * 1024;
+    double testDefaultQuota = InjectedBundle::shared().testRunner()->databaseDefaultQuota();
+    if (testDefaultQuota >= 0)
+        defaultQuota = testDefaultQuota;
+
+    unsigned long long newQuota = defaultQuota;
+
+    double maxQuota = InjectedBundle::shared().testRunner()->databaseMaxQuota();
+    if (maxQuota >= 0) {
+        if (defaultQuota < expectedUsageBytes && expectedUsageBytes <= maxQuota) {
+            newQuota = expectedUsageBytes;
+
+            StringBuilder stringBuilder;
+            stringBuilder.appendLiteral("UI DELEGATE DATABASE CALLBACK: increased quota to ");
+            stringBuilder.appendNumber(newQuota);
+            stringBuilder.append('\n');
+            InjectedBundle::shared().outputText(stringBuilder.toString());
+        }
+    }
+    return newQuota;
 }
 
 // Editor Client Callbacks
